/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.processus;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.commentaire.om.Commentaire;
import com.kosmos.commentaire.util.CommentaireUtil;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.utils.Chaine;
import com.univ.utils.EscapeString;

/**
 * Processus de gestion des commentaire en BO.
 */
public class SaisieCommentaire extends SaisieFiche {

    private static final Logger LOGGER = LoggerFactory.getLogger(SaisieCommentaire.class);

    /** The commentaire. */
    protected Commentaire commentaire = null;

    /**
     * Processus saisie Commentaire.
     *
     * @param ciu the infobean
     */
    public SaisieCommentaire(final InfoBean ciu) {
        super(ciu);
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     * @throws Exception the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0010");
        commentaire = new Commentaire();
        commentaire.init();
        traiterActionParDefaut(commentaire);
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
    }

    /**
     * Affichage de l'écran de saisie d'un Commentaire.
     *
     * @throws Exception the exception
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        if (infoBean.getString("ID_META") != null) {
            commentaire.setIdMeta(new Long(infoBean.getString("ID_META")));
        }
        try {
            FicheUniv fichePrincipale = commentaire.renvoyerFicheParente();
            infoBean.set("LIBELLE_ID_META", ReferentielObjets.getLibelleObjet(ReferentielObjets.getCodeObjetParClasse(fichePrincipale.getClass().getName())) + " : " + fichePrincipale.getLibelleAffichable());
        } catch (final Exception e) {
            LOGGER.debug("La fiche parente n'a pas été trouvée", e);
            infoBean.addMessageAlerte("La fiche parente n'a pas été trouvée");
        }
        // si on est en train de saisir dans le front, on prepare l'affichage des donnees
        infoBean.set("ID_COMMENTAIRE", commentaire.getIdCommentaire().toString());
        infoBean.set("TEXTE", supprimeHTMLLiensEtEmails(commentaire.getTexte()));
        infoBean.set("CODE_RUBRIQUE", commentaire.getCodeRubrique());
        infoBean.set("AUTEUR", commentaire.getAuteur());
        infoBean.set("EMAIL_AUTEUR", commentaire.getEmailAuteur());
        infoBean.set("URL_SITE", commentaire.getUrlSite());
        infoBean.set("GRS_SAISIE_ENCADRES", "0");
        infoBean.set("GRS_APERCU", "0");
        infoBean.set("LIBELLE_AFFICHABLE", commentaire.getLibelleAffichable());
        infoBean.set("SOUS_ONGLET", "PRINCIPAL");
        ControleurUniv.preparerPRINCIPAL(infoBean, commentaire, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        if (commentaire.getIdMeta().equals(0L)) {
            if (StringUtils.isEmpty(infoBean.getString("ID_META"))) {
                throw new ErreurApplicative(CommentaireUtil.getMessage("ST_COMMENT_ERREUR_CODEPARENT"));
            }
            //si le parent du commentaire a ete initialise, on recupere la structure du parent
            commentaire.setIdMeta(new Long((String) infoBean.get("ID_META")));
            if (StringUtils.isNotBlank(infoBean.getString("ID_PARENT"))) {
                commentaire.setIdParent(new Long((String) infoBean.get("ID_PARENT")));
            }
            if (StringUtils.isNotBlank(infoBean.getString("CODE_RATTACHEMENT"))) {
                commentaire.setCodeRattachement((String) infoBean.get("CODE_RATTACHEMENT"));
            }
            if (StringUtils.isNotBlank(infoBean.getString("CODE_RATTACHEMENT_AUTRES"))) {
                commentaire.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
            }
            if (StringUtils.isNotBlank(infoBean.getString("CODE_RUBRIQUE"))) {
                commentaire.setCodeRubrique((String) infoBean.get("CODE_RUBRIQUE"));
            }
        }
        final String text = EscapeString.escapeHtml(infoBean.getString("TEXTE"));
        commentaire.setTexte(Chaine.transformeEnHtml(text));
        commentaire.setAuteur((String) infoBean.get("AUTEUR"));
        commentaire.setEmailAuteur(infoBean.getString("EMAIL_AUTEUR"));
        commentaire.setUrlSite((String) infoBean.get("URL_SITE"));
    }

    /**
     * Gestion de l'écran de saisie d'un Commentaire.
     *
     * @throws Exception the exception
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            commentaire.init();
        } else {
            commentaire.setIdCommentaire(new Long(infoBean.getString("ID_COMMENTAIRE")));
            commentaire.retrieve();
        }
        if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(commentaire, false);
            alimenterDonneesFiche();
            final String typeEnregistement = (String) infoBean.get("TYPE_ENREGISTREMENT");
            //si le commentaire est en demande de validation et que l'enregistrement se fait dans le back
            if (commentaire.getEtatObjet().equals(EtatFiche.A_VALIDER.getEtat())) {
                final String mailAuteur = commentaire.getEmailAuteur();
                //si l'auteur a saisi un email
                if (mailAuteur.length() > 0) {
                    /* le mail doit tenir compte de la langue dans laquelle a ete saisi le commentaire */
                    if (StringUtils.equals(typeEnregistement, "3")) {
                        //le commentaire est supprime, pas de retour possible
                        infoBean.setActionUtilisateur(InfoBean.ACTION_SUPPRIMER);
                        final JSBMailbox mailbox = new JSBMailbox(false);
                        String message = CommentaireUtil.getMessage("ST_DEBUT_MAIL_REFUS") + "\"" + commentaire.getLibelleAffichable() + "\".\n";
                        if (infoBean.getString("MOTIF_RETOUR").length() > 0) {
                            message += CommentaireUtil.getMessage("ST_MOTIF_MAIL_REFUS") + " \n";
                            message += infoBean.getString("MOTIF_RETOUR") + ".\n\n";
                        } else {
                            message += CommentaireUtil.getMessage("ST_SANSMOTIF_MAIL_REFUS") + "\n";
                        }
                        message += CommentaireUtil.getMessage("ST_FIN_MAIL_REFUS");
                        mailbox.sendSystemMsg(mailAuteur, CommentaireUtil.getMessage("ST_SUJET_MAIL_REFUS"), message);
                        //envoi d'un mail pour dire que le commentaire a ete publie
                    } else if (StringUtils.equals(typeEnregistement, "2") && commentaire.getEtatObjet().equals(EtatFiche.A_VALIDER.getEtat())) {
                        final JSBMailbox mailbox = new JSBMailbox(false);
                        String message = CommentaireUtil.getMessage("ST_DEBUT_MAIL") + "\"" + commentaire.getLibelleAffichable() + "\"" + CommentaireUtil.getMessage("ST_MAIL_PUBLIE") + "\n" + CommentaireUtil.getMessage("ST_FIN_MAIL");
                        mailbox.sendSystemMsg(mailAuteur, CommentaireUtil.getMessage("ST_SUJET_MAIL"), message);
                    }
                } else {
                    // refus de validation
                    if (StringUtils.equals(typeEnregistement, "3")) {
                        //le commentaire est supprime, pas de retour possible
                        infoBean.setActionUtilisateur(InfoBean.ACTION_SUPPRIMER);
                    }
                }
            }
        }
        /*************************************/
        /* Appel traitement général          */
        /*************************************/
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, commentaire, this);
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }

    /**
     * Supprime les balises HTML autour des liens et des adresses e-mail (fait le traitement inverse de Chaine.transformeEnHtml => A déplacer dans cette classe ?)
     *
     * @param texte Texte à modifier
     * @return Le texte modifié
     */
    private String supprimeHTMLLiensEtEmails(final String texte) {
        return texte.replaceAll("<a href=[^>]+>([^<]*)</a>", "$1");
    }
}
