/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: dam
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.dam;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.univ.mediatheque.Mediatheque;

public class MediaUploadJob extends LogReportJob {

    private static final Logger LOG = LoggerFactory.getLogger(MediaUploadJob.class);

    private static final int MINIMUM_TIME_TO_PROCESS = 120000;

    /* (non-Javadoc)
             * @see java.lang.Runnable#run()
             */
    @Override
    public void perform() {
        Mediatheque mediatheque = Mediatheque.getInstance();
        InfoBean infoBean = new InfoBean();
        try {
            if (mediatheque == null) {
                throw new ErreurApplicative("Impossible de lancer le job d'upload de medias : mediatheque is null.");
            }
            // build work directory
            String uploadPath = mediatheque.getUploadPath();
            if (uploadPath.length() == 0) {
                uploadPath = WebAppUtil.getTempPath() + File.separator + "autoupload";
            }
            LOG.info("Répertoire source : " + uploadPath);
            File directory = new File(uploadPath);
            if (directory.exists()) {
                File[] files = directory.listFiles();
                if (files != null && files.length > 0) {
                    infoBean.set("AUTO_UPLOAD", "true");
                    String codeRedacteur = "ftp";
                    // lecture du fichier de properties
                    File propFile = new File(uploadPath + File.separator + "autoupload.properties");
                    if (propFile.exists()) {
                        Properties props = new Properties();
                        try (FileInputStream isProp = new FileInputStream(propFile);) {
                            props.load(isProp);
                            // test des propriétés
                            if (props.getProperty("CODE_REDACTEUR") != null) {
                                codeRedacteur = props.getProperty("CODE_REDACTEUR");
                            }
                            if (props.getProperty("CODE_RUBRIQUE") != null) {
                                infoBean.set("CODE_RUBRIQUE", props.getProperty("CODE_RUBRIQUE"));
                            }
                            if (props.getProperty("CODE_RATTACHEMENT") != null) {
                                infoBean.set("CODE_RATTACHEMENT", props.getProperty("CODE_RATTACHEMENT"));
                            }
                            if (props.getProperty("CODE_RUBRIQUE") != null) {
                                infoBean.set("CODE_RUBRIQUE", props.getProperty("CODE_RUBRIQUE"));
                            }
                            if (props.getProperty("THEMATIQUE") != null) {
                                infoBean.set("THEMATIQUE", props.getProperty("THEMATIQUE"));
                            }
                            if (props.getProperty("TITRE") != null) {
                                infoBean.set("TITRE", props.getProperty("TITRE"));
                            }
                            if (props.getProperty("LEGENDE") != null) {
                                infoBean.set("LEGENDE", props.getProperty("LEGENDE"));
                            }
                            if (props.getProperty("DESCRIPTION") != null) {
                                infoBean.set("DESCRIPTION", props.getProperty("DESCRIPTION"));
                            }
                            if (props.getProperty("AUTEUR") != null) {
                                infoBean.set("AUTEUR", props.getProperty("AUTEUR"));
                            }
                            if (props.getProperty("META_KEYWORDS") != null) {
                                infoBean.set("META_KEYWORDS", props.getProperty("META_KEYWORDS"));
                            }
                            if (props.getProperty("COPYRIGHT") != null) {
                                infoBean.set("COPYRIGHT", props.getProperty("COPYRIGHT"));
                            }
                        }
                    }
                    long timeToProcess = System.currentTimeMillis();
                    // process upload
                    mediatheque.uploadMediasByFolder( infoBean, directory, false, codeRedacteur);
                    timeToProcess = System.currentTimeMillis() - timeToProcess;
                    if (timeToProcess < MINIMUM_TIME_TO_PROCESS) {
                        timeToProcess = MINIMUM_TIME_TO_PROCESS;
                    }
                    // clean work directory
                    for (File file : files) {
                        performFileDeletion(timeToProcess, file);
                    }
                } else {
                    LOG.info("Le répertoire source est vide");
                    LOG.info("Aucun fichier n'a été uploadé");
                }
            } else {
                directory.mkdir();
                LOG.info("Le répertoire source a été crée");
                LOG.info("Aucun fichier n'a été uploadé");
            }
            if (infoBean.get("MSG_ERROR") != null) {
                LOG.info(infoBean.getString("MSG_ERROR"));
            }
        } catch (ErreurApplicative | IOException e) {
            LOG.error("erreur lors de l'execution du job d'upload de média", e);
        }
    }

    private void performFileDeletion(final long timeToProcess, final File file) {
        if (file != null && file.exists() && file.isFile()) {
            try {
                // on ne supprime pas les fichiers modifiés dans l'intervalle
                if ((System.currentTimeMillis() - file.lastModified()) > timeToProcess) {
                    file.delete();
                }
            } catch (SecurityException se) {
                LOG.debug("suppression du fichier impossible", se);
            }
        }
    }
}
