/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.processus;

import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.module.ModuleFil;
import com.kosmos.fil.service.impl.ServiceFil;
import com.kosmos.fil.util.FilUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.extension.module.ModuleHelper;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.Perimetre;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;

/**
 * processus saisie Fil.
 */
public class SaisieFil extends ProcessusBean {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieFil.class);

    /** The Constant MODE_AJOUT. */
    private static final int MODE_AJOUT = 0;

    /** The Constant MODE_MODIFICATION. */
    private static final int MODE_MODIFICATION = 1;

    /** The Constant MODE_SUPPRESSION. */
    private static final int MODE_SUPPRESSION = 2;

    /** The Constant ECRAN_PRINCIPAL. */
    private static final String ECRAN_PRINCIPAL = "PRINCIPAL";

    /** The Constant ECRAN_ACCUEIL. */
    private static final String ECRAN_ACCUEIL = "ACCUEIL";

    /** The Constant ECRAN_LISTE. */
    private static final String ECRAN_LISTE = "LISTE";

    /** The Constant ECRAN_TYPEFIL. */
    private static final String ECRAN_TYPEFIL = "TYPEFIL";

    private final ServiceFil serviceFil;

    /** The liste identifiant. */
    private final Long listeIdentifiant[] = null;

    /** The mode. */
    private int mode = -1;

    /** The fil. */
    protected FilBean fil;

    /** The autorisations. */
    private AutorisationBean autorisations = null;

    /**
     * processus saisie Fil.
     *
     * @param ciu
     *            com.jsbsoft.jtf.core.InfoBean
     */
    public SaisieFil(final InfoBean ciu) {
        super(ciu);
        serviceFil = ServiceManager.getServiceForBean(FilBean.class);
    }

    /**
     * Initialisation du processus.
     *
     * @throws Exception
     *             the exception
     */
    private void initialisation() throws Exception {
        fil = new FilBean();
        etat = EN_COURS;
        if (action.equals("AJOUTER")) {
            if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "C"))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            }
            mode = MODE_AJOUT;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
            if (infoBean.get("TYPE_FIL") != null) {
                preparerPRINCIPAL();
            } else {
                preparerLISTETYPES();
            }
        }
        if (action.equals("MODIFIERFILPARCODE")) {
            mode = MODE_MODIFICATION;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
            final String code = infoBean.getString("CODE_FIL");
            fil = serviceFil.getByCode(code);
            if (fil == null) {
                throw new ErreurApplicative(FilUtil.getMessage("LISTE_MANUELLE.ERREUR.FIL_INEXISTANT"));
            }
            if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "M"), new Perimetre(fil.getCodeRattachement(), fil.getCodeRubrique(), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            }
            preparerPRINCIPAL();
        }
        /* JSS 20040419 : suppression d'un fil par le code */
        if (action.equals("SUPPRIMERFILPARCODE")) {
            mode = MODE_SUPPRESSION;
            final String code = infoBean.getString("CODE_FIL");
            fil = serviceFil.getByCode(code);
            if (fil == null) {
                throw new ErreurApplicative(FilUtil.getMessage("LISTE_MANUELLE.ERREUR.FIL_INEXISTANT"));
            }
            if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "S"), new Perimetre(fil.getCodeRattachement(), fil.getCodeRubrique(), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            }
            serviceFil.delete(fil.getId());
            etat = FIN;
        } else if (action.equals("ACCUEIL")) {
            ecranLogique = ECRAN_ACCUEIL;
        }
        // Pour restrictions affichage des arbres rubriques et structures dans UnivFmt
        this.getDatas().put("AUTORISATIONS", autorisations);
    }

    /**
     * Affichage de l'écran de saisie d'un Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerPRINCIPAL() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("CODE", fil.getCode());
        infoBean.set("CODE_RATTACHEMENT", fil.getCodeRattachement());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(fil.getCodeRattachement(), fil.getLangue()));
        // JSS 20040409 : Ajout rubrique
        infoBean.set("CODE_RUBRIQUE", fil.getCodeRubrique());
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(fil.getCodeRubrique());
        String rubriqueLabel = StringUtils.EMPTY;
        if (rubriqueBean != null) {
            rubriqueLabel = rubriqueBean.getIntitule();
        }
        infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueLabel);
        // filtre des arbres
        infoBean.set("GRS_FILTRE_ARBRE_NOM_CODE_RUBRIQUE", "CODE_RUBRIQUE");
        infoBean.set("GRS_FILTRE_ARBRE_NOM_CODE_RATTACHEMENT", "CODE_RATTACHEMENT");
        infoBean.set("GRS_PERMISSION_TYPE", "TECH");
        infoBean.set("GRS_PERMISSION_OBJET", "fil");
        if (mode == MODE_AJOUT) {
            infoBean.set("GRS_PERMISSION_ACTION", "C");
        } else {
            infoBean.set("GRS_PERMISSION_ACTION", "M");
        }
        infoBean.set("INTITULE", fil.getIntitule());
        infoBean.set("LANGUE", fil.getLangue());
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        //infoBean.set("LISTE_TYPE_FIL", FilUtil.renvoyerTypesFils());
        infoBean.set("TYPE_FIL", fil.getTypeFil());
        infoBean.set("LIBELLE_TYPE_FIL", serviceFil.getLibelleTypeFil(fil.getTypeFil()));
        /* Liste des fiches rattachées au fil :
            syntaxe : ordre,code,langue;
         */
        infoBean.set("FICHE", fil.getListeFiches());
        String libelleFiches = "";
        final StringTokenizer st = new StringTokenizer(fil.getListeFiches(), ";");
        if (StringUtils.isNotBlank(fil.getTypeFil())) {
            String code = null;
            String langue = null;
            final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(fil.getTypeFil());
            ficheUniv.init();
            ficheUniv.setCtx(this);
            String libelle = "";
            while (st.hasMoreTokens()) {
                final String codelangue = st.nextToken();
                final StringTokenizer st2 = new StringTokenizer(codelangue, ",");
                //code de la fiche
                if (st2.hasMoreTokens()) {
                    code = st2.nextToken();
                }
                if (code != null) {
                    //langue de la fiche
                    if (st2.hasMoreTokens()) {
                        langue = st2.nextToken().substring(7);
                    }
                    if (langue == null) {
                        langue = "0";
                    }
                    int nbFiche = ficheUniv.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
                    if (nbFiche == 0) {
                        ficheUniv.selectCodeLangueEtat(code, langue, "");
                    }
                    //ajouter le libellé de la fiche
                    if (ficheUniv.nextItem()) {
                        libelle = ficheUniv.getLibelleAffichable();
                        if (!ficheUniv.getEtatObjet().equals("0003")) {
                            libelle += " (Etat : " + ReferentielObjets.getLibelleEtatObjet(ficheUniv.getEtatObjet()).trim() + ")";
                        }
                        if (libelleFiches.length() > 0) {
                            libelleFiches += ";";
                        }
                        libelleFiches += libelle;
                    } else {
                        if (libelleFiches.length() > 0) {
                            libelleFiches += ";";
                        }
                        libelleFiches += "(Fiche inexistante à supprimer)";
                    }
                }
            }
        }
        infoBean.set("LIBELLE_FICHE", libelleFiches);
        if (StringUtils.isNotBlank(fil.getIntitule())) {
            infoBean.setTitreEcran(fil.getIntitule());
        }
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        // JSS 20040419 : délégation
        autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations == null) {
            infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
            infoBean.setEcranLogique("LOGIN");
        } else {
            try {
                ecranLogique = infoBean.getEcranLogique();
                action = infoBean.getActionUtilisateur();
                if (etat == DEBUT) {
                    initialisation();
                } else if (ecranLogique.equals(ECRAN_LISTE)) {
                    traiterLISTE();
                } else if (ecranLogique.equals(ECRAN_TYPEFIL)) {
                    traiterLISTETYPES();
                } else if (ecranLogique.equals(ECRAN_PRINCIPAL)) {
                    traiterPRINCIPAL();
                }
                //placer l'état dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
            } catch (final Exception e) {
                LOG.error("Erreur sur le traitement des fils", e);
                infoBean.addMessageErreur(e.toString());
            }
        }
        // On continue si on n'est pas à la FIN !!!
        return etat == FIN;
    }

    /**
     * Traitement de l'écran de sélection d'un Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterLISTE() throws Exception {
        if (action.equals(InfoBean.ACTION_MODIFIER)) {
            final int indice = Integer.parseInt((String) infoBean.get("LISTE_INDICE"));
            fil = serviceFil.getById(listeIdentifiant[indice]);
            preparerPRINCIPAL();
        }
        if (action.equals(InfoBean.ACTION_VALIDER)) {
            final int indice = Integer.parseInt((String) infoBean.get("LISTE_INDICE"));
            fil = serviceFil.getById(listeIdentifiant[indice]);
            remplirDonneesRecherche();
            etat = FIN;
        }
        if (action.equals(InfoBean.ACTION_SUPPRIMER)) {
            final int indice = Integer.parseInt((String) infoBean.get("LISTE_INDICE"));
            serviceFil.delete(listeIdentifiant[indice]);
            etat = FIN;
        }
        if (action.equals(InfoBean.ACTION_ANNULER)) {
            infoBean.set("ID_FIL", null);
            etat = FIN;
        }
    }

    /**
     * Gestion de l'écran de saisie d'un Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterPRINCIPAL() throws Exception {
        if (action.equals(InfoBean.ACTION_VALIDER)) {
            //JSS 20040419 : test permissions
            if (mode == MODE_AJOUT) {
                fil.setCode(String.valueOf(System.currentTimeMillis()));
                if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "C"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                    throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_CREATION_HORS_PERIMETRE"));
                }
            }
            if (mode == MODE_MODIFICATION) {
                if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "M"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                    throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_MODIFICATION_HORS_PERIMETRE"));
                }
            }
            fil.setCodeRattachement((String) infoBean.get("CODE_RATTACHEMENT"));
            // JSS 20040409 : Ajout rubrique
            fil.setCodeRubrique((String) infoBean.get("CODE_RUBRIQUE"));
            fil.setLangue((String) infoBean.get("LANGUE"));
            fil.setIntitule((String) infoBean.get("INTITULE"));
            fil.setTypeFil((String) infoBean.get("TYPE_FIL"));
            fil.setListeFiches((String) infoBean.get("FICHE"));
            if (mode == MODE_AJOUT) {
                serviceFil.save(fil);
                final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_CREATION_LISTE_MANUELLE"), fil.getIntitule());
                infoBean.addMessageConfirmation(confirmation);
            }
            if (mode == MODE_MODIFICATION) {
                serviceFil.save(fil);
                final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_MODIFICATION_LISTE_MANUELLE"), fil.getIntitule());
                infoBean.addMessageConfirmation(confirmation);
            }
            etat = FIN;
        } else if (action.equals(InfoBean.ACTION_SUPPRIMER)) {
            if (!autorisations.possedePermission(new PermissionBean("TECH", "fil", "S"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            }
            serviceFil.delete(fil.getId());
            final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_SUPPRESSION_LISTE_MANUELLE"), fil.getIntitule());
            infoBean.addMessageConfirmation(confirmation);
            etat = FIN;
        }
    }

    /**
     * Préparation des données à renvoyer pour une rechercher Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void remplirDonneesRecherche() throws Exception {
        infoBean.set("ID_FIL", fil.getId());
        infoBean.set("CODE", fil.getCode());
        infoBean.set("LANGUE", fil.getLangue());
        infoBean.set("INTITULE", fil.getIntitule());
        infoBean.set("TYPE_FIL", fil.getTypeFil());
        infoBean.set("LISTE_FICHES", fil.getListeFiches());
    }

    /**
     * Affichage de l'écran de saisie d'un type de Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerLISTETYPES() throws Exception {
        final Map<String, String> liste = serviceFil.getTypesFils();
        if (liste.size() > 1) {
            ecranLogique = ECRAN_TYPEFIL;
            infoBean.set("LISTE_TYPE_FIL", liste);
        } else if (liste.size() == 1) {
            for(String currentKey : liste.keySet()) {
                fil.setTypeFil(currentKey);
            }
            preparerPRINCIPAL();
        } else {
            final ModuleFil module = (ModuleFil) ModuleHelper.getModule(ModuleFil.ID_EXTENSION, ModuleFil.ID_BEAN);
            infoBean.addMessageErreur(module.getMessage("AUCUN_OBJET_POUR_FIL"));
            etat = FIN;
        }
    }

    /**
     * Affichage de l'écran de saisie d'un type de Fil.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterLISTETYPES() throws Exception {
        if (action.equals(InfoBean.ACTION_SUITE)) {
            fil.setTypeFil((String) infoBean.get("TYPE_FIL"));
            preparerPRINCIPAL();
        }
        if (action.equals(InfoBean.ACTION_ANNULER)) {
            infoBean.set("ID_FIL", null);
            etat = FIN;
        }
    }
}
