/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.dao.impl;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.dao.impl.mysql.AbstractLegacyDAO;
import com.jsbsoft.jtf.datasource.exceptions.AddToDataSourceException;
import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.jsbsoft.jtf.datasource.utils.DaoUtils;
import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.dto.ContenuFormulaireDto;
import com.kosmos.formulaire.dto.FormulaireDto;

/**
 * DAO gérant la liste des formulaires saisies par les webmasters
 */
public class FormulaireDAO extends AbstractLegacyDAO<FormulaireBean> {

    private static final Logger LOG = LoggerFactory.getLogger(FormulaireDAO.class);

    private final RowMapper<FormulaireDto> formulaireDtoRowMapper = (rs, rowNum) -> {
        final FormulaireDto dto = new FormulaireDto();
        dto.setId(rs.getLong("ID_FORMULAIRE"));
        dto.setCode(rs.getString("CODE"));
        dto.setTitre(rs.getString("TITRE"));
        dto.setCodeRubrique(rs.getString("CODE_RUBRIQUE"));
        dto.setCodeRattachement(rs.getString("CODE_RATTACHEMENT"));
        dto.setModele(rs.getString("MODELE"));
        dto.setNombreEnregistrements(rs.getLong("NB_ENREG"));
        dto.setDureeConservation(rs.getInt("DUREE_CONSERVATION"));
        return dto;
    };

    private final RowMapper<ContenuFormulaireDto> contenuFormulaireDtoRowMapper = (rs, rowNum) -> {
        final ContenuFormulaireDto dto = new ContenuFormulaireDto();
        dto.setId(rs.getLong("ID_CONTENUFORMULAIRE"));
        dto.setCode(rs.getString("CODE"));
        dto.setCodeRubrique(rs.getString("CODE_RUBRIQUE"));
        dto.setCodeRattachement(rs.getString("CODE_RATTACHEMENT"));
        dto.setTitre(rs.getString("TITRE"));
        dto.setCodeUnique(rs.getString("CODE_UNIQUE"));
        dto.setInstance(rs.getString("INSTANCE"));
        dto.setDateCreation(rs.getDate("DATE_CREATION"));
        return dto;
    };

    public FormulaireDAO() {
        tableName = "FORMULAIRE";
    }

    /**
     * Méthode permettant de récupérer la liste des formulaires
     * @return Une liste de {@link FormulaireBean}
     */
    public List<FormulaireBean> getAll(final String orderByColumn) {
        List<FormulaireBean> forms;
        try {
            final StringBuilder query = new StringBuilder("SELECT * FROM FORMULAIRE");
            if(StringUtils.isNotBlank(orderByColumn)) {
                query.append(" ORDER BY ").append(orderByColumn.toUpperCase());
            }
            forms = namedParameterJdbcTemplate.query(query.toString(), rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting FORMULAIRE rows", dae);
        }
        return forms;
    }

    /**
     * Méthode permettant de récupérer le formulaire correspondant à un CONTENUFORMULAIRE, par l'ID du CONTENUFORMULAIRE.
     * @param id L'ID du CONTENUFORMULAIRE dont on souhaite récupérer le FORMULAIRE parent.
     * @return Le formulaire associé à l'ID du CONTENUFORMULAIRE
     */
    public FormulaireBean getByContenuId(long id) {
        FormulaireBean formulaire = null;
        try {
            final SqlParameterSource params = new MapSqlParameterSource("id", id);
            formulaire = namedParameterJdbcTemplate.queryForObject("SELECT F.* FROM FORMULAIRE F JOIN CONTENUFORMULAIRE CF ON F.CODE=CF.CODE_FORMULAIRE WHERE CF.ID_CONTENUFORMULAIRE = :id", params, rowMapper);
        } catch (final EmptyResultDataAccessException e) {
            LOG.debug(String.format("empty resultset size for id %d", id), e);
        } catch (final IncorrectResultSizeDataAccessException e) {
            LOG.debug("incorrect result size for query", e);
            LOG.error(String.format("incorrect result size for query with id %d : %d results found", id, e.getActualSize()));
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting FORMULAIRE rows", dae);
        }
        return formulaire;
    }

    /**
     * Méthode permettant de récupérer le formulaire par son code.
     * @param code le code du formulaire
     * @return Le formulaire
     */
    public FormulaireBean getByCode(String code) {
        FormulaireBean formulaire = null;
        try {
            final SqlParameterSource params = new MapSqlParameterSource("code", code);
            formulaire = namedParameterJdbcTemplate.queryForObject("SELECT F.* FROM FORMULAIRE F WHERE F.CODE = :code", params, rowMapper);
        } catch (final EmptyResultDataAccessException e) {
            LOG.debug(String.format("empty resultset size for code %s", code), e);
        } catch (final IncorrectResultSizeDataAccessException e) {
            LOG.debug("incorrect result size for query", e);
            LOG.error(String.format("incorrect result size for query with code %s : %d results found", code, e.getActualSize()));
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting FORMULAIRE rows", dae);
        }
        return formulaire;
    }

    /**
     * Méthode permettant de récupérer la liste des formulaires avec le nombre de réponses
     * @return Une liste de {@link FormulaireDto}
     */
    public List<FormulaireDto> getAllWithRecordsCount() {
        try {
            final StringBuilder query = new StringBuilder(307)
                .append("SELECT T1.ID_FORMULAIRE, T1.CODE, T1.TITRE, T1.CODE_RUBRIQUE, T1.CODE_RATTACHEMENT, T1.MODELE, T2.NB_ENREG, T1.DUREE_CONSERVATION ")
                .append(" FROM FORMULAIRE T1 ")
                .append(" LEFT OUTER JOIN (SELECT CODE_FORMULAIRE, COUNT(*) NB_ENREG FROM CONTENUFORMULAIRE GROUP BY CODE_FORMULAIRE) T2")
                .append(" ON T2.CODE_FORMULAIRE=T1.CODE")
                .append(" ORDER BY TITRE ");

            return namedParameterJdbcTemplate.query(query.toString(), formulaireDtoRowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting %s rows", tableName), dae);
        }
    }

    /**
     * Méthode permettant de récupérer la liste des formulaires avec les contenus contenant les valeurs en paramètre
     * @param params une MAP de code / valuer à rechercher dans les contenus de formaulaire
     * @return Une liste de {@link ContenuFormulaireDto}
     */
    public List<ContenuFormulaireDto> getAllForContent(final Map<String, String> params) {
        try {
            final MapSqlParameterSource queryParams = new MapSqlParameterSource();
            final StringBuilder query = new StringBuilder(350)
                    .append("SELECT T2.ID_CONTENUFORMULAIRE, T1.CODE, T1.CODE_RUBRIQUE, T1.CODE_RATTACHEMENT, T1.TITRE, T2.CODE_UNIQUE, T2.INSTANCE, T2.DATE_CREATION ")
                    .append(" FROM FORMULAIRE T1 ")
                    .append(" LEFT OUTER JOIN CONTENUFORMULAIRE T2 ON T2.CODE_FORMULAIRE=T1.CODE ");
            boolean firstParam = true;
            for (Map.Entry<String, String > entry : params.entrySet()) {
                if (firstParam) {
                    query.append(" WHERE ");
                    firstParam = false;
                } else {
                    query.append(" AND ");
                }
                query.append(" T2.INSTANCE LIKE :").append(entry.getKey());
                queryParams.addValue(entry.getKey(), "%\n" + entry.getKey() + "="+ entry.getValue() +"%");
            }
            return namedParameterJdbcTemplate.query(query.toString(), queryParams, contenuFormulaireDtoRowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting %s rows", tableName), dae);
        }
    }

    /**
     * Cette méthode permet de persister le bean passé en paramètre en forçant la valeur de son id.
     * @param formulaireBean le bean à persister.
     * @return le bean persisté.
     */
    public FormulaireBean addWithForcedId(FormulaireBean formulaireBean) {
        try {
            namedParameterJdbcTemplate.update(DaoUtils.getAddWithForcedIdQuery(tableName, getColumns(), columnNamingStrategy), getParameters(formulaireBean));
        } catch (DataAccessException dae) {
            throw new AddToDataSourceException(String.format("Unable to add [%s] to table \"%s\"", formulaireBean.toString(), tableName), dae);
        }
        return formulaireBean;
    }

    /**
     * Récupère la liste des formulaires liés à une rubrique.
     * @param sectionCode le code de la rubrique
     * @return la liste des formulaires correspondant
     */
    public List<FormulaireBean> getBySectionCode(final String sectionCode) {
        List<FormulaireBean> forms;
        try {
            final String query = "SELECT * FROM FORMULAIRE WHERE CODE_RUBRIQUE = :sectionCode";
            final SqlParameterSource params = new MapSqlParameterSource("sectionCode", sectionCode);
            forms = namedParameterJdbcTemplate.query(query, params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting FORMULAIRE rows", dae);
        }
        return forms;
    }
}
