/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.dao.impl;

import java.time.ZoneId;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.jsbsoft.jtf.datasource.dao.impl.mysql.AbstractLegacyDAO;
import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.newsletter.bean.MailingBean;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * Created on 23/04/15.
 */
public class MailingDAO extends AbstractLegacyDAO<MailingBean> {

    private static final Logger LOG = LoggerFactory.getLogger(MailingDAO.class);

    public MailingDAO() {
        tableName = "MAILING";
    }

    public List<MailingBean> getAll() {
        List<MailingBean> result;
        try {
            result = namedParameterJdbcTemplate.query("SELECT * FROM MAILING ORDER BY ID_MAILING", rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting for all MAILING row in the database", dae);
        }
        return result;
    }

    public List<MailingBean> getByIdNewsletter(String idNewsletter) {
        List<MailingBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("idNewsletter", idNewsletter);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MAILING WHERE ID_NEWSLETTER = :idNewsletter", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with newsletter id %s", idNewsletter), dae);
        }
        return result;
    }

    public List<MailingBean> getByState(String state) {
        List<MailingBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("state", state);
            result = namedParameterJdbcTemplate.query("SELECT DISTINCT * FROM MAILING WHERE ETAT = :state ORDER BY ID_MAILING", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with state %s", state), dae);
        }
        return result;
    }

    public List<MailingBean> getByIdNewsletterAndState(String idNewsletter, String state) {
        List<MailingBean> result;
        final StringBuilder query = new StringBuilder("SELECT * FROM MAILING WHERE");
        if(StringUtils.isNotBlank(idNewsletter)) {
            query.append(" ID_NEWSLETTER = :idNewsletter AND");
        }
        query.append(" ETAT = :state");
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("idNewsletter", idNewsletter);
            params.addValue("state", state);
            result = namedParameterJdbcTemplate.query(query.toString(), params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with state %s and newsletter id %s", state, idNewsletter), dae);
        }
        return result;
    }

    public List<MailingBean> getByIdNewsletterAndStateOrdered(String idNewsletter, String state, String ordered) {
        List<MailingBean> result;
        final StringBuilder query = new StringBuilder("SELECT * FROM MAILING WHERE");
        if(StringUtils.isNotBlank(idNewsletter)) {
            query.append(" ID_NEWSLETTER = :idNewsletter AND");
        }
        query.append(" ETAT = :state");
        if(StringUtils.isNotBlank(ordered) && ("DESC".equals(ordered) || "ASC".equals(ordered))) {
            query.append(String.format(" ORDER BY ID_MAILING %s", ordered));
        }
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("idNewsletter", idNewsletter);
            params.addValue("state", state);
            result = namedParameterJdbcTemplate.query(query.toString(), params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with state %s and newsletter id %s ordered by id %s", state, idNewsletter, ordered), dae);
        }
        return result;
    }

    public List<MailingBean> getByEmailAndIdNewsLetter(String email, String idNewsletter) {
        List<MailingBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("email", email);
            params.addValue("idNewsletter", idNewsletter);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MAILING WHERE EMAIL = :email AND ID_NEWSLETTER = :idNewsletter", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with email %s and newsletter id %s", email, idNewsletter), dae);
        }
        return result;
    }

    public List<MailingBean> getByCodeUtilisateurAndIdNewsLetter(String codeUtilisateur, String idNewsletter) {
        List<MailingBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("codeUtilisateur", codeUtilisateur);
            params.addValue("idNewsletter", idNewsletter);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MAILING WHERE CODE_UTILISATEUR LIKE :codeUtilisateur AND ID_NEWSLETTER = :idNewsletter", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with usercode %s and newsletter id %s", codeUtilisateur, idNewsletter), dae);
        }
        return result;
    }

    public List<MailingBean> getByObjectTypeCreationDatePublic(String object, String type, Date startDate, Date endDate) {
        return select(buildQueryForObjectTypeCreationDatePublic(object, type, startDate, endDate));
    }

    private String buildQueryForObjectTypeCreationDatePublic(String object, String type, Date startDate, Date endDate) {
        final RequeteSQL requeteSelect = new RequeteSQL();
        final ClauseWhere whereTypeDatePublic = new ClauseWhere();
        if (StringUtils.isNotEmpty(object)) {
            whereTypeDatePublic.setPremiereCondition(ConditionHelper.like("T1.OBJET", object, "%", "%"));
        }
        if (StringUtils.isNotEmpty(type) && !"0000".equals(type)) {
            whereTypeDatePublic.and(ConditionHelper.egalVarchar("T1.TYPE", type));
        }
        if (startDate != null) {
            whereTypeDatePublic.and(ConditionHelper.greaterEquals("T1.DATE_CREATION", startDate, TypeOperande.DATE));
        }
        if (endDate != null) {
            Date endDatePlusOneDay = Date.from(endDate.toInstant().atZone(ZoneId.systemDefault()).plusDays(1).toInstant());
            whereTypeDatePublic.and(ConditionHelper.lessEquals("T1.DATE_CREATION", endDatePlusOneDay, TypeOperande.DATE));
        }
        // on ne requete que les envois bien envoyés
        whereTypeDatePublic.and(ConditionHelper.egalVarchar("T1.ETAT", "SENT"));
        // les envois html
        whereTypeDatePublic.and(ConditionHelper.egalVarchar("T1.TYPE_MIME", "0"));
        // les archives publiques uniquement
        whereTypeDatePublic.and(ConditionHelper.egalVarchar("T1.PUBLIC_INTERNE", "0"));
        requeteSelect.where(whereTypeDatePublic);
        return requeteSelect.formaterRequete();
    }

    public Integer getCountOfIdNewsLetterAndState(String idNewsletter, String state) {
        Integer result = null;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("idNewsletter", idNewsletter);
            params.addValue("state", state);
            result = namedParameterJdbcTemplate.queryForObject("SELECT COUNT(*) FROM MAILING WHERE ID_NEWSLETTER = :idNewsletter AND ETAT = :state", params, Integer.class);
        } catch (EmptyResultDataAccessException e) {
            LOG.debug(String.format("empty resultset size for id %s and state %s on table %s", idNewsletter, state, tableName), e);
        } catch (IncorrectResultSizeDataAccessException e) {
            LOG.debug("incorrect resultset size", e);
            LOG.info("incorrect resultset size");
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with state %s and newsletter id %s", state, idNewsletter), dae);
        }
        return result;
    }

    public List<MailingBean> getByIdNewsletterAndState(String idNewsletter, String state, Integer limit, Integer offset) {
        List<MailingBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("idNewsletter", idNewsletter);
            params.addValue("state", state);
            params.addValue("limit", limit);
            params.addValue("offset", offset != null ? offset : 0);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MAILING WHERE ID_NEWSLETTER = :idNewsletter AND ETAT = :state LIMIT :limit OFFSET :offset", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MAILING row with state %s and newsletter id %s, limit %d and offset %d", state, idNewsletter, limit, offset), dae);
        }
        return result;
    }
}
