/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.processus;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.ProcessusHelper;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.newsletter.bean.DemandeAbonnementBean;
import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.service.ServiceDemandeAbonnement;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.utils.Chaine;
import com.univ.utils.EscapeString;
import com.univ.utils.MailUtil;
import com.univ.utils.URLResolver;

/**
 * Traitement des demandes d'abonnement à la newsletter
 */
public class TraitementDemandeAbonnement extends SaisieFiche {

    private static final Logger LOG = LoggerFactory.getLogger(TraitementDemandeAbonnement.class);

    /**
     * The Constant ECRAN_CONFIRMATION.
     */
    private static final String ECRAN_CONFIRMATION = "CONFIRMATION";

    /**
     * The Constant ECRAN_CHOIX.
     */
    private static final String ECRAN_CHOIX = "CHOIX";

    /**
     * The Constant ECRAN_ABONNEMENT.
     */
    private static final String ECRAN_ABONNEMENT = "ABONNEMENT";

    /**
     * The Constant ECRAN_VALIDATION_ABONNEMENT.
     */
    private static final String ECRAN_VALIDATION_ABONNEMENT = "VALIDER_ABONNEMENT";

    /**
     * The Constant ECRAN_DESABONNEMENT.
     */
    private static final String ECRAN_DESABONNEMENT = "DESABONNEMENT";

    /**
     * The Constant ECRAN_DESABONNEMENT.
     */
    private static final String ECRAN_DEMANDE_DESABONNEMENT = "DEMANDE_DESABONNEMENT";

    /**
     * The Constant ACTION_DEMANDE_ABONNEMENT.
     */
    private static final String ACTION_DEMANDE_ABONNEMENT = "DEMANDER_ABONNEMENT";

    /**
     * The Constant ACTION_ABONNER.
     */
    private static final String ACTION_ABONNER = "ABONNER";

    /**
     * The Constant ACTION_CONFIRMER.
     */
    private static final String ACTION_CONFIRMER = "CONFIRMER";

    /**
     * The Constant ACTION_CHOIX.
     */
    private static final String ACTION_CHOIX = "CHOISIR";

    /**
     * The Constant ACTION_DESABONNER.
     */
    private static final String ACTION_DESABONNER = "DESABONNER";

    /**
     * The Constant ACTION_DEMANDE_DESABONNEMENT.
     */
    private static final String ACTION_DEMANDE_DESABONNEMENT = "DEMANDER_DESABONNEMENT";

    private final ServiceUser serviceUser;

    private final ServiceDemandeAbonnement serviceDemandeAbonnement;

    /**
     * The autorisations.
     */
    AutorisationBean autorisations;

    /**
     * Constructeur
     *
     * @param infobean com.jsbsoft.jtf.core.InfoBean
     */
    public TraitementDemandeAbonnement(final InfoBean infobean) {
        super(infobean);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        serviceDemandeAbonnement = ServiceManager.getServiceForBean(DemandeAbonnementBean.class);
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     * @throws Exception the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        try {
            ecranLogique = ECRAN_VALIDATION_ABONNEMENT;
            action = infoBean.getActionUtilisateur();
            //demande d'abonnement : envoi mail pour confirmation
            if (action.equals(ACTION_DEMANDE_ABONNEMENT)) {
                traiterDEMANDERABONNEMENT();
            } else if (action.equals(ACTION_ABONNER)) {
                traiterABONNEMENT();
            } else if (action.equals(ACTION_CONFIRMER)) {
                traiterCONFIRMATION();
            } else if (action.equals(ACTION_CHOIX)) {
                traiterCHOIX();
            } else if (action.equals(ACTION_DESABONNER)) {
                traiterDESABONNEMENT();
            } else if (action.equals(ACTION_DEMANDE_DESABONNEMENT)) {
                preparerDESABONNEMENT();
            }
        } catch (final Exception e) {
            LOG.error("Erreur de traitement ", e);
            infoBean.addMessageErreur(e.toString());
        } finally {
            //placer l'état dans le composant d'infoBean
            infoBean.setEcranLogique(ecranLogique);
        }
        // On continue si on n'est pas à la FIN !!!
        return etat == FIN;
    }
    /**
     * Traiter demander abonnement.
     */
    protected void traiterDEMANDERABONNEMENT() {
        String allIds = (String) infoBean.get("ID_MODELE_MAIL");
        List<String> allSerializedIds = Arrays.asList(allIds.split(","));
        List<Long> ids = allSerializedIds.stream()
            .filter(StringUtils::isNumeric)
            .map(Long::valueOf)
            .collect(Collectors.toList());

        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        List<ModelemailBean> results = serviceModelemail.getByIds(ids);
        if (!results.isEmpty()) {
            infoBean.set("modeleMail", results);
            infoBean.set("nbModeleMail", results.size());
        }
        ecranLogique = ECRAN_ABONNEMENT;
    }

    /**
     * Traiter abonnement.
     *
     * @throws Exception the exception
     */
    protected void traiterABONNEMENT() throws Exception {
        ecranLogique = ECRAN_VALIDATION_ABONNEMENT;
        int nbNewsletters = 0;
        final String nbNewsletterDepuisInfoBean = infoBean.getString("NB_NEWSLETTERS");
        if (StringUtils.isNotEmpty(nbNewsletterDepuisInfoBean) && StringUtils.isNumeric(nbNewsletterDepuisInfoBean)) {
            nbNewsletters = Integer.parseInt(nbNewsletterDepuisInfoBean);
        }
        final String emailUtilisateur = infoBean.getString("EMAIL");
        final String formatSouhaite = StringUtils.defaultIfBlank(infoBean.getString("FORMAT_ENVOI"), "0");
        if (emailUtilisateur.equals("")) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_EMAIL_VIDE"));
        } else if (!MailUtil.verifMail(emailUtilisateur)) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_EMAIL_VALIDE"));
        }
        final String codeUtilisateur = infoBean.getString("CODE");
        final StringBuilder listeModelesChoisis = new StringBuilder();
        final StringBuilder listeLibellesModeles = new StringBuilder();
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        //récupération des newsletters choisies
        for (int i = 0; i <= nbNewsletters; i++) {
            String szIdModele = infoBean.getString("NEWSLETTER" + i);
            if (StringUtils.isNotEmpty(szIdModele) && StringUtils.isNumeric(szIdModele)) {
                Long idModele = Long.valueOf(szIdModele);
                ModelemailBean mm = serviceModelemail.getById(idModele);
                if (mm != null) {
                    listeModelesChoisis.append(idModele).append(";");
                    listeLibellesModeles.append("- ").append(mm.getIntitule()).append("\n");
                }
            }
        }
        if (listeModelesChoisis.length() > 0) {
            listeModelesChoisis.deleteCharAt(listeModelesChoisis.length() - 1);
        } else {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_SELECTION"));
        }
        final String mentions = infoBean.getString("NEWSLETTER_MENTIONS");
        if (StringUtils.isBlank(mentions)) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_MENTIONS"));
        }
        if (StringUtils.isNotBlank(codeUtilisateur)) {
            final UtilisateurBean utilisateurBean = serviceUser.getByCode(codeUtilisateur);
            if (utilisateurBean != null) {
                //si connecté et mail changé, ecran choix entre nouveau compte, ou utiliser celle du compte
                if (!utilisateurBean.getAdresseMail().equals(emailUtilisateur)) {
                    ecranLogique = ECRAN_CHOIX;
                    infoBean.set("CODE", codeUtilisateur);
                    infoBean.set("EMAIL", emailUtilisateur);
                    infoBean.set("EMAIL_UTILISATEUR", utilisateurBean.getAdresseMail());
                    infoBean.set("LISTE_MODELES_CHOISIS", listeModelesChoisis.toString());
                    infoBean.set("LISTE_LIBELLES_MODELES", listeLibellesModeles.toString());
                    infoBean.set("FORMAT_ENVOI", formatSouhaite);
                } else {
                    verifieSiDejaAbonne(listeModelesChoisis.toString(), utilisateurBean);
                    //si connecté
                    envoiDemandeAbonnement(emailUtilisateur, codeUtilisateur, listeModelesChoisis.toString(), listeLibellesModeles.toString(), formatSouhaite);
                }
            }
        } else {
            final List<UtilisateurBean> utilisateurs = serviceUser.getByMailAndCode(emailUtilisateur, null);
            if (CollectionUtils.isEmpty(utilisateurs)) {
                envoiDemandeAbonnement(emailUtilisateur, "", listeModelesChoisis.toString(), listeLibellesModeles.toString(), formatSouhaite);
            } else {
                final UtilisateurBean utilisateurBean = utilisateurs.get(0);
                verifieSiDejaAbonne(listeModelesChoisis.toString(), utilisateurBean);
                //sinon, on l'inscrit avec son login, afin que cette demande complète son compte
                envoiDemandeAbonnement(emailUtilisateur, utilisateurBean.getCode(), listeModelesChoisis.toString(), listeLibellesModeles.toString(), formatSouhaite);
            }
        }
    }

    /**
     * Envoi demande abonnement.
     *
     * @param emailUtilisateur the email utilisateur
     * @param codeUtilisateur the code utilisateur
     * @param listeModelesChoisis the liste modeles choisis
     * @param listeLibellesModeles the liste libelles modeles
     * @param formatSouhaite the format souhaite
     * @throws Exception the exception
     */
    private void envoiDemandeAbonnement(final String emailUtilisateur, final String codeUtilisateur, final String listeModelesChoisis, final String listeLibellesModeles,
        final String formatSouhaite) throws Exception {
        //enregistrement dans les demandes
        final DemandeAbonnementBean da = new DemandeAbonnementBean();
        da.setAdresseMailUtilisateur(emailUtilisateur);
        da.setCodeUtilisateur(codeUtilisateur);
        da.setListeModelesMails(listeModelesChoisis);
        da.setFormatSouhaite(formatSouhaite);
        serviceDemandeAbonnement.save(da);
        final String urlRelative = ProcessusHelper.getUrlProcessAction(null, NewsletterUtil.ID_EXTENSION, "TRAITEMENT_DEMANDE_ABONNEMENT", ACTION_CONFIRMER,
            new String[][] { { "ID_DEMANDE", String.valueOf(da.getIdDemandeAbonnement()) } });
        final String urlConfirmation = URLResolver.getAbsoluteUrl(EscapeString.unescapeHtml(urlRelative), this);
        //si non connecté, envoi mail confirmation + ecriture demande
        final JSBMailbox jsbMailbox = new JSBMailbox(false);
        String texte = NewsletterUtil.getMessage("ST_NEWSLETTER_EMAIL_CONFIRMATION");
        texte = texte.replaceAll("\\[0\\]", listeLibellesModeles);
        texte = texte.replaceAll("\\[1\\]", urlConfirmation);
        try {
            jsbMailbox.sendSystemMsg(emailUtilisateur, NewsletterUtil.getMessage("ST_NEWSLETTER_SUJET_EMAIL_CONFIRMATION"), texte);
        } catch (final EmailException e) {
            LOG.error("erreur lors de l'envoie du message", e);
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR"));
        }
    }

    /**
     * Preparer desabonnement.
     *
     * @throws Exception the exception
     */
    private void preparerDESABONNEMENT() throws Exception {
        ecranLogique = ECRAN_DESABONNEMENT;
        final String email = infoBean.getString("EMAIL");
        if (StringUtils.isEmpty(email)) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_EMAIL_VIDE"));
        }
        List<UtilisateurBean> utilisateurs = Collections.emptyList();
        if (StringUtils.isNotEmpty(infoBean.get("CODE_CONNECTE", String.class)) && StringUtils.equals(infoBean.get("EMAIL_CONNECTE", String.class), email)) {
            final String code = infoBean.getString("CODE_CONNECTE");
            utilisateurs = serviceUser.getByMailAndCode(email, code);
        } else {
            utilisateurs = serviceUser.getByMail(email);
        }
        // sélection de l'utilisateur avec son mail et éventuellement nom et login si deux comptes correspondent au mail
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        if (CollectionUtils.isNotEmpty(utilisateurs)) {
            int nbNewsletters = 0;
            final AtomicInteger i = new AtomicInteger(0);
            //récupération des modèles auxquels est abonné l'utilisateur (attention, il peut y avoir plusieurs utilisateurs avec le meme mail)
            for (UtilisateurBean currentUser : utilisateurs) {
                // On garde qu'une seule version par newsletter, les états ne sont donc pas pris en compte. 
                final List<ModelemailBean> listeModeles = serviceModelemail.getModelMailBeanByUser(currentUser)
                    .stream()
                    .filter(modelemailBean -> StringUtils.isEmpty(modelemailBean.getEtat()))
                    .collect(Collectors.toList());
                //aucun modèle, l'utilisateur n'est abonné à rien
                nbNewsletters = nbNewsletters + listeModeles.size();
                //préparation de la liste des modèles présentés à l'utilisateur qui souhaite se désabonner
                listeModeles.stream()
                    .forEach(modelemailBean -> {
                        infoBean.set("ID_MODELE_MAIL" + i, modelemailBean.getId());
                        infoBean.set("LIBELLE_MODELE_MAIL" + i, modelemailBean.getIntitule());
                        infoBean.set("CODE" + i, currentUser.getCode());
                        i.incrementAndGet();
                    });
            }
            if (nbNewsletters == 0) {
                throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_PAS_ABONNE"));
            } else {
                infoBean.set("NB_NEWSLETTERS", "" + nbNewsletters);
            }
            infoBean.set("EMAIL", email);
        } else {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_USER_INEXISTANT"));
        }
    }

    /**
     * Traiter desabonnement.
     *
     * @throws Exception the exception
     */
    protected void traiterDESABONNEMENT() throws Exception {
        ecranLogique = ECRAN_CONFIRMATION;
        final int nbNewsletters = Integer.parseInt(infoBean.getString("NB_NEWSLETTERS"));
        Collection<String> vGroupesUtilisateur = new ArrayList<>();
        String codeUtilisateurPrec = "";
        // boucle sur les newsletters auxquelles il est abonné
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        boolean isAucunModeleSelectionnee = Boolean.TRUE;
        for (int i = 0; i < nbNewsletters; i++) {
            final String idModele = infoBean.getString("ID_MODELE_MAIL" + i);
            if (idModele != null && StringUtils.isNumeric(idModele)) {
                ModelemailBean mm = serviceModelemail.getById(Long.valueOf(idModele));
                if (mm != null) {
                    final String codeUtilisateur = infoBean.getString("CODE" + i);
                    if (StringUtils.isEmpty(codeUtilisateur)) {
                        ecranLogique = ECRAN_DEMANDE_DESABONNEMENT;
                        return;
                    }
                    if (!codeUtilisateur.equals(codeUtilisateurPrec)) {
                        vGroupesUtilisateur = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(codeUtilisateur);
                    }
                    //si l'utilisateur est dans les visés, on l'enleve
                    boolean updateModele = false;
                    Collection<String> vUtilisateursVises = Chaine.getVecteurPointsVirgules(mm.getUtilisateursVises());
                    if (vUtilisateursVises.contains(codeUtilisateur)) {
                        vUtilisateursVises.remove(codeUtilisateur);
                        mm.setUtilisateursVises(StringUtils.join(vUtilisateursVises, ";"));
                        updateModele = true;
                    }
                    final String listeGroupesVises = mm.getDiffusionPublicVise();
                    //si l'utilisateur est dans un des groupes, on le vire du groupe
                    boolean updateGroupe = false;
                    if (CollectionUtils.isNotEmpty(vGroupesUtilisateur) && StringUtils.isNotEmpty(listeGroupesVises)) {
                        final Iterator<String> itGroups = vGroupesUtilisateur.iterator();
                        while(itGroups.hasNext()) {
                            final String currentGroup = itGroups.next();
                            if (listeGroupesVises.contains("/" + currentGroup + "]")) {
                                itGroups.remove();
                                updateGroupe = true;
                            }
                        }
                        if (updateGroupe) {
                            serviceGroupeUtilisateur.setGroupsForUser(codeUtilisateur, vGroupesUtilisateur);
                        }
                    }
                    if (updateModele) {
                        serviceModelemail.save(mm);
                    }
                    codeUtilisateurPrec = codeUtilisateur;
                    isAucunModeleSelectionnee = Boolean.FALSE;
                }
            }
        }//fin for sur les newsletters auxquelles il est abonné
        if (isAucunModeleSelectionnee) {
            preparerDESABONNEMENT();
            return;
        }
        // on vérifie si l'utilisateur n'est plus abonné à aucune newsletter
        if (nbNewsletters > 0) {
            final String codeUtilisateur = infoBean.getString("CODE0");
            final UtilisateurBean user = serviceUser.getByCode(codeUtilisateur);
            if (user != null) {
                final List<Long> listeModeles = serviceModelemail.getListeModelesParUtilisateur(user);
                if (CollectionUtils.isEmpty(listeModeles) && StringUtils.isBlank(user.getMotDePasse()) && StringUtils.isBlank(
                    user.getCodeLdap()) && user.getNom() != null && user.getNom().contains("@")) {
                    serviceUser.delete(user.getId());
                }
            }
        }
        infoBean.set("MESSAGE_CONFIRMATION", NewsletterUtil.getMessage("ST_NEWSLETTER_MESSAGE_DESABO_OK"));
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.processus.SaisieFiche#preparerLISTE()
     */
    @Override
    protected void preparerLISTE() throws Exception {
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {}

    /**
     * Traiter liste.
     *
     * @throws Exception the exception
     */
    protected void traiterLISTE() throws Exception {
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.processus.SaisieFiche#preparerPRINCIPAL()
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.processus.SaisieFiche#traiterPRINCIPAL()
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
    }

    @Override
    protected void preparerRECHERCHE() {
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.processus.SaisieFiche#traiterRECHERCHE()
     */
    @Override
    protected void traiterRECHERCHE() throws Exception {
    }

    /**
     * Traiter confirmation.
     *
     * @throws Exception the exception
     */
    private void traiterCONFIRMATION() throws Exception {
        final String idDemande = infoBean.getString("ID_DEMANDE");
        if (StringUtils.isEmpty(idDemande) || !StringUtils.isNumeric(idDemande)) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_DEMANDE_DEPASSEE"));
        }
        final Long idDemandeAbonnement = Long.valueOf(idDemande);
        final DemandeAbonnementBean da = serviceDemandeAbonnement.getById(idDemandeAbonnement);
        String codeUtilisateur, emailUtilisateur, listeModeles;
        if (da != null) {
            codeUtilisateur = da.getCodeUtilisateur();
            emailUtilisateur = da.getAdresseMailUtilisateur();
            listeModeles = da.getListeModelesMails();
            //nouvel utilisateur
            if (StringUtils.isBlank(codeUtilisateur)) {
                final UtilisateurBean user = new UtilisateurBean();
                user.setAdresseMail(emailUtilisateur);
                user.setNom(emailUtilisateur);
                codeUtilisateur = "" + System.currentTimeMillis();
                if (codeUtilisateur.length() > 8) {
                    codeUtilisateur = codeUtilisateur.substring(codeUtilisateur.length() - 8);
                }
                user.setCode(codeUtilisateur);
                //gère inscription aux groupes, ou dans les modèles
                inscriptionUtilisateur(listeModeles, user);
                user.setFormatEnvoi(da.getFormatSouhaite());
                serviceUser.save(user);
            } else {
                final UtilisateurBean user = serviceUser.getByCode(codeUtilisateur);
                if (user != null) {
                    // gère inscription aux groupes, ou dans les modèles
                    inscriptionUtilisateur(listeModeles, user);
                    user.setFormatEnvoi(da.getFormatSouhaite());
                    serviceUser.save(user);
                }
            }
            serviceDemandeAbonnement.delete(da.getId());
        } else {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_ERROR_DEMANDE_DEPASSEE"));
        }
        infoBean.set("MESSAGE_CONFIRMATION", NewsletterUtil.getMessage("ST_NEWSLETTER_MESSAGE_INSCRIT"));
        //     appel au traitement specifique (permet de surcharger le processus sans tout reecrire)
        traiterCONFIRMATIONSpecific();
        ecranLogique = ECRAN_CONFIRMATION;
    }

    /**
     * Gère l'inscription à l'utilisateur, en fonction de l'action de chaque modèle de mail.
     *
     * @param listeModeles the liste modeles
     * @throws Exception the exception
     */
    private void inscriptionUtilisateur(final String listeModeles, final UtilisateurBean user) throws Exception {
        final StringTokenizer st = new StringTokenizer(listeModeles, ";");
        final List<String> tableUtilisateurs = new ArrayList<>();
        final String codeUtilisateur = user.getCode();
        //boucle sur chaque modele de mail choisi
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        while (st.hasMoreTokens()) {
            String idModele = st.nextToken();
            ModelemailBean mm = serviceModelemail.getById(Long.valueOf(idModele));
            if (mm != null) {
                String action = mm.getActionInscription();
                String[] listeActions = action.split(";");
                for (String listeAction : listeActions) {
                    //si l'action souhaitée est d'ajouter l'utilisateur à la liste nominative d'utilisateurs visés
                    if (listeAction.equals("U")) {
                        String listeUtilisateurs = mm.getUtilisateursVises();
                        //si la liste contient déjà des utilisateurs
                        if (listeUtilisateurs.length() > 0) {
                            StringTokenizer stUtilisateurs = new StringTokenizer(listeUtilisateurs, ";");
                            while (stUtilisateurs.hasMoreTokens()) {
                                tableUtilisateurs.add(stUtilisateurs.nextToken());
                            }
                            // si l'utilisateur n'était pas déjà dans la liste, on l'ajoute
                            if (!tableUtilisateurs.contains(codeUtilisateur)) {
                                listeUtilisateurs += ";" + codeUtilisateur;
                            }
                        } else {
                            listeUtilisateurs = codeUtilisateur;
                        }
                        //mise à jour du modèle avec la nouvelle liste
                        mm.setUtilisateursVises(listeUtilisateurs);
                        serviceModelemail.save(mm);
                    } //si l'action est de rattacher l'utilisateur à un groupe
                    else if (listeAction.startsWith("G") && listeAction.contains(",")) {
                        String codeGroupe = listeAction.substring(listeAction.indexOf(',') + 1);
                        Collection<String> groupsCodes = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(user.getCode());
                        //si l'utilisateur n'appartient pas à ce groupe, on l'ajoute
                        if (!serviceUser.appartientAGroupeOuSousGroupes(codeGroupe, new ArrayList<>(groupsCodes))) {
                            groupsCodes.add(codeGroupe);
                            serviceGroupeUtilisateur.setGroupsForUser(user.getCode(), groupsCodes);
                        }
                    }
                }
            }
        }
    }

    /**
     * Traiter choix.
     *
     * @throws Exception the exception
     */
    private void traiterCHOIX() throws Exception {
        ecranLogique = ECRAN_ABONNEMENT;
        //choix entre nouveau compte ou utilisation du mail du compte existant
        final String choix_mail = infoBean.getString("CHOIX_MAIL");
        final String listeModeles = infoBean.getString("LISTE_MODELES_CHOISIS");
        final String listeLibellesModeles = infoBean.getString("LISTE_LIBELLES_MODELES");
        final String formatSouhaite = StringUtils.defaultIfBlank(infoBean.getString("FORMAT_ENVOI"), "1");
        // Si utilisateur connecté valide
        UtilisateurBean authUser = serviceUser.getByCode(infoBean.getString("CODE"));
        if (authUser != null) {
            // si nouvelle adresse mail, on créé un nouveau compte
            if (choix_mail.equals("2")) {
                UtilisateurBean foundUser = new UtilisateurBean();
                // on vérifie que ce mail ne correspond à aucun utilisateur
                final String email = infoBean.get("EMAIL", String.class);
                final List<UtilisateurBean> foundUsers = serviceUser.getByMailAndCode(email, StringUtils.EMPTY);
                if (CollectionUtils.isNotEmpty(foundUsers)) {
                    foundUser = foundUsers.get(0);
                } else {
                    foundUser.setAdresseMail(email);
                }
                authUser = foundUser;
            }
            verifieSiDejaAbonne(listeModeles, authUser);
            envoiDemandeAbonnement(authUser.getAdresseMail(), authUser.getCode(), listeModeles, listeLibellesModeles, formatSouhaite);
            ecranLogique = ECRAN_VALIDATION_ABONNEMENT;
        }
    }

    /**
     * Lorsque l'utilisateur est déjà enregistré dans la base, et qu'il est déjà abonné aux newsletters choisies, on lui renvoie un message comme quoi il est déjà abonné.
     *
     * @param _listeModelesChoisis the _liste modeles choisis
     * @throws Exception the exception
     */
    private void verifieSiDejaAbonne(final String _listeModelesChoisis, final UtilisateurBean user) throws Exception {
        boolean dejaAbonne = true;
        //vérification que la personne n'est pas déjà abonnée
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        final List<Long> listeModelesU = serviceModelemail.getListeModelesParUtilisateur(user);
        final StringTokenizer st = new StringTokenizer(_listeModelesChoisis, ";");
        while (st.hasMoreTokens()) {
            //il suffit d'un modele auquel il n'était pas encore abonné, pour qu'on lui envoie de nouveau un mail
            final String modeleChoisie = st.nextToken();
            if (StringUtils.isNotEmpty(modeleChoisie) && StringUtils.isNumeric(modeleChoisie) && !listeModelesU.contains(Long.parseLong(modeleChoisie))) {
                dejaAbonne = false;
                break;
            }
        }
        if (dejaAbonne) {
            throw new ErreurApplicative(NewsletterUtil.getMessage("ST_NEWSLETTER_DEJA_ABONNE"));
        }
    }

    /**
     * Gère les traitements spécifiques de la validation du formulaire de saisie d'un utilisateur (ne fait rien dans le cas standard, surcharger la méthode lorsque la classe est
     * étendue).
     *
     * @throws Exception the exception
     */
    protected void traiterCONFIRMATIONSpecific() throws Exception {
    }
}
