/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.service;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.dao.impl.ModelemailDAO;
import com.kosmos.newsletter.util.ModeleMailThread;
import com.kosmos.service.impl.AbstractServiceBean;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.AbstractPersistenceBean;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.utils.EscapeString;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * Created by olivier.camon on 13/05/15.
 */
public class ServiceModelemail extends AbstractServiceBean<ModelemailBean, ModelemailDAO> {

    public List<ModelemailBean> getByIds(List<Long> ids) {
        return dao.selectByIds(ids);
    }

    public List<ModelemailBean> getByState(String state) {
        return dao.selectByState(state);
    }

    public List<ModelemailBean> getByCodeRubrique(String codeRubrique) {
        return dao.selectByCodeRubrique(codeRubrique);
    }

    public List<ModelemailBean> getByCodesRubriques(List<String> codes) {
        return dao.selectByCodesRubriques(codes);
    }

    public List<ModelemailBean> getAll() {
        return dao.selectAll();
    }

    public List<ModelemailBean> getAllWithValidState() {
        return dao.selectAllWithValidState();
    }

    public void checkCodesLibelleMedia(final long idMedia) throws ErreurApplicative {
        List<ModelemailBean> results = dao.selectByIdBandeau(idMedia);
        if (results.size() > 0) {
            throw new ErreurApplicative("Impossible de supprimer cette ressource car elle est utilisée en tant que bandeau sur la newsletter : " + results.get(0)
                .getIntitule());
        }
    }

    /**
     * Envoi d'un mailing Cette fonction fonctionne en asynchrone (lancement d'un thread).
     */
    public void envoyerMailing(ModelemailBean bean) {
        final Thread thread = new Thread(new ModeleMailThread(bean));
        thread.start();
    }

    /**
     * selection d'un modèle de newsletter selon des critères choisis.
     *
     * @param idModelemail
     *            Long identifiant du modèle
     * @param intitule
     *            String intitulé du modèle
     * @param codeResponsable
     *            String code du responsable (code utilisateur)
     * @param type
     *            String code du type de modèle de newsletter
     * @param etat
     *            String etat de la newsletter (apercu ou pas)
     * @param codeRubrique
     *            String code de la rubrique
     * @param codeRattachement
     *            String code de rattachement
     *
     * @return int nombre de résultats
     *
     * @throws Exception
     *             the exception
     */
    public List<ModelemailBean> select(final Long idModelemail, final String intitule, String codeResponsable, final String type, final String etat, final String codeRubrique, final String codeRattachement) throws Exception {
        final RequeteSQL requeteSelect = new RequeteSQL();
        final ClauseWhere where = new ClauseWhere();
        final ClauseOrderBy orderBy = new ClauseOrderBy("INTITULE", ClauseOrderBy.SensDeTri.ASC);
        if (idModelemail != 0) {
            where.setPremiereCondition(ConditionHelper.egal("ID_MODELE_MAIL", idModelemail, TypeOperande.LONG));
        }
        if (StringUtils.isNotEmpty(intitule)) {
            where.and(ConditionHelper.rechercheMots("INTITULE", intitule));
        }
        if (StringUtils.isNotEmpty(codeResponsable) && !"0000".equals(codeResponsable)) {
            codeResponsable = EscapeString.escapeSql(codeResponsable);
            where.and(ConditionHelper.genericConditionSurColonne("CODE_RESPONSABLES", "'(^|.*;)" + codeResponsable + "($|;.*)'", TypeOperande.NON_ECHAPABLE, Operateur.REGEXP));
        }
        if (StringUtils.isNotEmpty(type) && !"0000".equals(type)) {
            where.and(ConditionHelper.egalVarchar("TYPE", type));
        }
        if (StringUtils.isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(codeRubrique) && !"0000".equals(codeRubrique)) {
            where.and(ConditionHelper.getConditionRubrique("CODE_RUBRIQUE", codeRubrique));
        }
        where.and(ConditionHelper.egalVarchar("ETAT", etat));
        requeteSelect.where(where)
            .orderBy(orderBy);
        return dao.select(requeteSelect.formaterRequete());
    }

    /**
     * Retourne la liste des modèles de mail auxquels un utilisateur donné est inscrit (soit par inscription directe, soit par rattachement à un groupe).
     *
     * @param user
     *            Utilisateur dont on souhaite obtenir les newsletters
     *
     * @return List liste des identifiants des modèles de mails
     *
     * @throws Exception
     *             the exception
     */
    public List<Long> getListeModelesParUtilisateur(final UtilisateurBean user) throws Exception {
        List<ModelemailBean> allMail = getAll();
        final String codeUtilisateur = user.getCode();
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final Collection<String> vGroupesUtilisateur = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(user.getCode());
        return allMail.stream()
            .filter(beanMail -> isUserRegisteredOnModel(codeUtilisateur, vGroupesUtilisateur, beanMail))
            .map(AbstractPersistenceBean::getId)
            .collect(Collectors.toList());
    }

    /**
     * Retourne tous les modelès mail ou est inscrit l'utilisateur.
     * @param user {@link UtilisateurBean} l'utilisateur inscrit 
     * @return la  {@link List} de {@link ModelemailBean} sur lesquelles l'utilisateur est inscrit
     * @throws Exception
     */
    public List<ModelemailBean> getModelMailBeanByUser(final UtilisateurBean user) throws Exception {
        List<ModelemailBean> allMail = getAll();
        final String codeUtilisateur = user.getCode();
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final Collection<String> vGroupesUtilisateur = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(user.getCode());
        return allMail.stream()
            .filter(beanMail -> isUserRegisteredOnModel(codeUtilisateur, vGroupesUtilisateur, beanMail))
            .collect(Collectors.toList());
    }

    /**
     * Détermine si l'utilisateur est inscrit à la newsletter.
     * @param codeUtilisateur Le code de l'utilisateur
     * @param vGroupesUtilisateur Les groupes de l'utilisateur
     * @param bean {@link ModelemailBean} le modèle de la newsletter
     * @return vrai si l'utilisateur est inscrit, faux dans le cas échéant.
     */
    boolean isUserRegisteredOnModel(final String codeUtilisateur, final Collection<String> vGroupesUtilisateur, final ModelemailBean bean) {
        String listeUtilisateursVises = bean.getUtilisateursVises();
        if (StringUtils.isNotEmpty(codeUtilisateur) && isListUserContainsCode(codeUtilisateur, listeUtilisateursVises)) {
            return true;
        }
        // groupes cibles
        String listeGroupesVises = bean.getDiffusionPublicVise();
        if (vGroupesUtilisateur.size() > 0 && !"".equals(listeGroupesVises)) {
            for (String codeGroupeCourant : vGroupesUtilisateur) {
                if (listeGroupesVises.contains("/" + codeGroupeCourant + "]")) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Détermine si le code utilisateur est présent dans la liste.
     * @param codeUtilisateur
     * @param listeUtilisateursVises
     * @return
     */
    private boolean isListUserContainsCode(final String codeUtilisateur, final String listeUtilisateursVises) {
        return Arrays.stream(listeUtilisateursVises.split(";"))
            .filter(code -> code.equals(codeUtilisateur))
            .findFirst()
            .isPresent();
    }

    public void addWithForcedId(ModelemailBean modelemailBean) throws Exception {
        dao.addWithForcedId(modelemailBean);
    }
}
