/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.util;

import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.jsoup.select.Selector.SelectorParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Classe pour https://assistance.kosmos.fr/browse/PRAOTMA-370
 *
 * @author laurent.wasylyszyn
 *
 */
public class NewsletterCSSUtil {

    private static final String STYLE_TAG = "style";

    private static final String RULE_DELIMITERS = "{}";

    private static final String PROPERTY_DELIMITER = ";";

    private static final String SELECTOR_SEPARATOR = ",";

    private static final String DOUBLE_QUOTE = "\"";

    private static final String SIMPLE_QUOTE = "'";

    private static final String LF = "\n";

    private static final Logger LOG = LoggerFactory.getLogger(NewsletterCSSUtil.class);

    private static final String CANNOT_PARSE_STYLE_ERROR = "Cannot parse style attribute.";

    /**
     * Transfert tous les styles CSS (user-agent, link, et style, dans les attributs style des balises HTML correspondantes)
     *
     * @param html
     *            le code html à parser pour récupérer les styles
     * @return le code html avec les styles mis en inline
     */
    public static String externalCssToInlineStyles(final String html) {
        final Document documentToParse = Jsoup.parse(html);
        final Elements styleElements = documentToParse.select(STYLE_TAG);
        for (final Element styleElement : styleElements) {
            final Element styleContent = getStyleContent(styleElement);
            final String style = StringUtils.replace(styleContent.data(), LF, StringUtils.EMPTY)
                .replaceAll("@media[^{]+\\{([\\s\\S]+?\\})\\s*\\}", StringUtils.EMPTY);
            final StringTokenizer st = new StringTokenizer(style, RULE_DELIMITERS);
            parseStyle(st, documentToParse);
        }
        return documentToParse.toString();
    }

    /**
     * Retourne le contenu de l'element style fourni en paramètre. Le contenu de l'élement est le premier element de la liste. la balise style ne contient pas de fils exepté le
     * textNode contenant la déclaration des styles donc on prend le premier élément uniquement.
     *
     * @param styleElement
     *            l'element style dont on souhaite récupérer le contenu
     * @return le contenu de l'element style
     */
    private static Element getStyleContent(final Element styleElement) {
        final Elements styleElementAndChilds = styleElement.getAllElements();
        return styleElementAndChilds.first();
    }

    private static void parseStyle(final StringTokenizer st, final Document doc) {
        while (st.countTokens() > 1) {
            final String mainSelector = st.nextToken();
            final String properties = st.nextToken();
            final String[] selectors = StringUtils.split(mainSelector, SELECTOR_SEPARATOR);
            parseSelectors(selectors, properties, doc);
        }
    }

    private static void parseSelectors(final String[] selectors, final String properties, final Document doc) {
        for (final String selector : selectors) {
            if (StringUtils.isNotBlank(selector)) {
                applyStyles(selector, properties, doc);
            }
        }
    }

    private static void applyStyles(final String selector, final String newProperties, final Document doc) {
        try {
            final Elements selectedElements = doc.select(selector);
            for (final Element target : selectedElements) {
                final String oldStyleAttribute = StringUtils.defaultString(target.attr(STYLE_TAG));
                target.attr(STYLE_TAG, concatenateStyles(oldStyleAttribute, newProperties));
            }
        } catch (final SelectorParseException e) {
            LOG.debug(CANNOT_PARSE_STYLE_ERROR, e);
        }
    }

    private static String concatenateStyles(String oldStyleAttribute, String newProperties) {
        oldStyleAttribute = oldStyleAttribute.trim();
        if (!oldStyleAttribute.endsWith(PROPERTY_DELIMITER) && oldStyleAttribute.length() > 0) {
            oldStyleAttribute += PROPERTY_DELIMITER;
        }
        if (!newProperties.trim().endsWith(PROPERTY_DELIMITER)) {
            newProperties += PROPERTY_DELIMITER;
        }
        String concat = oldStyleAttribute + newProperties;
        concat = concat.replace(DOUBLE_QUOTE, SIMPLE_QUOTE);
        return concat;
    }
}