/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.objetspartages.services.ServiceUser;

public class NewsletterUtil {

    private static final Logger LOG = LoggerFactory.getLogger(NewsletterUtil.class);

    public static final String ID_EXTENSION = "newsletter";

    public static final String CONTEXTE_NEWSLETTER = "newsletter";

    private static final String PROP_NEWSLETTER_STYLE = "newsletter.style";

    private static final String PROP_SITE_NEWSLETTER_STYLE = "newsletter_style";

    public static final String FORMAT_HTML = "0";

    public static boolean isAdministrateurNewsletter(final AutorisationBean autorisations) {
        boolean res = Boolean.FALSE;
        PermissionBean permission = new PermissionBean("TECH", "new", "C");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "new", "S");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "new", "M");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "new", "E");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        return res;
    }

    public static String getProperty(final String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static long getPropertyAsLong(final String key, final long defval) {
        return PropertyHelper.getPropertyAsLong(ID_EXTENSION, key, defval);
    }

    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    public static String getPathStyle(final String alias) {
        String res = "";
        String style = getProperty(PROP_NEWSLETTER_STYLE);
        if (StringUtils.isNotEmpty(style)) {
            final String[] styles = style.split(";");
            for (String s : styles) {
                File f = new File(s);
                if (!f.exists()) {
                    s = WebAppUtil.getAbsolutePath() + s;
                    f = new File(s);
                }
                if (f.exists()) {
                    if (StringUtils.isNotBlank(res)) {
                        res += ";";
                    }
                    res += f.getAbsolutePath();
                }
            }
        }
        if (StringUtils.isNotEmpty(alias)) {
            style = getProperty("site." + alias + "." + PROP_SITE_NEWSLETTER_STYLE);
            if (StringUtils.isNotEmpty(style)) {
                final String[] styles = style.split(";");
                for (String s : styles) {
                    File f = new File(s);
                    if (!f.exists()) {
                        s = WebAppUtil.getAbsolutePath() + s;
                        f = new File(s);
                    }
                    if (f.exists()) {
                        if (StringUtils.isNotBlank(res)) {
                            res += ";";
                        }
                        res += f.getAbsolutePath();
                    }
                }
            }
        }
        return res;
    }

    /**
     * Gets the newsletter style.
     *
     * @param alias
     *            l'alias du site dont dépend le style de la newsletter.
     * @return the newsletter style
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    public static String getNewsletterStyle(final String alias) throws IOException {
        final String style = getPathStyle(alias);
        String szStyle = "", oneStyle = "";
        final StringBuffer styleCss = new StringBuffer("");
        if (StringUtils.isNotBlank(style)) {
            try {
                final StringTokenizer stStyles = new StringTokenizer(style, ";");
                while (stStyles.hasMoreTokens()) {
                    oneStyle = stStyles.nextToken();
                    final File f = new File(oneStyle);
                    if (f.exists() && f.canRead()) {
                        File[] tabfiles = null;
                        if (f.isDirectory()) {
                            tabfiles = f.listFiles();
                        } else {
                            tabfiles = new File[1];
                            tabfiles[0] = f;
                        }
                        FileInputStream fis = null;
                        BufferedReader br = null;
                        String line = null;
                        for (int i = 0; i < tabfiles.length; i++) {
                            if (!tabfiles[i].isDirectory()) {
                                fis = new FileInputStream(tabfiles[i]);
                                br = new BufferedReader(new InputStreamReader(fis));
                                while ((line = br.readLine()) != null) {
                                    styleCss.append(line);
                                }
                                fis.close();
                            }
                        }
                    }
                }
                Pattern patternCSS = Pattern.compile("\\s");
                szStyle = patternCSS.matcher(styleCss.toString()).replaceAll(" ");
            } catch (final Exception e) {
                LOG.error("Exception lors de la lecture du fichier des styles css de la newsletter", e);
            }
        }
        return szStyle;
    }

    /**
     * Gets the newsletter style.
     *
     * @return the newsletter style
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    public static String getNewsletterStyle() throws IOException {
        return getNewsletterStyle("");
    }

    /**
     * Calcule le chemin relatif vers la newsletter.
     *
     * @return
     */
    public static String getNewsletterPath() {
        return WebAppUtil.getRelativeExtensionPath(NewsletterUtil.ID_EXTENSION);
    }

    /**
     * Retourne une liste de libelle afficahble d'utilisateur à partir d'un champ utilisateur composé de code utilisateur séparé par des ";"
     * @param codeUsers
     * @return
     */

    public static List<String> getAllLibelleUtilisateurFromCode(String codeUsers) {
        List<String> listeLibelle = new ArrayList<>();
        ServiceUser serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        if (StringUtils.isNotBlank(codeUsers)) {
            String[] codeUserArr = StringUtils.split(codeUsers, ";");
            for (String user : codeUserArr) {
                listeLibelle.add(serviceUser.getLibelle(user));
            }
        }
        return listeLibelle;
    }

    /**
     * Personnalise le corps de newsletter avec l'utilisateur passé en paramètre
     *
     * @param message : le message original
     * @param utilisateur : un objjet utilisateur
     *
     * @return le message mis à jour
     */
    public static String personnaliserMessage(final String message, final UtilisateurBean utilisateur) {

        String[] replacees = new String[] {
                Personnalisation.CODE.toString(),
                Personnalisation.NOM.toString(),
                Personnalisation.PRENOM.toString(),
                Personnalisation.EMAIL.toString(),
        };

        String[] replacors = new String[] {
                utilisateur.getCode(),
                utilisateur.getNom(),
                utilisateur.getPrenom(),
                utilisateur.getAdresseMail(),
        };

        return StringUtils.replaceEach(message, replacees, replacors);
    }

    /**
     * Converti un tableau d'octet en une chaine hexa
     *
     * @param bytes : Un tableau d'octet
     *
     * @return une chaine représeantant les octets en HexString.
     */
    public static String toHexString(byte[] bytes) {
        final StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            sb.append(Integer.toString((b & 0xff) + 0x100, 16).substring(1));
        }
        return sb.toString();
    }
}
