/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.offreformation.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.IExtension;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.ModuleHelper;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.jspbeans.FormationJspBean;
import com.ksup.objetspartages.jspbeans.sub.ParcoursDetailViewBean;
import com.ksup.objetspartages.jspbeans.sub.ParcoursViewBean;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.om.Parcours;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;

public final class OffreFormationUtil {

    public static final String ID_EXTENSION = "ofin";

    public static final String CODE_DROIT_ONGLET_CARACTERISTIQUE = "goc";

    public static final String CODE_DROIT_ONGLET_AFFICHAGE = "goa";

    private static final Logger LOG = LoggerFactory.getLogger(OffreFormationUtil.class);

    public static IExtension getExtension() {
        return ExtensionHelper.getExtensionManager().getExtension(ID_EXTENSION);
    }

    public static String getProperty(String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    public static IModule getModule(String idBean) {
        return ModuleHelper.getModule(ID_EXTENSION, idBean);
    }

    public static String buildLibelleCourtAnneeSemestreParcours(ParcoursBean parcoursBean) {
        return buildLibelleCourtAnneeSemestreParcours(parcoursBean, true);
    }

    /**
     * Méthode qui permet de calculer un libellé de année - semestre d'un parcours en fonction de celui-ci.
     *
     * <br/>
     * <strong>Règles métier : </strong><br/>
     *
     * <ul>
     *     <li>Parcours sur 1 semestre qui démarre au semestre 2 année L1 => L2 - S2</li>
     *      <li>Parcours sur 2 semestres qui démarre au semestre 3 année L2 => L2 - S3 & S4</li>
     *      <li>Parcours sur 3 semestres qui démarre au semestre 3 année L2 => L - S3 A S5</li>
     *      <li>Parcours sur 6 semestres qui démarre au semestre 1 année L1 =>  L - S1 A S6</li>
     * </ul>
     *   Rappel : le libellé année est paramétrable : on peut avoir L, L1, Licence, DUT, PACES etc ....
     *
     * @param parcours
     * Prend en entrée un objet {@link Parcours}, pour avoir la gestion de la langue dans le libellé et la gestion des méthodes "getLibelleXXX"
     * @return
     */
    public static String buildLibelleCourtAnneeSemestreParcours(ParcoursBean parcours, boolean abreviation) {
        if (parcours != null) {
            String espace = " ";
            StringBuilder resultat = new StringBuilder();
            resultat.append(StringUtils.defaultString(LabelUtils.getLibelle("2031", parcours.getAnnee(), parcours.getLangue())));

            if (resultat.length() > 0) {
                resultat.append(" - ");
            }
            if(isConstructionParcoursPresent()) {
                String semestrePrefix;
                if (abreviation) {
                    semestrePrefix = getMessage("FO.SEMESTRE_PREFIX");
                } else {
                    semestrePrefix = getMessage("FO.SEMESTRE_PREFIX_LONG") + " ";
                }
                //Cas duree parcours > 2 semestres
                //Start et duree ne sont normalement pas null
                Integer start = parcours.getNumeroPremierSemestre();
                Integer duree = parcours.getDureeSemestre();
                Integer fin = start + duree - 1;
                String semestreSeparator = getMessage("FO.SEMESTRE_SEPARATOR");
                if (duree > 2) {
                    semestreSeparator = getMessage("FO.SEMESTRE_SEPARATOR_A");
                }
                if (abreviation) {
                    resultat.append(semestrePrefix).append(start);
                    if (!start.equals(fin)) {
                        resultat.append(espace).append(semestreSeparator).append(espace).append(semestrePrefix).append(fin);
                    }
                } else {
                    resultat.append(semestrePrefix).append(start);
                    if (!start.equals(fin)) {
                        resultat.append(espace).append(semestreSeparator).append(espace).append(fin);
                    }
                }
            }else{
                resultat.append(LabelUtils.getLibelle(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), parcours.getLibelleParcoursSemestre(), parcours.getLangue()));
            }
            return resultat.toString();
        } else {
            return StringUtils.EMPTY;
        }
    }

    public static Map<String, String> getResponsablesAvecFonctionEtUrl(FormationBean formationBean) {
        Map<String, String> responsables = new HashMap<>();
        ContexteUniv ctx = ContexteUtil.getContexteUniv();
        String[] stResponsables = StringUtils.split(formationBean.getResponsablesFiche(), ";");
        String langueResponsable;
        UrlProvider urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        for (String codeLangue : stResponsables) {
            StringBuilder content = new StringBuilder();
            String[] codeLangueArray = StringUtils.split(codeLangue, ",");
            if (codeLangueArray.length > 0) {
                AnnuaireModele annuaire;
                try {
                    annuaire = FicheAnnuaire.getFicheAnnuaire(codeLangueArray[0], ctx.getLangue());
                } catch (Exception e) {
                    LOG.error("Erreur dans la récupération de la fiche annuaire", e);
                    continue;
                }
                if (annuaire != null) {
                    String url = urlProvider.provideAbsoluteUrl(annuaire.getPersistanceBean());
                    if (annuaire.getCivilite().length() > 0 && !annuaire.getCivilite().equals("0000")) {
                        content.append(annuaire.getCivilite()).append(" ");
                    }
                    content.append(annuaire.getPrenom()).append(" ").append(annuaire.getNom());
                    if (codeLangueArray.length > 1 && codeLangueArray[1] != null) {
                        String langue = codeLangueArray[1];
                        if ((StringUtils.isNotEmpty(codeLangueArray[1]) && !StringUtils.isNumeric(codeLangueArray[1])) && StringUtils.contains(codeLangueArray[1], ",LANGUE=")) {
                            langue = StringUtils.remove(codeLangueArray[1], ",LANGUE=");
                        }
                        if (StringUtils.isNotEmpty(langue) && StringUtils.isNumeric(langue)) {
                            langueResponsable = langue;
                        } else {
                            langueResponsable = String.valueOf(LangueUtil.getIndiceLocale(ctx.getLocale()));
                        }
                    } else {
                        langueResponsable = String.valueOf(LangueUtil.getIndiceLocale(ctx.getLocale()));
                    }
                    if (codeLangueArray.length > 2 && !"0000".equals(codeLangueArray[2])) {
                        content.append(" (").append(LabelUtils.getLibelle(OffreFormationTypeLibelle.TITRE_RESPONSABLE.getValeur(), codeLangueArray[2], langueResponsable)).append(")");
                    }
                    responsables.put(content.toString(), url);
                }
            }
        }
        return responsables;
    }

    public static boolean isSuitableForProgramme(Pair<ParcoursBean, String> parcoursLink) {
        boolean isNotConditionned = StringUtils.equals("0000", parcoursLink.getRight());
        return isNotConditionned;
    }

    /**
     * Verifie si l'utilisateur en cours a les droit de modifier les infos de l'onglet de la fiche
     * @param nomOnglet
     * @return vrai ou faux
     */
    public static Boolean getAfficherOnglet(final String nomOnglet) {
        final AutorisationBean autorisationsUser = ContexteUtil.getContexteUniv().getAutorisation();
        return (autorisationsUser != null && autorisationsUser.possedePermissionTech(nomOnglet));
    }

    public static String urlParcours(Long idFormation) {
        return URLResolver.getAbsoluteUrl(getProperty("FO.MON_PARCOURS.URL.SERVLET") + idFormation, ContexteUtil.getContexteUniv());
    }

    public static Set<ParcoursDetailViewBean> aggregateParcoursForDetailsArea(FormationJspBean formationJspBean) {
        if (formationJspBean.getFormationListParcours() != null && !formationJspBean.getFormationListParcours().getListeParcoursDiplomant().isEmpty()) {
            Set<ParcoursDetailViewBean> result = new HashSet<>();
            for (ParcoursViewBean parcoursDiplomant : formationJspBean.getFormationListParcours().getListeParcoursDiplomant()) {
                if (parcoursDiplomant.getListeParcoursAmont() != null) {
                    for (Set<ParcoursDetailViewBean> parcoursAmonts : parcoursDiplomant.getListeParcoursAmont().values()) {
                        result.addAll(parcoursAmonts);
                    }
                }
            }
            return result;
        } else {
            return Collections.emptySet();
        }
    }

    /**
     * Test si le module de construction de parcours est présent ou non.
     * @return true si le module est présent
     */
    public static boolean isConstructionParcoursPresent() {
        try {
            Object bean = ApplicationContextManager.getBean(ID_EXTENSION, "constructionparcours");
            return bean != null;
        } catch (NoSuchBeanDefinitionException e) {
            LOG.debug("Construction de parcours absent", e);
        }
        return false;
    }
}
