/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ofin.migration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.flywaydb.core.api.migration.MigrationChecksumProvider;
import org.flywaydb.core.api.migration.jdbc.JdbcMigration;

import com.kosmos.migration.exception.MigrationException;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.services.ServiceLabel;

/**
 * Migration des métiers en libellés.
 */
public class V2_7_0_5__migration_metiers implements JdbcMigration, MigrationChecksumProvider {

    private ServiceLabel serviceLabel;

    private final static String LANGUE_METIER = "0";

    @Override
    public Integer getChecksum() {
        return 1540811702;
    }

    @Override
    public void migrate(Connection connection) throws SQLException {
        serviceLabel = ServiceManager.getService(ServiceLabel.class);
        if (!hasToMigrateWork(connection)) {
            return;
        }
        List<MigrationWorkBean> works = getAllWorks(connection);
        Map<Long, String> migrationMap = migrateToLabel(works);
        migrationFormations(migrationMap, connection);
        dropTableMetier(connection);
    }

    /**
     * Suppression de la table métier.
     * @param connection la connexion à la base de données=
     */
    private void dropTableMetier(Connection connection) throws SQLException{
        String request = "DROP TABLE METIER";
        try (Statement statement = connection.createStatement()) {
            statement.execute(request);
        }
    }

    /**
     * Migre l'esemble des formations vers le nouveau format pour les métiers.
     * @param migrationMap la map de correspondance &lt;idMétier, codeLibellé&gt;
     * @param connection la connexion à la base de données
     * @throws SQLException exception de connexion à la base
     */
    private void migrationFormations(Map<Long, String> migrationMap, Connection connection) throws SQLException {
        Map<Long, String> formationConvertedWorks = new HashMap<>();
        String request = "select ID_FORMATION, METIERS from FORMATION WHERE METIERS IS NOT NULL AND METIERS <> \"\"";
        try (Statement statement = connection.createStatement(); ResultSet result = statement.executeQuery(request)) {
            while (result.next()) {
                String convertedWorks = convertWorkColumn(result.getString("METIERS"), migrationMap);
                formationConvertedWorks.put(result.getLong("ID_FORMATION"), convertedWorks);
            }
        }
        String updateRequest = "UPDATE FORMATION SET METIERS = ? WHERE ID_FORMATION = ?";
        try (PreparedStatement statement = connection.prepareStatement(updateRequest)) {
            for (Entry<Long, String> entry : formationConvertedWorks.entrySet()) {
                statement.setString(1, entry.getValue());
                statement.setLong(2, entry.getKey());
                statement.addBatch();
            }
            statement.executeBatch();
        }
    }

    /**
     * Migre le contenu d'une colonne métier vers le nouveau format.
     * <ul>
     *     <li>Ancien format : idMetier;idMetier;idMetier</li>
     *     <li>Nouveau format : codeLibelle;codeLibelle;codeLibelle</li>
     * </ul>
     * @param metiers la colonne à migrer.
     * @param migrationMap la table de migration
     * @return une colonne convertie
     */
    private String convertWorkColumn(String metiers, Map<Long, String> migrationMap) {
        if (StringUtils.isEmpty(metiers)) {
            return StringUtils.EMPTY;
        }
        List<String> convertedWorks = new ArrayList<>();
        for (String metier : metiers.split(";")) {
            convertedWorks.add(migrationMap.get(Long.valueOf(metier)));
        }
        return convertedWorks.stream()
            .collect(Collectors.joining(";"));
    }

    /**
     * Migre l'ensemble des métiers en libellés.
     * @param works les métiers
     * @return une map de correspondance entre idMétier et codeLibellé
     */
    private Map<Long, String> migrateToLabel(List<MigrationWorkBean> works) {
        Map<Long, String> correlationTable = new HashMap<>();
        works.stream()
            .filter(w -> Long.valueOf(0).equals(w.idMetierParent))
            .forEach(migrationWorkBean -> {
                LabelDto labelDto = new LabelDto(MetierSerializer.TYPE_LIBELLE_METIER, migrationWorkBean.codeRome, migrationWorkBean.appelationMetier, LANGUE_METIER);
                serviceLabel.save(labelDto);
                correlationTable.put(migrationWorkBean.idMetier, labelDto.getCode());
            });
        works.stream()
            .filter(w -> !Long.valueOf(0).equals(w.idMetierParent))
            .forEach(workWithParent -> {
                LabelDto labelDto = getLabelFromMetier(workWithParent, correlationTable.get(workWithParent.idMetierParent));
                serviceLabel.save(labelDto);
                correlationTable.put(workWithParent.idMetier, labelDto.getCode());
            });
        return correlationTable;
    }

    /**
     * Récupère un label depuis un métier et un parent.
     * @param migrationWorkBean le métier
     * @param labelParentCode le code du parent
     * @return le dto rempli
     */
    private LabelDto getLabelFromMetier(MigrationWorkBean migrationWorkBean, String labelParentCode) {
        LabelDto labelDto = new LabelDto(MetierSerializer.TYPE_LIBELLE_METIER, migrationWorkBean.codeOGR, migrationWorkBean.appelationMetier, LANGUE_METIER);
        LabelDto parentLabel = serviceLabel.getByTypeCodeLanguage(MetierSerializer.TYPE_LIBELLE_METIER, labelParentCode, LANGUE_METIER);
        if (parentLabel == null) {
            throw new MigrationException(String.format("Impossible de récupérer le libellé parent avec le code \"%s\"", labelParentCode));
        }
        labelDto.setIdLibelleParent(parentLabel.getId());
        return labelDto;
    }

    /**
     * Si pas de table métier, pas besoin de migrer.
     * @param connection la connexion à la base de données.
     * @return false si la table n'est pas présente
     * @throws SQLException exception de connexion à la base
     */
    private boolean hasToMigrateWork(Connection connection) throws SQLException {
        String request = "SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE table_schema = (SELECT database()) AND table_name = 'METIER'";
        try (Statement statement = connection.createStatement(); ResultSet result = statement.executeQuery(request)) {
            return result.next();
        }
    }

    /**
     * Récupère l'ensemble des métiers présents en base.
     * @param connection la connexion à la base de données.
     * @return les métiers
     * @throws SQLException exception de connexion à la base
     */
    private List<MigrationWorkBean> getAllWorks(Connection connection) throws SQLException {
        List<MigrationWorkBean> retour = new ArrayList<>();
        String request = "SELECT ID_METIER, CODE_ROME, APPELATION_METIER, CODE_OGR, ID_METIER_PARENT from METIER";
        try (Statement statement = connection.createStatement(); ResultSet result = statement.executeQuery(request)) {
            while (result.next()) {
                retour.add(new MigrationWorkBean(result.getLong("ID_METIER"), result.getString("CODE_ROME"), result.getString("APPELATION_METIER"), result.getString("CODE_OGR"), result.getLong("ID_METIER_PARENT")));
            }
        }
        return retour;
    }

    private static class MigrationWorkBean {

        private long idMetier;

        private String codeRome;

        private String appelationMetier;

        private String codeOGR;

        private Long idMetierParent;

        public MigrationWorkBean(final long idMetier, final String codeRome, final String appelationMetier, final String codeOGR, final Long idMetierParent) {
            this.idMetier = idMetier;
            this.codeRome = codeRome;
            this.appelationMetier = appelationMetier;
            this.codeOGR = codeOGR;
            this.idMetierParent = idMetierParent;
        }
    }
}
