/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.pdf;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections4.MultiValuedMap;
import org.apache.commons.collections4.multimap.ArrayListValuedHashMap;
import org.apache.commons.lang.builder.CompareToBuilder;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

import com.kosmos.layout.card.bean.CardBean;
import com.kosmos.layout.card.service.ServiceCard;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kosmos.service.ServiceSpecifiqueBean;
import com.kportal.pdf.ExportFichePDFContexte;
import com.kportal.pdf.PDFFreeMarkerConfig;
import com.kportal.pdf.service.ServiceExportFicheToPDF;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.ImageViewBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ResponseRechercheParcoursBean;
import com.ksup.objetspartages.jsputils.ServiceFormationViewBuilder;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.Parcours;
import com.ksup.objetspartages.service.ServiceRechercheParcours;
import com.ksup.objetspartages.util.FormationHelper;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.util.RessourceUtils;
import com.univ.utils.URLResolver;

import freemarker.template.TemplateModelException;

/**
 * Implémentation du service PDF spécifique à la Formation.
 */
public class ServicePDFFormation extends ServiceExportFicheToPDF implements ServiceSpecifiqueBean<FormationBean> {

    /**
     * Logger.
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(ServicePDFFormation.class);

    /**
     * Le service label.
     */
    private ServiceLabel serviceLabel;

    /**
     * Service de gestion des cartes.
     */
    private ServiceCard serviceCard;

    /**
     * Service utilitaire d'accès aux données de la fiche Formation pour l'affichage.
     */
    private ServiceFormationViewBuilder serviceFormationViewBuilder;

    /**
     * Service de recherche des parcours.
     */
    private ServiceRechercheParcours serviceRechercheParcours;

    public ServicePDFFormation() {
        super();
    }

    @Override
    protected void ajouterDonneesSpecifiques(final ExportFichePDFContexte exportPDFContexte, final Map<String, Object> modele) {
        final FicheUniv fiche = exportPDFContexte.getFiche();
        // Calcul des cartes
        modele.put("mapCartes", calculerMapCartes((Formation) fiche));
        // Calcul de l'URL de l'image
        ImageViewBean imageViewBean = serviceFormationViewBuilder.generateImageViewBean(fiche, "1", StringUtils.EMPTY);
        String imageUrl = StringUtils.EMPTY;
        if (!imageViewBean.isBeanEmpty()) {
            if (imageViewBean.getRessourceBean() != null) {
                imageUrl = RessourceUtils.getUrl(imageViewBean.getRessourceBean());
            } else {
                imageUrl = imageViewBean.getBackupUrl();
            }
            imageUrl = URLResolver.getAbsoluteUrl(imageUrl);
        }
        modele.put("urlImagePresentation", imageUrl);
        // Lecture de la liste des métiers
        List<LabelDto> listeMetiers = serviceLabel.getFromCodes(((Formation) fiche).getMetiers(), fiche.getLangue(), ";", MetierSerializer.TYPE_LIBELLE_METIER);
        modele.put("listeMetiers", listeMetiers);
        // Calcule de la map des parcours liés aux parcours diplômants
        modele.put("mapParcours", calculerMapParcours((Formation) fiche));
    }

    @Override
    protected void addUtilitiesInTemplateModel(final Map<String, Object> modele) {
        super.addUtilitiesInTemplateModel(modele);
        try {
            modele.put("FormationHelper", PDFFreeMarkerConfig.getReferencePourMethodesStatiques(FormationHelper.class));
        } catch (TemplateModelException e) {
            LOGGER.error("Erreur lors l'ajout de la classe FormationHelper au modèle.", e);
        }
        try {
            modele.put("OffreFormationUtil", PDFFreeMarkerConfig.getReferencePourMethodesStatiques(OffreFormationUtil.class));
        } catch (TemplateModelException e) {
            LOGGER.error("Erreur lors l'ajout de la classe OffreFormationUtil au modèle.", e);
        }
    }

    /**
     * Retourne la map des cartes de la formation classées par nom de la carte.
     * @param formation la formation
     * @return la liste des {@link CardBean} associés à la formation
     */
    protected MultiValuedMap<String, CardBean> calculerMapCartes(final Formation formation) {
        final Collection<CardBean> listeCartes = serviceCard.getCards(formation).values();
        // Constitution d'une map avec le nom de la carte comme clé
        MultiValuedMap<String, CardBean> mapCartes = new ArrayListValuedHashMap<>();
        for (CardBean carte : listeCartes) {
            mapCartes.put(StringUtils.replace(carte.getClass().getSimpleName(), "CardBean", StringUtils.EMPTY), carte);
        }
        return mapCartes;
    }

    /**
     * Retourne la map des parcours de la formation classées par parcours diplômants.
     * @param formation la formation
     * @return la liste des {@link com.ksup.objetspartages.om.Parcours} associés à la formation
     */
    protected Map<String, List<Parcours>> calculerMapParcours(final Formation formation) {
        // Constitution d'une map avec l'id du parcours diplômant comme clé
        Map<String, List<Parcours>> mapParcours = new ConcurrentHashMap<>();
        try {
            for (Parcours parcoursDiplomant : formation.getFichesParcoursFiche()) {
                mapParcours.put(String.valueOf(parcoursDiplomant.getIdParcours()), lireParcoursLies(parcoursDiplomant));
            }
        } catch (Exception e) {
            LOGGER.error("Erreur lors du calcul de la map des parcours pour la formation '{}", formation.getIdFormation(), e);
        }
        return mapParcours;
    }

    /**
     * Lit la liste des parcours liés à un parcours.
     * @param parcours le parcours pour lequel on souhaite récupérer les parcours liés.
     * @return la liste des {@link Parcours} liés
     */
    protected List<Parcours> lireParcoursLies(final Parcours parcours) {
        final List<Parcours> listeParcours = new ArrayList<>();
        try {
            final ResponseRechercheParcoursBean resultatRechercheParcours = serviceRechercheParcours.findTousParcoursForProgramme(parcours.getPersistenceBean());
            if (resultatRechercheParcours != null) {
                final Set<ParcoursBean> listeParcoursDuParcours = resultatRechercheParcours.getParcours();
                if (CollectionUtils.isNotEmpty(listeParcoursDuParcours)) {
                    for (ParcoursBean parcoursBean : listeParcoursDuParcours) {
                        listeParcours.add(new Parcours(parcoursBean));
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("Erreur lors de la récupération des parcours liés au parcours '{}", parcours.getIdParcours(), e);
        }
        // Tri des parcours
        Comparator<Parcours> parcoursComparator = (Parcours parcours1, Parcours parcours2) -> new CompareToBuilder().append(parcours1.getNumeroPremierSemestre(), parcours2.getNumeroPremierSemestre()).append(parcours1.getIntituleLong(), parcours2.getIntituleLong()).toComparison();
        Collections.sort(listeParcours, parcoursComparator);
        return listeParcours;
    }

    @Required
    public void setServiceCard(ServiceCard serviceCard) {
        this.serviceCard = serviceCard;
    }

    @Required
    public void setServiceFormationViewBuilder(final ServiceFormationViewBuilder serviceFormationViewBuilder) {
        this.serviceFormationViewBuilder = serviceFormationViewBuilder;
    }

    @Required
    public void setServiceRechercheParcours(final ServiceRechercheParcours serviceRechercheParcours) {
        this.serviceRechercheParcours = serviceRechercheParcours;
    }

    @Required
    public void setServiceLabel(final ServiceLabel serviceLabel) {
        this.serviceLabel = serviceLabel;
    }
}
