/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.batch;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.lang3.StringUtils;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.controllers.SGController;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kportal.cms.objetspartages.annotation.FicheAnnotationHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.kportal.tag.interpreteur.impl.InterpreteurLienIdFiche;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.url.bean.UrlBean;
import com.univ.url.provider.UrlProvider;
import com.univ.url.service.ServiceUrl;
import com.univ.utils.ContexteUniv;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.URLResolver;
import com.univ.xhtml.HTMLLinkExtractor;

public class ScanFiche extends LogReportJob implements Runnable {

    private class BrokenLinkReport {

        private long count = 0;

        private String reason = null;

        public BrokenLinkReport(final String reason) {
            this.reason = reason;
        }

        public String getReason() {
            return reason;
        }

        public void setReason(final String reason) {
            this.reason = reason;
        }

        public long getCount() {
            return count;
        }

        public void inc() {
            this.count++;
        }
    }

    private static final String PATTERN_TBOUNDARY = StringUtils.repeat('▄', 80);

    private static final String PATTERN_SECTION = StringUtils.repeat('═', 80);

    private static final String PATTERN_SEPARATOR = "   " + StringUtils.repeat('─', 77);

    private static final String PATTERN_BBOUNDARY = StringUtils.repeat('▀', 80);

    private static final String PATTERN_FICHE = "  • Fiche [{}]";

    private static final String PATTERN_ERROR = "    - Erreur lors de l''interrogation du lien [{0}] : {1}";

    private static final Logger LOG = LoggerFactory.getLogger(ScanFiche.class);

    private static final String MODE_REPONSE = "reponse";

    private static final String MODE_LIEN = "lien";

    private static final String SCAN_MODE = "scanMode";

    private static final int MAX_FICHES = 100;

    private static final int MAX_ERREUR = 10;

    private static final int MAX_ERREUR_LIEN_BRISE = 25;

    private static final int TIMEOUT = 5000;

    private static final int READTIMEOUT = 10000;

    /**
     * Code d'un objet pouvant contenir un champ de type URL qu'il faut vérifier.
     */
    private static final String CODE_OBJET_URL_SPECIFIQUE = "0018";

    /** The scan mode. */
    private String scanMode = MODE_LIEN;

    /** The max fiches. */
    private int maxFiches;

    /** The max erreurs. */
    private int maxErreurs;

    /** Le nombre de fois ou un même lien est comptabilisé. */
    private static int maxErreursLienBrise;

    /** The timeout. */
    private int timeout;

    /** The readtimeout. */
    private int readTimeout;

    private final ServiceMetatag serviceMetatag;

    private final ServiceUrl serviceUrl;

    private List<String> initMessages;

    private Map<String, BrokenLinkReport> lstLienInvalide = new HashMap<>();

    private Set<String> lstLienValide = new HashSet<>();

    private Pattern brokenLinkPattern;

    /**
     * Methode permettant de recuperer une valeur de properties avec une valeur par defaut, mais en informant l'utilisateur.
     *
     * @param key
     *            la clé de properties dont on vaut la valeur
     * @param defaultValue
     *            la valeur par défaut
     * @param messages
     *            Une liste pour mettre de côté des messages à ecrire dans la log.
     * @return la valeur correspondant à la key, defaultValue sinon.
     */
    private int getCorePropertyAsInt(final String key, final int defaultValue, final List<String> messages) {
        String value = null;
        int intValue;
        try {
            value = PropertyHelper.getCoreProperty(key);
            intValue = value != null ? Integer.valueOf(value) : defaultValue;
        } catch (NumberFormatException nfe) {
            messages.add(MessageFormat.format("La valeur [{0}] de la clé [{1}] n''est pas un entier. Utilisation de la valeur par défaut [{2}]", value, key, defaultValue));
            intValue = defaultValue;
        }
        return intValue;
    }

    public ScanFiche () {
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        serviceUrl = ServiceManager.getServiceForBean(UrlBean.class);
    }

    @Override
    public void init(final JobExecutionContext jobCtx) {
        scanMode = jobCtx.getMergedJobDataMap().getString(SCAN_MODE);
        initMessages = new ArrayList<>();
        maxFiches = getCorePropertyAsInt("scanFiche.maxFiches", MAX_FICHES, initMessages);
        maxErreurs = getCorePropertyAsInt("scanFiche.maxErreurs", MAX_ERREUR, initMessages);
        timeout = getCorePropertyAsInt("scanFiche.timeout", TIMEOUT, initMessages);
        readTimeout = getCorePropertyAsInt("scanFiche.readtimeout", READTIMEOUT, initMessages);
        brokenLinkPattern = Pattern.compile("<span class=\"" + InterpreteurLienIdFiche.BROKEN_LINK_CLASS + "\"[^>]+>([^<]+)</span>");
        maxErreursLienBrise = getCorePropertyAsInt("scanFiche.maxErreursLienBrise", MAX_ERREUR_LIEN_BRISE, initMessages);
        if (initMessages.size() == 0) {
            initMessages = null;
        }
    }

    @Override
    public void run() {
        if (scanMode.equals(MODE_LIEN)) {
            parcoursLiensFiche();
        } else if (scanMode.equals(MODE_REPONSE)) {
            testReponseFiche();
        }
    }

    @Override
    public void perform() {
        run();
    }

    /**
     * Test liens fiche.
     */
    private void parcoursLiensFiche() {
        try {
            //on commence par simuler une authentification via le processus d'identification
            final String leCookie = getCookieDeConnexion();
            if (StringUtils.isNotEmpty(leCookie)) {
                LOG.info(PATTERN_TBOUNDARY);
                LOG.info(" Démarrage de la détection des liens brisés");
                LOG.info(PATTERN_SEPARATOR);
                if (initMessages != null) {
                    LOG.info("  | Attention ! il y a des erreurs de configuration : ");
                    for (String message : initMessages) {
                        LOG.info("  |  • {}", message);
                    }
                    LOG.info(PATTERN_SEPARATOR);
                }
                LOG.info("  | Seule les " + maxErreursLienBrise + " premières erreurs pour chaque URL sont comptabilisées.");
                LOG.info("  | (Les autres fiches avec les mêmes liens brisés ne remonteront pas en erreur.)");
                LOG.info("  | Timeout de connexion : {} seconde(s)", (timeout / 1000));
                LOG.info("  | Timeout de réponse  : {} seconde(s)", (readTimeout / 1000));
                for (final String codeObjet : ReferentielObjets.getListeCodesObjet()) {
                    final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(codeObjet);
                    if (ficheUniv == null || !FicheAnnotationHelper.isFicheFrontOffice(ficheUniv)) {
                        continue;
                    }
                    final Collection<String> urlFichesAtester = calculerUrlFiches(codeObjet);
                    LOG.info(PATTERN_SECTION);
                    if (urlFichesAtester.size() > 0) {
                        LOG.info(" ■ Traitement des fiches \"{}\"", ReferentielObjets.getLibelleObjet(codeObjet));
                        appelsLiensFiches(urlFichesAtester, leCookie);
                    } else {
                        LOG.info(" ■ Traitement des fiches \"{}\" : aucune fiche à traiter", ReferentielObjets.getLibelleObjet(codeObjet));
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("Scanfiche : Echec de la détection des liens brisés", e);
        } finally {
            LOG.info(PATTERN_BBOUNDARY);
        }
    }

    /**
     * Renvoie le cookie de connexion au site
     *
     * @return le cookie de connexion au site
     * @throws MalformedURLException
     * @throws IOException
     */
    private String getCookieDeConnexion() throws IOException {
        String leCookie = StringUtils.EMPTY;
        try {
            final ContexteUniv ctx = new ContexteUniv(StringUtils.EMPTY);
            final String sUrl = URLResolver.getAbsoluteUrl(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=IDENTIFICATION&ACTION=CREER_COOKIE", ctx);
            final URLConnection urlConnection = (new URL(sUrl)).openConnection();
            urlConnection.setConnectTimeout(timeout);
            urlConnection.setReadTimeout(readTimeout);
            leCookie = urlConnection.getHeaderField("Set-Cookie");
        } catch (final Exception e) {
            LOG.error("Scanfiche : Echec de la recuperation du cookie de connexion", e);
        }
        return leCookie;
    }

    /**
     * Peuple la liste globale "listeInfosFiches" avec les informations correspondantes a l'objet passe en parametre Si l'objet est "Lien", peuple egalement la liste globale
     * "listeInfosLiens".
     *
     * @param codeObjet
     */
    private Collection<String> calculerUrlFiches(final String codeObjet) {
        Collection<String> urlFiches = new ArrayList<>();
        try {
            final List<MetatagBean> metas = serviceMetatag.getOnlineMetas(codeObjet);
            for (MetatagBean currentMeta : metas) {
                UrlBean canonicalUrl = serviceUrl.getCanonicalUrlByMetaId(currentMeta.getIdMetatag());
                urlFiches.add(URLResolver.getAbsoluteUrl(canonicalUrl));
                if (CODE_OBJET_URL_SPECIFIQUE.equals(codeObjet)) {
                    try {
                        final FicheUniv ficheUniv = FicheUnivHelper.getFicheParIdMeta(currentMeta.getMetaIdFiche());
                        if (ficheUniv == null || !FicheAnnotationHelper.isFicheFrontOffice(ficheUniv)) {
                            continue;
                        }
                        urlFiches.add(BeanUtils.getProperty(ficheUniv, "url"));
                    } catch (final Exception e) {
                        LOG.debug("Erreur lors de la récupération de l'attribut url de la fiche " + currentMeta.getMetaLibelleFiche() + " de type " + currentMeta.getMetaCodeObjet(), e);
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("impossible de calculer la liste des fiches de type " + codeObjet, e);
            urlFiches = Collections.emptyList();
        }
        return urlFiches;
    }

    /**
     * Teste les liens d'un groupe de fiches, pour un meme site
     * @param urls liste de liens à vérifier
     * @param leCookie cookie de session à transmettre dans le header
     * @throws Exception
     */
    private void appelsLiensFiches(final Collection<String> urls, final String leCookie) {
        int nbFicheErreur = 0;
        int nbErreurTotal = 0;
        for (final String sUrl : urls) {
            URLConnection urlConnection = null;
            try {
                final URL url = new URL(sUrl);
                urlConnection = (url).openConnection();
                urlConnection.setConnectTimeout(timeout);
                urlConnection.setReadTimeout(readTimeout);
                urlConnection.setRequestProperty("Cookie", leCookie);
                ((HttpURLConnection) urlConnection).setInstanceFollowRedirects(false);
                final int responseCode = ((HttpURLConnection) urlConnection).getResponseCode();
                if (responseCode == 302) {
                    //On ne suit pas les redirections sur une fiche
                    continue;
                } else if (responseCode != 200) {
                    nbErreurTotal++;
                    nbFicheErreur++;
                    String message = MessageFormat.format("    - Impossible de lire la fiche - erreur HTTP: {0} - {1}", responseCode, HttpStatus.getStatusText(responseCode));
                    ecrireErreurLien(sUrl, message, 1);
                } else {
                    lstLienValide.add(sUrl);
                    String content = lectureContenuDepuisUrl(urlConnection);
                    int nbErreur = analyseListeDesLiensExtraits(sUrl, StringUtils.substringAfter(sUrl, url.getAuthority()), content, leCookie);
                    nbErreur += analyseBrokenLinK(sUrl, content, nbErreur);
                    if (nbErreur > 0) {
                        nbErreurTotal += nbErreur;
                        nbFicheErreur++;
                    }
                }
            } catch (final Exception e) {
                nbErreurTotal++;
                nbFicheErreur++;
                String message = MessageFormat.format("    - Erreur technique lors de l''interrogation de la fiche : {0}", e.getMessage());
                ecrireErreurLien(sUrl, message, 1);
            } finally {
                if (urlConnection != null) {
                    ((HttpURLConnection) urlConnection).disconnect();
                }
            }
        }
        LOG.info(PATTERN_SEPARATOR);
        if (nbErreurTotal > 0) {
            LOG.info(MessageFormat.format("   Fin traitement : {0} erreur(s) dans {1} fiche(s) sur {2} fiche(s) traitée(s)", nbErreurTotal, nbFicheErreur, urls.size()));
        } else {
            LOG.info(MessageFormat.format("   Fin traitement : aucune erreur détectée sur {0} fiche(s) traitée(s)", urls.size()));
        }
    }

    private int analyseBrokenLinK(final String contentUrl, final String content, final int nbErreur) {
        int errorNumber = nbErreur;
        Matcher matcher = brokenLinkPattern.matcher(content);
        while (matcher.find()) {
            errorNumber++;
            ecrireErreurLien(contentUrl, "    - Erreur : lien brisé : \"" + matcher.group(1) + "\"", errorNumber);
        }
        return errorNumber;
    }

    /**
     * Analyse les liens extraits
     * @param sUrl
     * @param baseURL
     * @param contenu
     * @param leCookie
     */
    private int analyseListeDesLiensExtraits(final String sUrl, final String baseURL, final String contenu, final String leCookie) {
        final List<String> lstLien = HTMLLinkExtractor.GetInstance().extractLinksHtmlFromInput(baseURL, contenu, true);
        int nbErreur = 0;
        boolean isInvalid = false;
        String currentHost = null;
        boolean isToujoursOk = true;
        for (final String lien : lstLien) {
            String message = null;
            BrokenLinkReport report = null;
            // exclusion des processus métier
            if (lien.contains("/servlet/")) {
                continue;
            }
            String lienNettoye = nettoyerLien(lien);
            //Cas des URL relatives : on ajoute le domaine de la page courante
            if (StringUtils.startsWith(lienNettoye, "/")) {
                if (currentHost == null) {
                    try {
                        final URL url = new URL(sUrl);
                        currentHost = (url.getProtocol() != null ? url.getProtocol() : "http") + "://" + url.getAuthority();
                    } catch (MalformedURLException mue) {
                        //Ne peut pas arriver (traité dans la méthode appelante)
                    }
                }
                lienNettoye = currentHost + lienNettoye;
            }
            if (lstLienValide.contains(lienNettoye)) {
                //On a déjà testé le lien valide, on passe au suivant
                continue;
            } else if (StringUtils.isBlank(lienNettoye)) {
                isInvalid = true;
                nbErreur++;
                ecrireErreurLien(sUrl, "    - Erreur : lien vide trouvé - Modifiez le lien", nbErreur);
                continue;
            } else if (lstLienInvalide.containsKey(lienNettoye)) {
                //On a déjà testé le lien en erreur: on signale de nouveau le lien
                isInvalid = true;
                report = lstLienInvalide.get(lienNettoye);
                message = MessageFormat.format(PATTERN_ERROR, lien, report.getReason());
            } else {
                isInvalid = analyseDuLien(lienNettoye, leCookie, sUrl);
                if (isInvalid) {
                    report = lstLienInvalide.get(lienNettoye);
                    message = MessageFormat.format(PATTERN_ERROR, lien, report.getReason());
                }
            }
            if (isInvalid && report != null && report.getCount() < maxErreursLienBrise) {
                report.inc();
                nbErreur++;
                ecrireErreurLien(sUrl, message, nbErreur);
            }
        }
        return nbErreur;
    }

    private void ecrireErreurLien(final String urlFiche, final String message, final int nbErreur) {
        if (nbErreur == 1) {
            LOG.info(PATTERN_SEPARATOR);
            LOG.error(PATTERN_FICHE, urlFiche);
        }
        LOG.error(message);
    }

    /**
     * Nettoyage du lien passe en parametre
     * @param lien à nettoyer
     * @return lien nettoyé
     */
    private String nettoyerLien(final String lien) {
        String lienNettoye = lien;
        if (lienNettoye.contains("RF=")) {
            lienNettoye = lienNettoye.substring(0, lienNettoye.indexOf("RF="));
        }
        if (lienNettoye.contains("RH=")) {
            lienNettoye = lienNettoye.substring(0, lienNettoye.indexOf("RH="));
        }
        if (lienNettoye.endsWith("?")) {
            lienNettoye = lienNettoye.substring(0, lienNettoye.length() - 1);
        }
        return lienNettoye;
    }


    /**
     * Analyse le lien passe en parametre. Peuple les listes globales "lstLienAnalyse" et "lstLienCasse".
     *
     * @param lien le lien à analyser
     * @param leCookie le coockie de session à transmettre dans le header
     * @param sUrl l'url de la fiche source
     * @return
     */
    private boolean analyseDuLien(final String lien, final String leCookie, final String sUrl) {
        URLConnection urlConnection = null;
        boolean isInvalid = false;
        String cause = null;
        //analyse du lien
        try {
            final URL url = new URL(lien);
            urlConnection = url.openConnection();
            urlConnection.setRequestProperty("Cookie", leCookie);
            urlConnection.setConnectTimeout(timeout);
            urlConnection.setReadTimeout(readTimeout);
            final int responseCode = ((HttpURLConnection) urlConnection).getResponseCode();
            if (responseCode != 200 && responseCode != 302) {
                if (responseCode == 204) {
                    cause = "page sans contenu - Vérifiez le lien";
                } else if (responseCode == 206) {
                    cause = "page incomplète - Vérifiez le lien";
                } else if (responseCode == 301) {
                    cause = "page déplacée définitivement - Modifiez le lien";
                } else if (responseCode == 310) {
                    cause = MessageFormat.format("trop de redirections (erreur HTTP {0}) - Vérifiez le lien", responseCode);
                } else if (responseCode == 400 || responseCode == 414) {
                    cause = MessageFormat.format("syntaxe incorrecte (erreur HTTP {0} - {1}) - Vérifiez le lien", responseCode, HttpStatus.getStatusText(responseCode));
                } else if (responseCode == 401 || responseCode == 403 || responseCode == 407) {
                    cause = MessageFormat.format("page interdite ou protégée par une authentification (code HTTP:{0}) - Modifiez le lien", responseCode);
                } else if (responseCode == 404 || responseCode == 410) {
                    cause = MessageFormat.format("page introuvable (code HTTP {0}) - Modifiez le lien", responseCode);
                } else {
                    cause = MessageFormat.format("erreur technique (code HTTP {0} - {1}) - Vérifiez le lien", responseCode, HttpStatus.getStatusText(responseCode));
                }
                lstLienInvalide.put(lien, new BrokenLinkReport(cause));
                isInvalid = true;
            } else {
                lstLienValide.add(lien);
            }
        } catch (final Exception ee) {
            if (ee instanceof MalformedURLException) {
                cause = "URL invalide - Modifiez le lien";
            } else {
                cause = MessageFormat.format("Erreur technique ({0})- vérifiez le lien", ee.getMessage());
            }
            lstLienInvalide.put(lien, new BrokenLinkReport(cause));
            isInvalid = true;
        } finally {
            if (urlConnection != null && urlConnection instanceof HttpURLConnection) {
                ((HttpURLConnection) urlConnection).disconnect();
            }
        }
        return isInvalid;
    }


    /**
     * Renvoie le contenu renvoye par l'Url
     *
     * @param urlConnection url à laquelle récupérer le contenu
     * @return le contenu corrrespondant à l'url
     * @throws IOException
     */
    private String lectureContenuDepuisUrl(final URLConnection urlConnection) throws IOException {
        final StringBuilder input = new StringBuilder();
        final InputStream inputStream = urlConnection.getInputStream();
        final InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
        final BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
        String line;
        while ((line = bufferedReader.readLine()) != null) {
            input.append(line);
        }
        bufferedReader.close();
        inputStreamReader.close();
        inputStream.close();
        return input.toString();
    }

    /**
     * Test reponse fiche.
     */
    private void testReponseFiche() {
        final ContexteUniv ctx = new ContexteUniv("");
        ctx.setConnectionLongue(true);
        UrlProvider urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        try {
            LOG.info(PATTERN_TBOUNDARY);
            LOG.info(" ScanFiche : Début de l'interrogation des fiches");
            LOG.info(PATTERN_SECTION);
            if (initMessages != null) {
                LOG.info("  | Attention ! il y a des erreurs de configuration : ");
                for (String message : initMessages) {
                    LOG.info("  |  • {}", message);
                }
            }
            LOG.info(PATTERN_SEPARATOR);
            LOG.info("Nombre de fiches testées par type : {}", maxFiches);
            LOG.info("Maximum d'erreurs autorisées par type : {}", maxErreurs);
            LOG.info("Timeout de réponse : {} seconde(s)", (timeout / 1000));
            LOG.info(PATTERN_SEPARATOR);
            for (final String codeObjet : ReferentielObjets.getListeCodesObjet()) {
                final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(codeObjet);
                if (ficheUniv == null || !FicheAnnotationHelper.isFicheFrontOffice(ficheUniv)) {
                    continue;
                }
                LOG.info(" ■ Traitement {}", ReferentielObjets.getLibelleObjet(codeObjet));
                LOG.info(PATTERN_SEPARATOR);
                ficheUniv.setCtx(ctx);
                ficheUniv.init();
                final int count = ficheUniv.selectCodeLangueEtat("", "", "0003");
                final Random random = new Random();
                if (count > 0) {
                    /* Stockage des identifiants dans une liste */
                    final List<Long> listeGlobale = new ArrayList<>(count);
                    while (ficheUniv.nextItem()) {
                        if (!FicheUnivMgr.isFicheCollaborative(ficheUniv)) {
                            listeGlobale.add(ficheUniv.getIdFiche());
                        }
                    }
                    int nbLoops = 0;
                    int nbErreurs = 0;
                    while (listeGlobale.size() > 0 && nbLoops++ < maxFiches) {
                        // Choix aléatoire dans la liste
                        int indice = (int) (random.nextFloat() * listeGlobale.size());
                        if (indice == listeGlobale.size()) {
                            indice--;
                        }
                        // Lecture du la fiche
                        ficheUniv.setIdFiche(listeGlobale.get(indice));
                        ficheUniv.retrieve();
                        listeGlobale.remove(indice);
                        final ServiceInfosSiteProcessus serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
                        final InfosSite site = serviceInfosSite.determineSite(ficheUniv.getCodeRubrique(), true);
                        String sUrl = urlProvider.provideAbsoluteUrl(ficheUniv.getPersistanceBean());
                        sUrl = URLResolver.getAbsoluteUrl(sUrl, false, site);
                        sUrl = sUrl.replaceAll("STNAV=&RUBNAV=&", "");
                        sUrl = sUrl.replaceAll("&RH=", "");
                        try {
                            final HttpMethod method = new GetMethod(sUrl);
                            final HttpClient client = new HttpClient();
                            client.getParams().setSoTimeout(timeout);
                            final int status = client.executeMethod(method);
                            if (status != HttpStatus.SC_OK) {
                                LOG.error("Scanfiche : Erreur " + status + " (" + HttpStatus.getStatusText(status) + ") sur la fiche " + sUrl);
                                nbErreurs++;
                            }
                        } catch (final Exception e) {
                            LOG.error("Scanfiche : Erreur " + e.getMessage() + " sur la fiche " + sUrl);
                            nbErreurs++;
                        }
                        if (nbErreurs >= maxErreurs) {
                            LOG.info("!!! " + maxErreurs + " erreurs sur l'objet " + ReferentielObjets.getLibelleObjet(codeObjet) + " : passage à l'objet suivant");
                            break;
                        }
                    }
                }
            }
            LOG.info("***** ScanFiche : Fin de l'interrogation des fiches *****");
        } catch (final Exception e) {
            LOG.error("Echec test réponse fiche", e);
        } finally {
            LOG.info(PATTERN_SECTION);
            LOG.info(" ScanFiche : Fin de l'interrogation des fiches");
            LOG.info(PATTERN_TBOUNDARY);
            ctx.release();
        }
    }
}
