/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.sitemap;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.seo.util.SEOUtil;
import com.kosmos.service.impl.ServiceManager;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.url.bean.UrlBean;
import com.univ.url.service.ServiceUrl;
import com.univ.utils.URLResolver;

/**
 * Builder pour la génération de sitemap
 */
class SiteMapBuilder {

    private InfosSite infosSite;

    private List<String> allowedObjetCodes;

    private ServiceInfosSite serviceInfosSite;

    private ServiceRubrique serviceRubrique;

    private ServiceMetatag serviceMetatag;

    private ServiceUrl serviceUrl;

    private SimpleDateFormat sitemapDateFormater;

    /**
     * Constructeur de base.
     * @param siteCode code du site pour lequel on génère le sitemap
     */
    SiteMapBuilder(String siteCode) {
        initServices();
        sitemapDateFormater = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssXXX");
        infosSite = serviceInfosSite.getInfosSite(siteCode);
        allowedObjetCodes = SEOUtil.getSitemapObjects();
    }

    /**
     * Génère le sitemap.
     * @return le contenu du fichier sitemap
     */
    String generateSiteMap() {
        if(!doesSiteHaveSitemap()) {
            return StringUtils.EMPTY;
        }
        String sitemap = getSitemapHead();
        if(CollectionUtils.isNotEmpty(allowedObjetCodes)){
            sitemap += serviceUrl.getAllCanonicalUrlBySite(infosSite.getAlias()).stream()
                .filter(this::urlHaveToBeInSitemap)
                .map(this::getLine)
                .collect(Collectors.joining());
        }
        sitemap += getSitemapBottom();
        return sitemap;
    }

    /**
     * Génère une ligne dans le sitemap pour une url donnée.
     * @param urlBean l'url bean pour lequel on génère la ligne
     * @return la ligne correspondante à l'url
     */
    String getLine(UrlBean urlBean) {
        String absoluteUrl = URLResolver.getAbsoluteUrl(urlBean);
        if(StringUtils.isEmpty(absoluteUrl)) {
            return StringUtils.EMPTY;
        }
        String urlLine = "\t<url>\n\t\t<loc>" + absoluteUrl + "</loc>\n\t\t<lastmod>";
        urlLine += sitemapDateFormater.format(getUrlDate(urlBean));
        urlLine += "</lastmod>\n\t</url>\n";
        return urlLine;
    }

    /**
     * Renvoie la date à utiliser pour une url.
     * À savoir :
     * <ul>
     *     <li>Si c'est une url avec un meta on prend la date de modification du meta</li>
     *     <li>Sinon on prend la date de modification de l'url</li>
     * </ul>
     * @param urlBean l'url
     * @return la date à utiliser
     */
    Date getUrlDate(UrlBean urlBean) {
        if(urlBean.getIdMetatag() != null) {
            MetatagBean metatagBean = serviceMetatag.getById(urlBean.getIdMetatag());
            if(metatagBean != null && metatagBean.getMetaDateModification() != null) {
                return metatagBean.getMetaDateModification();
            }
        }
        return urlBean.getUpdateDate();
    }

    /**
     * Renvoi le pied du sitemap.
     * @return le pied du sitemap
     */
    String getSitemapBottom() {
        return "</urlset>";
    }

    /**
     * Renvoi l'entête du sitemap.
     * @return l'entête du sitemap
     */
    String getSitemapHead() {
        String sitemap = "<?xml version=\"1.0\" encoding=\"" + CharEncoding.DEFAULT + "\"?>\n";
        sitemap += "<urlset xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n";
        sitemap += "\t xsi:schemaLocation=\"http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd\"\n";
        sitemap += "\t xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">\n";
        return sitemap;
    }

    /**
     * Test si l'url doit être mis dans le sitemap. Règles à respecter :
     * <ul>
     *     <li>Le contenu doit être accepté dans le sitemap, voir {@link SEOUtil#getSitemapObjects()}</li>
     *     <li>Le contenu ne doit pas être marqué en tant que non indexé</li>
     *     <li>Leur rubrique ne doit pas être marquée en tant que non indexé</li>
     * </ul>
     * On gère également les urls qui n'ont pas d'id metatag, c'est à dire qu'ils ont une page de tête autre
     * @param urlBean l'url à tester
     * @return true si l'url doit être mis dans le sitemap
     */
    boolean urlHaveToBeInSitemap(UrlBean urlBean) {
        if(urlBean.getIdMetatag() != null) {
            MetatagBean metatagBean = serviceMetatag.getById(urlBean.getIdMetatag());
            if (metatagBean == null
                || !EtatFiche.EN_LIGNE.getEtat().equals(metatagBean.getMetaEtatObjet())
                || !allowedObjetCodes.contains(metatagBean.getMetaCodeObjet())
                || "1".equals(metatagBean.getMetaNonIndexe())) {
                return false;
            }
        }
        RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(urlBean.getSection());
        return rubriqueBean != null && !"1".equals(rubriqueBean.getNonIndexe());
    }

    /**
     * Détermine si le site peut avoir un sitemap.
     * <ul>
     *     <li>Le site doit être actif</li>
     *     <li>Le site doit être indexable</li>
     *     <li>Sa rubrique racine ne doit pas être sous dsi</li>
     * </ul>
     * @return true si le site peut avoir un sitemap
     */
    boolean doesSiteHaveSitemap() {
        if (infosSite == null || !infosSite.isActif() || infosSite.isNonIndexe()) {
            return false;
        }
        RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(infosSite.getCodeRubrique());
        if (rubriqueBean == null) {
            return false;
        }
        return !serviceRubrique.isUnderDsi(rubriqueBean);
    }

    private void initServices() {
        serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
        serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        serviceUrl = ServiceManager.getServiceForBean(UrlBean.class);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
    }
}
