/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.advancedFeatures;

import org.apache.commons.codec.binary.StringUtils;

import com.kosmos.uas.composant.UsineASiteComposant;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.advancedfeatures.UasAutorizer;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;

import fr.kosmos.front.utils.LocaleUtil;

/**
 * Autorité d'accès aux fonctionnalités avancées :
 * <ul>
 *     <li>Dupliquer</li>
 *     <li>Exporter</li>
 *     <li>Importer</li>
 * </ul>
 */
public class Autorizer implements UasAutorizer {

    /**
     * Id du bean dans le contexte.
     */
    public static final String ID_BEAN = "advancedFeaturesAutorizer";

    /**
     * Code de la permission pour l'action dupliquer.
     */
    public static final String ACTION_PERMISSION_DUPLIQUER = "D";
    /**
     * Code de la permission pour l'action exporter.
     */
    public static final String ACTION_PERMISSION_EXPORTER = "E";
    /**
     * Code de la permission pour l'action importer.
     */
    public static final String ACTION_PERMISSION_IMPORTER = "I";

    /**
     * Propriétés définissant si les fonctionnalités avancées sont activées ou non.
     */
    public static final String PROP_FONC_AVANCE = "uas.fonctionnaliteAvancee";

    /**
     * Valeur de la propriété permettant d'activer/désactiver les fonctionnalités avancées.
     */
    private static String advancedFeaturesPropValue = null;

    /**
     * Détermine s'il est possible d'accéder aux fonctionnalités avancées.
     * <ul>
     *     <li>Vérifie qu'elles soient bien activées</li>
     *     <li>Vérifie que l'autorisation passée en paramètre permet d'y accéder</li>
     * </ul>
     * @param autorisationBean les données d'autorisation
     * @param codePermissionAction String représentant l'action
     * @return true s'il est possible d'accéder aux fonctionnalités
     */
    public boolean canAccess(AutorisationBean autorisationBean, String codePermissionAction) {
        if (!isAdvancedFeatureEnabled()) {
            return false;
        }
        return autorisationBean.possedePermission(new PermissionBean(UsineASiteComposant.ID_BEAN.toUpperCase(LocaleUtil.getLocale()), UsineASiteComposant.CODE_PERMISSION, codePermissionAction));
    }

    @Override
    public boolean isAdvancedFeatureEnabled() {
        if (advancedFeaturesPropValue == null) {
            advancedFeaturesPropValue = UASUtil.getProperty(PROP_FONC_AVANCE);
        }
        return StringUtils.equals(advancedFeaturesPropValue, "1");
    }

    @Override
    public boolean canAccessAdvancedFeatures(final AutorisationBean autorisationBean) {
        return canAccess(autorisationBean, ACTION_PERMISSION_IMPORTER)
            || canAccess(autorisationBean, ACTION_PERMISSION_DUPLIQUER)
            || canAccess(autorisationBean, ACTION_PERMISSION_EXPORTER);
    }
}
