/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurDonneeNonTrouve;
import com.kosmos.execution.ExecutionStep;
import com.kosmos.execution.StepExecutionException;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.execution.DuplicationSiteHandler;
import com.kosmos.uas.execution.dto.SiteImportDto;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.service.PreSaveSiteCheck;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.kosmos.usinesite.template.service.ServiceTemplateSite;
import com.kosmos.usinesite.template.service.ServiceTemplateSiteFactory;
import com.kosmos.usinesite.utils.InfosSiteHelper;
import com.kportal.core.config.MessageHelper;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Étape de sauvegarde du site importé pour le cas de la duplication de site.
 */
public class DuplicateSaveSiteStep extends ExecutionStep<SiteImportDto, SiteImportDto, DuplicationSiteHandler> {

    private static final Logger LOG = LoggerFactory.getLogger(DuplicateSaveSiteStep.class);

    /**
     * Service de gestion de template de site.
     */
    private ServiceTemplateSite serviceTemplateSite;

    /**
     * Service de gestion des {@link com.univ.multisites.InfosSite}
     */
    private ServiceInfosSiteProcessus serviceInfosSite;

    /**
     * Constructeur de la classe.
     */
    public DuplicateSaveSiteStep(final DuplicationSiteHandler handler) {
        super(handler);
        serviceTemplateSite = ServiceTemplateSiteFactory.getServiceTemplateSite();
        serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
    }

    @Override
    public void execute() throws StepExecutionException {
        try {
            saveSite();
        } catch (Exception e) {
            throw new StepExecutionException(DuplicateSaveSiteStep.class, e);
        }
        output = input;
    }

    /**
     * Sauvegarde le site importé.
     * @throws Exception une exception
     */
    private void saveSite() throws Exception {
        InfosSiteImpl newInfosSite = handler.getTaskParameters().getNewInfoSite();
        final TemplateSite template = serviceTemplateSite.getTemplateSiteParCode(newInfosSite.getCodeTemplate());
        newInfosSite.setCodeRubrique(input.getSiteSection());
        try {
            PreSaveSiteCheck.getChecker().check(newInfosSite);
            serviceInfosSite.creer(newInfosSite, handler.getTaskParameters().getUserCode());
            final File oldSiteFilesDirectory = new File(InfosSiteHelper.getPathAbsoluFichiersTemplateDuSite(handler.getTaskParameters().getSourceSiteCode()));
            // Si le site dupliqué dispose d'un répertoire de contenu
            if (oldSiteFilesDirectory.exists()) {
                final File newSiteFilesDirectory = new File(InfosSiteHelper.getPathAbsoluFichiersTemplateDuSite(newInfosSite.getAlias()));
                FileUtils.copyDirectory(oldSiteFilesDirectory, newSiteFilesDirectory);
            }
        } catch (final ErreurDonneeNonTrouve e) {
            LOG.debug("Donnée non trouvée", e);
            throw new ErreursSaisieInfosSite(e.getMessage(), newInfosSite, template);
        } catch (final IOException e) {
            LOG.debug("Erreur durant la récupération du site", e);
            throw new ErreursSaisieInfosSite(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_IMPOSSIBLE_RECOPIER_SITE"), newInfosSite, template);
        }
    }

    @Override
    protected Class<? extends SiteImportDto> getInputType() {
        return SiteImportDto.class;
    }

    @Override
    protected Class<? extends SiteImportDto> getOutputType() {
        return SiteImportDto.class;
    }

}
