/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.exception.ErreurUtilisateurNonAuthentifie;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.uas.composant.UsineASiteComposant;
import com.kosmos.uas.processus.actions.ActionDto;
import com.kosmos.uas.processus.actions.SaisieInfoSiteAction;
import com.kosmos.uas.processus.actions.SaisieInfoSiteActionManager;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.kosmos.usinesite.template.service.ServiceTemplateSite;
import com.kosmos.usinesite.template.service.ServiceTemplateSiteFactory;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.om.AutorisationBean;

/**
 * Processus de saisie d'un {@link InfosSite} en back office. Ce processus permet de : creer, modifier, supprimer et lister. Ce processus est uniquement accessible au Webmaster.
 */
public class SaisieInfosSite extends ProcessusBean {

    public static final String INFOBEAN_CODE_ANCIEN_SITE = "CODE_SITE_DUPLIQUER";

    public static final String INFOBEAN_PLUGIN_SITE_DISPO = "PLUGINS_SITES";

    public static final String INFOBEAN_IMPORT_ZIP_LIST = "IMPORT_ZIP_LIST";

    public static final String INFOBEAN_IMPORT_ZIP_FILE = "FILE";

    public static final String INFOBEAN_SUPPRESSION_RUBRIQUE = "SUPPRESSION_RUBRIQUE";

    public static final String INFOBEAN_DELETE_ALL_CONTENT = "DELETE_ALL_CONTENT";

    public static final String INFOBEAN_METAS_PUBLICATION = "METAS_PUBLICATION";

    public static final String INFOBEAN_ETAT_INTERFACE_SAISIE = "ETAT_INTERFACE_SAISIE";

    public static final String INFOBEAN_LISTE_CODE_OBJET = "LISTE_CODE_OBJET";

    public static final String INFOBEAN_IS_GESTIONNAIRE_SITE = "IS_GESTIONNAIRE_SITE";

    public static final String INFOBEAN_CAN_DUPLICATE = "CAN_DUPLICATE_SITE";

    public static final String INFOBEAN_CAN_EXPORT = "CAN_EXPORT_SITE";

    public static final String INFOBEAN_VALEURS_LISTE_TEMPLATES_INFOSSITE = "VALEURS_LISTE_TEMPLATE_INFOSSITE";

    public static final String INFOBEAN_LISTE_INFOSSITES = "LISTE_INFOSSITES";

    public static final String INFOBEAN_INFOSSITE = "INFOSSITE";

    public static final String INFOBEAN_VALEURS_TEMPLATE_INFOSSITE = "VALEURS_TEMPLATE_INFOSSITE";

    public static final String INFOBEAN_TEMPLATE = "TEMPLATE";

    public static final String INFOBEAN_PRINCIPAL = "PRINCIPAL";

    public static final String INFOBEAN_INTITULE = "INTITULE";

    public static final String INFOBEAN_CODE = "CODE";

    public static final String INFOBEAN_SECURE = "SECURE";

    public static final String INFOBEAN_ACTIF = "ACTIF";

    public static final String INFOBEAN_PAGE_ACCUEIL = "PAGE_ACCUEIL";

    public static final String INFOBEAN_SSO = "SSO";

    public static final String INFOBEAN_NIVEAU_MONTANT_MAX = "NIVEAU_MONTANT_MAX";

    public static final String INFOBEAN_NIVEAU_DESCENDANT_MAX = "NIVEAU_DESCENDANT_MAX";

    public static final String INFOBEAN_NON_INDEXE = "NON_INDEXE";

    public static final String INFOBEAN_RESTREINT = "RESTREINT";

    public static final String INFOBEAN_HTTP_HOSTNAME = "HTTP_HOSTNAME";

    public static final String INFOBEAN_ALIAS = "ALIAS";

    public static final String INFOBEAN_CODE_RUBRIQUE = "CODE_RUBRIQUE";

    public static final String MESSAGE_ERREUR_CODE_NON_PRESENT_INFOBEAN = "BO_USINESITE_ERREUR_CODE_NON_PRESENT_INFOBEAN";

    private static final Logger LOG = LoggerFactory.getLogger(SaisieInfosSite.class);

    private String saisieSiteActionUtilisateur = SaisieSiteActionUtilisateur.AUCUNE.name();

    private String ecranLogiqueHistorique = null;

    private SaisieInfoSiteActionManager saisieInfoSiteActionManager;

    /**
     * Autorisation de l'utilisateur actuellement en train d'accéder au processus.
     */
    protected AutorisationBean autorisations;

    protected ServiceTemplateSite serviceTemplateSite;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieInfosSite(final InfoBean infoBean) {
        super(infoBean);
    }

    @Override
    protected boolean traiterAction() throws Exception {
        initialiserServices();
        saisieSiteActionUtilisateur = getSaisieSiteActionUtilisateurDepuisInfosBean();
        ecranLogiqueHistorique = getEcranLogiqueHistorique();
        String ecranDestination;
        try {
            verificationDroitUtilisateur();
            ecranDestination = traiterSaisieSiteActionUtilisateur();
        } catch (final ErreurApplicative e) {
            ecranDestination = traiterErreur(e);
        }
        return redirection(ecranDestination);
    }

    protected void verificationDroitUtilisateur() throws ErreurUtilisateurNonAuthentifie, ErreurActionNonAutorisee {
        if (autorisations == null || StringUtils.isEmpty(autorisations.getCode())) {
            throw new ErreurUtilisateurNonAuthentifie();
        }
        if (!UsineASiteComposant.isAutoriseParActionProcessus(autorisations, saisieSiteActionUtilisateur)) {
            throw new ErreurActionNonAutorisee();
        }
    }

    /**
     * Gestion des erreurs. Laisser remonter ou non un exception et/ou rediriger vers les écrans logique adéquat.
     *
     * @param e Exception à l'origine de l'appel.
     * @return l'ecran logique de redirection si cela est nécessaire.
     * @throws Exception erreur non traitée par le fonction que nous laissons remonter.
     */
    private String traiterErreur(final ErreurApplicative e) throws Exception {
        if (e instanceof ErreurUtilisateurNonAuthentifie) {
            infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
            return EcranLogique.LOGIN.name();
        }
        if (ecranLogiqueHistorique == EcranLogique.AUCUN.name()) {
            infoBean.addMessageErreur(e.getMessage());
            return EcranLogique.ERREUR.name();
        } else if (ecranLogiqueHistorique == EcranLogique.SAISIE.name()) {
            if (e instanceof ErreursSaisieInfosSite) {
                final ErreursSaisieInfosSite erreurs = (ErreursSaisieInfosSite) e;
                InfosSite infosSite = erreurs.getInfosSiteOrigineErreurs();
                final TemplateSite template = erreurs.getTemplateInfossiteOrigineErreurs();
                if (infosSite == null) {
                    infosSite = new InfosSiteImpl();
                }
                setDonneesPourSaisieDansInfoBean(infosSite, template);
                infoBean.addMessageErreur(StringUtils.join(erreurs.getListeMessagesErreur(), "\n"));
            } else {
                infoBean.addMessageErreur(e.getMessage());
            }
            if (SaisieSiteActionUtilisateur.VALIDER_AJOUTER.equals(saisieSiteActionUtilisateur)) {
                infoBean.set(INFOBEAN_ETAT_INTERFACE_SAISIE, EtatInterfaceSaisie.CREATION);
            } else {
                infoBean.set(INFOBEAN_ETAT_INTERFACE_SAISIE, EtatInterfaceSaisie.MODIFICATION);
            }
            return ecranLogiqueHistorique;
        } else {
            infoBean.addMessageErreur(e.getMessage());
            return ecranLogiqueHistorique;
        }
    }

    /**
     * Récupérer l'action faite par l'utilisateur.
     *
     * @return l'action si le paramétre SaisieSiteAction existe sinon {@link SaisieSiteActionUtilisateur#AUCUNE}
     */
    private String getSaisieSiteActionUtilisateurDepuisInfosBean() {
        String actionString = infoBean.getActionUtilisateur();
        if (StringUtils.isEmpty(actionString)) {
            return SaisieSiteActionUtilisateur.AUCUNE.name();
        }
        actionString = actionString.toUpperCase();
        SaisieInfoSiteAction action = saisieInfoSiteActionManager.getSaisieInfoSiteAction(actionString);
        if (action == null) {
            LOG.debug("impossible de faire la correspondance entre le String et l ennumeration SaisieSiteActionUtilisateur");
            return SaisieSiteActionUtilisateur.AUCUNE.name();
        }
        return actionString;
    }

    /**
     * Récupérer l'écran logique à l'origine de l'appel du processus.
     *
     * @return
     */
    private String getEcranLogiqueHistorique() {
        String ecranLogique = infoBean.getEcranLogique();
        if (StringUtils.isEmpty(ecranLogique)) {
            return EcranLogique.AUCUN.name();
        }
        ecranLogique = ecranLogique.toUpperCase();
        return ecranLogique;
    }

    /**
     * Traiter l'action utilisateur.
     *
     * @return
     * @throws Exception
     */
    private String traiterSaisieSiteActionUtilisateur() throws Exception {
        SaisieInfoSiteAction action = saisieInfoSiteActionManager.getSaisieInfoSiteAction(saisieSiteActionUtilisateur);
        if (action == null) {
            throw new ErreurActionNonAutorisee();
        }
        ActionDto actionDto = new ActionDto(infoBean, autorisations);
        action.doAction(actionDto);
        return action.getEcranLogique();
    }

    /**
     * Redirection vers l'écran logique spécifié.
     *
     * @param ecranDestination
     * @return {@link Boolean} qui indique la fin ou non du traitement.
     */
    private boolean redirection(final String ecranDestination) {
        if (ecranDestination == EcranLogique.FIN.name()) {
            etat = FIN;
        } else {
            ecranLogique = ecranDestination.toString();
            infoBean.setEcranLogique(ecranLogique);
        }
        return etat == FIN;
    }

    /**
     * Initialiser les variables de classe du processus.
     */
    protected void initialiserServices() {
        final SessionUtilisateur session = getGp().getSessionUtilisateur();
        if (session == null) {
            return;
        }
        autorisations = (AutorisationBean) session.getInfos().get(SessionUtilisateur.AUTORISATIONS);
        serviceTemplateSite = ServiceTemplateSiteFactory.getServiceTemplateSite();
        saisieInfoSiteActionManager = ApplicationContextManager.getBean(UASUtil.ID_EXTENSION, SaisieInfoSiteActionManager.ID_BEAN, SaisieInfoSiteActionManager.class);
    }

    /**
     * @param infosSite
     * @param template
     * @throws Exception Erreur durant la récupération des templates de sites.
     */
    protected void setDonneesPourSaisieDansInfoBean(final InfosSite infosSite, final TemplateSite template) throws Exception {
        infoBean.set(INFOBEAN_INFOSSITE, infosSite);
        infoBean.set(INFOBEAN_VALEURS_TEMPLATE_INFOSSITE, template);
        infoBean.set(INFOBEAN_VALEURS_LISTE_TEMPLATES_INFOSSITE, serviceTemplateSite.getListeTemplatesSite());
    }
}
