/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.utils.UASOmHelper;
import com.kportal.core.config.MessageHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.om.AutorisationBean;

/**
 * Action avancée de saisie de site. Actions soumise à contrôle, voir {@link Autorizer}.
 */
public abstract class AdvancedSaisieInfoSiteAction extends SaisieInfoSiteAction {

    /**
     * Autorité d'accès aux fonctionnalités avancées.
     */
    protected Autorizer autorizer;

    /**
     * Constructeur de la classe.
     */
    public AdvancedSaisieInfoSiteAction() {
        super();
        autorizer = ApplicationContextManager.getBean(UASUtil.ID_EXTENSION, Autorizer.ID_BEAN, Autorizer.class);
    }

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        if (!autorizer.canAccess(actionDto.getAutorisations(), getCodeActionPermission())) {
            throw new ErreurActionNonAutorisee();
        }
        doAdvancedAction(actionDto);
    }

    /**
     * Retourne le code de la permission pour l'action.
     * @return code de la permission
     */
    public abstract String getCodeActionPermission();

    /**
     * Effectue l'action avancée.
     * @param actionDto Dto contenant les informations nécessaire à l'exécution d'une action du processus.
     * @throws Exception une exception
     */
    public abstract void doAdvancedAction(final ActionDto actionDto) throws Exception;

    /**
     * Récupération d'un site dans le cas de la duplication et de l'export.
     * @param infoBean l'info bean
     * @param autorisationBean les autorisation de l'utilisateur
     * @return le site provenant de l'infobean.
     * @throws Exception
     */
    protected InfosSiteImpl recuperationSite(final InfoBean infoBean, final AutorisationBean autorisationBean) throws Exception {
        final String codeSiteADupliquer = infoBean.getString(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE);
        if (StringUtils.isEmpty(codeSiteADupliquer)) {
            throw new ErreurActionNonAutorisee(MessageHelper.getCoreMessage(SaisieInfosSite.MESSAGE_ERREUR_CODE_NON_PRESENT_INFOBEAN));
        }
        final InfosSiteImpl oldInfosSite = (InfosSiteImpl) serviceInfosSite.getInfosSite(codeSiteADupliquer);
        if (oldInfosSite == null || !possedePermissionGestionPourSite(oldInfosSite, autorisationBean)) {
            throw new ErreurActionNonAutorisee();
        }
        final InfosSiteImpl infosSite = new InfosSiteImpl();
        UASOmHelper.copyProperties(infosSite, oldInfosSite);
        return infosSite;
    }

    /**
     * Vérifie que les données du nouveau site soient unique.
     * @param futurInfoSite le futur site
     * @throws ErreurApplicative Erreur si le code ou le host n'est pas unique
     */
    protected void checkFuturSiteUnicity(InfosSiteImpl futurInfoSite) throws ErreurApplicative {
        if (serviceInfosSite.isSiteExisting(futurInfoSite.getAlias())) {
            throw new ErreurApplicative(String.format(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_CODE_EXISTANT"), futurInfoSite.getAlias()));
        }
        final InfosSite existingSite = serviceInfosSite.getSiteByHost(futurInfoSite.getHttpHostname());
        if (existingSite != null) {
            throw new ErreurApplicative(String.format(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_HOST_HTTP_EXISTANT"), futurInfoSite.getHttpHostname(), existingSite.getIntitule()));
        }
    }

}
