/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.util.Collection;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.taskscheduler.manager.TaskSchedulerManager;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.DeleteSiteUasTask;
import com.kosmos.uas.task.DeleteSiteUasTaskParameters;
import com.kosmos.uas.utils.SaisieInfosUASUtil;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.service.impl.DefaultServiceDeletion;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.utils.ServiceContenuModuleHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceRubrique;

/**
 * Action de suppression d'un site.
 */
class Delete extends SaisieInfoSiteAction {

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        final String code = getCodeDepuisInfoBean(actionDto.getInfoBean());
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        final ServiceLabel serviceLabel = ServiceManager.getService(ServiceLabel.class);
        if (currentLock!=null && !currentLock.isIdling()) {
            actionDto.getInfoBean().addMessageAlerte(String.format(UASUtil.getMessage("ERREUR.DELETE.TASK_IN_PROGRESS"), code));
            return;
        }
        final InfosSiteImpl infosSite = (InfosSiteImpl) serviceInfosSite.getInfosSite(code);
        final String pluginId = SaisieInfosUASUtil.getIdBeanDuplication(infosSite);
        if (!possedePermissionGestionPourSite(infosSite, actionDto.getAutorisations())) {
            throw new ErreurActionNonAutorisee();
        }
        if (infosSite.isSitePrincipal()) {
            throw new ErreursSaisieInfosSite(UASUtil.getMessage("USINESITE_SAISIE_SUPPRESSION_SITE_PRINCIPAL"));
        }
        if (StringUtils.isNotBlank(pluginId)) {
            validerSuppression(actionDto.getInfoBean());
        } else {
            validerSuppressionSansPlugin(actionDto.getInfoBean());
            serviceInfosSite.supprimer(code);
            serviceInfosSite.supprimerTousFichiers(infosSite);
        }
        serviceLabel.getBySiteCode(code).stream()
            .filter(labelDto -> labelDto.getCodeSites().size() == 1)
            .forEach(labelDto -> serviceLabel.delete(labelDto.getId()));
        serviceLabel.deleteLibelleSite(infosSite.getAlias());
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_SUPPRIMER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.FIN.name();
    }

    /**
     * Lance le traitement de suppression
     *
     * @param infoBean
     *            les données saisie
     * @throws Exception
     *             ...
     */
    private void validerSuppression(final InfoBean infoBean) throws Exception {
        final String codeSite = (String) infoBean.get(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE);
        final DeleteSiteUasTaskParameters parameters = new DeleteSiteUasTaskParameters();
        parameters.setCodeSite(codeSite);
        parameters.setDeleteContent((String) infoBean.get(SaisieInfosSite.INFOBEAN_DELETE_ALL_CONTENT));
        final DeleteSiteUasTask deleteTask = new DeleteSiteUasTask();
        deleteTask.setParameters(parameters);
        final TaskSchedulerManager taskSchedulerManager = (TaskSchedulerManager) ApplicationContextManager.getCoreContextBean("taskSchedulerManager");
        taskSchedulerManager.execute(deleteTask);
    }

    /**
     * Lance le traitement de suppression pour les sites n'étant pas issus d'une opération de duplication
     *
     * @param infoBean
     *            les données saisie
     * @throws Exception
     *             ...
     */
    private void validerSuppressionSansPlugin(InfoBean infoBean) throws Exception {
        final String codeSite = (String) infoBean.get(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE);
        final ServiceInfosSite serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        final InfosSite site = serviceInfosSite.getInfosSite(codeSite);
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubrique = serviceRubrique.getRubriqueByCode(site.getCodeRubrique());
        final String deleteContent = (String) infoBean.get(SaisieInfosSite.INFOBEAN_DELETE_ALL_CONTENT);
        if (rubrique != null && StringUtils.isNoneBlank(deleteContent, site.getCodeRubrique()) && UASUtil.isAdvancedFeatureEnabled()) {
            final DefaultServiceDeletion serviceDeletion = ApplicationContextManager.getCoreContextBean(DefaultServiceDeletion.ID_BEAN, DefaultServiceDeletion.class);
            final Collection<IModule> modulesToDelete = ServiceContenuModuleHelper.getModulesExportables();
            serviceDeletion.deleteContent(site.getCodeRubrique(), modulesToDelete);
        }
    }
}