/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.TypeMessage;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.jsbsoft.jtf.exception.ErreurDonneeNonTrouve;
import com.kosmos.controllers.SGController;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.UasTaskParameters;
import com.kosmos.usinesite.service.PreSaveSiteCheck;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kosmos.uas.utils.SaisieInfosSiteInfoBeanHelper;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.om.AutorisationBean;

/**
 * Action de modification d'un site.
 */
class Modify extends SaisieInfoSiteAction {

    private static final Logger LOG = LoggerFactory.getLogger(Modify.class);

    private String ecranLogique;

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        final String code = getCodeDepuisInfoBean(actionDto.getInfoBean());
        actionDto.getInfoBean().removeMessage(TypeMessage.ALERTE.getCode());
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        if (currentLock != null && !currentLock.isIdling() && StringUtils.equals(((UasTaskParameters) currentLock.getParameters()).getCodeSite(), code)){
            actionDto.getInfoBean().addMessageAlerte(String.format(UASUtil.getMessage("ERREUR.MODIFY.TASK_IN_PROGRESS"), code));
            final String urlModify = new StringBuilder(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY))
                .append("?PROC=").append("USINE_SITE")
                .append("&ACTION=").append(SaisieSiteActionUtilisateur.MODIFIER.name())
                .append("&CODE=").append(code)
                .toString();
            actionDto.getInfoBean().setEcranRedirection(urlModify);
            ecranLogique = EcranLogique.REDIRECT.name();
            return;
        }
        if (!verifierDroitModification(code, actionDto.getAutorisations())) {
            throw new ErreurActionNonAutorisee();
        }
        final TemplateSite template = getTemplateDepuisInfosBean(actionDto.getInfoBean());
        final InfosSiteImpl infosSite = new InfosSiteImpl();
        SaisieInfosSiteInfoBeanHelper.peuplerInfosSiteDepuisInfoBean(actionDto.getInfoBean(), infosSite, template);
        verifierDonneesInfosSite(infosSite, template);
        final InfosSite sitePrincipal = serviceInfosSite.getPrincipalSite();
        if (actionDto.getInfoBean().get(SaisieInfosSite.INFOBEAN_PRINCIPAL) != null && !code.equals(sitePrincipal.getAlias())) {
            if (!possedePermissionModificationPourSite(sitePrincipal, actionDto.getAutorisations()) && !possedePermissionGestionPourSite(sitePrincipal, actionDto.getAutorisations())) {
                throw new ErreurActionNonAutorisee();
            }
            final InfosSiteImpl ancienSitePrincipal = (InfosSiteImpl) sitePrincipal;
            ancienSitePrincipal.setSitePrincipal(Boolean.FALSE);
            PreSaveSiteCheck.getChecker().check(ancienSitePrincipal);
            serviceInfosSite.modifier(ancienSitePrincipal);
        } else {
            infosSite.setSitePrincipal(code.equals(sitePrincipal.getAlias()));
        }
        final String codeUtilisateur = actionDto.getAutorisations().getCode();
        try {
            PreSaveSiteCheck.getChecker().check(infosSite);
            serviceInfosSite.modifier(infosSite, codeUtilisateur);
            lancementTraitementsComplementairesTemplate(infosSite, template, actionDto.getInfoBean());
            serviceInfosSite.cleanFichiers(infosSite);
        } catch (final ErreurDonneeNonTrouve e) {
            LOG.debug("unable to find any data for this website", e);
            throw new ErreursSaisieInfosSite(e.getMessage(), infosSite, template);
        }
        actionDto.getInfoBean().addMessageConfirmation(String.format(MessageHelper.getCoreMessage("CONFIRMATION_MODIFICATION_SITE"), infosSite.getIntitule()));
        ecranLogique = EcranLogique.FIN.name();
    }

    /**
     * Vérification des droits pour modification.
     * @param code le code du site
     * @param autorisationBean les autorisations
     * @return true si l'utilisateur peut modifier
     */
    private boolean verifierDroitModification(final String code, final AutorisationBean autorisationBean) {
        final InfosSite infosSite = serviceInfosSite.getInfosSite(code);
        return possedePermissionModificationPourSite(infosSite, autorisationBean) || possedePermissionGestionPourSite(infosSite, autorisationBean);
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_MODIFICATION.name();
    }

    @Override
    public String getEcranLogique() {
        return ecranLogique;
    }
}
