/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.util.ArrayList;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.jsbsoft.jtf.exception.ErreurDonneeNonTrouve;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.taskscheduler.manager.LockManager;
import com.kosmos.uas.composant.UsineASiteComposant;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.utils.UASDisplayableModule;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.kosmos.usinesite.template.property.bean.TemplateSiteProperty;
import com.kosmos.usinesite.template.property.service.ServiceTemplateSiteProperty;
import com.kosmos.usinesite.template.property.service.ServiceTemplateSitePropertyFactory;
import com.kosmos.usinesite.template.property.traitement.TemplateSitePropertyTraitement;
import com.kosmos.usinesite.template.service.ServiceTemplateSite;
import com.kosmos.usinesite.template.service.ServiceTemplateSiteFactory;
import com.kosmos.usinesite.utils.InfosSiteHelper;
import com.kportal.core.config.MessageHelper;
import com.kportal.extension.module.service.ServiceContenuModuleContext;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.Perimetre;
import com.univ.objetspartages.om.PermissionBean;

/**
 * Interface définissant le fonctionnement d'une action.
 */
public abstract class SaisieInfoSiteAction {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieInfoSiteAction.class);

    /**
     * Service de template de site.
     */
    protected ServiceTemplateSite serviceTemplateSite;

    /**
     * Service de récupération de sites.
     */
    protected ServiceInfosSite serviceInfosSite;

    protected LockManager lockManager;

    /**
     * Constructeur de la classe.
     */
    SaisieInfoSiteAction() {
        serviceTemplateSite = ServiceTemplateSiteFactory.getServiceTemplateSite();
        serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        lockManager = ApplicationContextManager.getCoreContextBean(LockManager.ID_BEAN, LockManager.class);
    }

    /**
     * Exécution de l'action.
     * @param actionDto Dto contenant les informations nécessaire à l'exécution d'une action du processus.
     * @throws Exception une exception
     */
    public abstract void doAction(final ActionDto actionDto) throws Exception;

    /**
     * Récupération de la valeur définissant l'action.
     * @return un {@link String}
     */
    public abstract String getAction();

    /**
     * Récupération de l'écran logique vers lequel on renvoi à la fin de l'actoin.
     * @return un écran logique
     */
    public abstract String getEcranLogique();

    /**
     * @param infosSite
     * @param template
     * @throws Exception Erreur durant la récupération des templates de sites.
     */
    protected void setDonneesPourSaisieDansInfoBean(final InfosSite infosSite, final TemplateSite template, final InfoBean infoBean) throws Exception {
        infoBean.set(SaisieInfosSite.INFOBEAN_INFOSSITE, infosSite);
        infoBean.set(SaisieInfosSite.INFOBEAN_VALEURS_TEMPLATE_INFOSSITE, template);
        infoBean.set(SaisieInfosSite.INFOBEAN_VALEURS_LISTE_TEMPLATES_INFOSSITE, serviceTemplateSite.getListeTemplatesSite());
    }

    /**
     * récupérer le code du {@link InfosSite}. Si ce code n'est pas présent dans requête HTTP alors une exception est levée.
     *
     * @return Le code du {@link InfosSite} renseigné dans le {@link InfoBean} du processus.
     * @throws ErreurActionNonAutorisee Exception levée quand le code est introuvable dans le {@link InfoBean}
     */
    protected String getCodeDepuisInfoBean(final InfoBean infoBean) throws ErreurActionNonAutorisee {
        final String code = infoBean.getString(SaisieInfosSite.INFOBEAN_CODE);
        if (StringUtils.isEmpty(code)) {
            throw new ErreurActionNonAutorisee(MessageHelper.getCoreMessage(SaisieInfosSite.MESSAGE_ERREUR_CODE_NON_PRESENT_INFOBEAN));
        }
        return code;
    }

    /**
     * Vérifie si les autorisations courantes possèdent bien la permission de CRUD sur le périmètre donnée
     *
     * @param siteAVerifier
     * @return
     */
    protected boolean possedePermissionGestionPourSite(final InfosSite siteAVerifier, final AutorisationBean autorisationBean) {
        final PermissionBean permissionGestion = UsineASiteComposant.getPermissionGestion();
        final Perimetre perimetreFiche = new Perimetre(StringUtils.EMPTY, siteAVerifier.getCodeRubrique(), StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);
        return autorisationBean.isWebMaster() || autorisationBean.possedePermission(permissionGestion, perimetreFiche);
    }

    /**
     * Vérifie si les autorisations courantes possèdent bien la permission de modification sur le périmètre donnée
     *
     * @param siteAVerifier
     * @return
     */
    protected boolean possedePermissionModificationPourSite(final InfosSite siteAVerifier, final AutorisationBean autorisationBean) {
        final PermissionBean permissionModification = UsineASiteComposant.getPermissionModification();
        final Perimetre perimetreFiche = new Perimetre(StringUtils.EMPTY, siteAVerifier.getCodeRubrique(), StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);
        return autorisationBean.isWebMaster() || autorisationBean.possedePermission(permissionModification, perimetreFiche);
    }

    /**
     * Récupérer le template de site depuis l'infoBean.
     *
     * @return Le {@link TemplateSite} référencé dans le {@link InfoBean}
     * @throws Exception Erreur durant l'accés à la source de données des {@link TemplateSite}.
     */
    protected TemplateSite getTemplateDepuisInfosBean(final InfoBean infoBean) throws Exception {
        final String codeTemplate = infoBean.getString(SaisieInfosSite.INFOBEAN_TEMPLATE);
        if (StringUtils.isEmpty(codeTemplate)) {
            return null;
        }
        try {
            return serviceTemplateSite.getTemplateSiteParCode(codeTemplate);
        } catch (final ErreurDonneeNonTrouve e) {
            LOG.debug("le template n'existe pas", e);
            return null;
        }
    }

    /**
     * Vérification de la cohérence des données.
     *
     * @param infosSite
     * @param template
     * @throws ErreursSaisieInfosSite : Erreur dans la cohérance des données.
     */
    protected void verifierDonneesInfosSite(final InfosSiteImpl infosSite, final TemplateSite template) throws ErreursSaisieInfosSite {
        final ArrayList<String> fluxMessagesErreurs = new ArrayList<>();
        if (infosSite.isSitePrincipal() && !infosSite.isActif()) {
            fluxMessagesErreurs.add(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_SITE_PRINCIPAL_INACTIF"));
        }
        if (!InfosSiteHelper.isHostValid(infosSite.getHttpHostname())) {
            fluxMessagesErreurs.add(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_HOST_INVALIDE"));
        }
        if (!fluxMessagesErreurs.isEmpty()) {
            throw new ErreursSaisieInfosSite(fluxMessagesErreurs, infosSite, template);
        }
    }

    protected void lancementTraitementsComplementairesTemplate(final InfosSiteImpl infosSite, final TemplateSite template, final InfoBean infoBean) throws Exception {
        if (template == null) {
            return;
        }
        ServiceTemplateSiteProperty serviceTemplateSiteProperty = ServiceTemplateSitePropertyFactory.getServiceTemplateSite();
        for (final TemplateSiteProperty property : template.getListeProprietesComplementaires()) {
            try {
                final TemplateSitePropertyTraitement<TemplateSiteProperty> traitement = serviceTemplateSiteProperty.getTemplateSitePropertyTraitement(property);
                traitement.traiter(infosSite, template, property, infoBean);
            } catch (final ErreurDonneeNonTrouve e) {
                LOG.debug("la propriété n'a pas été trouvé", e);
            }
        }
    }

    /**
     * Prépare la liste des objets duplicables pour l'affichage
     */
    protected void setListeObjetsDansInfoBean(final InfoBean infoBean) {
        infoBean.set(SaisieInfosSite.INFOBEAN_LISTE_CODE_OBJET, UASDisplayableModule.getExportablesModules(ServiceContenuModuleContext.DEFAULT_EXECUTION_CONTEXT));
    }
}