/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.utils;

import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.unix4j.Unix4j;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.exception.ErreurDonneeNonTrouve;
import com.jsbsoft.jtf.exception.ErreurImportException;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.plugin.IPluginSite;
import com.kosmos.usinesite.reference.ExtensionFileFilter;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.utils.TransformationUtils;
import com.kosmos.usinesite.utils.UASServicesHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.util.compress.Zip;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;

public class PluginSiteHelper {

    private final static Logger LOG = LoggerFactory.getLogger(PluginSiteHelper.class);

    /**
     * Méthode de duplication des propriétés d'un site vers un répertoire
     *
     * @param oldSite
     *            : l'@{link com.univ.multisites.InfosSite} du site pour lequel on souhaite dupliqué ses propriétés
     * @param savePath
     *            : le chemin vers le répertoire dans lequel la duplication doit sauvegarder le fichier résultant
     * @throws IOException
     *             si le fichier ou le répertoire de sauvegarde ne sont pas accessibles
     */
    public static void duplicateProperties(final InfosSite oldSite, final String savePath) throws IOException {
        final File oldSiteProperties = new File(String.format("%s%s%s.properties", WebAppUtil.getConfigurationSitesPath(), File.separator, oldSite.getAlias()));
        if (!oldSiteProperties.exists()) {
            throw new IOException(String.format("Le fichier de properties \"%s\" n'existe pas au sein de l'application.", oldSiteProperties.getAbsolutePath()));
        }
        final File saveDirectory = new File(savePath);
        if (!saveDirectory.exists()) {
            throw new IOException(String.format("Le fichier \"%s\" n'existe pas.", saveDirectory.getAbsolutePath()));
        }
        FileUtils.copyFileToDirectory(oldSiteProperties, saveDirectory);
    }

    /**
     * Méthode de duplication des données associées à un template de site (répertoire du site contenant les fichiers du template).
     *
     * @param oldSite
     *            : l'@{link com.univ.multisites.InfosSite} du site pour lequel on souhaite dupliqué ses données
     * @param savePath
     *            : le chemin vers le répertoire dans lequel la duplication doit sauvegarder le dossier
     * @throws IOException
     *             si le fichier ou le répertoire de sauvegarde ne sont pas accessibles
     */
    public static void duplicateTemplateDatas(final InfosSite oldSite, final String savePath) throws IOException {
        final File oldSiteDatas = new File(String.format("%s%s%s%s", WebAppUtil.getConfigurationSitesPath(), WebAppUtil.DEFAULT_FICHIERS_UAS, oldSite.getAlias(), File.separator));
        if (!oldSiteDatas.exists()) {
            throw new IOException(String.format("Le répertorie de données \"%s\" n'existe pas au sein de l'application.", oldSiteDatas.getAbsolutePath()));
        }
        final File saveDirectory = new File(savePath);
        if (!saveDirectory.exists()) {
            throw new IOException(String.format("Le répertoire \"%s\" n'existe pas.", saveDirectory.getAbsolutePath()));
        }
        FileUtils.copyDirectory(oldSiteDatas, saveDirectory, new UASDirectoriesFilter(oldSite));
    }

    /**
     * Méthode permettant de compresser un répertoire. Le zip résultant est nommé selon le motfi suivant :
     *
     * &lt;WebAppUtil.getSauvegardePath()&gt;/&lt;WebAppUtil.getRelatifFichiersSitesPath()&gt;/&lt;codeSite&gt; - &lt;date&gt;.zip
     *
     * La date du nom de fichier est formattée selon le motif suivant : "yyyy-MM-dd-HH-mm-ss"
     *
     * @param oldSite
     *            : l'@{link com.univ.multisites.InfosSite} du site pour lequel on souhaite créer l'archive
     * @param savePath
     *            : le chemin dans lequel le fichier zip doit être sauvegardé
     * @throws IOException
     *             si le répertoire de sauvegarde n'est pas accessible ou si l'archive ne peut pas être créée
     * @return zipFile L'archive créée
     */
    public static File compressExportFolder(final InfosSite oldSite, final String savePath) throws IOException {
        final File saveDirectory = new File(savePath);
        if (!saveDirectory.exists()) {
            throw new IOException(String.format("Le fichier \"%s\" n'existe pas.", saveDirectory.getAbsolutePath()));
        }
        final Date currentDate = new Date(System.currentTimeMillis());
        final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss", Locale.FRANCE);
        final File zipFile = new File(String.format("%s%s%s%s%s-%s.zip", WebAppUtil.getSauvegardePath(), File.separator, WebAppUtil.DEFAULT_FICHIERS_UAS, File.separator, oldSite.getAlias(), df.format(currentDate)));
        FileUtils.touch(zipFile);
        Zip.compress(saveDirectory, zipFile, false);
        return zipFile;
    }

    /**
     * Méthode permettant d'importer le fichier properties d'un site à partir d'un répertoire.
     *
     * @param importPath
     *            : le chemin absolu du fichier ".properties" à importer
     * @return un {@link com.univ.multisites.bean.impl.InfosSiteImpl} correspondant au site importé
     * @throws Exception
     *             <ul>
     *             <li>Le fichier passé en paramètre ne possède pas d'extension ".properties"</li>
     *             <li>Le fichier passé en paramètre n'existe pas</li>
     *             <li>Le déplacement du fichier vers le répertoire des configurations de site n'a pas pu être effectué</li>
     *             <li>Le remplacement du code de site par le nouveau code n'a pas pu être effectué</li>
     *             <li>Le service InfosSite n'a pas pu récupérer le site importé</li>
     *             </ul>
     */
    public static InfosSiteImpl importProperties(final String importPath) throws Exception {
        final ServiceInfosSite serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        if (!"properties".equals(FilenameUtils.getExtension(importPath))) {
            throw new ErreurImportException(String.format("Le fichier \"%s\" n'est pas un fichier de properties valide.", importPath));
        }
        final File importFile = new File(importPath);
        if (!importFile.exists()) {
            throw new IOException(String.format("Le fichier \"%s\" n'existe pas.", importFile.getAbsolutePath()));
        }
        String codeSite = importFile.getName().replace("." + FilenameUtils.getExtension(importPath), StringUtils.EMPTY);
        final File siteConfDirectory = new File(WebAppUtil.getConfigurationSitesPath());
        // Si le site existe déjà
        if (serviceInfosSite.isSiteExisting(codeSite)) {
            final String newCodeSite = String.format("%s_%s", codeSite, Long.toString(System.nanoTime()));
            final File tempFile = new File(String.format("%s%s%s.properties", importFile.getParent(), File.separator, newCodeSite));
            FileUtils.touch(tempFile);
            try (FileOutputStream output = new FileOutputStream(tempFile)) {
                Unix4j.fromFile(importFile)
                    .sed(String.format("s/\\.%s\\./\\.%s\\./g", codeSite, newCodeSite))
                    .toOutputStream(output);

            }
            codeSite = newCodeSite;
            importFile.delete();
            FileUtils.moveFileToDirectory(tempFile, siteConfDirectory, false);
        } else {
            FileUtils.moveFileToDirectory(importFile, siteConfDirectory, false);
        }
        return (InfosSiteImpl) serviceInfosSite.getInfosSite(codeSite);
    }

    /**
     * Méthode permettant d'importer le fichier properties d'un site à partir d'un répertoire.
     *
     * @param importPath
     *            : le chemin absolu du répertoire contenant les données à importer
     * @throws Exception
     *             <ul>
     *             <li>Le répertoire passé en paramètre n'existe pas</li>
     *             <li>Le déplacement des fichiers vers le répertoire des configurations de site n'a pas pu être effectué</li>
     *             <li>Le service InfosSite n'a pas pu récupérer le site importé</li>
     *             </ul>
     */
    public static void importTemplateDatas(final InfosSite site, final String importPath) throws Exception {
        final File importDirectory = new File(importPath);
        final File siteDatas = new File(String.format("%s%s%s%s", WebAppUtil.getConfigurationSitesPath(), WebAppUtil.DEFAULT_FICHIERS_UAS, site.getAlias(), File.separator));
        if (!importDirectory.exists()) {
            throw new IOException(String.format("Le répertoire \"%s\" n'existe pas.", importDirectory.getAbsolutePath()));
        }
        final FileFilter filter = new UASDirectoriesFilter(site);
        for (final File currentDirectory : importDirectory.listFiles(filter)) {
            FileUtils.copyDirectoryToDirectory(currentDirectory, siteDatas);
        }
    }

    /**
     * Méthode per mettant de récupérer le path des properties d'un site à partir d'un répertoire.
     *
     * @param directory
     *            le chemin absolu vers le répertoire supposé contenir le fichier de properties.
     * @return Le chemin absolu vers le fichier de properties découvert.
     * @throws IOException
     *             <ul>
     *             <li>Si le répertoire fournit ne contient aucun fichier ".properties"</li>
     *             <li>Si le répertoire fournit contient plusieurs fichier ".properties"</li>
     *             </ul>
     */
    public static String getPropertiesPath(final File directory) throws IOException {
        final File[] children = directory.listFiles(new ExtensionFileFilter(ExtensionFileFilter.PROPERTIES));
        if (children.length == 0) {
            throw new IOException(String.format("Le répertoire \"%s\" ne contient pas de fichier properties.", directory.getAbsolutePath()));
        }
        // FIXME : Pas d'identification du fichier properties pour le moment. Si plusieurs fichiers properties sont remontés, abandon.
        if (children.length > 1) {
            throw new IOException(String.format("Le répertoire \"%s\" contient plusieurs fichier properties : %s.", directory.getAbsolutePath(), StringUtils.join(children)));
        }
        return children[0].getAbsolutePath();
    }

    /**
     * Méthode permettant de déplacer les fichiers d'un dossier dans le répertoire de duplication d'un site.
     *
     * @param site
     *            :
     * @link{com.univ.multisites.bean.impl.InfosSiteImpl identifiant le répertoire dans lequel le contenu doit être déplacé
     * @param saveDirectory
     *            :
     * @{link java.io.File} en chemin absolu représentant le répertoire à partir duquel les fichiers doivent être déplacés.
     * @return le répertoire de destination
     * @throws IOException
     *             <ul>
     *             <li>Si le répertoire source n'existe pas</li>
     *             <li>Si le répertoire de destination n'existe pas</li>
     *             </ul>
     */
    public static File moveFileToDuplicationDirectory(final InfosSiteImpl site, final File saveDirectory) throws IOException {
        final File duplicationDirectory = new File(TransformationUtils.getPathDossierDuplication(site));
        final File duplicationMediaDirectory = Paths.get(duplicationDirectory.getPath(), UASServicesHelper.SUB_FOLDER_NAME).toFile();
        final Iterator<File> itFiles = FileUtils.iterateFiles(saveDirectory, null, true);
        while (itFiles.hasNext()) {
            final File currentFile = itFiles.next();
            File dest = duplicationDirectory;
            if (StringUtils.equals(currentFile.getParentFile().getName(), UASServicesHelper.SUB_FOLDER_NAME)) {
                dest = duplicationMediaDirectory;
            }
            FileUtils.moveFileToDirectory(currentFile, dest, true);
        }
        FileUtils.forceDelete(saveDirectory);
        return duplicationDirectory;
    }

    /**
     * Méthode permettant de supprimer le contenu d'un répertoire ou le répertoire en lui-même.
     *
     * @param exportPath
     *            : le répertoire à nettoyer ou à supprimer
     * @param deleteDirectory
     *            : true pour supprimer le répertoire, false pour n'en supprimer que le contenu
     * @throws IOException
     *             si le répertoire à supprimer n'existe pas ou n'est pas accessible en écriture
     */
    public static void cleanUpDirectory(final String exportPath, final boolean deleteDirectory) throws IOException {
        final File directory = new File(exportPath);
        if (directory.exists()) {
            if (deleteDirectory) {
                FileUtils.deleteDirectory(directory);
            } else {
                FileUtils.cleanDirectory(directory);
            }
        }
    }

    /**
     * On récupère l'ensemble de type de plugin de site. (Duplication des contenus sans références, avec...)
     *
     * @return la liste des plugins de sites.
     */
    public static Collection<IPluginSite> getAllPluginsSite() {
        final Map<String, IPluginSite> plugins = ApplicationContextManager.getAllBeansOfType(IPluginSite.class);
        Collection<IPluginSite> pluginsSites = null;
        if (plugins == null) {
            pluginsSites = Collections.<IPluginSite>emptyList();
        } else {
            pluginsSites = plugins.values();
        }
        return pluginsSites;
    }

    /**
     * On récupère un plugin de site depuis son id
     *
     * @param pluginId
     *            l'id du plugin à récupérer
     * @return le plugin du site
     * @throws ErreurDonneeNonTrouve
     *             lorsque le plugin n'est pas trouvé...
     */
    public static IPluginSite getPluginById(final String pluginId) throws ErreurDonneeNonTrouve {
        final IPluginSite plugin = (IPluginSite) ApplicationContextManager.getEveryContextBean(pluginId);
        if (plugin == null) {
            throw new ErreurDonneeNonTrouve(String.format("Le Plugin de site ayant pour id \"%s\" n'a pas été trouvé.", pluginId));
        }
        return plugin;
    }
}