/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.usinesite.utils.UASServicesHelper;
import com.kportal.cms.objetspartages.Objetpartage;
import com.kportal.cms.objetspartages.annotation.FicheAnnotationHelper;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.ModuleHelper;
import com.kportal.extension.module.plugin.objetspartages.ExportPluginFiche;
import com.kportal.extension.module.service.ServiceContenuModule;
import com.kportal.extension.module.service.ServiceContenuModuleContext;
import com.kportal.extension.module.utils.ServiceContenuModuleHelper;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.utils.FicheUnivHelper;

public class UASDisplayableModule {

    private static final Logger LOG = LoggerFactory.getLogger(UASDisplayableModule.class);

    /**
     * Prépare la liste des objets duplicables
     * @param context le contexte d'exécution, voir {@link ServiceContenuModuleContext
     */
    public static Map<ServiceContenuModule.TypeContenu, List<IModule>> getExportablesModules(String context) {
        final Map<ServiceContenuModule.TypeContenu, List<IModule>> modules = new HashMap<>();
        preparerModuleServiceContenu(modules, context);
        preparerModuleExportablePluginFiche(modules, context);
        for (final List<IModule> currentList : modules.values()) {
            Collections.sort(currentList, new ModuleComparator());
        }
        return modules;
    }

    /**
     * Ajoute dans la liste des modules actifs les modules duplicables.
     * Un module est duplicable si la methode isModuleDuplicable retourne VRAI.
     * Si en plus le module est un objet partagé, on vérifie que le module peut être
     * instancié et que son contenu est duplicable.
     * @param activeModules la liste des modules duplicables actifs à completer
     * @param context le contexte d'exécution, voir {@link ServiceContenuModuleContext
     */
    private static void preparerModuleServiceContenu(final Map<ServiceContenuModule.TypeContenu, List<IModule>> activeModules, String context) {
        final Map<String, ServiceContenuModule> servicesContenusByModule = ServiceContenuModuleHelper.getServiceContenuModules(context);
        for (final Map.Entry<String, ServiceContenuModule> entry : servicesContenusByModule.entrySet()) {
            final ServiceContenuModule currentService = entry.getValue();
            if (currentService.isTechnical() || currentService.isPlugin()) {
                continue;
            }
            final String moduleId = entry.getKey();
            final IModule currentModule = ModuleHelper.getModule(moduleId);
            if (isModuleDuplicable(currentModule, context)) {
                if (currentModule instanceof Objetpartage) {
                    final FicheUniv ficheATester = FicheUnivHelper.instancierFiche(((Objetpartage) currentModule).getCodeObjet());
                    if (ficheATester == null || !FicheAnnotationHelper.isContenuDuplicable(ficheATester)) {
                        LOG.info("Impossible d'instancier la fiche {} ou le contenu n'est pas dupliquable : le module est ignoré", FicheUniv.class.getSimpleName());
                        continue;
                    }
                }
                activeModules.putIfAbsent(currentService.getTypeContenu(), new ArrayList<>());
                activeModules.get(currentService.getTypeContenu()).add(currentModule);
            }
        }
    }

    /**
     * Ajoute dans la liste des modules actifs les modules dupliquable de type plugin.
     * @param activeModules la liste des modules actifs à completer
     * @param context le contexte d'exécution, voir {@link ServiceContenuModuleContext
     */
    private static void preparerModuleExportablePluginFiche(final Map<ServiceContenuModule.TypeContenu, List<IModule>> activeModules, String context) {
        final Map<String, ServiceContenuModule> servicesContenusByModule = ServiceContenuModuleHelper.getServiceContenuModules(context);
        for (final Map.Entry<String, ServiceContenuModule> entry : servicesContenusByModule.entrySet()) {
            final ServiceContenuModule currentService = entry.getValue();
            if (!currentService.isPlugin()) {
                continue;
            }
            final String moduleId = entry.getKey();
            final IModule currentModule = ModuleHelper.getModule(moduleId);
            if (isExportable(currentModule, context)) {
                activeModules.putIfAbsent(ServiceContenuModule.TypeContenu.PLUGIN, new ArrayList<>());
                activeModules.get(ServiceContenuModule.TypeContenu.PLUGIN).add(currentModule);
            }
        }
    }

    /**
     * @param currentModule
     * @param context le contexte d'exécution, voir {@link ServiceContenuModuleContext}
     * @return true si le module est effectivement duplicable, false sinon
     */
    private static boolean isModuleDuplicable(final IModule currentModule, String context) {
        boolean moduleCollab = false;
        if (currentModule instanceof Objetpartage) {
            moduleCollab = ((Objetpartage) currentModule).isStrictlyCollaboratif();
        }
        return !moduleCollab && isExportable(currentModule, context);
    }

    /**
     * @param currentModule
     * @param context le contexte d'exécution, voir {@link ServiceContenuModuleContext}
     * @return true si le module est effectivement exportable, false sinon
     */
    private static boolean isExportable(final IModule currentModule, String context) {
        return currentModule.getEtat() == IModule.ETAT_ACTIF && !(currentModule instanceof ExportPluginFiche) && UASServicesHelper.hasAttachedService(currentModule, context);
    }
}
