/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.collections.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.service.ServiceFormation;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

/**
 * Classe représentant un objet formation.
 */
@FicheAnnotation(isSaisieFrontOffice = false)
public class Formation extends AbstractFicheRestriction<FormationBean> implements FicheRattachementsSecondaires {

    private static final Logger LOGGER = LoggerFactory.getLogger(Formation.class);

    /**
     * Constructeur vide.
     */
    public Formation() {
        super();
    }

    /**
     * Constructeur à partir d'un bean.
     * @param formationBean le bean
     */
    public Formation(FormationBean formationBean) {
        this();
        persistenceBean = formationBean;
    }

    /**
     * Renvoie le libellé à afficher (methode statique utilisée pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (StringUtils.isEmpty(codes)) {
            return res;
        }
        Formation formation = new Formation();
        formation.init();
        if (StringUtils.isEmpty(langue)) {
            langue = "0";
        }
        StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            String code = st.nextToken();
            final String[] values = code.split(",");
            // On cherche d'abord la version en ligne puis les autres versions
            int count = formation.selectCodeLangueEtat(values[0], langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) {
                count = formation.selectCodeLangueEtat(values[0], langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (formation.nextItem()) {
                    res += formation.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandée (méthode statique utilisée pour les jointures entre fiches).
     */
    public static Formation getFicheFormation(String code, String langue) throws Exception {
        Formation fiche = null;
        if (StringUtils.isNotEmpty(code)) {
            if (StringUtils.isEmpty(langue)) {
                langue = "0"; // francais par defaut
            }
            fiche = new Formation();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) { // si pas de version en ligne
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) { // si pas de version dans la langue demandée
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.toString());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    private ServiceFormation getServiceFormation() {
        return ServiceManager.getServiceForBean(FormationBean.class);
    }

    @Override
    public void add() {
        getServiceFormation().add(this.getPersistenceBean());
    }

    @Override
    public void addWithForcedId() throws Exception {
        getServiceFormation().addWithForcedId(getPersistenceBean());
    }

    @Override
    public void update() {
        getServiceFormation().update(getPersistenceBean());
    }

    @Override
    public void delete() throws Exception {
        getServiceFormation().delete(getIdFormation());
    }

    /**
     * Initialise l'objet metier.
     */
    public void init() {
        super.init();
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp.substring(timeStamp.length() - 8, timeStamp.length()));
        setCodeRattachementAutres(StringUtils.EMPTY);
        setCodeSiScolarite(StringUtils.EMPTY);
        setIntituleCourt(StringUtils.EMPTY);
        setIntituleLong(StringUtils.EMPTY);
        setCatalogue(StringUtils.EMPTY);
        setCodeFormationMere(StringUtils.EMPTY);
        setNatureFormation(StringUtils.EMPTY);
        setFormationDiplomante(StringUtils.EMPTY);
        setFormationLmd(StringUtils.EMPTY);
        setTypeDiplome(StringUtils.EMPTY);
        setGrade(StringUtils.EMPTY);
        setDomaineLmd(StringUtils.EMPTY);
        setDiscipline(StringUtils.EMPTY);
        setMention(StringUtils.EMPTY);
        setParcours(StringUtils.EMPTY);
        setNiveauRecrutement(StringUtils.EMPTY);
        setAnneePostbac(StringUtils.EMPTY);
        setNiveauSortie(StringUtils.EMPTY);
        setFormationInitiale(StringUtils.EMPTY);
        setRepriseEtudes(StringUtils.EMPTY);
        setTempsPlein(StringUtils.EMPTY);
        setCompatibleActivitePro(StringUtils.EMPTY);
        setTempsAmenage(StringUtils.EMPTY);
        setAlternance(StringUtils.EMPTY);
        setHorsContratAlternance(StringUtils.EMPTY);
        setContratApprentissage(StringUtils.EMPTY);
        setContratPro(StringUtils.EMPTY);
        setPresentiel(StringUtils.EMPTY);
        setPartiellementDistance(StringUtils.EMPTY);
        setTotalementDistance(StringUtils.EMPTY);
        setLieux(StringUtils.EMPTY);
        setLanguesEnseignement(StringUtils.EMPTY);
        setSelectif(StringUtils.EMPTY);
        setCapaciteAccueilLimitee(StringUtils.EMPTY);
        setDureeListe(StringUtils.EMPTY);
        setAutreDuree(StringUtils.EMPTY);
        setStages(StringUtils.EMPTY);
        setStagesOptionnel(StringUtils.EMPTY);
        setStagesOptionnelMultiple(StringUtils.EMPTY);
        setStagesOptionnelEtranger(StringUtils.EMPTY);
        setStagesObligatoire(StringUtils.EMPTY);
        setStagesObligatoireMultiple(StringUtils.EMPTY);
        setStagesObligatoireEtranger(StringUtils.EMPTY);
        setSejoursEtranger(StringUtils.EMPTY);
        setSejoursEtrangerOptionnel(StringUtils.EMPTY);
        setSejoursEtrangerOptionnelMultiples(StringUtils.EMPTY);
        setSejoursEtrangerObligatoire(StringUtils.EMPTY);
        setSejoursEtrangerObligatoireMultiples(StringUtils.EMPTY);
        setProjetsTutores(StringUtils.EMPTY);
        setProjetsTutoresOptionnel(StringUtils.EMPTY);
        setProjetsTutoresOptionnelMultiples(StringUtils.EMPTY);
        setProjetsTutoresObligatoire(StringUtils.EMPTY);
        setProjetsTutoresObligatoireMultiples(StringUtils.EMPTY);
        setSecteursActivites(StringUtils.EMPTY);
        setMetiers(StringUtils.EMPTY);
        setPrincipalesMatieres(StringUtils.EMPTY);
        setCentresInteret(StringUtils.EMPTY);
        setSerieBac(StringUtils.EMPTY);
        setDomaineErasmus(StringUtils.EMPTY);
        setAccroche(StringUtils.EMPTY);
        setResume(StringUtils.EMPTY);
        setDescription(StringUtils.EMPTY);
        setSpecificites(StringUtils.EMPTY);
        setLieuxComplement(StringUtils.EMPTY);
        setResponsablesFiche(StringUtils.EMPTY);
        setResponsablesComplement(StringUtils.EMPTY);
        setTemoignages(StringUtils.EMPTY);
        setLaboratoiresPartenaires(StringUtils.EMPTY);
        setEtablissementsPartenaires(StringUtils.EMPTY);
        setEntreprisesPartenaires(StringUtils.EMPTY);
        setConventionEtablissements(StringUtils.EMPTY);
        setFormationsRequises(StringUtils.EMPTY);
        setFormationsLiees(StringUtils.EMPTY);
        setPublicCible(StringUtils.EMPTY);
        setModalitesCandidatures(StringUtils.EMPTY);
        setModalitesCandidaturesSpecifiques(StringUtils.EMPTY);
        setConditionsAdmission(StringUtils.EMPTY);
        setProgramme(StringUtils.EMPTY);
        setParcoursFiche(StringUtils.EMPTY);
        setContenuFormation(StringUtils.EMPTY);
        setRythmePremiereAnnee(StringUtils.EMPTY);
        setEquipePedagogique(StringUtils.EMPTY);
        setTempsPleinDescription(StringUtils.EMPTY);
        setCompatibleProfessionnelleDescription(StringUtils.EMPTY);
        setTempsAmenageDescription(StringUtils.EMPTY);
        setAlternanceDescription(StringUtils.EMPTY);
        setPresentielDescription(StringUtils.EMPTY);
        setPartiellementDistanceDescription(StringUtils.EMPTY);
        setTotalementDistanceDescription(StringUtils.EMPTY);
        setStagesDescription(StringUtils.EMPTY);
        setSejoursEtrangerDescription(StringUtils.EMPTY);
        setProjetsTutoresComplement(StringUtils.EMPTY);
        setModalitesEvaluation(StringUtils.EMPTY);
        setFicheRncp(StringUtils.EMPTY);
        setCompetencesAttestees(StringUtils.EMPTY);
        setPoursuiteEtudes(StringUtils.EMPTY);
        setParcoursPoursuite(StringUtils.EMPTY);
        setFormationPoursuite(StringUtils.EMPTY);
        setDebouches(StringUtils.EMPTY);
        setInsertionProfessionnelle(StringUtils.EMPTY);
        setTarifComplement(StringUtils.EMPTY);
        setModalitesInscription(StringUtils.EMPTY);
        setPlaquettesCrm(StringUtils.EMPTY);
        setContactEmail(StringUtils.EMPTY);
        setContactTelephone(StringUtils.EMPTY);
        setContactCrm(StringUtils.EMPTY);
        setUrlExterneFormation(StringUtils.EMPTY);
        setRenseignements(StringUtils.EMPTY);
        setUrlInscription(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libellé pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la fiche rubrique correspondant à l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFicheCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final ArrayList<StructureModele> listCodeRattachementsAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementsAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementsAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementsAutres.nextToken(), getLangue());
            if (currentStructure != null) {
                listCodeRattachementsAutres.add(currentStructure);
            }
        }
        return listCodeRattachementsAutres;
    }

    /**
     * Renvoie le libellé pour l'attribut catalogue.
     */
    public String getLibelleCatalogue() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.CATALOGUE.getValeur(), getCatalogue(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut codeFormationMere.
     */
    public String getLibelleCodeFormationMere() throws Exception {
        return Formation.getLibelleAffichable(getCodeFormationMere(), getLangue());
    }

    /**
     * Renvoie la fiche formation correspondant à l'attribut codeFormationMere.
     */
    public Formation getFicheCodeFormationMere() throws Exception {
        return Formation.getFicheFormation(getCodeFormationMere(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut natureFormation.
     */
    public String getLibelleNatureFormation() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.NATURE_FORMATION.getValeur(), getNatureFormation(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut typeDiplome.
     */
    public String getLibelleTypeDiplome() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), getTypeDiplome(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut typeFormationContinue.
     */
    public String getLibelleTypeFormationContinue() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.TYPE_FORMATION_CONTINUE.getValeur(), getTypeFormationContinue(), LangueUtil.getLocale(Integer.parseInt(getLangue()))));
    }

    /**
     * Renvoie le libellé pour l'attribut grade.
     */
    public String getLibelleGrade() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.GRADE.getValeur(), getGrade(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut domaineLmd.
     */
    public String getLibelleDomaineLmd() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), getDomaineLmd(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut discipline.
     */
    public String getLibelleDiscipline() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), getDiscipline(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut mention.
     */
    public String getLibelleMention() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.MENTION.getValeur(), getMention(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut parcours.
     */
    public String getLibelleParcours() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.PARCOURS.getValeur(), getParcours(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut niveauRecrutement.
     */
    public String getLibelleNiveauRecrutement()  {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getNiveauRecrutement(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut anneePostbac.
     */
    public String getLibelleAnneePostbac() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.ANNEE_POSTBAC.getValeur(), getAnneePostbac(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut niveauSortie.
     */
    public String getLibelleNiveauSortie() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.NIVEAUX_SORTIE.getValeur(), getNiveauSortie(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut lieux.
     */
    public String getLibelleLieux()  {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.LIEUX.getValeur(), getLieux(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut languesEnseignement.
     */
    public String getLibelleLanguesEnseignement()  {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), getLanguesEnseignement(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut dureeListe.
     */
    public String getLibelleDureeListe() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.DUREE.getValeur(), getDureeListe(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut secteursActivites.
     */
    public String getLibelleSecteursActivites() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.SECTEUR_ACTIVITE.getValeur(), getSecteursActivites(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut principalesMatieres.
     */
    public String getLibellePrincipalesMatieres() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.PRINCIPALES_MATIERES.getValeur(), getPrincipalesMatieres(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut centresInteret.
     */
    public String getLibelleCentresInteret() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.CENTRES_INTERETS.getValeur(), getCentresInteret(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut serieBac.
     */
    public String getLibelleSerieBac()  {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.SERIE_BAC.getValeur(), getSerieBac(), getLangue()));
    }

    /**
     * Renvoie le libellé pour l'attribut domaineErasmus.
     */
    public String getLibelleDomaineErasmus() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), getDomaineErasmus(), getLangue()));
    }

    private String getConcatenatedListeLabel(List<String> labels) {
        return StringUtils.join(labels, ";");
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut resume.
     */
    public String getFormatedResume() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getResume());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut description.
     */
    public String getFormatedDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut specificites.
     */
    public String getFormatedSpecificites() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getSpecificites());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut lieuxComplement.
     */
    public String getFormatedLieuxComplement() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getLieuxComplement());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le libellé pour l'attribut responsablesFiche.
     */
    public String getLibelleResponsablesFiche() throws Exception {
        return FicheAnnuaire.getLibelleAffichable(getResponsablesFiche(), getLangue());
    }

    /**
     * Renvoie la fiche annuaireksup correspondant à l'attribut responsablesFiche.
     */
    public List<AnnuaireModele> getFicheResponsablesFiche() throws Exception {
        return FicheAnnuaire.getFichesAnnuaireDepuisReference(getResponsablesFiche());
    }

    /**
     * Renvoie la liste des fiches annuaireksup correspondant à l'attribut responsablesFiche, classées par titre de responsable.
     */
    public Map<String, List<AnnuaireModele>> getFichesAnnuaireDepuisReferenceParTypeResponsable() throws Exception {
        return FicheAnnuaire.getFichesAnnuaireDepuisReferenceParTypeResponsable(getResponsablesFiche());
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut responsablesComplement.
     */
    public String getFormatedResponsablesComplement() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getResponsablesComplement());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le libellé pour l'attribut temoignages.
     */
    public String getLibelleTemoignages() throws Exception {
        return FicheAnnuaire.getLibelleAffichable(getTemoignages(), getLangue());
    }

    /**
     * Renvoie la liste des fiches Annuaireanciensetudiants correspondant à l'attribut temoignages.
     */
    public List<AnnuaireModele> getFichesTemoignages() throws Exception {
        List<AnnuaireModele> listtemoignages = new ArrayList<>();
        if (StringUtils.isNotBlank(getTemoignages())) {
            StringTokenizer sttemoignages = new StringTokenizer(getTemoignages(), ";");
            while (sttemoignages.hasMoreTokens()) {
                listtemoignages.add(FicheAnnuaire.getFicheAnnuaire(sttemoignages.nextToken().split(",")[0], getLangue()));
            }
        }
        return listtemoignages;
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut laboratoiresPartenaires.
     */
    public String getFormatedLaboratoiresPartenaires() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getLaboratoiresPartenaires());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut etablissementsPartenaires.
     */
    public String getFormatedEtablissementsPartenaires() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getEtablissementsPartenaires());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut entreprisesPartenaires.
     */
    public String getFormatedEntreprisesPartenaires() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getEntreprisesPartenaires());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut conventionEtablissements.
     */
    public String getFormatedConventionEtablissements() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getConventionEtablissements());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut formationsRequises.
     */
    public String getFormatedFormationsRequises() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getFormationsRequises());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    public List<Formation> getFichesFormationsLieesFiche() throws Exception {
        List<Formation> listformationPoursuite = new ArrayList<Formation>();
        String formationsLiees = getFormationsLiees();
        if (StringUtils.isNotBlank(formationsLiees)) {
            StringTokenizer stformationPoursuite = new StringTokenizer(formationsLiees, ";");
            while (stformationPoursuite.hasMoreTokens()) {
                listformationPoursuite.add(Formation.getFicheFormation(stformationPoursuite.nextToken().split(",")[0], getLangue()));
            }
            return listformationPoursuite;
        }
        return ListUtils.EMPTY_LIST;
    }

    public String getLibelleFichesFormationsLieesFiche() throws Exception {
        return Formation.getLibelleAffichable(getFormationsLiees(), getLangue());
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut publicCible.
     */
    public String getFormatedPublicCible() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPublicCible());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut modalitesCandidatures.
     */
    public String getFormatedModalitesCandidatures() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getModalitesCandidatures());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut modalitesCandidaturesSpecifiques.
     */
    public String getFormatedModalitesCandidaturesSpecifiques() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getModalitesCandidaturesSpecifiques());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut conditionsAdmission.
     */
    public String getFormatedConditionsAdmission() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getConditionsAdmission());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut programme.
     */
    public String getFormatedProgramme() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getProgramme());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le libellé pour l'attribut parcoursFiche.
     */
    public String getLibelleParcoursFiche() throws Exception {
        return Parcours.getLibelleAffichable(getParcoursFiche(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut parcoursFiche.
     */
    public String getLibelleParcoursPoursuite() throws Exception {
        return Parcours.getLibelleAffichable(getParcoursPoursuite(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut parcoursFiche.
     */
    public String getLibelleFormationPoursuite() throws Exception {
        return Formation.getLibelleAffichable(getFormationPoursuite(), getLangue());
    }

    /**
     * Renvoie la liste des fiches Parcours correspondant à l'attribut parcoursFiche.
     */
    public List<Parcours> getFichesParcoursFiche() throws Exception {
        ArrayList<Parcours> listparcoursFiche = new ArrayList<Parcours>();
        if (StringUtils.isNotBlank(getParcoursFiche())) {
            StringTokenizer stparcoursFiche = new StringTokenizer(getParcoursFiche(), ";");
            while (stparcoursFiche.hasMoreTokens()) {
                listparcoursFiche.add(Parcours.getFicheParcours(stparcoursFiche.nextToken().split(",")[0], getLangue()));
            }
        }
        return listparcoursFiche;
    }

    /**
     * Renvoie la liste des fiches Parcours correspondant à l'attribut parcoursFiche.
     */
    public List<Parcours> getFichesParcoursPoursuite() throws Exception {
        ArrayList<Parcours> listparcoursPoursuite = new ArrayList<Parcours>();
        if (StringUtils.isNotBlank(getParcoursPoursuite())) {
            StringTokenizer stparcoursPoursuite = new StringTokenizer(getParcoursPoursuite(), ";");
            while (stparcoursPoursuite.hasMoreTokens()) {
                listparcoursPoursuite.add(Parcours.getFicheParcours(stparcoursPoursuite.nextToken().split(",")[0], getLangue()));
            }
        }
        return listparcoursPoursuite;
    }

    /**
     * Renvoie la liste des fiches Parcours correspondant à l'attribut parcoursFiche.
     */
    public List<Formation> getFichesFormationPoursuite() throws Exception {
        ArrayList<Formation> listformationPoursuite = new ArrayList<Formation>();
        if (StringUtils.isNotBlank(getFormationPoursuite())) {
            StringTokenizer stformationPoursuite = new StringTokenizer(getFormationPoursuite(), ";");
            while (stformationPoursuite.hasMoreTokens()) {
                listformationPoursuite.add(Formation.getFicheFormation(stformationPoursuite.nextToken().split(",")[0], getLangue()));
            }
        }
        return listformationPoursuite;
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut contenuFormation.
     */
    public String getFormatedContenuFormation() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContenuFormation());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut rythmePremiereAnnee.
     */
    public String getFormatedRythmePremiereAnnee() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getRythmePremiereAnnee());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut equipePedagogique.
     */
    public String getFormatedEquipePedagogique() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getEquipePedagogique());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut tempsPleinDescription.
     */
    public String getFormatedTempsPleinDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTempsPleinDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut compatibleProfessionnelleDescription.
     */
    public String getFormatedCompatibleProfessionnelleDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCompatibleProfessionnelleDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut tempsAmenageDescription.
     */
    public String getFormatedTempsAmenageDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTempsAmenageDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut alternanceDescription.
     */
    public String getFormatedAlternanceDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getAlternanceDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut presentielDescription.
     */
    public String getFormatedPresentielDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPresentielDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut partiellementDistanceDescription.
     */
    public String getFormatedPartiellementDistanceDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPartiellementDistanceDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut totalementDistanceDescription.
     */
    public String getFormatedTotalementDistanceDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTotalementDistanceDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut stagesDescription.
     */
    public String getFormatedStagesDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getStagesDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut sejoursEtrangerDescription.
     */
    public String getFormatedSejoursEtrangerDescription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getSejoursEtrangerDescription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut projetsTutores.
     */
    public String getFormatedProjetsTutoresComplement() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getProjetsTutoresComplement());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut modalitesEvaluation.
     */
    public String getFormatedModalitesEvaluation() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getModalitesEvaluation());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut ficheRncp.
     */
    public String getFormatedFicheRncp() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getFicheRncp());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut competencesAttestees.
     */
    public String getFormatedCompetencesAttestees() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCompetencesAttestees());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut poursuiteEtudes.
     */
    public String getFormatedPoursuiteEtudes() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPoursuiteEtudes());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut debouches.
     */
    public String getFormatedDebouches() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDebouches());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut insertionProfessionnelle.
     */
    public String getFormatedInsertionProfessionnelle() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getInsertionProfessionnelle());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut tarifComplement.
     */
    public String getFormatedTarifComplement() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTarifComplement());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut modalitesInscription.
     */
    public String getFormatedModalitesInscription() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getModalitesInscription());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut renseignements.
     */
    public String getFormatedRenseignements() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getRenseignements());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage(), e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @return the infos structure
     * @throws Exception the exception
     */
    public StructureModele getInfosStructure() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les métadonnées de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Renvoie une chaine correspondant à la concaténation des champs text de l'objet
     * (utilisé pour l'indexation Lucene).
     */
    public String getFullTextString() throws Exception {
        String s = "" + " " + getCodeSiScolarite() + " " + getIntituleCourt() + " " + getIntituleLong() + " " + getLibelleCatalogue() + " " + getLibelleNatureFormation() + " " + getLibelleTypeDiplome() + " " + getLibelleTypeFormationContinue() + " " + getLibelleGrade() + " " + getLibelleDomaineLmd().replaceAll(";", " ") + " " + getLibelleDiscipline().replaceAll(";", " ") + " " + getLibelleMention() + " " + getLibelleParcours().replaceAll(";", " ") + " " + getLibelleNiveauRecrutement().replaceAll(";", " ") + " " + getLibelleAnneePostbac().replaceAll(";", " ") + " " + getLibelleNiveauSortie().replaceAll(";", " ") + " " + getLibelleLieux().replaceAll(";", " ") + " " + getLibelleLanguesEnseignement().replaceAll(";", " ") + " " + getLibelleDureeListe().replaceAll(";", " ") + " " + getAutreDuree() + " " + getLibelleSecteursActivites().replaceAll(";", " ") + " " + getMetiers() + " " + getLibellePrincipalesMatieres().replaceAll(";", " ") + " " + getLibelleCentresInteret().replaceAll(";", " ") + " " + getLibelleSerieBac().replaceAll(";", " ") + " " + getLibelleDomaineErasmus().replaceAll(";", " ") + " " + getAccroche() + " " + getResume() + " " + getDescription() + " " + getSpecificites() + " " + getLieuxComplement() + " " + getResponsablesComplement() + " " + getLaboratoiresPartenaires() + " " + getEtablissementsPartenaires() + " " + getEntreprisesPartenaires() + " " + getConventionEtablissements() + " " + getFormationsRequises() + " " + getPublicCible() + " " + getModalitesCandidatures() + " " + getModalitesCandidaturesSpecifiques() + " " + getConditionsAdmission() + " " + getProgramme() + " " + getContenuFormation() + " " + getRythmePremiereAnnee() + " " + getEquipePedagogique() + " " + getTempsPleinDescription() + " " + getCompatibleProfessionnelleDescription() + " " + getTempsAmenageDescription() + " " + getAlternanceDescription() + " " + getPresentielDescription() + " " + getPartiellementDistanceDescription() + " " + getTotalementDistanceDescription() + " " + getStagesDescription() + " " + getSejoursEtrangerDescription() + " " + getProjetsTutoresComplement() + " " + getModalitesEvaluation() + " " + getFicheRncp() + " " + getCompetencesAttestees() + " " + getPoursuiteEtudes() + " " + getParcoursPoursuite() + " " + getFormationPoursuite() + " " + getDebouches() + " " + getInsertionProfessionnelle() + " " + getTarifComplement() + " " + getModalitesInscription() + " " + getPlaquettesCrm() + " " + getContactEmail() + " " + getContactTelephone() + " " + getContactCrm() + " " + getUrlExterneFormation() + " " + getRenseignements() + " " + getUrlInscription();
        return s;
    }

    /**
     * Traitement d'une requête sur l'objet.
     */
    public int traiterRequete(String requete) throws Exception {
        // Recuperation des paramètres de la requête
        String codeRattachementAutres = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT_AUTRES");
        String codeSiScolarite = RequeteUtil.renvoyerParametre(requete, "CODE_SI_SCOLARITE");
        String libelle = RequeteUtil.renvoyerParametre(requete, "LIBELLE");
        String catalogue = RequeteUtil.renvoyerParametre(requete, "CATALOGUE");
        String codeFormationMere = RequeteUtil.renvoyerParametre(requete, "CODE_FORMATION_MERE");
        String natureFormation = RequeteUtil.renvoyerParametre(requete, "NATURE_FORMATION");
        String formationDiplomante = RequeteUtil.renvoyerParametre(requete, "FORMATION_DIPLOMANTE");
        String formationLmd = RequeteUtil.renvoyerParametre(requete, "FORMATION_LMD");
        String typeDiplome = RequeteUtil.renvoyerParametre(requete, "TYPE_DIPLOME");
        String grade = RequeteUtil.renvoyerParametre(requete, "GRADE");
        String domaineLmd = RequeteUtil.renvoyerParametre(requete, "DOMAINE_LMD");
        String discipline = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE");
        String mention = RequeteUtil.renvoyerParametre(requete, "MENTION");
        String parcours = RequeteUtil.renvoyerParametre(requete, "PARCOURS");
        String niveauRecrutement = RequeteUtil.renvoyerParametre(requete, "NIVEAU_RECRUTEMENT");
        String anneePostbac = RequeteUtil.renvoyerParametre(requete, "ANNEE_POSTBAC");
        String niveauSortie = RequeteUtil.renvoyerParametre(requete, "NIVEAU_SORTIE");
        String formationInitiale = RequeteUtil.renvoyerParametre(requete, "FORMATION_INITIALE");
        String repriseEtudes = RequeteUtil.renvoyerParametre(requete, "REPRISE_ETUDES");
        String tempsPlein = RequeteUtil.renvoyerParametre(requete, "TEMPS_PLEIN");
        String compatibleActivitePro = RequeteUtil.renvoyerParametre(requete, "COMPATIBLE_ACTIVITE_PRO");
        String tempsAmenage = RequeteUtil.renvoyerParametre(requete, "TEMPS_AMENAGE");
        String alternance = RequeteUtil.renvoyerParametre(requete, "ALTERNANCE");
        String partiellementDistance = RequeteUtil.renvoyerParametre(requete, "PARTIELLEMENT_DISTANCE");
        String totalementDistance = RequeteUtil.renvoyerParametre(requete, "TOTALEMENT_DISTANCE");
        String responsablesFiche = RequeteUtil.renvoyerParametre(requete, "RESPONSABLES_FICHE");
        String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        String lieu = RequeteUtil.renvoyerParametre(requete, "LIEU");
        String ids = RequeteUtil.renvoyerParametre(requete, "IDS");
        return select(codeRattachementAutres, codeSiScolarite, libelle, catalogue, codeFormationMere, natureFormation, formationDiplomante, formationLmd, typeDiplome, grade, domaineLmd, discipline, mention, parcours, niveauRecrutement, anneePostbac, niveauSortie, formationInitiale, repriseEtudes, tempsPlein, compatibleActivitePro, tempsAmenage, alternance, partiellementDistance, totalementDistance, responsablesFiche, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, nombre, StringUtils.EMPTY, lieu, ids);
    }

    public int select(String codeRattachementAutres, String codeSiScolarite, String libelle, String catalogue, String codeFormationMere, String natureFormation, String formationDiplomante, String formationLmd, String typeDiplome, String grade, String domaineLmd, String discipline, String mention, String parcours, String niveauRecrutement, String anneePostbac, String niveauSortie, String formationInitiale, String repriseEtudes, String tempsPlein, String compatibleActivitePro, String tempsAmenage, String alternance, String partiellementDistance, String totalementDistance, String responsablesFiche, String codeRubrique, String codeRattachement, String langue, String etatObjet, String codeRedacteur, String code, String nombre, String orderByClause, String lieu, String ids) {
        final ClauseWhere where = new ClauseWhere();
        // Traitement du parametre codeSiScolarite
        if (StringUtils.isNotEmpty(codeSiScolarite)) {
            where.setPremiereCondition(ConditionHelper.rechercheMots("T1.CODE_SI_SCOLARITE", codeSiScolarite));
        }
        // Traitement du parametre codeRattachementAutres
        if (StringUtils.isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            //Cas où la recherche passe l'élément sur code rattachement mais le rattachement se trouve sur rattachement autres
            String codeRattachementTout = codeRattachement + ";" +codeRattachementAutres;
            if (StringUtils.isNotEmpty(codeRattachementTout) && !"0000".equals(codeRattachementTout)) {
                conditionStructure.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachementTout));
            }
            where.and(conditionStructure);
        } else if (StringUtils.isNotEmpty(codeRattachementAutres) && !"0000".equals(codeRattachementAutres)) {
            //Au cas où CODE_RATTACHEMENT pas là mais CODE_RATTACHEMENT_AUTRES oui
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachementAutres));
            where.and(conditionStructure);
        }
        // Traitement du parametre intituleCourt
        if (StringUtils.isNotEmpty(libelle)) {
            final ConditionList conditionLibelle = new ConditionList(ConditionHelper.rechercheMots("T1.INTITULE_COURT", libelle));
            conditionLibelle.or(ConditionHelper.rechercheMots("T1.INTITULE_LONG", libelle));
            where.and(conditionLibelle);
        }
        // Traitement du parametre catalogue
        if (StringUtils.isNotEmpty(catalogue) && !catalogue.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.CATALOGUE", catalogue));
        }
        // Traitement du parametre codeFormationMere
        if (StringUtils.isNotEmpty(codeFormationMere)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE_FORMATION_MERE", codeFormationMere));
        }
        // Traitement du parametre natureFormation
        if (StringUtils.isNotEmpty(natureFormation) && !natureFormation.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.NATURE_FORMATION", natureFormation));
        }
        // Traitement du parametre formationDiplomante
        if (StringUtils.isNotEmpty(formationDiplomante)) {
            where.and(ConditionHelper.egalVarchar("T1.FORMATION_DIPLOMANTE", formationDiplomante));
        }
        // Traitement du parametre formationLmd
        if (StringUtils.isNotEmpty(formationLmd)) {
            where.and(ConditionHelper.egalVarchar("T1.FORMATION_LMD", formationLmd));
        }
        // Traitement du parametre typeDiplome
        if (StringUtils.isNotEmpty(typeDiplome) && !typeDiplome.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_DIPLOME", typeDiplome));
        }
        // Traitement du parametre grade
        if (StringUtils.isNotEmpty(grade) && !grade.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.GRADE", grade));
        }
        // Traitement du parametre domaineLmd
        if (StringUtils.isNotEmpty(domaineLmd) && !domaineLmd.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DOMAINE_LMD", domaineLmd));
        }
        // Traitement du parametre discipline
        if (StringUtils.isNotEmpty(discipline) && !discipline.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DISCIPLINE", discipline));
        }
        //Traitements particuliers de recherche dans les intitulés de libellé
        traiterLibelleDansMentionEtParcours(where, libelle, langue);
        //Fin du traitement particulier


        // Traitement du parametre mention
        if (StringUtils.isNotEmpty(mention) && !mention.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.MENTION", mention));
        }
        // Traitement du parametre parcours
        if (StringUtils.isNotEmpty(parcours) && !niveauRecrutement.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.PARCOURS", parcours));
        }

        // Traitement du parametre niveauRecrutement
        if (StringUtils.isNotEmpty(niveauRecrutement) && !niveauRecrutement.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.NIVEAU_RECRUTEMENT", niveauRecrutement));
        }
        // Traitement du parametre anneePostbac
        if (StringUtils.isNotEmpty(anneePostbac) && !anneePostbac.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.ANNEE_POSTBAC", anneePostbac));
        }
        // Traitement du parametre niveauSortie
        if (StringUtils.isNotEmpty(niveauSortie) && !niveauSortie.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.NIVEAU_SORTIE", niveauSortie));
        }
        // Traitement du parametre formationInitiale
        if (StringUtils.isNotEmpty(formationInitiale)) {
            where.and(ConditionHelper.egalVarchar("T1.FORMATION_INITIALE", formationInitiale));
        }
        // Traitement du parametre repriseEtudes
        if (StringUtils.isNotEmpty(repriseEtudes)) {
            where.and(ConditionHelper.egalVarchar("T1.REPRISE_ETUDES", repriseEtudes));
        }
        // Traitement du parametre tempsPlein
        if (tempsPlein != null && tempsPlein.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TEMPS_PLEIN", tempsPlein));
        }
        // Traitement du parametre compatibleActivitePro
        if (compatibleActivitePro != null && compatibleActivitePro.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.COMPATIBLE_ACTIVITE_PRO", compatibleActivitePro));
        }
        // Traitement du parametre tempsAmenage
        if (tempsAmenage != null && tempsAmenage.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TEMPS_AMENAGE", tempsAmenage));
        }
        // Traitement du parametre alternance
        if (alternance != null && alternance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.ALTERNANCE", alternance));
        }
        // Traitement du parametre partiellementDistance
        if (partiellementDistance != null && partiellementDistance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.PARTIELLEMENT_DISTANCE", partiellementDistance));
        }
        // Traitement du parametre totalementDistance
        if (totalementDistance != null && totalementDistance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TOTALEMENT_DISTANCE", totalementDistance));
        }
        // Traitement du parametre responsablesFiche
        if (StringUtils.isNotEmpty(responsablesFiche)) {
            where.and(ConditionHelper.egalVarchar("T1.RESPONSABLES_FICHE", responsablesFiche));
        }
        // Traitement du parametre langue
        if (StringUtils.isNotEmpty(langue) && !langue.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        // Traitement du parametre etatObjet
        if (StringUtils.isNotEmpty(etatObjet) && !etatObjet.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        // Traitement du parametre codeRedacteur
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        // Traitement du parametre code
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        // Traitement du lieu
        if (StringUtils.isNotEmpty(lieu)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.LIEUX", lieu));
        }
        // Traitement des identifiants
        if (StringUtils.isNotEmpty(ids) && !"0000".equals(ids)) {
            where.and(ConditionHelper.in("ID_FORMATION", Chaine.getVecteurPointsVirgules(ids)));
        }
        // Ajout des criteres generiques
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(orderByClause)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(orderByClause);
        }
        orderBy.orderBy("T1.INTITULE_LONG", SensDeTri.ASC);
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Permet de traiter le cas où l'on veut aussi chercher dans les mentions et parcours
     * @param where
     * @param libelle
     * @param langue
     */
    private void traiterLibelleDansMentionEtParcours(ClauseWhere where, String libelle, String langue) {
        StringBuilder mentions = new StringBuilder();
        StringBuilder parcourss = new StringBuilder();
        if (StringUtils.isNotBlank(libelle)) {
            ServiceLabel serviceLabel = ServiceManager.getService(ServiceLabel.class);
            List<LabelDto> mentionLabels = serviceLabel.searchByTypeAndMatchingLibelle(OffreFormationTypeLibelle.MENTION.getValeur(), libelle);
            List<LabelDto> parcoursLabels = serviceLabel.searchByTypeAndMatchingLibelle(OffreFormationTypeLibelle.PARCOURS.getValeur(), libelle);
            for (LabelDto label : mentionLabels) {
                //Si la langue est présente dans les critères de recherche, on en tient compte
                if (StringUtils.isBlank(langue) || StringUtils.equals(label.getLangue(), langue)) {
                    mentions.append(label.getCode()).append("+");
                }
            }
            for (LabelDto label : parcoursLabels) {
                //Si la langue est présente dans les critères de recherche, on en tient compte
                if (StringUtils.isBlank(langue) || StringUtils.equals(label.getLangue(), langue)) {
                    parcourss.append(label.getCode()).append("+");
                }
            }
        }
        String mentionsFormatte = StringUtils.removeEnd(mentions.toString(), "+");
        String parcoursFormatte = StringUtils.removeEnd(parcourss.toString(), "+");
        // Traitement du parametre mention
        if (StringUtils.isNotBlank(mentionsFormatte)) {
            where.or(ConditionHelper.likePourValeursMultiple("T1.MENTION", mentionsFormatte));
        }
        // Traitement du parametre parcours
        if (StringUtils.isNotBlank(parcoursFormatte)) {
            where.or(ConditionHelper.likePourValeursMultiple("T1.PARCOURS", parcoursFormatte));
        }
    }

    /**
     * Renvoie la liste des références à cette fiche.
     */
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    public void setIdFormation(Long idFormation) {
        persistenceBean.setId(idFormation);
    }

    public Long getIdFormation() {
        return persistenceBean.getId();
    }

    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    public void setCodeSiScolarite(String codeSiScolarite) {
        persistenceBean.setCodeSiScolarite(codeSiScolarite);
    }

    public String getCodeSiScolarite() {
        return persistenceBean.getCodeSiScolarite();
    }

    public void setIntituleCourt(String intituleCourt) {
        persistenceBean.setIntituleCourt(intituleCourt);
    }

    public String getIntituleCourt() {
        return persistenceBean.getIntituleCourt();
    }

    public void setIntituleLong(String intituleLong) {
        persistenceBean.setIntituleLong(intituleLong);
    }

    public String getIntituleLong() {
        return persistenceBean.getIntituleLong();
    }

    public void setCatalogue(String catalogue) {
        persistenceBean.setCatalogue(catalogue);
    }

    public String getCatalogue() {
        return persistenceBean.getCatalogue();
    }

    public void setCodeFormationMere(String codeFormationMere) {
        persistenceBean.setCodeFormationMere(codeFormationMere);
    }

    public String getCodeFormationMere() {
        return persistenceBean.getCodeFormationMere();
    }

    public void setNatureFormation(String natureFormation) {
        persistenceBean.setNatureFormation(natureFormation);
    }

    public String getNatureFormation() {
        return persistenceBean.getNatureFormation();
    }

    public void setFormationDiplomante(String formationDiplomante) {
        persistenceBean.setFormationDiplomante(formationDiplomante);
    }

    /**
     * Formation diplomante (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getFormationDiplomante() {
        return persistenceBean.getFormationDiplomante();
    }

    public void setFormationLmd(String formationLmd) {
        persistenceBean.setFormationLmd(formationLmd);
    }

    /**
     * Formation LMD (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getFormationLmd() {
        return persistenceBean.getFormationLmd();
    }

    public void setTypeDiplome(String typeDiplome) {
        persistenceBean.setTypeDiplome(typeDiplome);
    }

    public String getTypeDiplome() {
        return persistenceBean.getTypeDiplome();
    }

    public void setGrade(String grade) {
        persistenceBean.setGrade(grade);
    }

    public String getGrade() {
        return persistenceBean.getGrade();
    }

    public void setDomaineLmd(String domaineLmd) {
        persistenceBean.setDomaineLmd(domaineLmd);
    }

    public String getDomaineLmd() {
        return persistenceBean.getDomaineLmd();
    }

    public void setDiscipline(String discipline) {
        persistenceBean.setDiscipline(discipline);
    }

    public String getDiscipline() {
        return persistenceBean.getDiscipline();
    }

    public void setMention(String mention) {
        persistenceBean.setMention(mention);
    }

    public String getMention() {
        return persistenceBean.getMention();
    }

    public void setParcours(String parcours) {
        persistenceBean.setParcours(parcours);
    }

    public String getParcours() {
        return persistenceBean.getParcours();
    }

    public void setNiveauRecrutement(String niveauRecrutement) {
        persistenceBean.setNiveauRecrutement(niveauRecrutement);
    }

    public String getNiveauRecrutement() {
        return persistenceBean.getNiveauRecrutement();
    }

    public void setAnneePostbac(String anneePostbac) {
        persistenceBean.setAnneePostbac(anneePostbac);
    }

    public String getAnneePostbac() {
        return persistenceBean.getAnneePostbac();
    }

    public void setNiveauSortie(String niveauSortie) {
        persistenceBean.setNiveauSortie(niveauSortie);
    }

    public String getNiveauSortie() {
        return persistenceBean.getNiveauSortie();
    }

    public void setFormationInitiale(String formationInitiale) {
        persistenceBean.setFormationInitiale(formationInitiale);
    }

    /**
     * Formation initiale (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getFormationInitiale() {
        return persistenceBean.getFormationInitiale();
    }

    public void setRepriseEtudes(String repriseEtudes) {
        persistenceBean.setRepriseEtudes(repriseEtudes);
    }

    /**
     * Formation accessible en reprise d'études (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getRepriseEtudes() {
        return persistenceBean.getRepriseEtudes();
    }

    public void setTempsPlein(String tempsPlein) {
        persistenceBean.setTempsPlein(tempsPlein);
    }

    /**
     * Formation en temps plein (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getTempsPlein() {
        return persistenceBean.getTempsPlein();
    }

    public void setCompatibleActivitePro(String compatibleActivitePro) {
        persistenceBean.setCompatibleActivitePro(compatibleActivitePro);
    }

    /**
     * Formation compatible avec une activité pro (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getCompatibleActivitePro() {
        return persistenceBean.getCompatibleActivitePro();
    }

    public void setTempsAmenage(String tempsAmenage) {
        persistenceBean.setTempsAmenage(tempsAmenage);
    }

    /**
     * Formation avec temps aménagé (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getTempsAmenage() {
        return persistenceBean.getTempsAmenage();
    }

    public void setAlternance(String alternance) {
        persistenceBean.setAlternance(alternance);
    }

    /**
     * Formation en alternance (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getAlternance() {
        return persistenceBean.getAlternance();
    }

    public void setHorsContratAlternance(String horsContratAlternance) {
        persistenceBean.setHorsContratAlternance(horsContratAlternance);
    }

    /**
     * Formation en hors contrat d'alternance (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getHorsContratAlternance() {
        return persistenceBean.getHorsContratAlternance();
    }

    public void setContratApprentissage(String contratApprentissage) {
        persistenceBean.setContratApprentissage(contratApprentissage);
    }

    /**
     * Formation avec contrat d'apprentissage (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getContratApprentissage() {
        return persistenceBean.getContratApprentissage();
    }

    public void setContratPro(String contratPro) {
        persistenceBean.setContratPro(contratPro);
    }

    /**
     * Formation en contrat pro (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getContratPro() {
        return persistenceBean.getContratPro();
    }

    public void setPresentiel(String presentiel) {
        persistenceBean.setPresentiel(presentiel);
    }

    /**
     * Formation en présentiel (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getPresentiel() {
        return persistenceBean.getPresentiel();
    }

    public void setPartiellementDistance(String partiellementDistance) {
        persistenceBean.setPartiellementDistance(partiellementDistance);
    }


    /**
     * Formation accessible partiellement à distance (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getPartiellementDistance() {
        return persistenceBean.getPartiellementDistance();
    }

    public void setTotalementDistance(String totalementDistance) {
        persistenceBean.setTotalementDistance(totalementDistance);
    }

    /**
     * Formation accessible totalement à distance (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getTotalementDistance() {
        return persistenceBean.getTotalementDistance();
    }

    public boolean isMixteDistance() {
        return "1".equals(persistenceBean.getPartiellementDistance()) || "1".equals(persistenceBean.getTotalementDistance());
    }

    public void setLieux(String lieux) {
        persistenceBean.setLieux(lieux);
    }

    public String getLieux() {
        return persistenceBean.getLieux();
    }

    public void setLanguesEnseignement(String languesEnseignement) {
        persistenceBean.setLanguesEnseignement(languesEnseignement);
    }

    public String getLanguesEnseignement() {
        return persistenceBean.getLanguesEnseignement();
    }

    public void setSelectif(String selectif) {
        persistenceBean.setSelectif(selectif);
    }

    /**
     * Formation sélective (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSelectif() {
        return persistenceBean.getSelectif();
    }

    public void setCapaciteAccueilLimitee(String capaciteAccueilLimitee) {
        persistenceBean.setCapaciteAccueilLimitee(capaciteAccueilLimitee);
    }

    /**
     * Formation avec capacité d'accueil limitée (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getCapaciteAccueilLimitee() {
        return persistenceBean.getCapaciteAccueilLimitee();
    }

    public void setDureeListe(String dureeListe) {
        persistenceBean.setDureeListe(dureeListe);
    }

    public String getDureeListe() {
        return persistenceBean.getDureeListe();
    }

    public void setAutreDuree(String autreDuree) {
        persistenceBean.setAutreDuree(autreDuree);
    }

    public String getAutreDuree() {
        return persistenceBean.getAutreDuree();
    }

    public void setStages(String stages) {
        persistenceBean.setStages(stages);
    }

    /**
     * Formation avec stages (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStages() {
        return persistenceBean.getStages();
    }

    public void setStagesOptionnel(String stagesOptionnel) {
        persistenceBean.setStagesOptionnel(stagesOptionnel);
    }

    /**
     * Formation avec stages optionnels (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesOptionnel() {
        return persistenceBean.getStagesOptionnel();
    }

    public void setStagesOptionnelMultiple(String stagesOptionnelMultiple) {
        persistenceBean.setStagesOptionnelMultiple(stagesOptionnelMultiple);
    }

    /**
     * Formation avec stages optionnels multiples (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesOptionnelMultiple() {
        return persistenceBean.getStagesOptionnelMultiple();
    }

    public void setStagesOptionnelEtranger(String stagesOptionnelEtranger) {
        persistenceBean.setStagesOptionnelEtranger(stagesOptionnelEtranger);
    }

    /**
     * Formation avec stages optionnels à l'étranger (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesOptionnelEtranger() {
        return persistenceBean.getStagesOptionnelEtranger();
    }

    public void setStagesObligatoire(String stagesObligatoire) {
        persistenceBean.setStagesObligatoire(stagesObligatoire);
    }

    /**
     * Formation avec stages obligatoires (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesObligatoire() {
        return persistenceBean.getStagesObligatoire();
    }

    public void setStagesObligatoireMultiple(String stagesObligatoireMultiple) {
        persistenceBean.setStagesObligatoireMultiple(stagesObligatoireMultiple);
    }

    /**
     * Formation avec stages obligatoires multiples (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesObligatoireMultiple() {
        return persistenceBean.getStagesObligatoireMultiple();
    }

    public void setStagesObligatoireEtranger(String stagesObligatoireEtranger) {
        persistenceBean.setStagesObligatoireEtranger(stagesObligatoireEtranger);
    }

    /**
     * Formation avec stages obligatoires à l'étranger (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getStagesObligatoireEtranger() {
        return persistenceBean.getStagesObligatoireEtranger();
    }

    public boolean isStageOptionnel() {
        return "1".equals(persistenceBean.getStagesOptionnel()) || "1".equals(persistenceBean.getStagesOptionnelMultiple()) || "1".equals(persistenceBean.getStagesOptionnelEtranger());
    }

    public boolean isStageEtranger() {
        return "1".equals(persistenceBean.getStagesObligatoireEtranger()) || "1".equals(persistenceBean.getStagesOptionnelEtranger());
    }

    public boolean isStageObligatoire() {
        return "1".equals(persistenceBean.getStagesObligatoire()) || "1".equals(persistenceBean.getStagesObligatoireMultiple()) || "1".equals(persistenceBean.getStagesObligatoireEtranger());
    }

    public void setSejoursEtranger(String sejoursEtranger) {
        persistenceBean.setSejoursEtranger(sejoursEtranger);
    }

    /**
     * Formation avec séjours à l'étranger (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSejoursEtranger() {
        return persistenceBean.getSejoursEtranger();
    }

    public void setSejoursEtrangerOptionnel(String sejoursEtrangerOptionnel) {
        persistenceBean.setSejoursEtrangerOptionnel(sejoursEtrangerOptionnel);
    }

    /**
     * Formation avec séjours à l'étranger optionnels (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSejoursEtrangerOptionnel() {
        return persistenceBean.getSejoursEtrangerOptionnel();
    }

    public void setSejoursEtrangerOptionnelMultiples(String sejoursEtrangerOptionnelMultiples) {
        persistenceBean.setSejoursEtrangerOptionnelMultiples(sejoursEtrangerOptionnelMultiples);
    }

    /**
     * Formation avec séjours à l'étranger optionnels multiples (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSejoursEtrangerOptionnelMultiples() {
        return persistenceBean.getSejoursEtrangerOptionnelMultiples();
    }

    public void setSejoursEtrangerObligatoire(String sejoursEtrangerObligatoire) {
        persistenceBean.setSejoursEtrangerObligatoire(sejoursEtrangerObligatoire);
    }

    /**
     * Formation avec séjours à l'étranger obligatoires (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSejoursEtrangerObligatoire() {
        return persistenceBean.getSejoursEtrangerObligatoire();
    }

    public void setSejoursEtrangerObligatoireMultiples(String sejoursEtrangerObligatoireMultiples) {
        persistenceBean.setSejoursEtrangerObligatoireMultiples(sejoursEtrangerObligatoireMultiples);
    }

    /**
     * Formation avec séjours à l'étranger obligatoires multiples (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getSejoursEtrangerObligatoireMultiples() {
        return persistenceBean.getSejoursEtrangerObligatoireMultiples();
    }

    public void setProjetsTutores(String projetsTutores) {
        persistenceBean.setProjetsTutores(projetsTutores);
    }

    /**
     * Formation avec projets tutorés (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getProjetsTutores() {
        return persistenceBean.getProjetsTutores();
    }

    public void setProjetsTutoresOptionnel(String projetsTutoresOptionnel) {
        persistenceBean.setProjetsTutoresOptionnel(projetsTutoresOptionnel);
    }

    /**
     * Formation avec projets tutorés optionnels (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getProjetsTutoresOptionnel() {
        return persistenceBean.getProjetsTutoresOptionnel();
    }

    public void setProjetsTutoresOptionnelMultiples(String projetsTutoresOptionnelMultiples) {
        persistenceBean.setProjetsTutoresOptionnelMultiples(projetsTutoresOptionnelMultiples);
    }

    /**
     * Formation avec projets tutorés optionnels multiples (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getProjetsTutoresOptionnelMultiples() {
        return persistenceBean.getProjetsTutoresOptionnelMultiples();
    }

    public void setProjetsTutoresObligatoire(String projetsTutoresObligatoire) {
        persistenceBean.setProjetsTutoresObligatoire(projetsTutoresObligatoire);
    }

    /**
     * Formation avec projets tutorés obligatoires (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getProjetsTutoresObligatoire() {
        return persistenceBean.getProjetsTutoresObligatoire();
    }

    public void setProjetsTutoresObligatoireMultiples(String projetsTutoresObligatoireMultiples) {
        persistenceBean.setProjetsTutoresObligatoireMultiples(projetsTutoresObligatoireMultiples);
    }

    public String getProjetsTutoresObligatoireMultiples() {
        return persistenceBean.getProjetsTutoresObligatoireMultiples();
    }

    public void setSecteursActivites(String secteursActivites) {
        persistenceBean.setSecteursActivites(secteursActivites);
    }

    public String getSecteursActivites() {
        return persistenceBean.getSecteursActivites();
    }

    public void setMetiers(String metiers) {
        persistenceBean.setMetiers(metiers);
    }

    public String getMetiers() {
        return persistenceBean.getMetiers();
    }

    public void setPrincipalesMatieres(String principalesMatieres) {
        persistenceBean.setPrincipalesMatieres(principalesMatieres);
    }

    public String getPrincipalesMatieres() {
        return persistenceBean.getPrincipalesMatieres();
    }

    public void setCentresInteret(String centresInteret) {
        persistenceBean.setCentresInteret(centresInteret);
    }

    public String getCentresInteret() {
        return persistenceBean.getCentresInteret();
    }

    public void setSerieBac(String serieBac) {
        persistenceBean.setSerieBac(serieBac);
    }

    public String getSerieBac() {
        return persistenceBean.getSerieBac();
    }

    public void setDomaineErasmus(String domaineErasmus) {
        persistenceBean.setDomaineErasmus(domaineErasmus);
    }

    public String getDomaineErasmus() {
        return persistenceBean.getDomaineErasmus();
    }

    public void setAccroche(String accroche) {
        persistenceBean.setAccroche(accroche);
    }

    public String getAccroche() {
        return persistenceBean.getAccroche();
    }

    public void setResume(String resume) {
        persistenceBean.setResume(resume);
    }

    public String getResume() {
        return persistenceBean.getResume();
    }

    public void setDescription(String description) {
        persistenceBean.setDescription(description);
    }

    public String getDescription() {
        return persistenceBean.getDescription();
    }

    public void setSpecificites(String specificites) {
        persistenceBean.setSpecificites(specificites);
    }

    public String getSpecificites() {
        return persistenceBean.getSpecificites();
    }

    public void setLieuxComplement(String lieuxComplement) {
        persistenceBean.setLieuxComplement(lieuxComplement);
    }

    public String getLieuxComplement() {
        return persistenceBean.getLieuxComplement();
    }

    public void setResponsablesFiche(String responsablesFiche) {
        persistenceBean.setResponsablesFiche(responsablesFiche);
    }

    public String getResponsablesFiche() {
        return persistenceBean.getResponsablesFiche();
    }

    public void setResponsablesComplement(String responsablesComplement) {
        persistenceBean.setResponsablesComplement(responsablesComplement);
    }

    public String getResponsablesComplement() {
        return persistenceBean.getResponsablesComplement();
    }

    public void setTemoignages(String temoignages) {
        persistenceBean.setTemoignages(temoignages);
    }

    public String getTemoignages() {
        return persistenceBean.getTemoignages();
    }

    public void setLaboratoiresPartenaires(String laboratoiresPartenaires) {
        persistenceBean.setLaboratoiresPartenaires(laboratoiresPartenaires);
    }

    public String getLaboratoiresPartenaires() {
        return persistenceBean.getLaboratoiresPartenaires();
    }

    public void setEtablissementsPartenaires(String etablissementsPartenaires) {
        persistenceBean.setEtablissementsPartenaires(etablissementsPartenaires);
    }

    public String getEtablissementsPartenaires() {
        return persistenceBean.getEtablissementsPartenaires();
    }

    public void setEntreprisesPartenaires(String entreprisesPartenaires) {
        persistenceBean.setEntreprisesPartenaires(entreprisesPartenaires);
    }

    public String getEntreprisesPartenaires() {
        return persistenceBean.getEntreprisesPartenaires();
    }

    public void setConventionEtablissements(String conventionEtablissements) {
        persistenceBean.setConventionEtablissements(conventionEtablissements);
    }

    public String getConventionEtablissements() {
        return persistenceBean.getConventionEtablissements();
    }

    public void setFormationsRequises(String formationsRequises) {
        persistenceBean.setFormationsRequises(formationsRequises);
    }

    public String getFormationsRequises() {
        return persistenceBean.getFormationsRequises();
    }

    public String getFormationsLiees() {
        return persistenceBean.getFormationsLiees();
    }

    public void setFormationsLiees(String formationsLiees) {
        persistenceBean.setFormationsLiees(formationsLiees);
    }

    public void setPublicCible(String publicCible) {
        persistenceBean.setPublicCible(publicCible);
    }

    public String getPublicCible() {
        return persistenceBean.getPublicCible();
    }

    public void setModalitesCandidatures(String modalitesCandidatures) {
        persistenceBean.setModalitesCandidatures(modalitesCandidatures);
    }

    public String getModalitesCandidatures() {
        return persistenceBean.getModalitesCandidatures();
    }

    public void setModalitesCandidaturesSpecifiques(String modalitesCandidaturesSpecifiques) {
        persistenceBean.setModalitesCandidaturesSpecifiques(modalitesCandidaturesSpecifiques);
    }

    public String getModalitesCandidaturesSpecifiques() {
        return persistenceBean.getModalitesCandidaturesSpecifiques();
    }

    public void setConditionsAdmission(String conditionsAdmission) {
        persistenceBean.setConditionsAdmission(conditionsAdmission);
    }

    public String getConditionsAdmission() {
        return persistenceBean.getConditionsAdmission();
    }

    public void setProgramme(String programme) {
        persistenceBean.setProgramme(programme);
    }

    public String getProgramme() {
        return persistenceBean.getProgramme();
    }

    public void setParcoursFiche(String parcoursFiche) {
        persistenceBean.setParcoursFiche(parcoursFiche);
    }

    public String getParcoursFiche() {
        return persistenceBean.getParcoursFiche();
    }

    public void setContenuFormation(String contenuFormation) {
        persistenceBean.setContenuFormation(contenuFormation);
    }

    public String getContenuFormation() {
        return persistenceBean.getContenuFormation();
    }

    public void setRythmePremiereAnnee(String rythmePremiereAnnee) {
        persistenceBean.setRythmePremiereAnnee(rythmePremiereAnnee);
    }

    public String getRythmePremiereAnnee() {
        return persistenceBean.getRythmePremiereAnnee();
    }

    public void setEquipePedagogique(String equipePedagogique) {
        persistenceBean.setEquipePedagogique(equipePedagogique);
    }

    public String getEquipePedagogique() {
        return persistenceBean.getEquipePedagogique();
    }

    public void setTempsPleinDescription(String tempsPleinDescription) {
        persistenceBean.setTempsPleinDescription(tempsPleinDescription);
    }

    public String getTempsPleinDescription() {
        return persistenceBean.getTempsPleinDescription();
    }

    public void setCompatibleProfessionnelleDescription(String compatibleProfessionnelleDescription) {
        persistenceBean.setCompatibleProfessionnelleDescription(compatibleProfessionnelleDescription);
    }

    public String getCompatibleProfessionnelleDescription() {
        return persistenceBean.getCompatibleProfessionnelleDescription();
    }

    public void setTempsAmenageDescription(String tempsAmenageDescription) {
        persistenceBean.setTempsAmenageDescription(tempsAmenageDescription);
    }

    public String getTempsAmenageDescription() {
        return persistenceBean.getTempsAmenageDescription();
    }

    public void setAlternanceDescription(String alternanceDescription) {
        persistenceBean.setAlternanceDescription(alternanceDescription);
    }

    public String getAlternanceDescription() {
        return persistenceBean.getAlternanceDescription();
    }

    public void setPresentielDescription(String presentielDescription) {
        persistenceBean.setPresentielDescription(presentielDescription);
    }

    public String getPresentielDescription() {
        return persistenceBean.getPresentielDescription();
    }

    public void setPartiellementDistanceDescription(String partiellementDistanceDescription) {
        persistenceBean.setPartiellementDistanceDescription(partiellementDistanceDescription);
    }

    public String getPartiellementDistanceDescription() {
        return persistenceBean.getPartiellementDistanceDescription();
    }

    public void setTotalementDistanceDescription(String totalementDistanceDescription) {
        persistenceBean.setTotalementDistanceDescription(totalementDistanceDescription);
    }

    public String getTotalementDistanceDescription() {
        return persistenceBean.getTotalementDistanceDescription();
    }

    public void setStagesDescription(String stagesDescription) {
        persistenceBean.setStagesDescription(stagesDescription);
    }

    public String getStagesDescription() {
        return persistenceBean.getStagesDescription();
    }

    public void setSejoursEtrangerDescription(String sejoursEtrangerDescription) {
        persistenceBean.setSejoursEtrangerDescription(sejoursEtrangerDescription);
    }

    public String getSejoursEtrangerDescription() {
        return persistenceBean.getSejoursEtrangerDescription();
    }

    public void setProjetsTutoresComplement(String projetsTutoresComplement) {
        persistenceBean.setProjetsTutoresComplement(projetsTutoresComplement);
    }

    public String getProjetsTutoresComplement() {
        return persistenceBean.getProjetsTutoresComplement();
    }

    public void setModalitesEvaluation(String modalitesEvaluation) {
        persistenceBean.setModalitesEvaluation(modalitesEvaluation);
    }

    public String getModalitesEvaluation() {
        return persistenceBean.getModalitesEvaluation();
    }

    public void setFicheRncp(String ficheRncp) {
        persistenceBean.setFicheRncp(ficheRncp);
    }

    public String getFicheRncp() {
        return persistenceBean.getFicheRncp();
    }

    public void setCompetencesAttestees(String competencesAttestees) {
        persistenceBean.setCompetencesAttestees(competencesAttestees);
    }

    public String getCompetencesAttestees() {
        return persistenceBean.getCompetencesAttestees();
    }

    public void setPoursuiteEtudes(String poursuiteEtudes) {
        persistenceBean.setPoursuiteEtudes(poursuiteEtudes);
    }

    public String getPoursuiteEtudes() {
        return persistenceBean.getPoursuiteEtudes();
    }

    public void setParcoursPoursuite(String parcoursPoursuite) {
        persistenceBean.setParcoursPoursuite(parcoursPoursuite);
    }

    public String getParcoursPoursuite() {
        return persistenceBean.getParcoursPoursuite();
    }

    public void setFormationPoursuite(String formationPoursuite) {
        persistenceBean.setFormationPoursuite(formationPoursuite);
    }

    public String getFormationPoursuite() {
        return persistenceBean.getFormationPoursuite();
    }

    public void setDebouches(String debouches) {
        persistenceBean.setDebouches(debouches);
    }

    public String getDebouches() {
        return persistenceBean.getDebouches();
    }

    public void setInsertionProfessionnelle(String insertionProfessionnelle) {
        persistenceBean.setInsertionProfessionnelle(insertionProfessionnelle);
    }

    public String getInsertionProfessionnelle() {
        return persistenceBean.getInsertionProfessionnelle();
    }

    public void setTarifComplement(String tarifComplement) {
        persistenceBean.setTarifComplement(tarifComplement);
    }

    public String getTarifComplement() {
        return persistenceBean.getTarifComplement();
    }

    public void setModalitesInscription(String modalitesInscription) {
        persistenceBean.setModalitesInscription(modalitesInscription);
    }

    public String getModalitesInscription() {
        return persistenceBean.getModalitesInscription();
    }

    public void setPlaquettesCrm(String plaquettesCrm) {
        persistenceBean.setPlaquettesCrm(plaquettesCrm);
    }

    public String getPlaquettesCrm() {
        return persistenceBean.getPlaquettesCrm();
    }

    public void setContactEmail(String contactEmail) {
        persistenceBean.setContactEmail(contactEmail);
    }

    public String getContactEmail() {
        return persistenceBean.getContactEmail();
    }

    public void setContactTelephone(String contactTelephone) {
        persistenceBean.setContactTelephone(contactTelephone);
    }

    public String getContactTelephone() {
        return persistenceBean.getContactTelephone();
    }

    public void setContactCrm(String contactCrm) {
        persistenceBean.setContactCrm(contactCrm);
    }

    public String getContactCrm() {
        return persistenceBean.getContactCrm();
    }

    public void setUrlExterneFormation(String urlExterneFormation) {
        persistenceBean.setUrlExterneFormation(urlExterneFormation);
    }

    public String getUrlExterneFormation() {
        return persistenceBean.getUrlExterneFormation();
    }

    public void setRenseignements(String renseignements) {
        persistenceBean.setRenseignements(renseignements);
    }

    public String getRenseignements() {
        return persistenceBean.getRenseignements();
    }

    public void setUrlInscription(String urlInscription) {
        persistenceBean.setUrlInscription(urlInscription);
    }

    public String getUrlInscription() {
        return persistenceBean.getUrlInscription();
    }

    /**
     * Afficher le programme de la formation (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getAfficherProgramme() {
        return persistenceBean.getAfficherProgramme();
    }

    public void setAfficherProgramme(final String afficherProgramme) {
        persistenceBean.setAfficherProgramme(afficherProgramme);
    }

    /**
     * Activer la construction de parcours (booléen).
     * @return "1" si vrai, "0" si faux
     */
    public String getActiverConstruireParcours() {
        return persistenceBean.getActiverConstruireParcours();
    }

    public void setActiverConstruireParcours(final String activerConstruireParcours) {
        persistenceBean.setActiverConstruireParcours(activerConstruireParcours);
    }

    public String getFormationContinue() {
        return persistenceBean.getFormationContinue();
    }

    public void setFormationContinue(final String formationContinue) {
        persistenceBean.setFormationContinue(formationContinue);
    }

    public String getTypeFormationContinue() {
        return persistenceBean.getTypeFormationContinue();
    }

    public void setTypeFormationContinue(final String typeFormationContinue) {
        persistenceBean.setTypeFormationContinue(typeFormationContinue);
    }
}

