/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: rechercheetlabo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.LaboratoireBean;
import com.ksup.objetspartages.service.ServiceLaboratoire;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotBlank;
import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet laboratoire.
 */
@FicheAnnotation()
public class Laboratoire extends AbstractFicheRestriction<LaboratoireBean> implements StructureModele {

    private static final long serialVersionUID = -2123054438151768624L;

    private transient final ServiceRessource serviceRessource;

    public Laboratoire() {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
    }

    @Override
    public void addWithForcedId() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceStructure.updateCache(this);
        super.addWithForcedId();
    }

    @Override
    public void add() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceStructure.updateCache(this);
        super.add();
    }

    @Override
    public void delete() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final ServiceLaboratoire serviceLaboratoire = ServiceManager.getServiceForBean(LaboratoireBean.class);
        serviceStructure.updateCache(this);
        serviceLaboratoire.delete(this.getIdFiche());
    }

    @Override
    public void update() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        super.update();
        serviceStructure.updateCache(this);
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     *
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Laboratoire#getFicheLaboratoire(String, String)}
     */
    @Deprecated
    public static Laboratoire getFicheLaboratoire(final OMContext ctx, final String code, String langue) throws Exception {
        Laboratoire fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Laboratoire();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Laboratoire getFicheLaboratoire(final String code, String langue) throws Exception {
        Laboratoire fiche = null;
        if (StringUtils.isNotBlank(code)) {
            langue = StringUtils.defaultString(langue, "0");
            fiche = new Laboratoire();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     *
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Laboratoire#getLibelleAffichable(String, String)}
     */
    @Deprecated
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        return getLibelleAffichable(codes, langue);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (StringUtils.isBlank(codes)) {
            return res;
        }
        langue = StringUtils.defaultString(langue, "0");
        final Laboratoire laboratoire = new Laboratoire();
        laboratoire.init();
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = laboratoire.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = laboratoire.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (laboratoire.nextItem()) {
                    res += laboratoire.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    @Override
    public void setCtx(OMContext ctx) {
        this.ctx = ctx;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        // Temps systeme en ms
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setLibelleCourt(StringUtils.EMPTY);
        setLibelleLong(StringUtils.EMPTY);
        setTypeLaboratoire("0000");
        setStructureEnseignement("1");
        setReferenceUmr(StringUtils.EMPTY);
        setReconnaissance(StringUtils.EMPTY);
        setDirectionScientifique(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setRattachementExterne(StringUtils.EMPTY);
        setIdBandeau((long) 0);
        setOnglets(StringUtils.EMPTY);
        setCouleurTitre(StringUtils.EMPTY);
        setCouleurFond(StringUtils.EMPTY);
        setAdresse(StringUtils.EMPTY);
        setCodePostal(StringUtils.EMPTY);
        setVille(StringUtils.EMPTY);
        setVilles(StringUtils.EMPTY);
        setTelephone(StringUtils.EMPTY);
        setTelecopie(StringUtils.EMPTY);
        setAdresseMail(StringUtils.EMPTY);
        setSiteWeb("http://");
        setCodeResponsable(StringUtils.EMPTY);
        setComposition(StringUtils.EMPTY);
        setAxeRechercheEtablissement(StringUtils.EMPTY);
        setAxesRecherche(StringUtils.EMPTY);
        setDomaineApplication(StringUtils.EMPTY);
        setLanguesParlees(StringUtils.EMPTY);
        setFormations(StringUtils.EMPTY);
        setPublications(StringUtils.EMPTY);
        setEquipements(StringUtils.EMPTY);
        setCooperationsInternationales(StringUtils.EMPTY);
        setCollaborationsUniversitaires(StringUtils.EMPTY);
        setCooperationsIndustrielles(StringUtils.EMPTY);
        setChiffresCles(StringUtils.EMPTY);
        setComplementCoordonnees(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setIdPlanAcces((long) 0);
        setIdPhoto((long) 0);
    }

    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle pour l'attribut typeLaboratoire.
     */
    public String getLibelleTypeLaboratoire() {
        return LabelUtils.getLibelle("63", getTypeLaboratoire(), getLangue());
    }

    @Override
    public String getLibelleTypeStructure() {
        return getLibelleTypeLaboratoire();
    }

    /**
     * Renvoie le libelle pour l'attribut directionScientifique.
     */
    public String getLibelleDirectionScientifique() {
        return FicheUnivHelper.getLibelle(this, "direction_scientifique", getDirectionScientifique());
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final ArrayList<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if(currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut onglets.
     */
    public String getFormatedOnglets() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getOnglets());
    }

    /**
     * Renvoie le libelle pour l'attribut villes. Si plusieurs villes, chaque libellé est séparé par la balise HTML br
     */
    public String getLibelleVilles() {
        return LabelUtils.getLibelle("22", getVilles(), getLangue()).replaceAll(";", "<br />");
    }

    /**
     * Renvoie le libelle pour l'attribut codeResponsable.
     */
    public String getLibelleCodeResponsable() throws Exception {
        String listeLibelles = StringUtils.EMPTY;
        final StringTokenizer stResponsables = new StringTokenizer(getCodeResponsable(), ";");
        int idxResponsable = 0;
        // la liste des responsables est composée de la manière suivante :
        // code,langue,titre
        while (stResponsables.hasMoreTokens()) {
            String code = StringUtils.EMPTY;
            String langue = "0";
            String titre = StringUtils.EMPTY;
            StringTokenizer stResponsableCourant = new StringTokenizer(stResponsables.nextToken(), ",");
            int token = 0;
            while (stResponsableCourant.hasMoreTokens()) {
                switch (token) {
                    case 0:
                        code = stResponsableCourant.nextToken();
                        break;
                    case 1:
                        langue = stResponsableCourant.nextToken();
                        break;
                    case 2:
                        titre = stResponsableCourant.nextToken();
                        break;
                    default:
                        stResponsableCourant.nextToken();
                        break;
                }
                token++;
            }
            if (idxResponsable > 0) {
                listeLibelles += ";";
            }
            if (!code.equals(StringUtils.EMPTY)) {
                final AnnuaireModele annuaire = FicheAnnuaire.getFicheAnnuaire(code, langue);
                if (annuaire != null) {
                    listeLibelles += annuaire.getLibelleAffichable();
                }
            }
            if (!titre.equals(StringUtils.EMPTY)) {
                listeLibelles += " [" + LabelUtils.getLibelle("64", titre, langue) + "]";
            }
            idxResponsable++;
        }
        String sCumullibelles = listeLibelles.replaceAll("\"", "\\\"");
        sCumullibelles = sCumullibelles.replaceAll("\'", "\\\'");
        return sCumullibelles;
    }

    /**
     * Renvoie la fiche annuaire correspondant a l'attribut codeResponsable.
     */
    public FicheUniv getFicheCodeResponsable() throws Exception {
        return FicheAnnuaire.getFicheAnnuaire(getCodeResponsable(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut composition.
     */
    public String getFormatedComposition() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComposition());
    }

    /**
     * Renvoie le libelle pour l'attribut axeRechercheEtablissement.
     */
    public String getLibelleAxeRechercheEtablissement() {
        return LabelUtils.getLibelle("61", getAxeRechercheEtablissement(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut axeRecherche.
     */
    public String getFormatedAxeRecherche() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getAxesRecherche());
    }

    /**
     * Renvoie le libelle pour l'attribut domaineApplication.
     */
    public String getLibelleDomaineApplication() {
        return LabelUtils.getLibelle("62", getDomaineApplication(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut formations.
     */
    public String getFormatedFormations() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getFormations());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut publications.
     */
    public String getFormatedPublications() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPublications());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut equipements.
     */
    public String getFormatedEquipements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getEquipements());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut cooperationsInternationales.
     */
    public String getFormatedCooperationsInternationales() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCooperationsInternationales());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut collaborationsUniversitaires.
     */
    public String getFormatedCollaborationsUniversitaires() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCollaborationsUniversitaires());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut cooperationsIndustrielles.
     */
    public String getFormatedCooperationsIndustrielles() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCooperationsIndustrielles());
    }

    /**
     * Renvoie le fichier joint correspondant a l'attribut photo.
     */
    public RessourceBean getPhoto() throws Exception {
        return serviceRessource.getFile(this);
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut chiffresCles.
     */
    public String getFormatedChiffresCles() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getChiffresCles());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complementCoordonnees.
     */
    public String getFormatedComplementCoordonnees() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplementCoordonnees());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String libelle = RequeteUtil.renvoyerParametre(requete, "LIBELLE");
        final String theme = RequeteUtil.renvoyerParametre(requete, "THEME");
        String typeLaboratoire = RequeteUtil.renvoyerParametre(requete, "TYPE_LABORATOIRE");
        // sur la recherche tout type de structure, le paramètre est
        // TYPE_STRUCTURE
        if (typeLaboratoire.length() == 0) {
            typeLaboratoire = RequeteUtil.renvoyerParametre(requete, "TYPE_STRUCTURE");
        }
        final String referenceUmr = RequeteUtil.renvoyerParametre(requete, "REFERENCE_UMR");
        final String directionScientifique = RequeteUtil.renvoyerParametre(requete, "DIRECTION_SCIENTIFIQUE");
        final String axeRechercheEtablissement = RequeteUtil.renvoyerParametre(requete, "AXE_RECHERCHE_ETABLISSEMENT");
        final String domaineApplication = RequeteUtil.renvoyerParametre(requete, "DOMAINE_APPLICATION");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        String order = StringUtils.EMPTY;
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(libelle, theme, typeLaboratoire, referenceUmr, directionScientifique, axeRechercheEtablissement, domaineApplication, StringUtils.EMPTY, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Laboratoire a partir de l'ensemble des criteres.
     */
    public int select(final String libelle, final String theme, final String typeLaboratoire, final String referenceUmr, final String directionScientifique, final String axeRechercheEtablissement, final String domaineApplication, final String structureEnseignement, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (isNotEmpty(libelle) && isNotBlank(libelle)) {
            final ConditionList conditionLiblle = new ConditionList(ConditionHelper.rechercheMots("T1.LIBELLE_COURT", libelle));
            conditionLiblle.or(ConditionHelper.rechercheMots("T1.LIBELLE_LONG", libelle));
            where.and(conditionLiblle);
        }
        if (isNotEmpty(typeLaboratoire) && !"0000".equals(typeLaboratoire)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_LABORATOIRE", typeLaboratoire));
        }
        if (isNotEmpty(referenceUmr)) {
            where.and(ConditionHelper.rechercheMots("T1.REFERENCE_UMR", referenceUmr));
        }
        if (isNotEmpty(directionScientifique) && !"0000".equals(directionScientifique)) {
            where.and(ConditionHelper.egalVarchar("T1.DIRECTION_SCIENTIFIQUE", directionScientifique));
        }
        if (isNotEmpty(axeRechercheEtablissement) && !"0000".equals(axeRechercheEtablissement)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.AXE_RECHERCHE_ETABLISSEMENT", axeRechercheEtablissement));
        }
        if (isNotEmpty(domaineApplication) && !"0000".equals(domaineApplication)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DOMAINE_APPLICATION", domaineApplication));
        }
        if (isNotEmpty(structureEnseignement)) {
            where.and(ConditionHelper.egalVarchar("T1.STRUCTURE_ENSEIGNEMENT", structureEnseignement));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructure.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructure);
        }
        if (isNotEmpty(theme)) {
            final ConditionList conditionTheme = new ConditionList(ConditionHelper.like("T1.AXES_RECHERCHE", theme, "%", "%"));
            conditionTheme.or(ConditionHelper.like("T1.META_KEYWORDS", theme, "%", "%"));
            where.and(conditionTheme);
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ContexteUtil.getContexteUniv(), this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.LIBELLE_LONG", SensDeTri.ASC);
        }
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ContexteUtil.getContexteUniv(), nombre);
        requeteSelect.orderBy(orderBy).limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    @Override
    public String getTypeStructure() {
        return getTypeLaboratoire();
    }

    @Override
    public void setTypeStructure(final String typeStructure) {
        setTypeLaboratoire(typeStructure);
    }

    @Override
    public String getAttributSpecifique1() {
        return getStructureEnseignement();
    }

    @Override
    public String getAttributSpecifique2() {
        return getTypeLaboratoire();
    }

    @Override
    public String getAttributSpecifique3() {
        return StringUtils.EMPTY;
    }

    @Override
    public String getAttributSpecifique4() {
        return StringUtils.EMPTY;
    }

    @Override
    public String getAttributSpecifique5() {
        return StringUtils.EMPTY;
    }

    @Override
    public String getActivite() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setActivite(final String activite) {
        // nothing
    }

    public Long getIdLaboratoire(){
        return persistenceBean.getId();
    }

    public void setIdLaboratoire(Long id) {
        persistenceBean.setId(id);
    }

    /**
     * Gets the libelle court.
     *
     * @return the libelle court
     */
    public String getLibelleCourt() {
        return persistenceBean.getLibelleCourt();
    }

    /**
     * Sets the libelle court.
     *
     * @param libelleCourt
     *            the new libelle court
     */
    public void setLibelleCourt(String libelleCourt) {
        persistenceBean.setLibelleCourt(libelleCourt);
    }

    /**
     * Gets the libelle long.
     *
     * @return the libelle long
     */
    public String getLibelleLong() {
        return persistenceBean.getLibelleLong();
    }

    /**
     * Sets the libelle long.
     *
     * @param libelleLong
     *            the new libelle long
     */
    public void setLibelleLong(String libelleLong) {
        persistenceBean.setLibelleLong(libelleLong);
    }

    /**
     * Gets the type laboratoire.
     *
     * @return the type laboratoire
     */
    public String getTypeLaboratoire() {
        return persistenceBean.getTypeLaboratoire();
    }

    /**
     * Sets the type laboratoire.
     *
     * @param typeLaboratoire
     *            the new type laboratoire
     */
    public void setTypeLaboratoire(String typeLaboratoire) {
        persistenceBean.setTypeLaboratoire(typeLaboratoire);
    }

    /**
     * Gets the structure enseignement.
     *
     * @return the structure enseignement
     */
    public String getStructureEnseignement() {
        return persistenceBean.getStructureEnseignement();
    }

    /**
     * Sets the structure enseignement.
     *
     * @param structureEnseignement
     *            the new structure enseignement
     */
    public void setStructureEnseignement(String structureEnseignement) {
        persistenceBean.setStructureEnseignement(structureEnseignement);
    }

    /**
     * Gets the reference umr.
     *
     * @return the reference umr
     */
    public String getReferenceUmr() {
        return persistenceBean.getReferenceUmr();
    }

    /**
     * Sets the reference umr.
     *
     * @param referenceUmr
     *            the new reference umr
     */
    public void setReferenceUmr(String referenceUmr) {
        persistenceBean.setReferenceUmr(referenceUmr);
    }

    /**
     * Gets the reconnaissance.
     *
     * @return the reconnaissance
     */
    public String getReconnaissance() {
        return persistenceBean.getReconnaissance();
    }

    /**
     * Sets the reconnaissance.
     *
     * @param reconnaissance
     *            the new reconnaissance
     */
    public void setReconnaissance(String reconnaissance) {
        persistenceBean.setReconnaissance(reconnaissance);
    }

    /**
     * Gets the direction scientifique.
     *
     * @return the direction scientifique
     */
    public String getDirectionScientifique() {
        return persistenceBean.getDirectionScientifique();
    }

    /**
     * Sets the direction scientifique.
     *
     * @param directionScientifique
     *            the new direction scientifique
     */
    public void setDirectionScientifique(String directionScientifique) {
        persistenceBean.setDirectionScientifique(directionScientifique);
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementAutres
     *            the new code rattachement autres
     */
    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    /**
     * Gets the rattachement externe.
     *
     * @return the rattachement externe
     */
    public String getRattachementExterne() {
        return persistenceBean.getRattachementExterne();
    }

    /**
     * Sets the rattachement externe.
     *
     * @param rattachementExterne
     *            the new rattachement externe
     */
    public void setRattachementExterne(String rattachementExterne) {
        persistenceBean.setRattachementExterne(rattachementExterne);
    }

    /**
     * Gets the id bandeau.
     *
     * @return the id bandeau
     */
    
    public Long getIdBandeau() {
        return persistenceBean.getIdBandeau();
    }

    /**
     * Sets the id bandeau.
     *
     * @param idBandeau
     *            the new id bandeau
     */
    public void setIdBandeau(Long idBandeau) {
        persistenceBean.setIdBandeau(idBandeau);
    }

    /**
     * Gets the onglets.
     *
     * @return the onglets
     */
    public String getOnglets() {
        return persistenceBean.getOnglets();
    }

    /**
     * Sets the onglets.
     *
     * @param onglets
     *            the new onglets
     */
    
    public void setOnglets(String onglets) {
        persistenceBean.setOnglets(onglets);
    }

    /**
     * Gets the couleur titre.
     *
     * @return the couleur titre
     */
    public String getCouleurTitre() {
        return persistenceBean.getCouleurTitre();
    }

    /**
     * Sets the couleur titre.
     *
     * @param couleurTitre
     *            the new couleur titre
     */
    public void setCouleurTitre(String couleurTitre) {
        persistenceBean.setCouleurTitre(couleurTitre);
    }

    /**
     * Gets the couleur fond.
     *
     * @return the couleur fond
     */
    public String getCouleurFond() {
        return persistenceBean.getCouleurFond();
    }

    /**
     * Sets the couleur fond.
     *
     * @param couleurFond
     *            the new couleur fond
     */
    public void setCouleurFond(String couleurFond) {
        persistenceBean.setCouleurFond(couleurFond);
    }

    /**
     * Gets the adresse.
     *
     * @return the adresse
     */
    public String getAdresse() {
        return persistenceBean.getAdresse();
    }

    /**
     * Sets the adresse.
     *
     * @param adresse
     *            the new adresse
     */
    public void setAdresse(String adresse) {
        persistenceBean.setAdresse(adresse);
    }

    /**
     * Gets the code postal.
     *
     * @return the code postal
     */
    public String getCodePostal() {
        return persistenceBean.getCodePostal();
    }

    /**
     * Sets the code postal.
     *
     * @param codePostal
     *            the new code postal
     */
    public void setCodePostal(String codePostal) {
        persistenceBean.setCodePostal(codePostal);
    }

    /**
     * Gets the ville.
     *
     * @return the ville
     */
    public String getVille() {
        return persistenceBean.getVille();
    }

    /**
     * Sets the ville.
     *
     * @param ville
     *            the new ville
     */
    public void setVille(String ville) {
        persistenceBean.setVille(ville);
    }

    /**
     * Gets the villes.
     *
     * @return the villes
     */
    public String getVilles() {
        return persistenceBean.getVilles();
    }

    /**
     * Sets the villes.
     *
     * @param villes
     *            the new villes
     */
    public void setVilles(String villes) {
        persistenceBean.setVilles(villes);
    }

    /**
     * Gets the telephone.
     *
     * @return the telephone
     */
    public String getTelephone() {
        return persistenceBean.getTelephone();
    }

    /**
     * Sets the telephone.
     *
     * @param telephone
     *            the new telephone
     */
    public void setTelephone(String telephone) {
        persistenceBean.setTelephone(telephone);
    }

    /**
     * Gets the telecopie.
     *
     * @return the telecopie
     */
    public String getTelecopie() {
        return persistenceBean.getTelecopie();
    }

    /**
     * Sets the telecopie.
     *
     * @param telecopie
     *            the new telecopie
     */
    public void setTelecopie(String telecopie) {
        persistenceBean.setTelecopie(telecopie);
    }

    /**
     * Gets the adresse mail.
     *
     * @return the adresse mail
     */
    public String getAdresseMail() {
        return persistenceBean.getAdresseMail();
    }

    /**
     * Sets the adresse mail.
     *
     * @param adresseMail
     *            the new adresse mail
     */
    public void setAdresseMail(String adresseMail) {
        persistenceBean.setAdresseMail(adresseMail);
    }

    /**
     * Gets the site web.
     *
     * @return the site web
     */
    public String getSiteWeb() {
        return persistenceBean.getSiteWeb();
    }

    /**
     * Sets the site web.
     *
     * @param siteWeb
     *            the new site web
     */
    public void setSiteWeb(String siteWeb) {
        persistenceBean.setSiteWeb(siteWeb);
    }

    /**
     * Gets the code responsable.
     *
     * @return the code responsable
     */
    public String getCodeResponsable() {
        return persistenceBean.getCodeResponsable();
    }

    /**
     * Sets the code responsable.
     *
     * @param codeResponsable
     *            the new code responsable
     */
    public void setCodeResponsable(String codeResponsable) {
        persistenceBean.setCodeResponsable(codeResponsable);
    }

    /**
     * Gets the composition.
     *
     * @return the composition
     */
    
    public String getComposition() {
        return persistenceBean.getComposition();
    }

    /**
     * Sets the composition.
     *
     * @param composition
     *            the new composition
     */
    
    public void setComposition(String composition) {
        persistenceBean.setComposition(composition);
    }

    /**
     * Gets the axe recherche etablissement.
     *
     * @return the axe recherche etablissement
     */
    public String getAxeRechercheEtablissement() {
        return persistenceBean.getAxeRechercheEtablissement();
    }

    /**
     * Sets the axe recherche etablissement.
     *
     * @param axeRechercheEtablissement
     *            the new axe recherche etablissement
     */
    public void setAxeRechercheEtablissement(String axeRechercheEtablissement) {
        persistenceBean.setAxeRechercheEtablissement(axeRechercheEtablissement);
    }

    /**
     * Gets the axes recherche.
     *
     * @return the axes recherche
     */
    
    public String getAxesRecherche() {
        return persistenceBean.getAxesRecherche();
    }

    /**
     * Sets the axes recherche.
     *
     * @param axesRecherche
     *            the new axes recherche
     */
    
    public void setAxesRecherche(String axesRecherche) {
        persistenceBean.setAxesRecherche(axesRecherche);
    }

    /**
     * Gets the domaine application.
     *
     * @return the domaine application
     */
    public String getDomaineApplication() {
        return persistenceBean.getDomaineApplication();
    }

    /**
     * Sets the domaine application.
     *
     * @param domaineApplication
     *            the new domaine application
     */
    public void setDomaineApplication(String domaineApplication) {
        persistenceBean.setDomaineApplication(domaineApplication);
    }

    /**
     * Gets the langues parlees.
     *
     * @return the langues parlees
     */
    public String getLanguesParlees() {
        return persistenceBean.getLanguesParlees();
    }

    /**
     * Sets the langues parlees.
     *
     * @param languesParlees
     *            the new langues parlees
     */
    public void setLanguesParlees(String languesParlees) {
        persistenceBean.setLanguesParlees(languesParlees);
    }

    /**
     * Gets the formations.
     *
     * @return the formations
     */
    
    public String getFormations() {
        return persistenceBean.getFormations();
    }

    /**
     * Sets the formations.
     *
     * @param formations
     *            the new formations
     */
    
    public void setFormations(String formations) {
        persistenceBean.setFormations(formations);
    }

    /**
     * Gets the publications.
     *
     * @return the publications
     */
    
    public String getPublications() {
        return persistenceBean.getPublications();
    }

    /**
     * Sets the publications.
     *
     * @param publications
     *            the new publications
     */
    
    public void setPublications(String publications) {
        persistenceBean.setPublications(publications);
    }

    /**
     * Gets the equipements.
     *
     * @return the equipements
     */
    
    public String getEquipements() {
        return persistenceBean.getEquipements();
    }

    /**
     * Sets the equipements.
     *
     * @param equipements
     *            the new equipements
     */
    
    public void setEquipements(String equipements) {
        persistenceBean.setEquipements(equipements);
    }

    /**
     * Gets the cooperations internationales.
     *
     * @return the cooperations internationales
     */
    
    public String getCooperationsInternationales() {
        return persistenceBean.getCooperationsInternationales();
    }

    /**
     * Sets the cooperations internationales.
     *
     * @param cooperationsInternationales
     *            the new cooperations internationales
     */
    
    public void setCooperationsInternationales(String cooperationsInternationales) {
        persistenceBean.setCooperationsInternationales(cooperationsInternationales);
    }

    /**
     * Gets the collaborations universitaires.
     *
     * @return the collaborations universitaires
     */
    public String getCollaborationsUniversitaires() {
        return persistenceBean.getCollaborationsUniversitaires();
    }

    /**
     * Sets the collaborations universitaires.
     *
     * @param collaborationsUniversitaires
     *            the new collaborations universitaires
     */
    public void setCollaborationsUniversitaires(String collaborationsUniversitaires) {
        persistenceBean.setCollaborationsUniversitaires(collaborationsUniversitaires);
    }

    /**
     * Gets the cooperations industrielles.
     *
     * @return the cooperations industrielles
     */
    public String getCooperationsIndustrielles() {
        return persistenceBean.getCooperationsIndustrielles();
    }

    /**
     * Sets the cooperations industrielles.
     *
     * @param cooperationsIndustrielles
     *            the new cooperations industrielles
     */
    public void setCooperationsIndustrielles(String cooperationsIndustrielles) {
        persistenceBean.setCooperationsIndustrielles(cooperationsIndustrielles);
    }

    /**
     * Gets the chiffres cles.
     *
     * @return the chiffres cles
     */
    public String getChiffresCles() {
        return persistenceBean.getChiffresCles();
    }

    /**
     * Sets the chiffres cles.
     *
     * @param chiffresCles
     *            the new chiffres cles
     */
    public void setChiffresCles(String chiffresCles) {
        persistenceBean.setChiffresCles(chiffresCles);
    }

    /**
     * Gets the complement coordonnees.
     *
     * @return the complement coordonnees
     */
    public String getComplementCoordonnees() {
        return persistenceBean.getComplementCoordonnees();
    }

    /**
     * Sets the complement coordonnees.
     *
     * @param complementCoordonnees
     *            the new complement coordonnees
     */
    public void setComplementCoordonnees(String complementCoordonnees) {
        persistenceBean.setComplementCoordonnees(complementCoordonnees);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the id plan acces.
     *
     * @return the id plan acces
     */
    public Long getIdPlanAcces() {
        return persistenceBean.getIdPlanAcces();
    }

    /**
     * Sets the id plan acces.
     *
     * @param idPlanAcces
     *            the new id plan acces
     */
    public void setIdPlanAcces(Long idPlanAcces) {
        persistenceBean.setIdPlanAcces(idPlanAcces);
    }

    /**
     * Gets the id photo.
     *
     * @return the id photo
     */
    public Long getIdPhoto() {
        return persistenceBean.getIdPhoto();
    }

    /**
     * Sets the id photo.
     *
     * @param idPhoto
     *            the new id photo
     */
    public void setIdPhoto(Long idPhoto) {
        persistenceBean.setIdPhoto(idPhoto);
    }

    /**
     * FIXME : correctif pour CORE-1746 à corriger en suppprimant ce genre de cache
     * @param o
     * @throws IOException
     */
    private void writeObject(final ObjectOutputStream o) throws IOException {
        o.writeObject(this.getPersistenceBean());
    }

    /**
     * FIXME : correctif pour CORE-1746 à corriger en suppprimant ce genre de cache
     * @param o
     * @throws IOException
     * @throws ClassNotFoundException
     */
    private void readObject(final ObjectInputStream o) throws IOException, ClassNotFoundException {
        this.persistenceBean = (LaboratoireBean) o.readObject();
    }
}
