/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: offrestageemploi
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.CodeLibelle;
import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.bean.OffresstagesemploisBean;
import com.ksup.objetspartages.service.ServiceOffresstagesemplois;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.DateUtil;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseJoin;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ClauseJoinHelper;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet offresstagesemplois.
 */
@FicheAnnotation()
public class Offresstagesemplois extends AbstractFicheRestriction<OffresstagesemploisBean> {

    public static final String SEQUENCE_LANGUE = ",LANGUE=";

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     * @param codes Liste des codes séparés par un ";". Les codes peuvent être de la forme "12;13" ou bien avec la langue : "1,LANGUE=0;3,LANGUE=0;65,LANGUE=1" 
     * @param langue Langue utilisée pour la recherche des fiches
     * @return La liste des libellés séparés par un ";"
     */
    public static String getLibelleAffichable(final String codes, String langue) {
        ServiceOffresstagesemplois serviceOffresstagesemplois = ServiceManager.getServiceForBean(OffresstagesemploisBean.class);
        if (StringUtils.isEmpty(codes)) {
            return StringUtils.EMPTY;
        }
        return Arrays.stream(codes.split(";"))
            .map(code -> serviceOffresstagesemplois.getFiche(getCodeFiche(code), getLangueFiche(code, langue)))
            .map(Offresstagesemplois::getLibelleAffichable)
            .collect(Collectors.joining(";"));
    }

    /**
     * Renvoie le libelle affichable de la fiche.
     * @param offresstagesemploisBean {@link OffresstagesemploisBean}
     * @return le libelle affichable de la fiche si le bean n'est pas null. Renvoie "-" si le bean est null
     */
    static final String getLibelleAffichable(OffresstagesemploisBean offresstagesemploisBean) {
        if (offresstagesemploisBean == null) {
            return "-";
        }
        ServiceOffresstagesemplois serviceOffresstagesemplois = ServiceManager.getServiceForBean(OffresstagesemploisBean.class);
        return serviceOffresstagesemplois.getLibelleAffichable(offresstagesemploisBean);
    }

    /**
     * Renvoie un code fiche. Si la séquence ",langue=" n'est pas présente alors le code passé en paramètre sera renvoyé. 
     * Dans le cas échéant, la chaîne de caractère présente avant la séquence sera renvoyée.
     * @param code le code en chaîne de caractère.
     * @return le code
     */
    static final String getCodeFiche(final String code) {
        if (StringUtils.contains(code, SEQUENCE_LANGUE)) {
            return StringUtils.substringBefore(code, SEQUENCE_LANGUE);
        }
        return code;
    }

    /**
     * Renvoie la langue . Si la séquence ",langue=" n'est pas présente alors la langue passée en paramèter sera renvoyée 
     * Dans le cas échéant, la chaîne de caractère présente après la séquence sera renvoyée.
     * @param code le code en chaîne de caractère.
     * @param langueFallback la langue renvoyée si le code ne comprend pas la chaîne ",LANGUE="             
     * @return la langue
     */
    static final String getLangueFiche(final String code, final String langueFallback) {
        if (StringUtils.contains(code, SEQUENCE_LANGUE)) {
            return StringUtils.substringAfter(code, SEQUENCE_LANGUE);
        }
        return langueFallback;
    }
    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Offresstagesemplois getFicheOffresstagesemplois(final OMContext ctx, final String code, String langue) throws Exception {
        Offresstagesemplois fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Offresstagesemplois();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setTypeOffre(StringUtils.EMPTY);
        setTypeStage(StringUtils.EMPTY);
        setTypeEmploi(StringUtils.EMPTY);
        setTypeJob(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setSecteursActivites(StringUtils.EMPTY);
        setDescriptif(StringUtils.EMPTY);
        setDuree(StringUtils.EMPTY);
        setCommentaireDuree(StringUtils.EMPTY);
        setDebut(StringUtils.EMPTY);
        setCommentaireDebut(StringUtils.EMPTY);
        setLieu(StringUtils.EMPTY);
        setPays(StringUtils.EMPTY);
        setVilles(StringUtils.EMPTY);
        setRemunerationStage(StringUtils.EMPTY);
        setCommentaireRemunerationStage(StringUtils.EMPTY);
        setUrlDetailOffre("http://");
        setFiliere(StringUtils.EMPTY);
        setNiveauEtude(StringUtils.EMPTY);
        setFormationsRequises(StringUtils.EMPTY);
        setCompetencesRequises(StringUtils.EMPTY);
        setRaisonSociale(StringUtils.EMPTY);
        setActivite(StringUtils.EMPTY);
        setResponsable(StringUtils.EMPTY);
        setCoordonnees(StringUtils.EMPTY);
        setCodePostal(StringUtils.EMPTY);
        setVille(StringUtils.EMPTY);
        setTelephone(StringUtils.EMPTY);
        setTelecopie(StringUtils.EMPTY);
        setEmailResponsable(StringUtils.EMPTY);
        setSiteWeb("http://");
        setActivationReponse(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setTheme(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if(currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie le libelle pour l'attribut secteursActivites.
     */
    public String getLibelleSecteursActivites() {
        return LabelUtils.getLibelle("65", getSecteursActivites(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut duree.
     */
    public String getLibelleDuree() {
        return LabelUtils.getLibelle("32", getDuree(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut debut.
     */
    public String getLibelleDebut() {
        return LabelUtils.getLibelle("33", getDebut(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut pays.
     */
    public String getLibellePays() {
        return FicheUnivHelper.getLibelle(this, "international_pays", getPays());
    }

    /**
     * Renvoie le libelle pour l'attribut filiere.
     */
    public String getLibelleFiliere() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getFiliere(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut filiere.
     */
    public List<StructureModele> getFichesFiliere() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listFiliere = new ArrayList<>();
        final StringTokenizer stFiliere = new StringTokenizer(getFiliere(), ";");
        while (stFiliere.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stFiliere.nextToken(), getLangue());
            if(currentStructure != null) {
                listFiliere.add(currentStructure);
            }
        }
        return listFiliere;
    }

    /**
     * Renvoie le libelle pour l'attribut niveauEtude.
     */
    public String getLibelleNiveauEtude() {
        return FicheUnivHelper.getLibelle(this, "niveau_recrutement", getNiveauEtude());
    }

    /**
     * Renvoie le libelle pour l'attribut typeOffre
     */
    public String getLibelleTypeOffre() {
        return FicheUnivHelper.getLibelle(this, "type_offre", getTypeOffre());
    }

    /**
     * Renvoie le libelle pour l'attribut typeStage
     */
    public String getLibelleTypeStage() {
        return LabelUtils.getLibelle("35", getTypeStage(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut typeJob
     */
    public String getLibelleTypeJob() {
        return LabelUtils.getLibelle("36", getTypeJob(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut typeStage
     */
    public String getLibelleTypeEmploi() {
        return LabelUtils.getLibelle("37", getTypeEmploi(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut theme.
     */
    public String getFormatedTheme() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTheme());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut descriptif.
     */
    public String getFormatedDescriptif() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDescriptif());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut formationsRequises.
     */
    public String getFormatedFormationsRequises() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getFormationsRequises());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut competencesRequises.
     */
    public String getFormatedCompetencesRequises() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCompetencesRequises());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut activite.
     */
    public String getFormatedActivite() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getActivite());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        String typeOffre = RequeteUtil.renvoyerParametre(requete, "TYPE_OFFRE");
        final Hashtable<String, String> listeTypesOffre = CodeLibelle.lireTable(ReferentielObjets.getExtension(this), "type_offre", ContexteUtil.getContexteUniv().getLocale());
        for (String key : listeTypesOffre.keySet()) {
            key = key.toUpperCase();
            if ("1".equals(RequeteUtil.renvoyerParametre(requete, key))) {
                if (typeOffre.length() > 0) {
                    typeOffre += "+";
                }
                typeOffre += key.toLowerCase();
            }
        }
        final String typeEmploi = RequeteUtil.renvoyerParametre(requete, "TYPE_EMPLOI");
        final String typeStage = RequeteUtil.renvoyerParametre(requete, "TYPE_STAGE");
        final String typeJob = RequeteUtil.renvoyerParametre(requete, "TYPE_JOB");
        final String secteursActivites = RequeteUtil.renvoyerParametre(requete, "SECTEURS_ACTIVITES");
        final String duree = RequeteUtil.renvoyerParametre(requete, "DUREE");
        final String debut = RequeteUtil.renvoyerParametre(requete, "DEBUT");
        final String villes = RequeteUtil.renvoyerParametre(requete, "VILLES");
        final String pays = RequeteUtil.renvoyerParametre(requete, "PAYS");
        final String filiere = RequeteUtil.renvoyerParametre(requete, "FILIERE");
        final String niveauEtude = RequeteUtil.renvoyerParametre(requete, "NIVEAU_ETUDE");
        final String remunerationStage = RequeteUtil.renvoyerParametre(requete, "REMUNERATION_STAGE");
        final String raisonSociale = RequeteUtil.renvoyerParametre(requete, "RAISON_SOCIALE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String sDateDebut = RequeteUtil.renvoyerParametre(requete, "DATE_DEBUT");
        final String sDateFin = RequeteUtil.renvoyerParametre(requete, "DATE_FIN");
        final String nbJours = RequeteUtil.renvoyerParametre(requete, "JOUR");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        Date dateDebut = null;
        Date dateFin = null;
        String order = "";
        final Date today = new Date(System.currentTimeMillis());
        if (selection.length() > 0) {
            // Offres du jour
            if (selection.equals("0004")) {
                // date début dans la journée ou date de fin dans la journée
                dateDebut = today; // aujourd'hui
                dateFin = today; // aujourd'hui
                order = "META.META_DATE_MISE_EN_LIGNE";
            }
            // Offres de la semaine
            else if (selection.equals("0005")) {
                // date début dans la semaine ou date de fin dans la semaine
                dateDebut = DateUtil.getFirstDayOfWeek(today);
                dateFin = DateUtil.getLastDayOfWeek(today);
                order = "META.META_DATE_MISE_EN_LIGNE";
            }
            // Offres du mois
            else if (selection.equals("0006")) {
                // date début dans le mois ou date de fin dans le mois
                dateDebut = DateUtil.getFirstDayOfMonth(today);
                dateFin = DateUtil.getLastDayOfMonth(today);
                order = "META.META_DATE_MISE_EN_LIGNE";
            }
            // Dernières offres parues
            else if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        // publiées depuis nbJours jours
        else if (nbJours.length() > 0) {
            dateDebut = DateUtil.addDays(today, -Integer.parseInt(nbJours));
            order = "META.META_DATE_MISE_EN_LIGNE";
        }
        //on ne trie par date que si une date a été renseignée.
        else if (sDateDebut.length() > 0 || sDateFin.length() > 0) {
            dateDebut = DateUtil.parseDate(sDateDebut);
            dateFin = DateUtil.parseDate(sDateFin);
            order = "META.META_DATE_MISE_EN_LIGNE";
        }
        return select(titre, typeOffre, typeEmploi, typeStage, typeJob, dateDebut, dateFin, secteursActivites, duree, debut, villes, pays, filiere, niveauEtude, remunerationStage, raisonSociale, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Offresstagesemplois a partir de l'ensemble des criteres.
     */
    public int select(final String titre, String typeOffre, final String typeEmploi, final String typeStage, final String typeJob, final Date dateDebut, final Date dateFin, final String secteursActivites, final String duree, final String debut, final String villes, final String pays, final String filiere, final String niveauEtude, final String remunerationStage, final String raisonSociale, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        ClauseJoin joinMeta = new ClauseJoin();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (isNotEmpty(titre)) {
            final ConditionList conditionTitre = new ConditionList(ConditionHelper.rechercheMots("T1.TITRE", titre));
            conditionTitre.or(ConditionHelper.rechercheMots("T1.DESCRIPTIF", titre));
            where.and(conditionTitre);
        }
        if (isNotEmpty(typeOffre)) {
            typeOffre = typeOffre.replaceAll("\\+", ";");
            final Collection<String> codesTypeOffre = Arrays.asList(typeOffre.split(";"));
            where.and(ConditionHelper.in("T1.TYPE_OFFRE", codesTypeOffre));
        }
        if (isNotEmpty(typeEmploi) && !"0000".equals(typeEmploi)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_EMPLOI", typeEmploi));
        }
        if (isNotEmpty(typeStage) && !"0000".equals(typeStage)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_STAGE", typeStage));
        }
        if (isNotEmpty(typeJob) && !"0000".equals(typeJob)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_JOB", typeJob));
        }
        if (Formateur.estSaisie(dateDebut)) {
            if (Formateur.estSaisie(dateFin)) {
                where.and(ConditionHelper.greaterEquals("T1.DATE_LIMITE_OFFRE", dateDebut, TypeOperande.DATE));
                where.and(ConditionHelper.lessEquals("META.META_DATE_MISE_EN_LIGNE", dateFin, TypeOperande.DATE));
            } else {
                where.and(ConditionHelper.greaterEquals("META.META_DATE_MISE_EN_LIGNE", dateDebut, TypeOperande.DATE));
                where.and(ConditionHelper.greaterEquals("T1.DATE_LIMITE_OFFRE", new Date(System.currentTimeMillis()), TypeOperande.DATE));
            }
            joinMeta = ClauseJoinHelper.creerJointureMetaTag(this);
        }
        if (isNotEmpty(secteursActivites) && !"0000".equals(secteursActivites)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.SECTEURS_ACTIVITES", secteursActivites));
        }
        if (isNotEmpty(duree) && !"0000".equals(duree)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DUREE", duree));
        }
        if (isNotEmpty(debut) && !"0000".equals(debut)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DEBUT", debut));
        }
        if (isNotEmpty(villes)) {
            where.and(ConditionHelper.rechercheMots("T1.VILLES", villes));
        }
        if (isNotEmpty(pays) && !"0000".equals(pays)) {
            where.and(ConditionHelper.egalVarchar("T1.PAYS", pays));
        }
        if (isNotEmpty(filiere) && !"0000".equals(filiere)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.FILIERE", filiere));
        }
        if (isNotEmpty(niveauEtude) && !"0000".equals(niveauEtude)) {
            where.and(ConditionHelper.egalVarchar("T1.NIVEAU_ETUDE", niveauEtude));
        }
        if (isNotEmpty(remunerationStage)) {
            where.and(ConditionHelper.egalVarchar("T1.REMUNERATION_STAGE", remunerationStage));
        }
        if (isNotEmpty(raisonSociale)) {
            where.and(ConditionHelper.rechercheMots("T1.RAISON_SOCIALE", raisonSociale));
        }
        if (isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
        }
        requeteSelect.join(joinMeta).orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    public Long getIdOffresstagesemplois(){
        return persistenceBean.getId();
    }

    public void setIdOffresstagesemplois(Long id) {
        persistenceBean.setId(id);
    }

    /**
     * Gets the type offre.
     *
     * @return the type offre
     */
    public String getTypeOffre() {
        return persistenceBean.getTypeOffre();
    }

    /**
     * Sets the type offre.
     *
     * @param typeOffre
     *            the new type offre
     */
    public void setTypeOffre(String typeOffre) {
        persistenceBean.setTypeOffre(typeOffre);
    }

    /**
     * Gets the type stage.
     *
     * @return the type stage
     */
    public String getTypeStage() {
        return persistenceBean.getTypeStage();
    }

    /**
     * Sets the type stage.
     *
     * @param typeStage
     *            the new type stage
     */
    public void setTypeStage(String typeStage) {
        persistenceBean.setTypeStage(typeStage);
    }

    /**
     * Gets the type emploi.
     *
     * @return the type emploi
     */
    public String getTypeEmploi() {
        return persistenceBean.getTypeEmploi();
    }

    /**
     * Sets the type emploi.
     *
     * @param typeEmploi
     *            the new type emploi
     */
    public void setTypeEmploi(String typeEmploi) {
        persistenceBean.setTypeEmploi(typeEmploi);
    }

    /**
     * Gets the type job.
     *
     * @return the type job
     */
    public String getTypeJob() {
        return persistenceBean.getTypeJob();
    }

    /**
     * Sets the type job.
     *
     * @param typeJob the new type job
     */
    public void setTypeJob(String typeJob) {
        persistenceBean.setTypeJob(typeJob);
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementAutres the new code rattachement autres
     */
    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    /**
     * Gets the secteurs activites.
     *
     * @return the secteurs activites
     */
    public String getSecteursActivites() {
        return persistenceBean.getSecteursActivites();
    }

    /**
     * Sets the secteurs activites.
     *
     * @param secteursActivites the new secteurs activites
     */
    public void setSecteursActivites(String secteursActivites) {
        persistenceBean.setSecteursActivites(secteursActivites);
    }

    /**
     * Gets the descriptif.
     *
     * @return the descriptif
     */
    @GetterAnnotation(isToolbox = true)
    public String getDescriptif() {
        return persistenceBean.getDescriptif();
    }

    /**
     * Sets the descriptif.
     *
     * @param descriptif
     *            the new descriptif
     */
    public void setDescriptif(String descriptif) {
        persistenceBean.setDescriptif(descriptif);
    }

    /**
     * Gets the duree.
     *
     * @return the duree
     */
    public String getDuree() {
        return persistenceBean.getDuree();
    }

    /**
     * Sets the duree.
     *
     * @param duree the new duree
     */
    public void setDuree(String duree) {
        persistenceBean.setDuree(duree);
    }

    /**
     * Gets the commentaire duree.
     *
     * @return the commentaire duree
     */
    public String getCommentaireDuree() {
        return persistenceBean.getCommentaireDuree();
    }

    /**
     * Sets the commentaire duree.
     *
     * @param commentaireDuree
     *            the new commentaire duree
     */
    public void setCommentaireDuree(String commentaireDuree) {
        persistenceBean.setCommentaireDuree(commentaireDuree);
    }

    /**
     * Gets the debut.
     *
     * @return the debut
     */
    public String getDebut() {
        return persistenceBean.getDebut();
    }

    /**
     * Sets the debut.
     *
     * @param debut the new debut
     */
    public void setDebut(String debut) {
        persistenceBean.setDebut(debut);
    }

    /**
     * Gets the commentaire debut.
     *
     * @return the commentaire debut
     */
    public String getCommentaireDebut() {
        return persistenceBean.getCommentaireDebut();
    }

    /**
     * Sets the commentaire debut.
     *
     * @param commentaireDebut
     *            the new commentaire debut
     */
    public void setCommentaireDebut(String commentaireDebut) {
        persistenceBean.setCommentaireDebut(commentaireDebut);
    }

    /**
     * Gets the lieu.
     *
     * @return the lieu
     */
    public String getLieu() {
        return persistenceBean.getLieu();
    }

    /**
     * Sets the lieu.
     *
     * @param lieu
     *            the new lieu
     */
    public void setLieu(String lieu) {
        persistenceBean.setLieu(lieu);
    }

    /**
     * Gets the pays.
     *
     * @return the pays
     */
    public String getPays() {
        return persistenceBean.getPays();
    }

    /**
     * Sets the pays.
     *
     * @param pays
     *            the new pays
     */
    public void setPays(String pays) {
        persistenceBean.setPays(pays);
    }

    /**
     * Gets the villes.
     *
     * @return the villes
     */
    public String getVilles() {
        return persistenceBean.getVilles();
    }

    /**
     * Sets the villes.
     *
     * @param villes the new villes
     */
    public void setVilles(String villes) {
        persistenceBean.setVilles(villes);
    }

    /**
     * Gets the remuneration stage.
     *
     * @return the remuneration stage
     */
    public String getRemunerationStage() {
        return persistenceBean.getRemunerationStage();
    }

    /**
     * Sets the remuneration stage.
     *
     * @param remunerationStage
     *            the new remuneration stage
     */
    public void setRemunerationStage(String remunerationStage) {
        persistenceBean.setRemunerationStage(remunerationStage);
    }

    /**
     * Gets the commentaire remuneration stage.
     *
     * @return the commentaire remuneration stage
     */
    public String getCommentaireRemunerationStage() {
        return persistenceBean.getCommentaireRemunerationStage();
    }

    /**
     * Sets the commentaire remuneration stage.
     *
     * @param commentaireRemunerationStage
     *            the new commentaire remuneration stage
     */
    public void setCommentaireRemunerationStage(String commentaireRemunerationStage) {
        persistenceBean.setCommentaireRemunerationStage(commentaireRemunerationStage);
    }

    /**
     * Gets the date limite offre.
     *
     * @return the date limite offre
     */
    public Date getDateLimiteOffre() {
        return persistenceBean.getDateLimiteOffre();
    }

    /**
     * Sets the date limite offre.
     *
     * @param dateLimiteOffre
     *            the new date limite offre
     */
    public void setDateLimiteOffre(Date dateLimiteOffre) {
        persistenceBean.setDateLimiteOffre(dateLimiteOffre);
    }

    /**
     * Gets the url detail offre.
     *
     * @return the url detail offre
     */
    public String getUrlDetailOffre() {
        return persistenceBean.getUrlDetailOffre();
    }

    /**
     * Sets the url detail offre.
     *
     * @param urlDetailOffre the new url detail offre
     */
    public void setUrlDetailOffre(String urlDetailOffre) {
        persistenceBean.setUrlDetailOffre(urlDetailOffre);
    }

    /**
     * Gets the filiere.
     *
     * @return the filiere
     */
    public String getFiliere() {
        return persistenceBean.getFiliere();
    }

    /**
     * Sets the filiere.
     *
     * @param filiere the new filiere
     */
    public void setFiliere(String filiere) {
        persistenceBean.setFiliere(filiere);
    }

    /**
     * Gets the niveau etude.
     *
     * @return the niveau etude
     */
    public String getNiveauEtude() {
        return persistenceBean.getNiveauEtude();
    }

    /**
     * Sets the niveau etude.
     *
     * @param niveauEtude
     *            the new niveau etude
     */
    public void setNiveauEtude(String niveauEtude) {
        persistenceBean.setNiveauEtude(niveauEtude);
    }

    /**
     * Gets the formations requises.
     *
     * @return the formations requises
     */
    @GetterAnnotation(isToolbox = true)
    public String getFormationsRequises() {
        return persistenceBean.getFormationsRequises();
    }

    /**
     * Sets the formations requises.
     *
     * @param formationsRequises
     *            the new formations requises
     */
    public void setFormationsRequises(String formationsRequises) {
        persistenceBean.setFormationsRequises(formationsRequises);
    }

    /**
     * Gets the competences requises.
     *
     * @return the competences requises
     */
    @GetterAnnotation(isToolbox = true)
    public String getCompetencesRequises() {
        return persistenceBean.getCompetencesRequises();
    }

    /**
     * Sets the competences requises.
     *
     * @param competencesRequises
     *            the new competences requises
     */
    public void setCompetencesRequises(String competencesRequises) {
        persistenceBean.setCompetencesRequises(competencesRequises);
    }

    /**
     * Gets the raison sociale.
     *
     * @return the raison sociale
     */
    public String getRaisonSociale() {
        return persistenceBean.getRaisonSociale();
    }

    /**
     * Sets the raison sociale.
     *
     * @param raisonSociale
     *            the new raison sociale
     */
    public void setRaisonSociale(String raisonSociale) {
        persistenceBean.setRaisonSociale(raisonSociale);
    }

    /**
     * Gets the activite.
     *
     * @return the activite
     */
    @GetterAnnotation(isToolbox = true)
    public String getActivite() {
        return persistenceBean.getActivite();
    }

    /**
     * Sets the activite.
     *
     * @param activite the new activite
     */
    public void setActivite(String activite) {
        persistenceBean.setActivite(activite);
    }

    /**
     * Gets the responsable.
     *
     * @return the responsable
     */
    public String getResponsable() {
        return persistenceBean.getResponsable();
    }

    /**
     * Sets the responsable.
     *
     * @param responsable
     *            the new responsable
     */
    public void setResponsable(String responsable) {
        persistenceBean.setResponsable(responsable);
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    public String getCoordonnees() {
        return persistenceBean.getCoordonnees();
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    public void setCoordonnees(String coordonnees) {
        persistenceBean.setCoordonnees(coordonnees);
    }

    /**
     * Gets the code postal.
     *
     * @return the code postal
     */
    public String getCodePostal() {
        return persistenceBean.getCodePostal();
    }

    /**
     * Sets the code postal.
     *
     * @param codePostal the new code postal
     */
    public void setCodePostal(String codePostal) {
        persistenceBean.setCodePostal(codePostal);
    }

    /**
     * Gets the ville.
     *
     * @return the ville
     */
    public String getVille() {
        return persistenceBean.getVille();
    }

    /**
     * Sets the ville.
     *
     * @param ville the new ville
     */
    public void setVille(String ville) {
        persistenceBean.setVille(ville);
    }

    /**
     * Gets the telephone.
     *
     * @return the telephone
     */
    public String getTelephone() {
        return persistenceBean.getTelephone();
    }

    /**
     * Sets the telephone.
     *
     * @param telephone
     *            the new telephone
     */
    public void setTelephone(String telephone) {
        persistenceBean.setTelephone(telephone);
    }

    /**
     * Gets the telecopie.
     *
     * @return the telecopie
     */
    public String getTelecopie() {
        return persistenceBean.getTelecopie();
    }

    /**
     * Sets the telecopie.
     *
     * @param telecopie
     *            the new telecopie
     */
    public void setTelecopie(String telecopie) {
        persistenceBean.setTelecopie(telecopie);
    }

    /**
     * Gets the email responsable.
     *
     * @return the email responsable
     */
    public String getEmailResponsable() {
        return persistenceBean.getEmailResponsable();
    }

    /**
     * Sets the email responsable.
     *
     * @param emailResponsable
     *            the new email responsable
     */
    public void setEmailResponsable(String emailResponsable) {
        persistenceBean.setEmailResponsable(emailResponsable);
    }

    /**
     * Gets the site web.
     *
     * @return the site web
     */
    public String getSiteWeb() {
        return persistenceBean.getSiteWeb();
    }

    /**
     * Sets the site web.
     *
     * @param siteWeb
     *            the new site web
     */
    public void setSiteWeb(String siteWeb) {
        persistenceBean.setSiteWeb(siteWeb);
    }

    /**
     * Gets the activation reponse.
     *
     * @return the activation reponse
     */
    public String getActivationReponse() {
        return persistenceBean.getActivationReponse();
    }

    /**
     * Sets the activation reponse.
     *
     * @param activationReponse the new activation reponse
     */
    public void setActivationReponse(String activationReponse) {
        persistenceBean.setActivationReponse(activationReponse);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the theme.
     *
     * @return the theme
     */
    public String getTheme() {
        return persistenceBean.getTheme();
    }

    /**
     * Sets the theme.
     *
     * @param theme
     *            the new theme
     */
    public void setTheme(String theme) {
        persistenceBean.setTheme(theme);
    }

    /**
     * Gets the date suppression.
     *
     * @return the date suppression
     */
    public Date getDateSuppression() {
        return persistenceBean.getDateSuppression();
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the new date suppression
     */
    public void setDateSuppression(Date dateSuppression) {
        persistenceBean.setDateSuppression(dateSuppression);
    }
    
}
