/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ParcoursLink;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.service.ServiceParcours;
import com.ksup.objetspartages.service.ServiceUeup;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

/**
 * Classe représentant un objet parcours.
 */
@FicheAnnotation(isFicheFrontOffice = false, isEncadreRechercheEmbarquable = false, isIndexable = false, isLienRequete = false, isSaisieFrontOffice = false, isLienInterne = false)
public class Parcours extends AbstractFicheRestriction<ParcoursBean> implements FicheRattachementsSecondaires {

    private static final Logger LOG = LoggerFactory.getLogger(Parcours.class);

    /**
     * Constructeur vide.
     */
    public Parcours() {
        super();
    }

    /**
     * Constructeur à partir d'un bean.
     * @param parcoursBean le bean
     */
    public Parcours(ParcoursBean parcoursBean) {
        this();
        persistenceBean = parcoursBean;
    }

    /**
     * Renvoie le libellé à afficher (methode statique utilisée pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (StringUtils.isEmpty(codes)) {
            return res;
        }
        Parcours parcours = new Parcours();
        parcours.init();
        if (StringUtils.isEmpty(langue)) {
            langue = "0";
        }
        StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            String code = st.nextToken();
            final String[] values = code.split(",");
            // CODE,LANGUE=0
            if (values.length == 2) {
                code = values[0];
                langue = values[1].substring(7);
            }
            // CODE,LANGUE,TITRE
            if (values.length == 3) {
                code = values[0];
                langue = values[1];
            }
            // On cherche d'abord la version en ligne puis les autres versions
            int count = parcours.selectCodeLangueEtat(values[0], langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) {
                count = parcours.selectCodeLangueEtat(values[0], langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (parcours.nextItem()) {
                    res += parcours.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandée (méthode statique utilisée pour les jointures entre fiches).
     */
    public static Parcours getFicheParcours(String code, String langue) throws Exception {
        Parcours fiche = null;
        if (StringUtils.isNotEmpty(code)) {
            if (StringUtils.isEmpty(langue)) {
                langue = "0"; // francais par defaut
            }
            fiche = new Parcours();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) { // si pas de version en ligne
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) { // si pas de version dans la langue demandée
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.toString());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    private ServiceParcours getServiceParcours() {
        return ServiceManager.getServiceForBean(ParcoursBean.class);
    }

    @Override
    public void add() {
        getServiceParcours().add(this.getPersistenceBean());
    }

    @Override
    public void addWithForcedId() throws Exception {
        getServiceParcours().addWithForcedId(getPersistenceBean());
    }

    @Override
    public void update() {
        getServiceParcours().update(getPersistenceBean());
    }

    @Override
    public void delete() throws Exception {
        getServiceParcours().delete(getIdParcours());
    }

    /**
     * Initialise l'objet metier.
     */
    public void init() {
        super.init();
        setCodeRattachementAutres(StringUtils.EMPTY);
        setCodeSiScolarite(StringUtils.EMPTY);
        setIntituleCourt(StringUtils.EMPTY);
        setIntituleLong(StringUtils.EMPTY);
        setParcoursFictif(StringUtils.EMPTY);
        setDiplomantEtape(StringUtils.EMPTY);
        setNiveauRecrutement(StringUtils.EMPTY);
        setNiveauDuDiplome(StringUtils.EMPTY);
        setAccesSousCondition(StringUtils.EMPTY);
        setTempsPlein(StringUtils.EMPTY);
        setCompatibleActivitePro(StringUtils.EMPTY);
        setTempsAmenage(StringUtils.EMPTY);
        setAlternance(StringUtils.EMPTY);
        setHorsContratAlternance(StringUtils.EMPTY);
        setContratApprentissage(StringUtils.EMPTY);
        setContratPro(StringUtils.EMPTY);
        setUeups(StringUtils.EMPTY);
        setPresentation(StringUtils.EMPTY);
        setSecteur(StringUtils.EMPTY);
        setParcoursAmont(StringUtils.EMPTY);
        setParcoursAval(StringUtils.EMPTY);
        setLibelleParcoursSemestre(StringUtils.EMPTY);
        setDureeSemestre(0);
        setIntituleParcoursDiplomant(StringUtils.EMPTY);
        setParcoursAmontBeans(null);
        setParcoursAvalBeans(null);
        setUeupsBeans(null);
        setNumeroPremierSemestre(0);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Renvoie le libellé pour l'attribut niveauRecrutement.
     */
    public String getLibelleNiveauRecrutement() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getNiveauRecrutement(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut niveauDuDiplome.
     */
    public String getLibelleNiveauDuDiplome() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), getNiveauDuDiplome(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut libelleParcoursSemestre.
     */
    public String getLibelleLibelleParcoursSemestre() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), getLibelleParcoursSemestre(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut annee.
     */
    public String getLibelleAnnee() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.ANNEE.getValeur(), getAnnee(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut semestre1.
     */
    public String getLibelleUeups() throws Exception {
        return Ueup.getLibelleAffichable(getUeups(), getLangue());
    }

    /**
     * Renvoie le contenu formaté en HTML pour l'attribut presentation.
     */
    public String getFormatedPresentation() {
        try {
            return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPresentation());
        } catch (Exception e) {
            LOG.error("Une exception lors du formatage de la toolbox Presentation", e);
            return StringUtils.EMPTY;
        }
    }

    /**
     * Renvoie le libellé pour l'attribut secteur.
     */
    public String getLibelleSecteur() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.SECTEUR.getValeur(), getSecteur(), getLangue());
    }

    /**
     * Renvoie le libellé pour l'attribut parcoursAmont.
     */
    public String getLibelleParcoursAmont() throws Exception {
        return Parcours.getLibelleAffichable(getParcoursAmont(), getLangue());
    }

    /**
     * Renvoie la liste des fiches Parcours correspondant à l'attribut parcoursAmont.
     */
    public List<Parcours> getFichesParcoursAmont() throws Exception {
        ArrayList<Parcours> listparcoursAmont = new ArrayList<Parcours>();
        StringTokenizer stparcoursAmont = new StringTokenizer(getParcoursAmont(), ";");
        while (stparcoursAmont.hasMoreTokens()) {
            listparcoursAmont.add(Parcours.getFicheParcours(StringUtils.split(stparcoursAmont.nextToken(), ",")[0], getLangue()));
        }
        return listparcoursAmont;
    }

    /**
     * Renvoie le libellé pour l'attribut parcoursAval.
     */
    public String getLibelleParcoursAval() throws Exception {
        return Parcours.getLibelleAffichable(getParcoursAval(), getLangue());
    }

    /**
     * Renvoie la liste des fiches Parcours correspondant à l'attribut parcoursAval.
     */
    public List<Parcours> getFichesParcoursAval() throws Exception {
        ArrayList<Parcours> listparcoursAval = new ArrayList<Parcours>();
        if (StringUtils.isNotEmpty(getParcoursAval())) {
            StringTokenizer stparcoursAval = new StringTokenizer(getParcoursAval(), ";");
            while (stparcoursAval.hasMoreTokens()) {
                listparcoursAval.add(Parcours.getFicheParcours(StringUtils.split(stparcoursAval.nextToken(), ",")[0], getLangue()));
            }
        }
        return listparcoursAval;
    }

    /**
     * Renvoie le libellé pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @return the infos structure
     * @throws Exception the exception
     */
    public StructureModele getInfosStructure() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les métadonnées de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Renvoie une chaine correspondant à la concaténation des champs text de l'objet
     * (utilisé pour l'indexation Lucene).
     */
    public String getFullTextString() throws Exception {
        String s = "" + " " + getCodeSiScolarite() + " " + getIntituleCourt() + " " + getIntituleLong() + " " + getLibelleNiveauRecrutement().replaceAll(";", " ") + " " + getLibelleNiveauDuDiplome() + " " + getLibelleAnnee() + " " + getPresentation() + " " + getLibelleSecteur() + " " + getLibelleParcoursSemestre() + " " + getDureeSemestre();
        return s;
    }

    /**
     * Traitement d'une requête sur l'objet.
     */
    public int traiterRequete(String requete) throws Exception {
        // Recuperation des paramètres de la requête
        String codeRattachementAutres = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT_AUTRES");
        String codeSiScolarite = RequeteUtil.renvoyerParametre(requete, "CODE_SI_SCOLARITE");
        String intituleCourt = RequeteUtil.renvoyerParametre(requete, "INTITULE_COURT");
        String intituleLong = RequeteUtil.renvoyerParametre(requete, "INTITULE_LONG");
        String intituleParcoursDiplomant = RequeteUtil.renvoyerParametre(requete, "INTITULE_PARCOURS_DIPLOMANT");
        String parcoursFictif = RequeteUtil.renvoyerParametre(requete, "PARCOURS_FICTIF");
        String diplomantEtape = RequeteUtil.renvoyerParametre(requete, "DIPLOMANT_ETAPE");
        String niveauRecrutement = RequeteUtil.renvoyerParametre(requete, "NIVEAU_RECRUTEMENT");
        String niveauDuDiplome = RequeteUtil.renvoyerParametre(requete, "NIVEAU_DU_DIPLOME");
        String accesSousCondition = RequeteUtil.renvoyerParametre(requete, "ACCES_SOUS_CONDITION");
        String annee = RequeteUtil.renvoyerParametre(requete, "ANNEE");
        String tempsPlein = RequeteUtil.renvoyerParametre(requete, "TEMPS_PLEIN");
        String compatibleActivitePro = RequeteUtil.renvoyerParametre(requete, "COMPATIBLE_ACTIVITE_PRO");
        String tempsAmenage = RequeteUtil.renvoyerParametre(requete, "TEMPS_AMENAGE");
        String alternance = RequeteUtil.renvoyerParametre(requete, "ALTERNANCE");
        String partiellementDistance = RequeteUtil.renvoyerParametre(requete, "PARTIELLEMENT_DISTANCE");
        String totalementDistance = RequeteUtil.renvoyerParametre(requete, "TOTALEMENT_DISTANCE");
        String semestre1 = RequeteUtil.renvoyerParametre(requete, "UEUPS");
        String secteur = RequeteUtil.renvoyerParametre(requete, "SECTEUR");
        String parcoursAmont = RequeteUtil.renvoyerParametre(requete, "PARCOURS_AMONT");
        String parcoursAval = RequeteUtil.renvoyerParametre(requete, "PARCOURS_AVAL");
        String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        return select(codeRattachementAutres, codeSiScolarite, intituleCourt, intituleLong, intituleParcoursDiplomant, parcoursFictif, diplomantEtape, niveauRecrutement, niveauDuDiplome, accesSousCondition, tempsPlein, compatibleActivitePro, tempsAmenage, alternance, partiellementDistance, totalementDistance, annee, semestre1, secteur, parcoursAmont, parcoursAval, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, nombre, StringUtils.EMPTY);
    }

    public int select(String codeRattachementAutres, String codeSiScolarite, String intituleCourt, String intituleLong, String intituleParcoursDiplomant, String parcoursFictif, String diplomantEtape, String niveauRecrutement, String niveauDuDiplome, String accesSousCondition, String tempsPlein, String compatibleActivitePro, String tempsAmenage, String alternance, String partiellementDistance, String totalementDistance, String annee, String semestre1, String secteur, String parcoursAmont, String parcoursAval, String codeRubrique, String codeRattachement, String langue, String etatObjet, String codeRedacteur, String code, String nombre, String orderByClause) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        // Traitement du parametre codeSiScolarite
        if (StringUtils.isNotEmpty(codeSiScolarite)) {
            where.and(ConditionHelper.rechercheMots("T1.CODE_SI_SCOLARITE", codeSiScolarite));
        }
        // Traitement du parametre codeRattachementAutres
        if (StringUtils.isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            if (StringUtils.isNotEmpty(codeRattachementAutres) && !"0000".equals(codeRattachementAutres)) {
                conditionStructure.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachementAutres));
            }
            where.and(conditionStructure);
        } else if (StringUtils.isNotEmpty(codeRattachementAutres) && !"0000".equals(codeRattachementAutres)) {
            //Au cas où CODE_RATTACHEMENT pas là mais CODE_RATTACHEMENT_AUTRES oui
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachementAutres));
            where.and(conditionStructure);
        }
        // Traitement du parametre intituleCourt
        if (StringUtils.isNotEmpty(intituleCourt)) {
            where.and(ConditionHelper.rechercheMots("T1.INTITULE_COURT", intituleCourt));
        }
        // Traitement du parametre intituleLong
        if (StringUtils.isNotEmpty(intituleLong)) {
            where.and(ConditionHelper.rechercheMots("T1.INTITULE_LONG", intituleLong));
        }
        // Traitement du parametre intituleDiplomant
        if (StringUtils.isNotEmpty(intituleParcoursDiplomant)) {
            where.and(ConditionHelper.rechercheMots("T1.INTITULE_PARCOURS_DIPLOMANT", intituleParcoursDiplomant));
        }
        // Traitement du parametre parcoursFictif
        if (parcoursFictif != null && parcoursFictif.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.PARCOURS_FICTIF", parcoursFictif));
        }
        // Traitement du parametre diplomantEtape
        if (diplomantEtape != null && diplomantEtape.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.DIPLOMANT_ETAPE", diplomantEtape));
        }
        // Traitement du parametre niveauRecrutement
        if (StringUtils.isNotEmpty(niveauRecrutement) && !niveauRecrutement.equals("0000")) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.NIVEAU_RECRUTEMENT", niveauRecrutement));
        }
        // Traitement du parametre niveauDuDiplome
        if (StringUtils.isNotEmpty(niveauDuDiplome) && !niveauDuDiplome.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.NIVEAU_DU_DIPLOME", niveauDuDiplome));
        }
        // Traitement du parametre accesSousCondition
        if (StringUtils.isNotEmpty(accesSousCondition)) {
            where.and(ConditionHelper.egalVarchar("T1.ACCES_SOUS_CONDITION", accesSousCondition));
        }
        // Traitement du parametre tempsPlein
        if (tempsPlein != null && tempsPlein.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TEMPS_PLEIN", tempsPlein));
        }
        // Traitement du parametre compatibleActivitePro
        if (compatibleActivitePro != null && compatibleActivitePro.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.COMPATIBLE_ACTIVITE_PRO", compatibleActivitePro));
        }
        // Traitement du parametre tempsAmenage
        if (tempsAmenage != null && tempsAmenage.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TEMPS_AMENAGE", tempsAmenage));
        }
        // Traitement du parametre alternance
        if (alternance != null && alternance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.ALTERNANCE", alternance));
        }
        // Traitement du parametre partiellementDistance
        if (partiellementDistance != null && partiellementDistance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.PARTIELLEMENT_DISTANCE", partiellementDistance));
        }
        // Traitement du parametre totalementDistance
        if (totalementDistance != null && totalementDistance.equals("1")) {
            where.and(ConditionHelper.rechercheMots("T1.TOTALEMENT_DISTANCE", totalementDistance));
        }
        // Traitement du parametre annee
        if (StringUtils.isNotEmpty(annee) && !annee.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.ANNEE", annee));
        }
        // Traitement du parametre semestre1
        if (StringUtils.isNotEmpty(semestre1)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.UEUPS", semestre1));
        }
        // Traitement du parametre secteur
        if (StringUtils.isNotEmpty(secteur) && !secteur.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.SECTEUR", secteur));
        }
        // Traitement du parametre parcoursAmont
        if (StringUtils.isNotEmpty(parcoursAmont)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.PARCOURS_AMONT", parcoursAmont));
        }
        // Traitement du parametre parcoursAval
        if (StringUtils.isNotEmpty(parcoursAval)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.PARCOURS_AVAL", parcoursAval));
        }
        // Traitement du parametre codeRattachement
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        // Traitement du parametre langue
        if (StringUtils.isNotEmpty(langue) && !langue.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        // Traitement du parametre etatObjet
        if (StringUtils.isNotEmpty(etatObjet) && !etatObjet.equals("0000")) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        // Traitement du parametre codeRedacteur
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        // Traitement du parametre code
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        // Ajout des criteres generiques
        final RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotBlank(orderByClause)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(orderByClause);
        } else {
            orderBy.orderBy("T1.CODE_SI_SCOLARITE", ClauseOrderBy.SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        // Gestion du nombre de resultats
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Renvoie la liste des références à cette fiche.
     */
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    public void setIdParcours(Long idParcours) {
        persistenceBean.setId(idParcours);
    }

    public Long getIdParcours() {
        return persistenceBean.getId();
    }

    public void setCodeSiScolarite(String codeSiScolarite) {
        persistenceBean.setCodeSiScolarite(codeSiScolarite);
    }

    public void setDureeSemestre(Integer dureeSemestre) {
        persistenceBean.setDureeSemestre(dureeSemestre);
    }

    public String getCodeSiScolarite() {
        return persistenceBean.getCodeSiScolarite();
    }

    public Integer getDureeSemestre() {
        return persistenceBean.getDureeSemestre();
    }

    public void setIntituleCourt(String intituleCourt) {
        persistenceBean.setIntituleCourt(intituleCourt);
    }

    public String getIntituleCourt() {
        return persistenceBean.getIntituleCourt();
    }

    public void setIntituleLong(String intituleLong) {
        persistenceBean.setIntituleLong(intituleLong);
    }

    public String getIntituleLong() {
        return persistenceBean.getIntituleLong();
    }

    public void setParcoursFictif(String parcoursFictif) {
        persistenceBean.setParcoursFictif(parcoursFictif);
    }

    public String getParcoursFictif() {
        return persistenceBean.getParcoursFictif();
    }

    public void setDiplomantEtape(String diplomantEtape) {
        persistenceBean.setDiplomantEtape(diplomantEtape);
    }

    public String getDiplomantEtape() {
        return persistenceBean.getDiplomantEtape();
    }

    public boolean isDiplomantEtape() {
        return "1".equals(getDiplomantEtape());
    }

    public void setNiveauRecrutement(String niveauRecrutement) {
        persistenceBean.setNiveauRecrutement(niveauRecrutement);
    }

    public String getNiveauRecrutement() {
        return persistenceBean.getNiveauRecrutement();
    }

    public void setNiveauDuDiplome(String niveauDuDiplome) {
        persistenceBean.setNiveauDuDiplome(niveauDuDiplome);
    }

    public String getNiveauDuDiplome() {
        return persistenceBean.getNiveauDuDiplome();
    }

    public void setAccesSousCondition(String accesSousCondition) {
        persistenceBean.setAccesSousCondition(accesSousCondition);
    }

    public String getAccesSousCondition() {
        return persistenceBean.getAccesSousCondition();
    }

    public void setTempsPlein(String tempsPlein) {
        persistenceBean.setTempsPlein(tempsPlein);
    }

    public String getTempsPlein() {
        return persistenceBean.getTempsPlein();
    }

    public boolean isTempsPlein() {
        return "1".equals(persistenceBean.getTempsPlein());
    }

    public void setCompatibleActivitePro(String compatibleActivitePro) {
        persistenceBean.setCompatibleActivitePro(compatibleActivitePro);
    }

    public String getCompatibleActivitePro() {
        return persistenceBean.getCompatibleActivitePro();
    }

    public boolean isCompatibleActivitePro() {
        return "1".equals(persistenceBean.getCompatibleActivitePro());
    }

    public void setTempsAmenage(String tempsAmenage) {
        persistenceBean.setTempsAmenage(tempsAmenage);
    }

    public String getTempsAmenage() {
        return persistenceBean.getTempsAmenage();
    }

    public boolean isTempsAmenage() {
        return "1".equals(persistenceBean.getTempsAmenage());
    }

    public void setAlternance(String alternance) {
        persistenceBean.setAlternance(alternance);
    }

    public String getAlternance() {
        return persistenceBean.getAlternance();
    }

    public boolean isAlternance() {
        return "1".equals(persistenceBean.getAlternance());
    }

    public boolean isFormationAlternance() {
        return "1".equals(persistenceBean.getAlternance());
    }

    public void setHorsContratAlternance(String horsContratAlternance) {
        persistenceBean.setHorsContratAlternance(horsContratAlternance);
    }

    public String getHorsContratAlternance() {
        return persistenceBean.getHorsContratAlternance();
    }

    public boolean isHorsContratAlternance() {
        return "1".equals(persistenceBean.getHorsContratAlternance());
    }

    public void setContratApprentissage(String contratApprentissage) {
        persistenceBean.setContratApprentissage(contratApprentissage);
    }

    public String getContratApprentissage() {
        return persistenceBean.getContratApprentissage();
    }

    public boolean isContratApprentissage() {
        return "1".equals(persistenceBean.getContratApprentissage());
    }

    public void setContratPro(String contratPro) {
        persistenceBean.setContratPro(contratPro);
    }

    public String getContratPro() {
        return persistenceBean.getContratPro();
    }

    public boolean isContratPro() {
        return "1".equals(persistenceBean.getContratPro());
    }

    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    public String getLibelleParcoursSemestre() {
        return persistenceBean.getLibelleParcoursSemestre();
    }

    public void setAnnee(String annee) {
        persistenceBean.setAnnee(annee);
    }

    public String getAnnee() {
        return persistenceBean.getAnnee();
    }

    public void setUeups(String ueups) {
        persistenceBean.setUeups(ueups);
    }

    public void setUeupsBeans(final List<UeupBean> ueups) {
        persistenceBean.setUeupsBeans(ueups);
    }

    public String getUeups() {
        return persistenceBean.getUeups();
    }

    public void setPresentation(String presentation) {
        persistenceBean.setPresentation(presentation);
    }

    public String getPresentation() {
        return persistenceBean.getPresentation();
    }

    public void setSecteur(String secteur) {
        persistenceBean.setSecteur(secteur);
    }

    public String getSecteur() {
        return persistenceBean.getSecteur();
    }

    public void setParcoursAmont(String parcoursAmont) {
        persistenceBean.setParcoursAmont(parcoursAmont);
    }

    public void setParcoursAmontBeans(final List<ParcoursLink> parcoursAmont) {
        persistenceBean.setParcoursAmontBeans(parcoursAmont);
    }

    public String getParcoursAmont() {
        return persistenceBean.getParcoursAmont();
    }

    public void setParcoursAval(String parcoursAval) {
        persistenceBean.setParcoursAval(parcoursAval);
    }

    public void setParcoursAvalBeans(final List<ParcoursLink> parcoursAval) {
        persistenceBean.setParcoursAvalBeans(parcoursAval);
    }

    public String getParcoursAval() {
        return persistenceBean.getParcoursAval();
    }

    public void setLibelleParcoursSemestre(String libelleParcoursSemestre) {
        persistenceBean.setLibelleParcoursSemestre(libelleParcoursSemestre);
    }

    public String getIntituleParcoursDiplomant() {
        return this.persistenceBean.getIntituleParcoursDiplomant();
    }

    public void setIntituleParcoursDiplomant(String intituleParcoursDiplomant) {
        this.persistenceBean.setIntituleParcoursDiplomant(intituleParcoursDiplomant);
    }

    public Integer getNumeroPremierSemestre() {
        return this.persistenceBean.getNumeroPremierSemestre();
    }

    public void setNumeroPremierSemestre(Integer numeroPremierSemestre) {
        this.persistenceBean.setNumeroPremierSemestre(numeroPremierSemestre);
    }

    public List<Ueup> getFicheUeups() {
        List<Ueup> listsemestre1 = new ArrayList<>();
        ServiceUeup serviceUeup = ServiceManager.getServiceForBean(UeupBean.class);
        if(CollectionUtils.isNotEmpty(persistenceBean.getUeupsBeans())) {
            for (UeupBean ueupBean : persistenceBean.getUeupsBeans()) {
                Ueup ueup = new Ueup();
                UeupBean databaseUeupBean = serviceUeup.getFiche(ueupBean.getCode(), ueupBean.getLangue());
                if (databaseUeupBean != null) {
                    ueup.setPersistenceBean(databaseUeupBean);
                    listsemestre1.add(ueup);
                }
            }
        }
        return listsemestre1;
    }
}




















