/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.univ.services.conversion.ServiceConversionLabelAutoCompletion;
import com.univ.autocomplete.bean.AutoCompleteSuggestion;
import com.univ.autocomplete.bean.AutoCompletionObject;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.MenuSautsUtil;

/**
 * Processus de gestion des fiches Formation.
 */
public class SaisieFormation extends SaisieFiche {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieFormation.class);

    private static final String BUSINESS_SEPARATOR = ";";

    /**
     * The Constant MAPPER.
     */
    private static final ObjectMapper JSON_HELPER = new ObjectMapper();

    static {
        JSON_HELPER.setSerializationInclusion(JsonInclude.Include.NON_DEFAULT);
    }

    /**
     * The serviceLabel.
     */
    private final ServiceLabel serviceLabel;

    /**
     * The serviceConversionLabelAutoCompletion.
     */
    private final ServiceConversionLabelAutoCompletion serviceConversionLabelAutoCompletion;

    protected Formation formation = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieFormation(final InfoBean infoBean) {
        super(infoBean);
        this.serviceLabel = ServiceManager.getService(ServiceLabel.class);
        this.serviceConversionLabelAutoCompletion = ServiceManager.getService(ServiceConversionLabelAutoCompletion.class);
    }

    /**
     * Point d'entrée du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0002");
        formation = new Formation();
        formation.init();
        traiterActionParDefaut(formation);
        return (etat == FIN);
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean codeRattachementAutresFiche = serviceRubrique.getRubriqueByCode((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        if (codeRattachementAutresFiche != null) {
            infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", codeRattachementAutresFiche.getIntitule());
        }
        infoBean.set("LISTE_CATALOGUES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.CATALOGUE.getValeur(), getLocale()));
        infoBean.set("LISTE_NATURE_FORMATIONS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NATURE_FORMATION.getValeur(), getLocale()));
        final Map<String, String> hdiplome = LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), this.getLocale());
        infoBean.set("LISTE_TYPE_DIPLOMES", generateDiplomesMap(hdiplome));
        infoBean.set("LISTE_GRADES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.GRADE.getValeur(), getLocale()));
        infoBean.set("LISTE_DOMAINE_LMDS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), getLocale()));
        infoBean.set("LISTE_MENTIONS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.MENTION.getValeur(), getLocale()));
        infoBean.set("LISTE_PARCOURSS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.PARCOURS.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_RECRUTEMENTS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_ANNEE_POSTBACS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.ANNEE_POSTBAC.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_SORTIES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAUX_SORTIE.getValeur(), getLocale()));
        infoBean.set("LISTE_LIEUXS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.LIEUX.getValeur(), getLocale()));
        infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_DUREE_LISTES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DUREE.getValeur(), getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.SECTEUR_ACTIVITE.getValeur(), getLocale()));
        infoBean.set("LISTE_PRINCIPALES_MATIERESS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.PRINCIPALES_MATIERES.getValeur(), getLocale()));
        infoBean.set("LISTE_CENTRES_INTERETS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.CENTRES_INTERETS.getValeur(), getLocale()));
        infoBean.set("LISTE_SERIE_BACS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.SERIE_BAC.getValeur(), getLocale()));
        infoBean.set("LISTE_DOMAINE_ERASMUSS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), getLocale()));
        infoBean.set("LISTE_TYPE_FORMATION_CONTINUE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_FORMATION_CONTINUE.getValeur(), getLocale()));
        try {
            infoBean.set("LIBELLE_CODE_FORMATION_MERE", Formation.getLibelleAffichable((String) infoBean.get("CODE_FORMATION_MERE"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_RESPONSABLES_FICHE", prepareLibelleReponsables());
            infoBean.set("LIBELLE_TEMOIGNAGES", prepareLibelleTemoignage());
            infoBean.set("LIBELLE_PARCOURS_FICHE", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_FICHE"), StringUtils.EMPTY));
        } catch (Exception e) {
            LOG.error("Erreur de récupération de libellés", e);
        }
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean codeRattachementAutresFiche = serviceRubrique.getRubriqueByCode((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        if (codeRattachementAutresFiche != null) {
            infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", codeRattachementAutresFiche.getIntitule());
        }
        infoBean.set("LISTE_CATALOGUES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.CATALOGUE.getValeur(), getLocale()));
        infoBean.set("LISTE_NATURE_FORMATIONS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.NATURE_FORMATION.getValeur(), getLocale()));
        final Map<String, String> hdiplome = LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), this.getLocale());
        infoBean.set("LISTE_TYPE_DIPLOMES", generateDiplomesMap(hdiplome));
        infoBean.set("LISTE_GRADES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.GRADE.getValeur(), getLocale()));
        infoBean.set("LISTE_DOMAINE_LMDS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), getLocale()));
        infoBean.set("LISTE_MENTIONS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.MENTION.getValeur(), getLocale()));
        infoBean.set("LISTE_PARCOURSS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.PARCOURS.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_RECRUTEMENTS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_ANNEE_POSTBACS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.ANNEE_POSTBAC.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_SORTIES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.NIVEAUX_SORTIE.getValeur(), getLocale()));
        infoBean.set("LISTE_LIEUXS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.LIEUX.getValeur(), getLocale()));
        infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_DUREE_LISTES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.DUREE.getValeur(), getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.SECTEUR_ACTIVITE.getValeur(), getLocale()));
        infoBean.set("LISTE_PRINCIPALES_MATIERESS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.PRINCIPALES_MATIERES.getValeur(), getLocale()));
        infoBean.set("LISTE_CENTRES_INTERETS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.CENTRES_INTERETS.getValeur(), getLocale()));
        infoBean.set("LISTE_SERIE_BACS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.SERIE_BAC.getValeur(), getLocale()));
        infoBean.set("LISTE_DOMAINE_ERASMUSS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), getLocale()));
        infoBean.set("LISTE_TYPE_FORMATION_CONTINUE", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.TYPE_FORMATION_CONTINUE.getValeur(), getLocale()));
        try {
            infoBean.set("LIBELLE_CODE_FORMATION_MERE", Formation.getLibelleAffichable((String) infoBean.get("CODE_FORMATION_MERE"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_RESPONSABLES_FICHE", prepareLibelleReponsables());
            infoBean.set("LIBELLE_TEMOIGNAGES", prepareLibelleTemoignage());
            infoBean.set("LIBELLE_PARCOURS_FICHE", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_FICHE"), StringUtils.EMPTY));
        } catch (Exception e) {
            LOG.error("Erreur de récupération de libellés", e);
        }
    }

    /**
     * Positionne dans l'infobean les valeurs
     * @param fieldName
     * @param typeLibelle
     * @param values
     */
    private void prepareKmultiSelectField(final String fieldName, final OffreFormationTypeLibelle typeLibelle, final String values) {
        final String language = formation.getLangue();
        final String sectionCode = getCurrentSection(formation.getPersistanceBean());
        final List<String> selectedLabels = LabelUtils.getListLibelleWithSection(typeLibelle.getValeur(), values, language, sectionCode);
        infoBean.set(fieldName, values);
        infoBean.set("LIBELLE_" + fieldName, StringUtils.join( selectedLabels, ";"));
        infoBean.set("LISTE_" + fieldName + "S", LabelUtils.getLabelForComboForSection(typeLibelle.getValeur(), language, sectionCode));
    }

    /**
     * Affichage de l'écran de saisie d'une fiche Formation.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_FORMATION", formation.getIdFormation().toString());
        infoBean.set("CODE_RATTACHEMENT_AUTRES", formation.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", formation.getLibelleCodeRattachementAutres());
        infoBean.set("CODE_SI_SCOLARITE", formation.getCodeSiScolarite());
        infoBean.set("INTITULE_COURT", formation.getIntituleCourt());
        infoBean.set("INTITULE_LONG", formation.getIntituleLong());
        infoBean.set("CATALOGUE", formation.getCatalogue());
        infoBean.set("LISTE_CATALOGUES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.CATALOGUE.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        infoBean.set("CODE_FORMATION_MERE", formation.getCodeFormationMere());
        infoBean.set("LIBELLE_CODE_FORMATION_MERE", formation.getLibelleCodeFormationMere());
        infoBean.set("NATURE_FORMATION", formation.getNatureFormation());
        infoBean.set("LISTE_NATURE_FORMATIONS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.NATURE_FORMATION.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        infoBean.set("FORMATION_DIPLOMANTE", formation.getFormationDiplomante());
        infoBean.set("FORMATION_LMD", formation.getFormationLmd());
        infoBean.set("TYPE_DIPLOME", formation.getTypeDiplome());
        final Map<String, String> hdiplome = LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        int i = 1;
        while (hdiplome.get("#" + i + "0000#") != null) {
            hdiplome.put(i + "0000", "#GROUPE#" + hdiplome.get("#" + i + "0000#"));
            hdiplome.put(i + "9999", "#/GROUPE#");
            hdiplome.remove("#" + i + "0000#");
            i++;
        }
        infoBean.set("LISTE_TYPE_DIPLOMES", hdiplome);
        infoBean.set("GRADE", formation.getGrade());
        infoBean.set("LISTE_GRADES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.GRADE.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        prepareKmultiSelectField("DOMAINE_LMD", OffreFormationTypeLibelle.DOMAINE_LMD, formation.getDomaineLmd());
        prepareKmultiSelectField("DISCIPLINE", OffreFormationTypeLibelle.DISCIPLINE, formation.getDiscipline());
        infoBean.set("MENTION", formation.getMention());
        infoBean.set("LIBELLE_MENTION", this.preparerMention());
        infoBean.set("PARCOURS", formation.getParcours());
        infoBean.set("LIBELLE_PARCOURS", this.preparerParcours());
        infoBean.set("LISTE_PARCOURSS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.PARCOURS.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        prepareKmultiSelectField("NIVEAU_RECRUTEMENT", OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT, formation.getNiveauRecrutement());
        final String libelleNiveauRecrutement = LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), formation.getNiveauRecrutement(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        infoBean.set("LIBELLE_AFFICHABLE_NIVEAU_RECRUTEMENT", preparerLibelleAffichableFromLibelle(libelleNiveauRecrutement));
        prepareKmultiSelectField("ANNEE_POSTBAC", OffreFormationTypeLibelle.ANNEE_POSTBAC, formation.getAnneePostbac());
        prepareKmultiSelectField("NIVEAU_SORTIE", OffreFormationTypeLibelle.NIVEAUX_SORTIE, formation.getNiveauSortie());
        infoBean.set("FORMATION_INITIALE", formation.getFormationInitiale());
        infoBean.set("REPRISE_ETUDES", formation.getRepriseEtudes());
        infoBean.set("TEMPS_PLEIN", formation.getTempsPlein());
        infoBean.set("COMPATIBLE_ACTIVITE_PRO", formation.getCompatibleActivitePro());
        infoBean.set("TEMPS_AMENAGE", formation.getTempsAmenage());
        infoBean.set("ALTERNANCE", formation.getAlternance());
        infoBean.set("HORS_CONTRAT_ALTERNANCE", formation.getHorsContratAlternance());
        infoBean.set("CONTRAT_APPRENTISSAGE", formation.getContratApprentissage());
        infoBean.set("CONTRAT_PRO", formation.getContratPro());
        infoBean.set("PRESENTIEL", formation.getPresentiel());
        infoBean.set("PARTIELLEMENT_DISTANCE", formation.getPartiellementDistance());
        infoBean.set("TOTALEMENT_DISTANCE", formation.getTotalementDistance());
        infoBean.set("LIEUX", formation.getLieux());
        String libelleLieux = LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.LIEUX.getValeur(), formation.getLieux(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        infoBean.set("LIBELLE_LIEUX", libelleLieux);
        infoBean.set("LIBELLE_AFFICHABLE_LIEUX", preparerLibelleAffichableFromLibelle(libelleLieux));
        infoBean.set("LISTE_LIEUXS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.LIEUX.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        final String libelleLanguesEnseignement = LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), formation.getLanguesEnseignement(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        infoBean.set("LIBELLE_AFFICHABLE_LANGUES_ENSEIGNEMENT", preparerLibelleAffichableFromLibelle(libelleLanguesEnseignement));
        prepareKmultiSelectField("LANGUES_ENSEIGNEMENT", OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT, formation.getLanguesEnseignement());
        infoBean.set("SELECTIF", formation.getSelectif());
        infoBean.set("CAPACITE_ACCUEIL_LIMITEE", formation.getCapaciteAccueilLimitee());
        String duree = preparerLibelleAffichableFromLibelle(formation.getLibelleDureeListe());
        duree += StringUtils.isNoneBlank(formation.getAutreDuree(), duree) ? ", " : StringUtils.EMPTY;
        duree += StringUtils.defaultString(formation.getAutreDuree());
        infoBean.set("LIBELLE_AFFICHABLE_DUREE_LISTE", duree);
        prepareKmultiSelectField("DUREE_LISTE", OffreFormationTypeLibelle.DUREE, formation.getDureeListe());
        infoBean.set("AUTRE_DUREE", formation.getAutreDuree());
        infoBean.set("STAGES", formation.getStages());
        infoBean.set("STAGES_OPTIONNEL", formation.getStagesOptionnel());
        infoBean.set("STAGES_OPTIONNEL_MULTIPLE", formation.getStagesOptionnelMultiple());
        infoBean.set("STAGES_OPTIONNEL_ETRANGER", formation.getStagesOptionnelEtranger());
        infoBean.set("STAGES_OBLIGATOIRE", formation.getStagesObligatoire());
        infoBean.set("STAGES_OBLIGATOIRE_MULTIPLE", formation.getStagesObligatoireMultiple());
        infoBean.set("STAGES_OBLIGATOIRE_ETRANGER", formation.getStagesObligatoireEtranger());
        infoBean.set("SEJOURS_ETRANGER", formation.getSejoursEtranger());
        infoBean.set("SEJOURS_ETRANGER_OPTIONNEL", formation.getSejoursEtrangerOptionnel());
        infoBean.set("SEJOURS_ETRANGER_OPTIONNEL_MULTIPLES", formation.getSejoursEtrangerOptionnelMultiples());
        infoBean.set("SEJOURS_ETRANGER_OBLIGATOIRE", formation.getSejoursEtrangerObligatoire());
        infoBean.set("SEJOURS_ETRANGER_OBLIGATOIRE_MULTIPLES", formation.getSejoursEtrangerObligatoireMultiples());
        infoBean.set("PROJETS_TUTORES", formation.getProjetsTutores());
        infoBean.set("PROJETS_TUTORES_OPTIONNEL", formation.getProjetsTutoresOptionnel());
        infoBean.set("PROJETS_TUTORES_OPTIONNEL_MULTIPLES", formation.getProjetsTutoresOptionnelMultiples());
        infoBean.set("PROJETS_TUTORES_OBLIGATOIRE", formation.getProjetsTutoresObligatoire());
        infoBean.set("PROJETS_TUTORES_OBLIGATOIRE_MULTIPLES", formation.getProjetsTutoresObligatoireMultiples());
        infoBean.set("SECTEURS_ACTIVITES", formation.getSecteursActivites());
        infoBean.set("LIBELLE_SECTEURS_ACTIVITES", this.preparerSecteurDActivite());
        infoBean.set("LIBELLE_AFFICHABLE_SECTEURS_ACTIVITES", this.libelleAffichableSecteurActivite());
        final String libellePrincipalesMatieres = LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.PRINCIPALES_MATIERES.getValeur(), formation.getPrincipalesMatieres(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        infoBean.set("LIBELLE_AFFICHABLE_PRINCIPALES_MATIERES", preparerLibelleAffichableFromLibelle(libellePrincipalesMatieres));
        prepareKmultiSelectField("PRINCIPALES_MATIERES", OffreFormationTypeLibelle.PRINCIPALES_MATIERES, formation.getPrincipalesMatieres());
        prepareKmultiSelectField("CENTRES_INTERET", OffreFormationTypeLibelle.CENTRES_INTERETS, formation.getCentresInteret());
        final String libelleSerieBac = LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.SERIE_BAC.getValeur(), formation.getSerieBac(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        infoBean.set("LIBELLE_AFFICHABLE_SERIE_BAC", preparerLibelleAffichableFromLibelle(libelleSerieBac));
        prepareKmultiSelectField("SERIE_BAC", OffreFormationTypeLibelle.SERIE_BAC, formation.getSerieBac());
        prepareKmultiSelectField("DOMAINE_ERASMUS", OffreFormationTypeLibelle.DOMAINE_ERASMUS, formation.getDomaineErasmus());
        infoBean.set("ACCROCHE", formation.getAccroche());
        infoBean.set("RESUME", formation.getResume());
        infoBean.set("DESCRIPTION", formation.getDescription());
        infoBean.set("SPECIFICITES", formation.getSpecificites());
        infoBean.set("LIEUX_COMPLEMENT", formation.getLieuxComplement());
        infoBean.set("RESPONSABLES_FICHE", formation.getResponsablesFiche());
        infoBean.set("LIBELLE_RESPONSABLES_FICHE", prepareLibelleReponsables());
        infoBean.set("RESPONSABLES_COMPLEMENT", formation.getResponsablesComplement());
        infoBean.set("TEMOIGNAGES", formation.getTemoignages());
        infoBean.set("LIBELLE_TEMOIGNAGES", formation.getLibelleTemoignages());
        infoBean.set("LABORATOIRES_PARTENAIRES", formation.getLaboratoiresPartenaires());
        infoBean.set("ETABLISSEMENTS_PARTENAIRES", formation.getEtablissementsPartenaires());
        infoBean.set("ENTREPRISES_PARTENAIRES", formation.getEntreprisesPartenaires());
        infoBean.set("CONVENTION_ETABLISSEMENTS", formation.getConventionEtablissements());
        infoBean.set("FORMATIONS_REQUISES", formation.getFormationsRequises());
        infoBean.set("FORMATIONS_LIEES", formation.getFormationsLiees());
        infoBean.set("LIBELLE_FORMATIONS_LIEES", formation.getLibelleFichesFormationsLieesFiche());
        infoBean.set("PUBLIC_CIBLE", formation.getPublicCible());
        infoBean.set("MODALITES_CANDIDATURES", formation.getModalitesCandidatures());
        infoBean.set("MODALITES_CANDIDATURES_SPECIFIQUES", formation.getModalitesCandidaturesSpecifiques());
        infoBean.set("CONDITIONS_ADMISSION", formation.getConditionsAdmission());
        infoBean.set("PROGRAMME", formation.getProgramme());
        infoBean.set("PARCOURS_FICHE", formation.getParcoursFiche());
        infoBean.set("LIBELLE_PARCOURS_FICHE", formation.getLibelleParcoursFiche());
        infoBean.set("CONTENU_FORMATION", formation.getContenuFormation());
        infoBean.set("RYTHME_PREMIERE_ANNEE", formation.getRythmePremiereAnnee());
        infoBean.set("EQUIPE_PEDAGOGIQUE", formation.getEquipePedagogique());
        infoBean.set("TEMPS_PLEIN_DESCRIPTION", formation.getTempsPleinDescription());
        infoBean.set("COMPATIBLE_PROFESSIONNELLE_DESCRIPTION", formation.getCompatibleProfessionnelleDescription());
        infoBean.set("TEMPS_AMENAGE_DESCRIPTION", formation.getTempsAmenageDescription());
        infoBean.set("ALTERNANCE_DESCRIPTION", formation.getAlternanceDescription());
        infoBean.set("PRESENTIEL_DESCRIPTION", formation.getPresentielDescription());
        infoBean.set("PARTIELLEMENT_DISTANCE_DESCRIPTION", formation.getPartiellementDistanceDescription());
        infoBean.set("TOTALEMENT_DISTANCE_DESCRIPTION", formation.getTotalementDistanceDescription());
        infoBean.set("STAGES_DESCRIPTION", formation.getStagesDescription());
        infoBean.set("SEJOURS_ETRANGER_DESCRIPTION", formation.getSejoursEtrangerDescription());
        infoBean.set("PROJETS_TUTORES_COMPLEMENT", formation.getProjetsTutoresComplement());
        infoBean.set("MODALITES_EVALUATION", formation.getModalitesEvaluation());
        infoBean.set("FICHE_RNCP", formation.getFicheRncp());
        infoBean.set("COMPETENCES_ATTESTEES", formation.getCompetencesAttestees());
        infoBean.set("POURSUITE_ETUDES", formation.getPoursuiteEtudes());
        infoBean.set("LIBELLE_PARCOURS_POURSUITE", formation.getLibelleParcoursPoursuite());
        infoBean.set("PARCOURS_POURSUITE", formation.getParcoursPoursuite());
        infoBean.set("LIBELLE_FORMATION_POURSUITE", formation.getLibelleFormationPoursuite());
        infoBean.set("FORMATION_POURSUITE", formation.getFormationPoursuite());
        infoBean.set("DEBOUCHES", formation.getDebouches());
        infoBean.set("INSERTION_PROFESSIONNELLE", formation.getInsertionProfessionnelle());
        infoBean.set("TARIF_COMPLEMENT", formation.getTarifComplement());
        infoBean.set("METIERS", formation.getMetiers());
        infoBean.set("LIBELLE_METIERS", this.preparerMetiers());
        infoBean.set("LIBELLE_AFFICHABLE_METIERS", this.libelleAffichableMetier());
        infoBean.set("MODALITES_INSCRIPTION", formation.getModalitesInscription());
        infoBean.set("PLAQUETTES_CRM", formation.getPlaquettesCrm());
        infoBean.set("CONTACT_EMAIL", formation.getContactEmail());
        infoBean.set("CONTACT_TELEPHONE", formation.getContactTelephone());
        infoBean.set("CONTACT_CRM", formation.getContactCrm());
        infoBean.set("URL_EXTERNE_FORMATION", formation.getUrlExterneFormation());
        infoBean.set("RENSEIGNEMENTS", formation.getRenseignements());
        infoBean.set("URL_INSCRIPTION", formation.getUrlInscription());
        infoBean.set("AFFICHER_PROGRAMME", formation.getAfficherProgramme());
        infoBean.set("CODE_RATTACHEMENT", formation.getCodeRattachement());
        infoBean.set("FORMATION_CONTINUE", formation.getFormationContinue());
        infoBean.set("TYPE_FORMATION_CONTINUE", formation.getTypeFormationContinue());
        infoBean.set("LISTE_TYPE_FORMATION_CONTINUE", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_FORMATION_CONTINUE.getValeur(), formation.getLangue(), getCurrentSection(formation.getPersistanceBean())));
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", formation.getInfosStructure() != null ? formation.getInfosStructure().getLibelleAffichable() : "");
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("LIBELLE_AFFICHABLE", formation.getLibelleAffichable());
            if (infoBean.get("SOUS_ONGLET") == null) {
                // DSI sur les onglets, on affiche l'onglet suivant si on a pas les droits
                if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE)) {
                    infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
                } else if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE)) {
                    infoBean.set("SOUS_ONGLET", "AFFICHAGE"); // onglet par defaut
                } else if (StringUtils.isBlank(infoBean.getString("GRS_SAISIE_ENCADRES")) || StringUtils.equals(infoBean.getString("GRS_SAISIE_ENCADRES"), "0")) {
                    //Cas pas de droits sur les premiers onglets on affiche les encadrés si possibles.
                    infoBean.set("SOUS_ONGLET", "encadres");
                } else {
                    throw new ErreurApplicative(MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_AUCUN_DROIT"));
                }
            }
        }
        infoBean.set("PRINCIPAL", OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE));
        infoBean.set("AFFICHAGE", OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE));
        infoBean.set("CALL_TO_ACTION", OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE));
        infoBean.set("LISTE_ONGLET_FORMATION", getListeOnglet());
        if (OffreFormationUtil.isConstructionParcoursPresent()) {
            infoBean.set("ACTIVER_CONSTRUIRE_PARCOURS", formation.getActiverConstruireParcours());
        }
        infoBean.set(MenuSautsUtil.INFOBEAN_FIELD, ApplicationContextManager.getBean(OffreFormationUtil.ID_EXTENSION, "offreformationMenuSauts"));
        ControleurUniv.preparerPRINCIPAL(infoBean, formation, this);
    }

    private String preparerLibelleAffichableFromLibelle(final String libellesInput) {
        final String[] libelles = StringUtils.split(libellesInput, ";");
        return StringUtils.join(libelles, ", ");
    }

    private Map<String, String> getListeOnglet() {
        final Map<String, String> listeOnglet = new HashMap<>();
        // Si droit "onglet caracteristiques
        if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE)) {
            listeOnglet.put("PRINCIPAL", MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_CARACTERISTIQUES"));
        }
        // Si droit "onglet Affichage (affichage + call + action + encadre)
        if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE)) {
            listeOnglet.put("AFFICHAGE", MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_AFFICHAGE"));
            listeOnglet.put("CALL_TO_ACTION", MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_CALL_TO_ACTION"));
        }
        return listeOnglet;
    }

    private String prepareLibelleReponsables() throws Exception {
        final Collection<String> labels = new ArrayList<>();
        final String[] codes = StringUtils.split(formation.getResponsablesFiche(), ";");
        for (final String currentCode : codes) {
            final String[] split = currentCode.split(",");
            final StringBuilder currentLabel = new StringBuilder(FicheAnnuaire.getLibelleAffichable(currentCode, formation.getLangue()));
            if (split.length == 3 && !"0000".equals(split[2])) {
                currentLabel.append(String.format(" [%s]", LabelUtils.getLibelleWithSection("64", split[2], formation.getLangue(), getCurrentSection(formation.getPersistanceBean()))));
            }
            labels.add(currentLabel.toString());
        }
        return StringUtils.join(labels, ";");
    }

    private String prepareLibelleTemoignage() throws Exception {
        final Collection<String> labels = new ArrayList<>();
        final String[] codes = StringUtils.split(formation.getTemoignages(), ";");
        for (final String currentCode : codes) {
            final String[] split = currentCode.split(",");
            final StringBuilder currentLabel = new StringBuilder(FicheAnnuaire.getLibelleAffichable(currentCode, formation.getLangue()));
            if (split.length == 3 && !"0000".equals(split[2])) {
                currentLabel.append(String.format(" [%s]", LabelUtils.getLibelleWithSection("64", split[2], formation.getLangue(), getCurrentSection(formation.getPersistanceBean()))));
            }
            labels.add(currentLabel.toString());
        }
        return StringUtils.join(labels, ";");
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        formation.setCodeRattachementAutres(infoBean.getString("CODE_RATTACHEMENT_AUTRES"));
        formation.setCodeSiScolarite(infoBean.getString("CODE_SI_SCOLARITE"));
        formation.setIntituleCourt(infoBean.getString("INTITULE_COURT"));
        formation.setIntituleLong(infoBean.getString("INTITULE_LONG"));
        formation.setCatalogue(infoBean.getString("CATALOGUE"));
        formation.setCodeFormationMere(infoBean.getString("CODE_FORMATION_MERE"));
        formation.setNatureFormation(infoBean.getString("NATURE_FORMATION"));
        formation.setFormationDiplomante(infoBean.getString("FORMATION_DIPLOMANTE"));
        formation.setFormationLmd(infoBean.getString("FORMATION_LMD"));
        formation.setTypeDiplome(infoBean.getString("TYPE_DIPLOME"));
        formation.setGrade(infoBean.getString("GRADE"));
        formation.setDomaineLmd(infoBean.getString("DOMAINE_LMD"));
        formation.setDiscipline(infoBean.getString("DISCIPLINE"));
        formation.setMention(infoBean.getString("MENTION"));
        formation.setParcours(infoBean.getString("PARCOURS"));
        formation.setNiveauRecrutement(infoBean.getString("NIVEAU_RECRUTEMENT"));
        formation.setAnneePostbac(infoBean.getString("ANNEE_POSTBAC"));
        formation.setNiveauSortie(infoBean.getString("NIVEAU_SORTIE"));
        formation.setFormationInitiale(infoBean.getString("FORMATION_INITIALE"));
        formation.setRepriseEtudes(infoBean.getString("REPRISE_ETUDES"));
        formation.setTempsPlein(infoBean.getString("TEMPS_PLEIN"));
        formation.setCompatibleActivitePro(infoBean.getString("COMPATIBLE_ACTIVITE_PRO"));
        formation.setTempsAmenage(infoBean.getString("TEMPS_AMENAGE"));
        formation.setAlternance(infoBean.getString("ALTERNANCE"));
        formation.setHorsContratAlternance(infoBean.getString("HORS_CONTRAT_ALTERNANCE"));
        formation.setContratApprentissage(infoBean.getString("CONTRAT_APPRENTISSAGE"));
        formation.setContratPro(infoBean.getString("CONTRAT_PRO"));
        formation.setPresentiel(infoBean.getString("PRESENTIEL"));
        formation.setPartiellementDistance(infoBean.getString("PARTIELLEMENT_DISTANCE"));
        formation.setTotalementDistance(infoBean.getString("TOTALEMENT_DISTANCE"));
        formation.setLieux(infoBean.getString("LIEUX"));
        formation.setLanguesEnseignement(infoBean.getString("LANGUES_ENSEIGNEMENT"));
        formation.setSelectif(infoBean.getString("SELECTIF"));
        formation.setCapaciteAccueilLimitee(infoBean.getString("CAPACITE_ACCUEIL_LIMITEE"));
        formation.setDureeListe(infoBean.getString("DUREE_LISTE"));
        formation.setAutreDuree(infoBean.getString("AUTRE_DUREE"));
        formation.setStages(infoBean.getString("STAGES"));
        formation.setStagesOptionnel(infoBean.getString("STAGES_OPTIONNEL"));
        formation.setStagesOptionnelMultiple(infoBean.getString("STAGES_OPTIONNEL_MULTIPLE"));
        formation.setStagesOptionnelEtranger(infoBean.getString("STAGES_OPTIONNEL_ETRANGER"));
        formation.setStagesObligatoire(infoBean.getString("STAGES_OBLIGATOIRE"));
        formation.setStagesObligatoireMultiple(infoBean.getString("STAGES_OBLIGATOIRE_MULTIPLE"));
        formation.setStagesObligatoireEtranger(infoBean.getString("STAGES_OBLIGATOIRE_ETRANGER"));
        formation.setSejoursEtranger(infoBean.getString("SEJOURS_ETRANGER"));
        formation.setSejoursEtrangerOptionnel(infoBean.getString("SEJOURS_ETRANGER_OPTIONNEL"));
        formation.setSejoursEtrangerOptionnelMultiples(infoBean.getString("SEJOURS_ETRANGER_OPTIONNEL_MULTIPLES"));
        formation.setSejoursEtrangerObligatoire(infoBean.getString("SEJOURS_ETRANGER_OBLIGATOIRE"));
        formation.setSejoursEtrangerObligatoireMultiples(infoBean.getString("SEJOURS_ETRANGER_OBLIGATOIRE_MULTIPLES"));
        formation.setProjetsTutores(infoBean.getString("PROJETS_TUTORES"));
        formation.setProjetsTutoresOptionnel(infoBean.getString("PROJETS_TUTORES_OPTIONNEL"));
        formation.setProjetsTutoresOptionnelMultiples(infoBean.getString("PROJETS_TUTORES_OPTIONNEL_MULTIPLES"));
        formation.setProjetsTutoresObligatoire(infoBean.getString("PROJETS_TUTORES_OBLIGATOIRE"));
        formation.setProjetsTutoresObligatoireMultiples(infoBean.getString("PROJETS_TUTORES_OBLIGATOIRE_MULTIPLES"));
        formation.setSecteursActivites(infoBean.getString("SECTEURS_ACTIVITES"));
        formation.setMetiers(infoBean.getString("METIERS"));
        formation.setPrincipalesMatieres(infoBean.getString("PRINCIPALES_MATIERES"));
        formation.setCentresInteret(infoBean.getString("CENTRES_INTERET"));
        formation.setSerieBac(infoBean.getString("SERIE_BAC"));
        formation.setDomaineErasmus(infoBean.getString("DOMAINE_ERASMUS"));
        formation.setAccroche(infoBean.getString("ACCROCHE"));
        formation.setResume(infoBean.getString("RESUME"));
        formation.setDescription(infoBean.getString("DESCRIPTION"));
        formation.setSpecificites(infoBean.getString("SPECIFICITES"));
        formation.setLieuxComplement(infoBean.getString("LIEUX_COMPLEMENT"));
        formation.setResponsablesFiche(infoBean.getString("RESPONSABLES_FICHE"));
        formation.setResponsablesComplement(infoBean.getString("RESPONSABLES_COMPLEMENT"));
        formation.setTemoignages(infoBean.getString("TEMOIGNAGES"));
        formation.setLaboratoiresPartenaires(infoBean.getString("LABORATOIRES_PARTENAIRES"));
        formation.setEtablissementsPartenaires(infoBean.getString("ETABLISSEMENTS_PARTENAIRES"));
        formation.setEntreprisesPartenaires(infoBean.getString("ENTREPRISES_PARTENAIRES"));
        formation.setConventionEtablissements(infoBean.getString("CONVENTION_ETABLISSEMENTS"));
        formation.setFormationsRequises(infoBean.getString("FORMATIONS_REQUISES"));
        formation.setFormationsLiees(infoBean.getString("FORMATIONS_LIEES"));
        formation.setPublicCible(infoBean.getString("PUBLIC_CIBLE"));
        formation.setModalitesCandidatures(infoBean.getString("MODALITES_CANDIDATURES"));
        formation.setModalitesCandidaturesSpecifiques(infoBean.getString("MODALITES_CANDIDATURES_SPECIFIQUES"));
        formation.setConditionsAdmission(infoBean.getString("CONDITIONS_ADMISSION"));
        formation.setProgramme(infoBean.getString("PROGRAMME"));
        formation.setParcoursFiche(infoBean.getString("PARCOURS_FICHE"));
        formation.setContenuFormation(infoBean.getString("CONTENU_FORMATION"));
        formation.setRythmePremiereAnnee(infoBean.getString("RYTHME_PREMIERE_ANNEE"));
        formation.setEquipePedagogique(infoBean.getString("EQUIPE_PEDAGOGIQUE"));
        formation.setTempsPleinDescription(infoBean.getString("TEMPS_PLEIN_DESCRIPTION"));
        formation.setCompatibleProfessionnelleDescription(infoBean.getString("COMPATIBLE_PROFESSIONNELLE_DESCRIPTION"));
        formation.setTempsAmenageDescription(infoBean.getString("TEMPS_AMENAGE_DESCRIPTION"));
        formation.setAlternanceDescription(infoBean.getString("ALTERNANCE_DESCRIPTION"));
        formation.setPresentielDescription(infoBean.getString("PRESENTIEL_DESCRIPTION"));
        formation.setPartiellementDistanceDescription(infoBean.getString("PARTIELLEMENT_DISTANCE_DESCRIPTION"));
        formation.setTotalementDistanceDescription(infoBean.getString("TOTALEMENT_DISTANCE_DESCRIPTION"));
        formation.setStagesDescription(infoBean.getString("STAGES_DESCRIPTION"));
        formation.setSejoursEtrangerDescription(infoBean.getString("SEJOURS_ETRANGER_DESCRIPTION"));
        formation.setProjetsTutoresComplement(infoBean.getString("PROJETS_TUTORES_COMPLEMENT"));
        formation.setModalitesEvaluation(infoBean.getString("MODALITES_EVALUATION"));
        formation.setFicheRncp(infoBean.getString("FICHE_RNCP"));
        formation.setCompetencesAttestees(infoBean.getString("COMPETENCES_ATTESTEES"));
        formation.setPoursuiteEtudes(infoBean.getString("POURSUITE_ETUDES"));
        formation.setParcoursPoursuite(infoBean.getString("PARCOURS_POURSUITE"));
        formation.setFormationPoursuite(infoBean.getString("FORMATION_POURSUITE"));
        formation.setDebouches(infoBean.getString("DEBOUCHES"));
        formation.setInsertionProfessionnelle(infoBean.getString("INSERTION_PROFESSIONNELLE"));
        formation.setTarifComplement(infoBean.getString("TARIF_COMPLEMENT"));
        formation.setModalitesInscription(infoBean.getString("MODALITES_INSCRIPTION"));
        formation.setPlaquettesCrm(infoBean.getString("PLAQUETTES_CRM"));
        formation.setContactEmail(infoBean.getString("CONTACT_EMAIL"));
        formation.setContactTelephone(infoBean.getString("CONTACT_TELEPHONE"));
        formation.setContactCrm(infoBean.getString("CONTACT_CRM"));
        formation.setUrlExterneFormation(infoBean.getString("URL_EXTERNE_FORMATION"));
        formation.setRenseignements(infoBean.getString("RENSEIGNEMENTS"));
        formation.setUrlInscription(infoBean.getString("URL_INSCRIPTION"));
        formation.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        formation.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        formation.setAfficherProgramme(infoBean.getString("AFFICHER_PROGRAMME"));
        formation.setFormationContinue(infoBean.getString("FORMATION_CONTINUE"));
        formation.setTypeFormationContinue(infoBean.getString("TYPE_FORMATION_CONTINUE"));
        if (OffreFormationUtil.isConstructionParcoursPresent()) {
            formation.setActiverConstruireParcours(infoBean.getString("ACTIVER_CONSTRUIRE_PARCOURS"));
        }
    }

    /**
     * Traite l'écran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            formation.init();
        } else {
            formation.setIdFormation(Long.valueOf(infoBean.getString("ID_FORMATION")));
            formation.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            //si le code doit etre controle car saisi manuellement ou bien
            //parce que l'objet implemente l'interface Structure ou Annuaire : passer la valeur true
            alimenteDonneesCreation(formation, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, formation, this);
        if (StringUtils.isEmpty(ecranLogique)) {
            etat = FIN;
        }
    }

    /**
     * {@link SaisieFormation#preparerPRINCIPAL()}.
     */
    private String preparerMetiers() throws JsonProcessingException {
        final List<String> codes = new ArrayList<>(findMetierCodes(formation));
        if (CollectionUtils.isEmpty(codes)) {
            return StringUtils.EMPTY;
        }
        final List<LabelDto> metiers = serviceLabel.getDtoByTypeCodesLanguageAndSite(MetierSerializer.TYPE_LIBELLE_METIER, codes, formation.getLangue(), getCurrentSection(formation.getPersistanceBean()));
        final List<AutoCompleteSuggestion> suggestions = this.serviceConversionLabelAutoCompletion.convert(metiers);
        final AutoCompletionObject autoComplete = new AutoCompletionObject();
        autoComplete.setSuggestions(suggestions);
        return JSON_HELPER.writeValueAsString(autoComplete);
    }

    private Collection<String> findMetierCodes(final Formation formation) {
        return serviceConversionLabelAutoCompletion.extractIds(formation.getMetiers(), BUSINESS_SEPARATOR);
    }

    private String libelleAffichableMetier() {
        final List<String> codes = new ArrayList<>(findMetierCodes(formation));
        if (CollectionUtils.isEmpty(codes)) {
            return StringUtils.EMPTY;
        }
        String langue = ContexteUtil.getContexteUniv().getLangue();
        return serviceLabel.getByTypeCodesLanguageAndSection(MetierSerializer.TYPE_LIBELLE_METIER, codes, langue, formation.getCodeRubrique()).stream()
            .collect(Collectors.joining(", "));
    }

    private String libelleAffichableSecteurActivite() {
        final Collection<String> ids = this.serviceConversionLabelAutoCompletion.extractIds(formation.getSecteursActivites(), ";");
        if (CollectionUtils.isEmpty(ids)) {
            return StringUtils.EMPTY;
        }
        return serviceLabel.getByTypeCodesLanguageAndSection(MetierSerializer.TYPE_LIBELLE_METIER, new ArrayList<>(ids), formation.getLangue(), formation.getCodeRubrique()).stream()
            .collect(Collectors.joining(", "));
    }

    /**
     * {@link SaisieFormation#preparerPRINCIPAL()}.
     */
    private String preparerMention() throws JsonProcessingException {
        final String locale = formation.getLangue();
        final Collection<String> ids = this.serviceConversionLabelAutoCompletion.extractIds(formation.getMention(), ";");
        if (CollectionUtils.isEmpty(ids)) {
            return StringUtils.EMPTY;
        }
        final List<LabelDto> mentions = this.serviceLabel.getDtoByTypeCodesLanguageAndSite(OffreFormationTypeLibelle.MENTION.getValeur(), new ArrayList<>(ids), locale, formation.getCodeRubrique());
        final List<AutoCompleteSuggestion> suggestions = this.serviceConversionLabelAutoCompletion.convert(mentions);
        final AutoCompletionObject autoComplete = new AutoCompletionObject();
        autoComplete.setSuggestions(suggestions);
        return JSON_HELPER.writeValueAsString(autoComplete);
    }

    /**
     * {@link SaisieFormation#preparerPRINCIPAL()}.
     */
    private String preparerParcours() throws JsonProcessingException {
        final String locale = formation.getLangue();
        final Collection<String> ids = this.serviceConversionLabelAutoCompletion.extractIds(formation.getParcours(), ";");
        if (CollectionUtils.isEmpty(ids)) {
            return StringUtils.EMPTY;
        }
        final List<LabelDto> mentions = this.serviceLabel.getDtoByTypeCodesLanguageAndSite(OffreFormationTypeLibelle.PARCOURS.getValeur(), new ArrayList<>(ids), locale, formation.getCodeRubrique());
        final List<AutoCompleteSuggestion> suggestions = this.serviceConversionLabelAutoCompletion.convert(mentions);
        final AutoCompletionObject autoComplete = new AutoCompletionObject();
        autoComplete.setSuggestions(suggestions);
        return JSON_HELPER.writeValueAsString(autoComplete);
    }

    private String preparerSecteurDActivite() throws JsonProcessingException {
        final String locale = formation.getLangue();
        final Collection<String> ids = this.serviceConversionLabelAutoCompletion.extractIds(formation.getSecteursActivites(), ";");
        if (CollectionUtils.isEmpty(ids)) {
            return StringUtils.EMPTY;
        }
        final List<LabelDto> secteurActivite = this.serviceLabel.getDtoByTypeCodesLanguageAndSite(OffreFormationTypeLibelle.SECTEUR_ACTIVITE.getValeur(), new ArrayList<>(ids), locale, formation.getCodeRubrique());
        final List<AutoCompleteSuggestion> suggestions = this.serviceConversionLabelAutoCompletion.convert(secteurActivite);
        final AutoCompletionObject autoComplete = new AutoCompletionObject();
        autoComplete.setSuggestions(suggestions);
        return JSON_HELPER.writeValueAsString(autoComplete);
    }

    /**
     * Génère la map des diplômes à faire apparaître dans la recherche.
     * @param hdiplome la map des diplômes non travaillées
     * @return la map des diplômes agrémentée
     */
    private Map<String, String> generateDiplomesMap(final Map<String, String> hdiplome) {
        final Map<String, String> outputMap = new HashMap<>(hdiplome);
        int i = 1;
        while (outputMap.get("#" + i + "0000#") != null) {
            outputMap.put(i + "0000", "#GROUPE#" + hdiplome.get("#" + i + "0000#"));
            outputMap.put(i + "9999", "#/GROUPE#");
            outputMap.remove("#" + i + "0000#");
            i++;
        }
        return outputMap;
    }
}
