/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.cache.annotation.Cacheable;

import com.kosmos.service.Service;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.dao.impl.OffreFormationDAO;
import com.ksup.objetspartages.dao.impl.UeupDAO;
import com.univ.objetspartages.om.EtatFiche;

/**
 * Classe de service permettant de travailler sur toute l'extension (peut utiliser des DAO spécifiques pour requêter sur toutes les tables à la fois)
 *
 * Created by alexandre.baillif on 10/04/17.
 */
public class ServiceOffreFormation implements Service {

    private OffreFormationDAO offreFormationDAO;

    private ServiceFormation serviceFormation;

    private ServiceRechercheParcours serviceRechercheParcours;

    private ServiceUeup serviceUeup;

    private UeupDAO ueupDAO;

    public void setServiceUeup(ServiceUeup serviceUeup) {
        this.serviceUeup = serviceUeup;
    }

    public void setOffreFormationDAO(OffreFormationDAO offreFormationDAO) {
        this.offreFormationDAO = offreFormationDAO;
    }

    public void setUeupDAO(UeupDAO ueupDAO) {
        this.ueupDAO = ueupDAO;
    }

    public void setServiceFormation(final ServiceFormation serviceFormation) {
        this.serviceFormation = serviceFormation;
    }

    public void setServiceRechercheParcours(final ServiceRechercheParcours serviceRechercheParcours) {
        this.serviceRechercheParcours = serviceRechercheParcours;
    }

    public List<FormationBean> findAllFormationByUeups(List<String> codes, String langue) {
        return offreFormationDAO.findFormationBeansByUeupCodes(codes, langue);
    }

    /**
     * Permet de retrouver toutes les formations liées à un code ueup
     * @param codeUeup
     * @param langue
     * @return
     * @throws Exception
     */
    @Cacheable(value = "ServiceOffreFormation.findFormationByUeup", key = "#codeUeup + #langue")
    public Set<FormationBean> findAllFormationByCodeUeup(String codeUeup, String langue) {
        List<String> codesUeupsParents = serviceUeup.findCodesUeupParents(codeUeup, langue);
        //Si oui
        if (!codesUeupsParents.isEmpty()) {
            //On ajoute le codeUeup de départ
            codesUeupsParents.add(codeUeup);
            //Puis on requête pour récupérer les formations à partir de ces codes (service dédié)
            return new HashSet<>(findAllFormationByUeups(codesUeupsParents, langue));
        } else {
            //Si non, on cherche directement par la méthode de service dédiée
            return new HashSet<>(findAllFormationByUeups(Collections.singletonList(codeUeup), langue));
        }
    }

    /**
     * Permets de retrouver toutes les formations liés à un code cours
     * @param codeCours
     * @param langue
     * @return
     */
    @Cacheable(value = "ServiceOffreFormation.findFormationByCours", key = "#codeCours + #langue")
    public Set<FormationBean> findFormationByCours(String codeCours, String langue) {
        Set<FormationBean> result = new HashSet<>();
        //On récupère tous codes ueups qui ont ce cours
        List<String> codesUeups = offreFormationDAO.findUeupsByCodeCours(codeCours, langue);
        for (String codeUeup : codesUeups) {
            result.addAll(findAllFormationByCodeUeup(codeUeup, langue));
        }
        return result;
    }

    @Cacheable(value = "ServiceOffreFormation.findUeupByCours", key = "#codeCours + #langue")
    public Set<UeupBean> findUeupByCours(String codeCours, String langue) {
        List<String> codesUeups = offreFormationDAO.findUeupsByCodeCours(codeCours, langue);
        return ueupDAO.selectByCodesAndLangueAndEtat(new HashSet<>(codesUeups), langue, EtatFiche.EN_LIGNE.getEtat());
    }

    /**
     * Methode permettant de récupérer les UEUP à partir du code et de la langue d'une fiche formation.
     * cela va récupérer la fiche formation puis récupère l'ensemble des parcours diplomants de la formation pour ensuite aller chercher les UEUP liés
     * @param codeFormation le code de la formation parente
     * @param langue la langue de la formation parente
     * @return un Set des codes des UEUPs liés ou un Set vide si non trouvé
     */
    public Set<String> findUeupByFormation(String codeFormation, String langue) {
        Set<String> result = new HashSet<>();
        final FormationBean formationParente = serviceFormation.getFiche(codeFormation, langue);
        if (formationParente != null) {
            Set<ParcoursBean> parcoursDiplomants = serviceRechercheParcours.getParcoursBeanFromFormationBean(formationParente);
            for (ParcoursBean parcoursCourant : parcoursDiplomants) {
                if(CollectionUtils.isNotEmpty(parcoursCourant.getUeupsBeans())) {
                    for (UeupBean ueCourant : parcoursCourant.getUeupsBeans()) {
                        result.addAll(serviceUeup.findAllChild(ueCourant));
                        result.add(ueCourant.getCode());
                    }
                }
            }
        }
        return result;
    }
}
