/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;

import com.kosmos.service.impl.ServiceFiche;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.dao.impl.ParcoursDAO;
import com.ksup.objetspartages.dao.impl.ParcoursJointuresDAO;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

public class ServiceParcours extends ServiceFiche<ParcoursBean, ParcoursDAO, FicheDefaultViewModel> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceParcours.class);

    private ParcoursJointuresDAO parcoursJointuresDAO;

    public void setParcoursJointuresDAO(ParcoursJointuresDAO parcoursJointuresDAO) {
        this.parcoursJointuresDAO = parcoursJointuresDAO;
    }

    /**
     * Lorsqu'on modifie un parcours c'est tous les programmes (de toutes les formations) qui peuvent dans l'absolu changer. Du coup on vide le cache complet.
     */
    @Caching(evict = {@CacheEvict(value = "ServiceOffreFormation.findFormationByCours", allEntries = true), @CacheEvict(value = "ServiceOffreFormation.findFormationByUeup", allEntries = true), @CacheEvict(value = "ServiceRechercheParcours.findParcoursByParcours", allEntries = true)})
    public void updateCacheProgramme() {
        LOG.debug("Eviction du cache pour le programme");
    }

    public void add(ParcoursBean parcoursBean) {
        dao.add(parcoursBean);
        saveJoinedDatas(parcoursBean);
        updateCacheProgramme();
    }

    public void addWithForcedId(ParcoursBean parcoursBean) {
        dao.addWithForcedId(parcoursBean);
        saveJoinedDatas(parcoursBean);
        updateCacheProgramme();
    }

    public void update(ParcoursBean parcoursBean) {
        dao.update(parcoursBean);
        saveJoinedDatas(parcoursBean);
        updateCacheProgramme();
    }

    protected void saveJoinedDatas(ParcoursBean parcoursBean) {
        prepareJoinedParcours(parcoursBean);
        parcoursJointuresDAO.saveParcoursToParcoursLiens(parcoursBean);
        parcoursJointuresDAO.saveParcoursToUeups(parcoursBean);
    }

    private void prepareJoinedParcours(ParcoursBean parcoursBean) {
        //Parcours amont
        if(CollectionUtils.isNotEmpty(parcoursBean.getParcoursAmontBeans())) {
            for (Pair<ParcoursBean, String> pairAmont : parcoursBean.getParcoursAmontBeans()) {
                ParcoursBean amont = pairAmont.getLeft();
                amont.setId(getIdByCodeLangueEtat(amont));
            }
        }
        if(CollectionUtils.isNotEmpty(parcoursBean.getParcoursAvalBeans())) {
            //Parcours aval
            for (Pair<ParcoursBean, String> pairAval : parcoursBean.getParcoursAvalBeans()) {
                ParcoursBean aval = pairAval.getLeft();
                aval.setId(getIdByCodeLangueEtat(aval));
            }
        }
    }

    private Long getIdByCodeLangueEtat(ParcoursBean parcoursBean) {
        ParcoursBean res = getByCodeLangueEtat(parcoursBean.getCode(), parcoursBean.getLangue(), parcoursBean.getEtatObjet());
        if (res == null || res.getId().equals(0L)) {
            return parcoursBean.getId();
        }
        return res.getId();
    }

    @Override
    public String getLibelleAffichable(ParcoursBean bean) {
        String res = bean.getIntituleLong();
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    /**
     * Retoune un parcours par son code, sa langue, son état en se basant sur la méthode de dao (
     *
     * Retourne null si pas trouver.
     * @param code
     * @param langue
     * @param etatObjet
     * @return
     */
    public ParcoursBean getByCodeLangueEtat(String code, String langue, String etatObjet) {
        List<ParcoursBean> listParcours = dao.selectCodeLangueEtat(code, langue, etatObjet);
        return CollectionUtils.isNotEmpty(listParcours) ? listParcours.get(0) : null;
    }

    @Override
    public ParcoursBean getById(final Long id) {
        return dao.getById(id);
    }

    /**
     *
     * @param id
     * @deprecated use {@link #delete(ParcoursBean parcours)} instead of.
     */
    @Deprecated
    public void delete(Long id) {
        ParcoursBean currentParcours = getById(id);
        delete(currentParcours);
    }

    public void delete(ParcoursBean parcours) {
        parcoursJointuresDAO.deleteParcoursToParcoursLiens(parcours);
        parcoursJointuresDAO.deleteParcoursToUeups(parcours);
        dao.delete(parcours.getId());
        updateCacheProgramme();
    }

    /**
     * Cette méthode permet de recalculer des données liées aux parcours. à savoir (parcours amont et aval liés, numero premier semestre et duree semestre)
     * @return
     * 	true si la mise à jour à réussi.
     */
    public void resyncDataForParcours() {
        List<ParcoursBean> allParcours = dao.select(StringUtils.EMPTY);
        for (ParcoursBean parcours : allParcours) {
            this.update(parcours);
        }
    }

    @Override
    public String getReferences(ParcoursBean bean){
        String references = StringUtils.defaultString(bean.getPresentation());
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Parcours", bean.getParcoursAmont()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Parcours", bean.getParcoursAval()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Ueup", bean.getUeups()));
        references += StringUtils.defaultString(bean.getPresentation());
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement()));
        return references;
    }

    @Override
    public boolean isContentVisible(ParcoursBean bean) {
        return false;
    }
}
