/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.Cacheable;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.service.Service;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ResponseRechercheParcoursBean;
import com.ksup.objetspartages.om.Parcours;
import com.univ.objetspartages.om.EtatFiche;

/**
 * Service de creation de parcours de formation.
 *
 * @author frederic.falcao
 */
public class ServiceRechercheParcours implements Service {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceRechercheParcours.class);

    private ServiceParcours serviceParcours;

    public void setServiceParcours(ServiceParcours serviceParcours) {
        this.serviceParcours = serviceParcours;
    }

    /**
     * Trouver tous les parcours amont d'un parcours
     */
    public ResponseRechercheParcoursBean findTousParcoursForParcours(ParcoursBean parcours) {
        return findParcours(parcours, false);
    }

    /**
     * Trouver tous les parcours amont d'un parcours
     */
    public ResponseRechercheParcoursBean findTousParcoursForProgramme(ParcoursBean parcours) {
        return findParcours(parcours, true);
    }

    /**
     *
     * Trouve tous les parcours liés à un parcours en excluant les <strong>parcours fictifs</strong> et les parcours liés avec un lien <strong>sous conditions</strong>
     *
     * @param parcours
     * @param isForProgramme
     *     Cas où on exclue les parcours fictifs et les parcours liés sous conditions
     * @return
     */
    protected ResponseRechercheParcoursBean findParcours(ParcoursBean parcours, boolean isForProgramme) {
        Set<ParcoursBean> allParcours = findParcoursByParcours(parcours, isForProgramme);
        ResponseRechercheParcoursBean responseRechercheParcoursBean = new ResponseRechercheParcoursBean(allParcours);
        return responseRechercheParcoursBean;
    }

    @Cacheable(value = "ServiceRechercheParcours.findParcoursByParcours", key = "#parcours.code + #parcours.langue + #isForProgramme")
    public Set<ParcoursBean> findParcoursByParcours(ParcoursBean parcours, boolean isForProgramme) {
        Set<ParcoursBean> list1Tmp = new HashSet<>(); //Liste qui va servir de tampon pour la recherche
        Set<ParcoursBean> listResultat = new HashSet<>();
        listResultat.add(parcours);//On ajoute le parcours diplomant car il fait partie de l'arbre des parcours et donc de la liste de résultats.
        int compteurSecu = 0; // Compteur servant à sortir d'une boucle infinie si les données ne sont pas cohérente

        //On initialise la liste tampon avec le parcours diplomant (point de départ de la recherche)
        list1Tmp.add(parcours);
        do {
            Set<String> listeCodesParcoursAmontAmont = new HashSet<>();
            for (ParcoursBean parcoursTmp : list1Tmp) {
                // On recupere la liste des code des parcours amont du parcours courant
                if(CollectionUtils.isNotEmpty(parcoursTmp.getParcoursAmontBeans())) {
                    for (Pair<ParcoursBean, String> parcoursAmontLink : parcoursTmp.getParcoursAmontBeans()) {
                        if (!isForProgramme || OffreFormationUtil.isSuitableForProgramme(parcoursAmontLink)) {
                            listeCodesParcoursAmontAmont.add(parcoursAmontLink.getLeft().getCode());
                        }
                    }
                }
            }
            list1Tmp.clear();
            if (!listeCodesParcoursAmontAmont.isEmpty()) {
                for (String code : listeCodesParcoursAmontAmont) {
                    //On sélectionne uniquement les parcours en ligne
                    ParcoursBean pc = serviceParcours.getByCodeLangueEtat(code, parcours.getLangue(), EtatFiche.EN_LIGNE.getEtat());
                    if (pc != null) {
                        list1Tmp.add(pc);
                    }
                }
                if (!list1Tmp.isEmpty()) {
                    // Ajout dans la liste de résultat si pas déjà dedans
                    ajouterParcoursToListe(list1Tmp, listResultat, isForProgramme);
                }
            }
            //Le compteur secu évite les boucles infinies en cas de modélisation non conformes (i.e. cycles)
            compteurSecu++;
        } while (!list1Tmp.isEmpty() && compteurSecu < 7);

        return listResultat;
    }

    /**
     * Ajoute les parcoursBean d'une liste à une autre liste si pas déjà présente dans la liste de retour en excluant sur demande les parcours fictif.
     *
     * @param listTmp
     * @param listResultat
     * @param excludeFictif Si true exclue les parcours fictifs
     */
    private void ajouterParcoursToListe(Set<ParcoursBean> listTmp, Set<ParcoursBean> listResultat, boolean excludeFictif) {
        for (ParcoursBean parcTmp : listTmp) {
            if (!listResultat.contains(parcTmp)) {
                if (!excludeFictif || !StringUtils.equals("1", parcTmp.getParcoursFictif())) {
                    listResultat.add(parcTmp);
                }
            }
        }
    }

    /**
     * recuperer les fiches parcours rattachées à une formation
     *
     * @param formationBean
     * @return Un liste de ParcoursBean
     */
    public Set<ParcoursBean> getParcoursBeanFromFormationBean(FormationBean formationBean) {
        final Set<ParcoursBean> result = new LinkedHashSet<>();
        for (ParcoursBean parcours : formationBean.getParcoursFicheBeans()) {
            ParcoursBean current = serviceParcours.getByCodeLangueEtat(parcours.getCode(), parcours.getLangue(), parcours.getEtatObjet());
            if (current != null) {
                result.add(current);
            }
        }
        return result;
    }

    public List<ParcoursBean> findParcoursByParamSemestre(List<ParcoursBean> listeParcoursDiplomant) {
        List<ParcoursBean> listeComplete = new ArrayList<>();
        // ajout de tous les parcours amont
        for (ParcoursBean parcTmp : listeParcoursDiplomant) {
            ResponseRechercheParcoursBean reponse = findTousParcoursForParcours(parcTmp);
            if (reponse != null) {
                listeComplete.addAll(reponse.getParcours());
            }
        }
        return listeComplete;
    }

    /**
     * Recupérer les parcours aval d'une fiche parcours
     * @return liste de parcours
     */
    public List<ParcoursBean> findParcoursAvals(Long idLastChosenParcours) {
        if (idLastChosenParcours != null) {
            List<ParcoursBean> listeAval = new ArrayList<>();
            ParcoursBean parcoursBeanEnCours = serviceParcours.getById(idLastChosenParcours);
            if(CollectionUtils.isNotEmpty(parcoursBeanEnCours.getParcoursAvalBeans())) {
                for (Pair<ParcoursBean, String> parcoursBeanPair : parcoursBeanEnCours.getParcoursAvalBeans()) {
                    ParcoursBean pc = serviceParcours.getByCodeLangueEtat(parcoursBeanPair.getLeft().getCode(), parcoursBeanEnCours.getLangue(), EtatFiche.EN_LIGNE.getEtat());
                    if (pc != null) {
                        listeAval.add(pc);
                    }
                }
            }
            return listeAval;
        } else {
            return Collections.emptyList();
        }
    }
}
