/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;

import com.kosmos.service.impl.ServiceFiche;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.dao.impl.UeupDAO;
import com.ksup.objetspartages.dao.impl.UeupJointuresDAO;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

/**
 * Created by alexandre.baillif on 15/02/17.
 */
public class ServiceUeup extends ServiceFiche<UeupBean, UeupDAO, FicheDefaultViewModel> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceUeup.class);

    private UeupJointuresDAO ueupJointuresDAO;

    public void setUeupJointuresDAO(UeupJointuresDAO ueupJointureDAO) {
        this.ueupJointuresDAO = ueupJointureDAO;
    }

    @Caching(evict = {@CacheEvict(value = "ServiceOffreFormation.findFormationByCours", allEntries = true), @CacheEvict(value = "ServiceOffreFormation.findFormationByUeup", key = "#bean.code + #bean.langue")})
    public void updateCacheFormation(UeupBean bean) {
        LOG.debug("Eviction du cache pour le ueup [" + bean.getLibelleLong() + ", " + bean.getCode() + "]");
    }

    /**
     * Met à jour les caches qui contient des formationbean mais qui n'ont pas de key utilisable à cet endroit
     */
    @Caching(evict = {@CacheEvict(value = "ServiceOffreFormation.findUeupByCours", allEntries = true)})
    public void updateCacheLies() {
        LOG.debug("Eviction du cache ServiceOffreFormation.findUeupByCours");
    }

    public void add(UeupBean ueupBean) {
        dao.add(ueupBean);
        saveLiens(ueupBean);
        updateCacheLies();
    }

    public void addWithForcedId(UeupBean ueupBean) {
        dao.addWithForcedId(ueupBean);
        saveLiens(ueupBean);
        updateCacheLies();
    }

    public void update(UeupBean ueupBean) {
        dao.update(ueupBean);
        saveLiens(ueupBean);
        updateCacheFormation(ueupBean);
        updateCacheLies();
    }

    protected void saveLiens(UeupBean ueupBean) {
        ueupJointuresDAO.saveUeupFils(ueupBean);
        ueupJointuresDAO.saveUeupsRequises(ueupBean);
        ueupJointuresDAO.saveUeupsToCours(ueupBean);
    }

    /**
     *
     * @param id
     * @deprecated use {@link #delete(UeupBean)} instead
     */
    @Deprecated
    public void delete(Long id) {
        UeupBean ueupBean = getById(id);
        delete(ueupBean);
    }

    public void delete(UeupBean ueupBean) {
        if (ueupBean != null) {
            updateCacheFormation(ueupBean);
            dao.delete(ueupBean.getId());
            ueupJointuresDAO.deleteUeupToUeups(ueupBean);
            ueupJointuresDAO.deleteUeupToCours(ueupBean);
            updateCacheLies();
        }
    }

    public List<String> findCodesUeupParents(String codeUeup, String langue) {
        return ueupJointuresDAO.findCodesUeupParents(codeUeup, langue);
    }

    /**
     * À partir d'un code d'ueup
     * @param code
     * @return
     */
    public Set<String> findAllChild(final String code) {
        final Set<String> result = new HashSet<>();
        if (StringUtils.isNotBlank(code)) {
            final List<UeupBean> ueups = dao.selectCodeLangueEtat(StringUtils.substringBefore(code, ",LANGUE="), StringUtils.substringAfter(code, ",LANGUE="), EtatFiche.EN_LIGNE.getEtat());
            if (!ueups.isEmpty()) {
                result.addAll(findAllChild(ueups.get(0)));
            }
        }
        return result;
    }

    /**
     * À partir d'un code d'ueup
     * @param parent
     * @return
     */
    public Set<String> findAllChild(final UeupBean parent) {
        final Set<String> result = new HashSet<>();
        if (parent != null) {
            result.addAll(ueupJointuresDAO.findAllChildsCode(parent.getCode(), parent.getLangue()));
        }
        return result;
    }

    @Override
    public String getLibelleAffichable(UeupBean bean) {
        String res = bean.getLibelleLong();
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    @Override
    public String getReferences(UeupBean bean){
        String references = FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        references += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachementAutres());
        references += FicheUnivMgr.getReferenceParJointure("annuaire", bean.getCodeResponsables());
        references += StringUtils.defaultString(bean.getPrerequis());
        references += StringUtils.defaultString(bean.getComplementsResponsable());
        references += StringUtils.defaultString(bean.getCoordonnees());
        references += StringUtils.defaultString(bean.getObjectifs());
        references += StringUtils.defaultString(bean.getContenu());
        references += StringUtils.defaultString(bean.getCalendrier());
        references += StringUtils.defaultString(bean.getControleConnaissances());
        references += StringUtils.defaultString(bean.getBibliographie());
        references += StringUtils.defaultString(bean.getComplements());
        if(StringUtils.isNotEmpty(bean.getUeupfils())){
            for(String codeUe : bean.getUeupfils().split(";")){
                references += FicheUnivMgr.getReferenceParJointure("ueup", codeUe);
            }
        }
        if(StringUtils.isNotEmpty(bean.getUeupsRequises())){
            for(String codeUe : bean.getUeupsRequises().split(";")){
                references += FicheUnivMgr.getReferenceParJointure("ueup", codeUe);
            }
        }
        if(StringUtils.isNotEmpty(bean.getCours())){
            for(String cours : bean.getUeupsRequises().split(";")){
                references += FicheUnivMgr.getReferenceParJointure("cours", cours);
            }
        }
        return references;
    }
}
