/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Created on 11 mars 2004
 *
 */
package com.univ.batch.imports;

import com.jsbsoft.jtf.webutils.ContextePage;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.univ.utils.ContexteUtil;
import com.univ.xml.TransformateurXSL;
import com.univ.xml.imports.ImportManager;
import com.univ.xml.util.InfosImport;
import org.apache.commons.lang3.StringUtils;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Import de données XML à partir d'un dossier ou d'un fichier. S'il s'agit d'un dossier, on importe tous les fichiers qui ont l'extension .xml. S'il s'agit d'un fichier, on
 * importe ce fichier. Le processus d'import est le même que lorsqu'il est lancé manuellement depuis le back-office.
 *
 * @author malice
 */
public class ImportXML extends LogReportJob implements Runnable {

    private static final Logger LOG = LoggerFactory.getLogger(ImportXML.class);

    /**
     * clé permettant de retrouver le paramètre fichier dans la map du job
     */
    public static String SELECTED_FILE = "SELECTED_FILE";

    /**
     * Clé permettant de retrouver le paramètre "Transformation XML" dans la map du job.
     */
    public static String TRANSFORMATION_XML = "TRANSFORMATION_XML";

    /**
     * Clé permettant de retrouver le paramètre "Pattern fichiers" dans la map du job.
     */
    public static String PATTERN_FICHIERS = "PATTERN_FICHIERS";

    /**
     * Valeur indiquant qu'aucune transformation XML n'est demandée (car la saisie d'un paramètre est obligatoire).
     */
    public static String AUCUNE_TRANSFORMATION_XML = "AUCUNE";

    /**
     * Valeur indiquant qu'aucun pattern de fichiers n'est demandé (car la saisie d'un paramètre est obligatoire).
     */
    public static String TOUS_LES_FICHIERS = "TOUS";

    /** The s folder or file. */
    private String sFolderOrFile = "";

    /**
     * Le nom du fichier XSL à utiliser pour la transformation XML à appliquer avant l'import.
     */
    private String nomFichierTransformationXML;

    /**
     * Le pattern des fichiers à prendre en compte pour l'import.
     */
    private String patternFichiers;

    /**
     * The main method.
     *
     * @param args
     *            the arguments
     */
    public static void main(final String[] args) {
        final ImportXML iXML = new ImportXML();
        iXML.setSFolderOrFile(args[0]);
        final Thread thread = new Thread(iXML);
        thread.start();
    }

    /**
     * Lancement de la procédure d'import.
     */
    @Override
    public void run() {
        // ajout d'un contexte dans le thread local
        ContexteUtil.setContexteSansRequete();
        final ContextePage ctx = ContexteUtil.getContexteUniv();
        ctx.setConnectionLongue(true);
        try {
            new Long(System.currentTimeMillis());
            String path = "";
            if (StringUtils.isNotEmpty(sFolderOrFile)) {
                path = sFolderOrFile;
            } else {
                path = WebAppUtil.getImportSourcePath();
            }
            final InfosImport infosImport = new InfosImport();
            infosImport.setPathFichier(path);

            // Gestion de la transformation XML
            TransformateurXSL transXSL = null;
            if (StringUtils.isNotBlank(getNomFichierTransformationXML()) && !AUCUNE_TRANSFORMATION_XML.equals(getNomFichierTransformationXML())) {
                transXSL = new TransformateurXSL();
                infosImport.getTransformations().add(getNomFichierTransformationXML());
            }

            // Gestion des patterns de fichiers à prendre en compte
            if (StringUtils.isNotBlank(getPatternFichiers()) && !ImportXML.TOUS_LES_FICHIERS.equals(getPatternFichiers())) {
                infosImport.setPatternFichiers(getPatternFichiers());
            }

            final ImportManager importManager = new ImportManager(infosImport, transXSL);
            importManager.setLogger(LOG);
            importManager.execute(ctx);
        } catch (final Exception e) {
            LOG.error(e.getMessage(), e);
        } finally {
            // suppression du contexte du thread local
            ContexteUtil.releaseContexteUniv();
        }
    }

    @Override
    public String getReportSubject() {
        return "[IMPORTS XML] Synthèse";
    }

    @Override
    public boolean isReportByMail() {
        return "1".equals(PropertyHelper.getProperty("importExport", "import.rapport_batch"));
    }

    @Override
    public void perform() {
        run();
    }

    @Override
    public void init(final JobExecutionContext context) {

        // Récupération des paramètres du job
        sFolderOrFile = (String) context.getMergedJobDataMap().get(SELECTED_FILE);
        nomFichierTransformationXML = (String) context.getMergedJobDataMap().get(TRANSFORMATION_XML);
        patternFichiers = (String) context.getMergedJobDataMap().get(PATTERN_FICHIERS);
    }

    public String getSFolderOrFile() {
        return sFolderOrFile;
    }

    public void setSFolderOrFile(final String folderOrFile) {
        sFolderOrFile = folderOrFile;
    }

    public String getNomFichierTransformationXML() {
        return nomFichierTransformationXML;
    }

    public String getPatternFichiers() {
        return patternFichiers;
    }
}
