/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: actualite
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.om;

import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.Predicate;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.univ.mediatheque.UsingLibelleMedia;
import com.univ.objetspartages.bean.ActualiteBean;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.service.ServiceActualite;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.ActualiteHelper;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUtil;
import com.univ.utils.DateUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseJoin;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ClauseJoinHelper;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

/**
 * Classe representant un objet actualite.
 */
@FicheAnnotation()
public class Actualite extends AbstractFicheRestriction<ActualiteBean> implements UsingLibelleMedia {

    private static final Logger LOG = LoggerFactory.getLogger(Actualite.class);

    private final ServiceMedia serviceMedia;

    public Actualite() {
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     *
     * @param ctx
     *            the ctx
     * @param codes
     *            the codes
     * @param langue
     *            the langue
     *
     * @return the libelle affichable
     *
     * @throws Exception
     *             the exception
     */
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Actualite actualite = new Actualite();
        actualite.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = actualite.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = actualite.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (actualite.nextItem()) {
                    res += actualite.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     *
     * @param ctx
     *            the ctx
     * @param code
     *            the code
     * @param langue
     *            the langue
     *
     * @return the fiche actualite
     *
     * @throws Exception
     *             the exception
     */
    public static Actualite getFicheActualite(final OMContext ctx, final String code, String langue) throws Exception {
        Actualite fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Actualite();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void  init() {
        super.init();
        setResume(StringUtils.EMPTY);
        setLieu(StringUtils.EMPTY);
        setIdPlanAcces((long) 0);
        setTypeEvenement(StringUtils.EMPTY);
        setThematique(StringUtils.EMPTY);
        setDescription(StringUtils.EMPTY);
        setResponsableEvenement(StringUtils.EMPTY);
        setEmailResponsableEvenement(StringUtils.EMPTY);
        setRattachementBandeau("0");
        setComplementDate(StringUtils.EMPTY);
        setSiteEvenement(StringUtils.EMPTY);
        setIdVignette((long) 0);
        setPartenaires(StringUtils.EMPTY);
        setInsertionAgenda(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     *
     * @return the libelle affichable
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean) ;
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut lieu.
     *
     * @return the formated lieu
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedLieu() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getLieu());
    }

    /**
     * Renvoie le libelle pour l'attribut typeEvenement.
     *
     * @return the libelle type evenement
     *
     */
    public String getLibelleTypeEvenement() {
        return LabelUtils.getLibelle("03", getTypeEvenement(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut thematique.
     *
     * @return the libelle thematique
     *
     */
    public String getLibelleThematique() {
        return LabelUtils.getLibelle("04", getThematique(), getLangue());
    }

    /**
     * Renvoie les thematiques sous forme de liste pour qu'elles soient affichees sous forme de liste en front.
     */
    public List<String> getListLibelleThematique() {
        return LabelUtils.getLibelleList("04", getThematique(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut description.
     *
     * @return the formated description
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedDescription() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDescription());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complementDate.
     *
     * @return the formated complement date
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedComplementDate() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplementDate());
    }

    /**
     * Renvoie le libelle pour l'attribut siteEvenement.
     *
     * @return the libelle site evenement
     *
     */
    public String getLibelleSiteEvenement() {
        return LabelUtils.getLibelle("08", getSiteEvenement(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut partenaires.
     *
     * @return the formated partenaires
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedPartenaires() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPartenaires());
    }

    /**
     * Renvoie la rubrique de la fiche.
     *
     * @return the infos rubrique
     *
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @return the infos structure
     *
     * @throws Exception
     *             the exception
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     *
     * @return the meta
     *
     * @throws Exception
     *             the exception
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     *
     * @param requete
     *            the requete
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        final String typeEvenement = RequeteUtil.renvoyerParametre(requete, "TYPE_EVENEMENT");
        final String thematique = RequeteUtil.renvoyerParametre(requete, "THEMATIQUE");
        final String siteEvenement = RequeteUtil.renvoyerParametre(requete, "SITE");
        final String mois = RequeteUtil.renvoyerParametre(requete, "MOIS");
        final String sDateDebut = RequeteUtil.renvoyerParametre(requete, "DATE_EVENEMENT_DEBUT");
        final String sDateFin = RequeteUtil.renvoyerParametre(requete, "DATE_EVENEMENT_FIN");
        final String nbJours = RequeteUtil.renvoyerParametre(requete, "JOUR");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String tri = RequeteUtil.renvoyerParametre(requete, "TRI_DATE");
        final String triDesc = RequeteUtil.renvoyerParametre(requete, "TRI_DATE_DESC");
        final String ids = RequeteUtil.renvoyerParametre(requete, "IDS");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        Date dateDebut = null;
        Date dateFin = null;
        String order = "T1.DATE_EVENEMENT";
        String orderTri = "";
        if (selection.length() > 0) {
            final Date today = new Date(System.currentTimeMillis());
            // Agenda (évènements à venir)
            if ("0001".equals(selection)) {
                dateDebut = today; // aujourd'hui
            } else if ("0003".equals(selection)) {
                dateFin = today; // aujourd'hui
                orderTri = "DESC";
            }
            // Actualités du jour
            else if ("0004".equals(selection)) {
                // date début dans la journée ou date de fin dans la journée
                dateDebut = today; // aujourd'hui
                dateFin = today; // aujourd'hui
            }
            // Actualités de la semaine
            else if ("0005".equals(selection)) {
                // date début dans la semaine ou date de fin dans la semaine
                dateDebut = DateUtil.getFirstDayOfWeek(today);
                dateFin = DateUtil.getLastDayOfWeek(today);
            }
            // Actualités du mois
            else if ("0006".equals(selection)) {
                // date début dans le mois ou date de fin dans le mois
                dateDebut = DateUtil.getFirstDayOfMonth(today);
                dateFin = DateUtil.getLastDayOfMonth(today);
            }
            // Dernières actualités publiées
            else if ("0007".equals(selection)) {
                order = "META.META_DATE_MISE_EN_LIGNE";
                orderTri = "DESC";
            }
            // Dernières actualités modifiées
            else if ("0008".equals(selection)) {
                order = "T1.DATE_MODIFICATION";
                orderTri = "DESC";
            }
        } else if (mois.length() > 0 && !"0000".equals(mois)) {
            final GregorianCalendar cal = new GregorianCalendar();
            cal.setTimeInMillis(System.currentTimeMillis());
            cal.set(Calendar.MONTH, Integer.parseInt(mois) - 1);
            dateDebut = DateUtil.getFirstDayOfMonth(new Date(cal.getTimeInMillis()));
            dateFin = DateUtil.getLastDayOfMonth(new Date(cal.getTimeInMillis()));
        } else if (nbJours.length() > 0) {
            dateDebut = DateUtil.addDays(new Date(System.currentTimeMillis()), 1);
            dateFin = DateUtil.addDays(new Date(System.currentTimeMillis()), Integer.parseInt(nbJours)); // aujourd'hui + nbJours
        } else {
            dateDebut = DateUtil.parseDate(sDateDebut);
            dateFin = DateUtil.parseDate(sDateFin);
        }
        if ("0".equals(tri) || "DATE_DESC".equals(tri) || "1".equals(triDesc)) {
            orderTri = "DESC";
        }
        if ("1".equals(tri) || "DATE_ASC".equals(tri)) {
            orderTri = "ASC";
        }
        return select(code, titre, typeEvenement, thematique, dateDebut, dateFin, codeRattachement, codeRubrique, langue, codeRedacteur, etatObjet, siteEvenement, nombre, order + " " + orderTri, ids);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     *
     * @return the references
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Actualite a partir de l'ensemble des criteres.
     *
     * @param code
     *            the code
     * @param titre
     *            the titre
     * @param typeEvenement
     *            the type evenement
     * @param thematique
     *            the thematique
     * @param dateEvenementDebut
     *            the date evenement debut
     * @param dateEvenementFin
     *            the date evenement fin
     * @param codeRattachement
     *            the code rattachement
     * @param codeRubrique
     *            the code rubrique
     * @param langue
     *            the langue
     * @param codeRedacteur
     *            the code redacteur
     * @param etatObjet
     *            the etat objet
     * @param siteEvenement
     *            the site evenement
     * @param nombre
     *            the nombre
     * @param order
     *            the order
     * @param ids
     *            the ids
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    public int select(final String code, final String titre, final String typeEvenement, final String thematique, final Date dateEvenementDebut, final Date dateEvenementFin, final String codeRattachement, final String codeRubrique, final String langue, final String codeRedacteur, final String etatObjet, final String siteEvenement, final String nombre, final String order, final String ids) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (StringUtils.isNotEmpty(titre)) {
            where.setPremiereCondition(ConditionHelper.rechercheMots("TITRE", titre));
        }
        if (StringUtils.isNotEmpty(typeEvenement) && !"0000".equals(typeEvenement)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.TYPE_EVENEMENT", typeEvenement));
        }
        if (StringUtils.isNotEmpty(thematique) && !"0000".equals(thematique)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.THEMATIQUE", thematique));
        }
        if (Formateur.estSaisie(dateEvenementDebut)) {
            where.and(ConditionHelper.critereDateDebut("DATE_FIN_EVENEMENT", dateEvenementDebut));
        }
        if (Formateur.estSaisie(dateEvenementFin)) {
            where.and(ConditionHelper.critereDateFin("DATE_EVENEMENT", dateEvenementFin));
        }
        if (StringUtils.isNotEmpty(siteEvenement) && !"0000".equals(siteEvenement)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.SITE_EVENEMENT", siteEvenement));
        }
        if (StringUtils.isNotEmpty(ids) && !"0000".equals(ids)) {
            where.and(ConditionHelper.in("ID_ACTUALITE", Chaine.getVecteurPointsVirgules(ids)));
        }
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotBlank(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.TITRE", ClauseOrderBy.SensDeTri.ASC);
        }
        if (orderBy != null && StringUtils.contains(orderBy.formaterSQL(), "META.")) {
            final boolean existJoinMeta = CollectionUtils.exists(requeteSelect.getJointures(), new Predicate() {

                @Override
                public boolean evaluate(final Object o) {
                    final ClauseJoin jointure = (ClauseJoin) o;
                    return StringUtils.contains(jointure.getNomTable(), "METATAG");
                }
            });
            if (!existJoinMeta) {
                requeteSelect.join(ClauseJoinHelper.creerJointureMetaTag(this));
            }
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Renvoie la description de la fiche (utilise dans SaisieCommentaire).
     *
     * @return the contenu
     *
     * @see Actualite#getDescription()
     */
    public String getContenu() {
        return getDescription();
    }

    /**
     * Renvoie la liste des types d'evenements sous forme de vecteur.
     *
     * @return the vecteur types evenement
     */
    public Vector<String> getVecteurTypesEvenement() {
        final Vector<String> v = new Vector<>();
        final StringTokenizer st = new StringTokenizer(getTypeEvenement(), ";");
        while (st.hasMoreTokens()) {
            v.add(st.nextToken());
        }
        return v;
    }

    /**
     * Valorise la liste des types d'evenement sous forme de vecteur.
     *
     * @param v
     *            the v
     */
    public void setVecteurTypesEvenements(final Vector<String> v) {
        String liste = "";
        final Enumeration<String> en = v.elements();
        while (en.hasMoreElements()) {
            liste += en.nextElement() + ";";
        }
        setTypeEvenement(liste);
    }

    @Override
    public List<String> getCodesLibelleMedia() {
        // 0003, le plan d'acces est un type photo de libelle utilise dans la mediatheque
        return Collections.singletonList("0003");
    }

    @Override
    public void checkCodesLibelleMedia(final long idMedia) throws ErreurApplicative {
        List<ActualiteBean> results = ((ServiceActualite) getServiceFiche()).getByAccesPlanId(idMedia);
        if (CollectionUtils.isNotEmpty(results)) {
            throw new ErreurApplicative(String.format(ActualiteHelper.getMessage("ACTUALITE.ERREUR.RESSOURCE_NON_SUPPRIMABLE"), results.get(0).getTitre()));

        }
    }

    public Long getIdActualite() {
        return persistenceBean.getId();
    }

    public void setIdActualite(Long idActualite) {
        persistenceBean.setId(idActualite);
    }

    @GetterAnnotation(isToolbox = true)
    public String getComplementDate() {
        return persistenceBean.getComplementDate();
    }

    /**
     * Sets the complement date.
     *
     * @param complementDate
     *            the complementDate to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplementDate(String complementDate) {
        persistenceBean.setComplementDate(complementDate);
    }

    /**
     * Gets the date declenchement.
     *
     * @return the dateDeclenchement
     */
    public Date getDateDeclenchement() {
        return persistenceBean.getDateDeclenchement();
    }

    /**
     * Sets the date declenchement.
     *
     * @param dateDeclenchement
     *            the dateDeclenchement to set
     */
    public void setDateDeclenchement(Date dateDeclenchement) {
        persistenceBean.setDateDeclenchement(dateDeclenchement);
    }

    /**
     * Gets the date evenement.
     *
     * @return the dateEvenement
     */
    public Date getDateEvenement() {
        return persistenceBean.getDateEvenement();
    }

    /**
     * Sets the date evenement.
     *
     * @param dateEvenement
     *            the dateEvenement to set
     */
    public void setDateEvenement(Date dateEvenement) {
        persistenceBean.setDateEvenement(dateEvenement);
    }

    /**
     * Gets the date fin evenement.
     *
     * @return the dateFinEvenement
     */
    public Date getDateFinEvenement() {
        return persistenceBean.getDateFinEvenement();
    }

    /**
     * Sets the date fin evenement.
     *
     * @param dateFinEvenement
     *            the dateFinEvenement to set
     */
    public void setDateFinEvenement(Date dateFinEvenement) {
        persistenceBean.setDateFinEvenement(dateFinEvenement);
    }

    /**
     * Gets the date suppression.
     *
     * @return the dateSuppression
     */
    public Date getDateSuppression() {
        return persistenceBean.getDateSuppression();
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the dateSuppression to set
     */
    public void setDateSuppression(Date dateSuppression) {
        persistenceBean.setDateSuppression(dateSuppression);
    }

    /**
     * Gets the description.
     *
     * @return the description
     */
    @GetterAnnotation(isToolbox = true)
    public String getDescription() {
        return persistenceBean.getDescription();
    }

    /**
     * Sets the description.
     *
     * @param description
     *            the description to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setDescription(String description) {
        persistenceBean.setDescription(description);
    }

    /**
     * Gets the email responsable evenement.
     *
     * @return the emailResponsableEvenement
     */
    public String getEmailResponsableEvenement() {
        return persistenceBean.getEmailResponsableEvenement();
    }

    /**
     * Sets the email responsable evenement.
     *
     * @param emailResponsableEvenement
     *            the emailResponsableEvenement to set
     */
    public void setEmailResponsableEvenement(String emailResponsableEvenement) {
        persistenceBean.setEmailResponsableEvenement(emailResponsableEvenement);
    }

    /**
     * Gets the id plan acces.
     *
     * @return the idPlanAcces
     */
    @GetterAnnotation(isIdMedia = true)
    public Long getIdPlanAcces() {
        return persistenceBean.getIdPlanAcces();
    }

    /**
     * Sets the id plan acces.
     *
     * @param idPlanAcces
     *            the idPlanAcces to set
     */
    public void setIdPlanAcces(Long idPlanAcces) {
        persistenceBean.setIdPlanAcces(idPlanAcces);
    }

    /**
     * Gets the id vignette.
     *
     * @return the idVignette
     */
    public Long getIdVignette() {
        return persistenceBean.getIdVignette();
    }

    /**
     * Sets the id vignette.
     *
     * @param idVignette
     *            the idVignette to set
     */
    public void setIdVignette(Long idVignette) {
        persistenceBean.setIdVignette(idVignette);
    }

    /**
     * Gets the insertion agenda.
     *
     * @return the insertionAgenda
     */
    public String getInsertionAgenda() {
        return persistenceBean.getInsertionAgenda();
    }

    /**
     * Sets the insertion agenda.
     *
     * @param insertionAgenda
     *            the insertionAgenda to set
     */
    public void setInsertionAgenda(String insertionAgenda) {
        persistenceBean.setInsertionAgenda(insertionAgenda);
    }

    /**
     * Gets the lieu.
     *
     * @return the lieu
     */
    @GetterAnnotation(isToolbox = true)
    public String getLieu() {
        return persistenceBean.getLieu();
    }

    /**
     * Sets the lieu.
     *
     * @param lieu
     *            the lieu to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setLieu(String lieu) {
        persistenceBean.setLieu(lieu);
    }

    /**
     * Gets the partenaires.
     *
     * @return the partenaires
     */
    @GetterAnnotation(isToolbox = true)
    public String getPartenaires() {
        return persistenceBean.getPartenaires();
    }

    /**
     * Sets the partenaires.
     *
     * @param partenaires
     *            the partenaires to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setPartenaires(String partenaires) {
        persistenceBean.setPartenaires(partenaires);
    }

    /**
     * Gets the rattachement bandeau.
     *
     * @return the rattachementBandeau
     */
    public String getRattachementBandeau() {
        return persistenceBean.getRattachementBandeau();
    }

    /**
     * Sets the rattachement bandeau.
     *
     * @param rattachementBandeau
     *            the rattachementBandeau to set
     */
    public void setRattachementBandeau(String rattachementBandeau) {
        persistenceBean.setRattachementBandeau(rattachementBandeau);
    }

    /**
     * Gets the responsable evenement.
     *
     * @return the responsableEvenement
     */
    public String getResponsableEvenement() {
        return persistenceBean.getResponsableEvenement();
    }

    /**
     * Sets the responsable evenement.
     *
     * @param responsableEvenement
     *            the responsableEvenement to set
     */
    public void setResponsableEvenement(String responsableEvenement) {
        persistenceBean.setResponsableEvenement(responsableEvenement);
    }

    /**
     * Gets the resume.
     *
     * @return the resume
     */
    public String getResume() {
        return persistenceBean.getResume();
    }

    /**
     * Sets the resume.
     *
     * @param resume
     *            the resume to set
     */
    public void setResume(String resume) {
        persistenceBean.setResume(resume);
    }

    /**
     * Gets the site evenement.
     *
     * @return the siteEvenement
     */
    public String getSiteEvenement() {
        return persistenceBean.getSiteEvenement();
    }

    /**
     * Sets the site evenement.
     *
     * @param siteEvenement
     *            the siteEvenement to set
     */
    public void setSiteEvenement(String siteEvenement) {
        persistenceBean.setSiteEvenement(siteEvenement);
    }

    /**
     * Gets the thematique.
     *
     * @return the thematique
     */
    public String getThematique() {
        return persistenceBean.getThematique();
    }

    /**
     * Sets the thematique.
     *
     * @param thematique
     *            the thematique to set
     */
    public void setThematique(String thematique) {
        persistenceBean.setThematique(thematique);
    }

    /**
     * Gets the type evenement.
     *
     * @return the typeEvenement
     */
    public String getTypeEvenement() {
        return persistenceBean.getTypeEvenement();
    }

    /**
     * Sets the type evenement.
     *
     * @param typeEvenement
     *            the typeEvenement to set
     */
    public void setTypeEvenement(String typeEvenement) {
        persistenceBean.setTypeEvenement(typeEvenement);
    }
}
