/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ged
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.om;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.univ.objetspartages.bean.DocumentBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUtil;
import com.univ.utils.DateUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * Classe representant un objet document.
 */
@FicheAnnotation()
public class Document extends AbstractFicheRestriction<DocumentBean> {

    private final ServiceRessource serviceRessource;

    public Document() {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     *
     * @param ctx
     *            the ctx
     * @param codes
     *            the codes
     * @param langue
     *            the langue
     *
     * @return the libelle affichable
     *
     * @throws Exception
     *             the exception
     */
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Document document = new Document();
        document.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = document.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = document.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (document.nextItem()) {
                    res += document.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     *
     * @param ctx
     *            the ctx
     * @param code
     *            the code
     * @param langue
     *            the langue
     *
     * @return the fiche document
     *
     * @throws Exception
     *             the exception
     */
    public static Document getFicheDocument(final OMContext ctx, final String code, String langue) throws Exception {
        Document fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Document();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setSousTitre(StringUtils.EMPTY);
        setResume(StringUtils.EMPTY);
        setDateCreationDocument(new Date());
        setTypeDocument(StringUtils.EMPTY);
        setSourceSupport(StringUtils.EMPTY);
        setAuteurs(StringUtils.EMPTY);
        setEditeur(StringUtils.EMPTY);
        setDescription(StringUtils.EMPTY);
        setReferencesIsdn(StringUtils.EMPTY);
        setProchaineMajPeriodicite(StringUtils.EMPTY);
        setThematique(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        // champs plus utilises, initialises a vide
        setTheme(StringUtils.EMPTY);
        setComiteSelection(StringUtils.EMPTY);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle a afficher.
     *
     * @return the libelle affichable
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Renvoie la liste des fichiers joints de la fiche.
     *
     * @return the liste fichiers
     *
     * @throws Exception
     *             the exception
     */
    public List<RessourceBean> getListeFichiers() throws Exception {
        return serviceRessource.getFiles(this);
    }

    /**
     * Renvoie le libelle pour l'attribut typeDocument.
     *
     * @return the libelle type document
     *
     */
    public String getLibelleTypeDocument() {
        return LabelUtils.getLibelle("40", getTypeDocument(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut thematique.
     *
     * @return the libelle thematique
     *
     */
    public String getLibelleThematique() {
        return LabelUtils.getLibelle("04", getThematique(), getLangue());
    }

    /**
     * Renvoie les thematiques sous forme de liste pour qu'elles soient affichees sous forme de liste en front.
     */
    public List<String> getListLibelleThematique() {
        return LabelUtils.getLibelleList("04", getThematique(), getLangue());
    }

    /**
     * Renvoie la rubrique de la fiche.
     *
     * @return the infos rubrique
     *
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @return the infos structure
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     *
     * @return the meta
     *
     * @throws Exception
     *             the exception
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut auteurs.
     *
     * @return the formated auteurs
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedAuteurs() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getAuteurs());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     *
     * @return the formated complements
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut description.
     *
     * @return the formated description
     *
     * @throws Exception
     *             the exception
     */
    public String getFormatedDescription() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDescription());
    }

    /**
     * Traitement d'une requete sur l'objet.
     *
     * @param requete
     *            the requete
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        final String auteur = RequeteUtil.renvoyerParametre(requete, "AUTEUR");
        final String typeDocument = RequeteUtil.renvoyerParametre(requete, "TYPE_DOCUMENT");
        final String thematique = RequeteUtil.renvoyerParametre(requete, "THEMATIQUE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String ids = RequeteUtil.renvoyerParametre(requete, "IDS");
        final String nbMax = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String period = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String tri = RequeteUtil.renvoyerParametre(requete, "TRI_DATE");
        final String nbDays = RequeteUtil.renvoyerParametre(requete, "NB_DAYS");
        final String sDateDebut = RequeteUtil.renvoyerParametre(requete, "DATE_DEBUT");
        final String sDateFin = RequeteUtil.renvoyerParametre(requete, "DATE_FIN");
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        Date dateDebut = null;
        Date dateFin = null;
        String orderDate = StringUtils.EMPTY;
        String orderTri = StringUtils.EMPTY;
        if (period.length() > 0) {
            orderDate = "T1.DATE_CREATION_DOCUMENT";
            final Date today = new Date(System.currentTimeMillis());
            if (period.endsWith("01")) // derniers documents
            {
                dateFin = today; // aujourd'hui
                orderTri = "DESC";
            } else if (period.endsWith("02")) // documents du jour
            {
                dateDebut = today; // aujourd'hui
                dateFin = today; // aujourd'hui
            } else if (period.endsWith("03")) // documents de la semaine
            {
                dateDebut = DateUtil.getFirstDayOfWeek(today);
                dateFin = DateUtil.getLastDayOfWeek(today);
            } else if (period.endsWith("04")) // documents du mois
            {
                dateDebut = DateUtil.getFirstDayOfMonth(today);
                dateFin = DateUtil.getLastDayOfMonth(today);
            }
            // Derniers documents publiés
            else if (period.endsWith("07")) {
                orderDate = "META.META_DATE_MISE_EN_LIGNE";
                orderTri = "DESC";
            }
        } else if (nbDays.length() > 0) // documents mis en ligne depuis n jours
        {
            //construction de la date : aujourd'hui - nb de jours saisis
            final GregorianCalendar gCal = new GregorianCalendar();
            gCal.setTimeInMillis(System.currentTimeMillis());
            gCal.add(Calendar.DAY_OF_MONTH, -Integer.parseInt(nbDays));
            dateDebut = new Date(gCal.getTimeInMillis());
            dateFin = new Date(System.currentTimeMillis()); // aujourd'hui
            orderDate = "T1.DATE_CREATION_DOCUMENT";
        } else if (sDateDebut.length() > 0 || sDateFin.length() > 0) {
            dateDebut = DateUtil.parseDate(sDateDebut);
            dateFin = DateUtil.parseDate(sDateFin);
            orderDate = "T1.DATE_CREATION_DOCUMENT";
        }
        if ("DATE_DESC".equals(tri)) {
            orderTri = "DESC";
        }
        if ("DATE_ASC".equals(tri)) {
            orderTri = "ASC";
        }
        String order = "T1.TITRE";
        if (orderDate.length() > 0) {
            order = orderDate + " " + orderTri + ", " + order;
        }
        return select(code, titre, typeDocument, auteur, thematique, codeRattachement, codeRubrique, langue, codeRedacteur, etatObjet, dateDebut, dateFin, nbMax, order, ids);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     *
     * @return the references
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Document a partir de l'ensemble des criteres.
     *
     * @param code
     *            the code
     * @param titre
     *            the titre
     * @param typeDocument
     *            the type document
     * @param auteurs
     *            the auteurs
     * @param thematique
     *            the thematique
     * @param codeRattachement
     *            the code rattachement
     * @param codeRubrique
     *            the code rubrique
     * @param langue
     *            the langue
     * @param codeRedacteur
     *            the code redacteur
     * @param etatObjet
     *            the etat objet
     * @param dtstart
     *            the dtstart
     * @param dtend
     *            the dtend
     * @param nbMax
     *            the nb max
     * @param order
     *            the order
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    public int select(final String code, final String titre, final String typeDocument, final String auteurs, final String thematique, final String codeRattachement, final String codeRubrique, final String langue, final String codeRedacteur, final String etatObjet, final Date dtstart, final Date dtend, final String nbMax, final String order, final String ids) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (StringUtils.isNotEmpty(titre)) {
            where.setPremiereCondition(ConditionHelper.rechercheMots("TITRE", titre));
        }
        if (StringUtils.isNotEmpty(auteurs)) {
            where.and(ConditionHelper.rechercheMots("AUTEURS", auteurs));
        }
        if (StringUtils.isNotEmpty(typeDocument) && !"0000".equals(typeDocument)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_DOCUMENT", typeDocument));
        }
        if (StringUtils.isNotEmpty(thematique) && !"0000".equals(thematique)) {
            where.and(ConditionHelper.likePourValeursMultiple("THEMATIQUE", thematique));
        }
        if (dtstart != null && Formateur.estSaisie(dtstart)) {
            where.and(ConditionHelper.critereDateDebut("DATE_CREATION_DOCUMENT", dtstart));
        }
        // FIXME : le comportement de dtend n'est pas le même que sur les autres objets (calcul d'interval)
        if (dtend != null && Formateur.estSaisie(dtend)) {
            where.and(ConditionHelper.lessEquals("DATE_CREATION_DOCUMENT", dtend, TypeOperande.DATE));
        }
        if (StringUtils.isNotEmpty(ids) && !"0000".equals(ids)) {
            where.and(ConditionHelper.in("ID_DOCUMENT", Chaine.getVecteurPointsVirgules(ids)));
        }
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nbMax);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    public Long getIdDocument() {
        return persistenceBean.getId();
    }

    public void setIdDocument(Long id) {
        persistenceBean.setId(id);
    }

    /**
     * Gets the auteurs.
     *
     * @return the auteurs
     */
    public String getAuteurs() {
        return persistenceBean.getAuteurs();
    }

    /**
     * Sets the auteurs.
     *
     * @param auteurs
     *            the auteurs to set
     */
    public void setAuteurs(String auteurs) {
        this.persistenceBean.setAuteurs(auteurs);
    }

    /**
     * Gets the comite selection.
     *
     * @return the comiteSelection
     */
    public String getComiteSelection() {
        return persistenceBean.getComiteSelection();
    }

    /**
     * Sets the comite selection.
     *
     * @param comiteSelection
     *            the comiteSelection to set
     */
    public void setComiteSelection(String comiteSelection) {
        this.persistenceBean.setComiteSelection(comiteSelection);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the complements to set
     */
    public void setComplements(String complements) {
        this.persistenceBean.setComplements(complements);
    }

    /**
     * Gets the date archivage document.
     *
     * @return the dateArchivageDocument
     */
    public Date getDateArchivageDocument() {
        return persistenceBean.getDateArchivageDocument();
    }

    /**
     * Sets the date archivage document.
     *
     * @param dateArchivageDocument
     *            the dateArchivageDocument to set
     */
    public void setDateArchivageDocument(Date dateArchivageDocument) {
        this.persistenceBean.setDateArchivageDocument(dateArchivageDocument);
    }

    /**
     * Gets the date creation document.
     *
     * @return the dateCreationDocument
     */
    public Date getDateCreationDocument() {
        return persistenceBean.getDateCreationDocument();
    }

    /**
     * Sets the date creation document.
     *
     * @param dateCreationDocument
     *            the dateCreationDocument to set
     */
    public void setDateCreationDocument(Date dateCreationDocument) {
        this.persistenceBean.setDateCreationDocument(dateCreationDocument);
    }

    /**
     * Gets the date derniere maj document.
     *
     * @return the dateDerniereMajDocument
     */
    public Date getDateDerniereMajDocument() {
        return persistenceBean.getDateDerniereMajDocument();
    }

    /**
     * Sets the date derniere maj document.
     *
     * @param dateDerniereMajDocument
     *            the dateDerniereMajDocument to set
     */
    public void setDateDerniereMajDocument(Date dateDerniereMajDocument) {
        this.persistenceBean.setDateDerniereMajDocument(dateDerniereMajDocument);
    }

    /**
     * Gets the date parution document.
     *
     * @return the dateParutionDocument
     */
    public Date getDateParutionDocument() {
        return persistenceBean.getDateParutionDocument();
    }

    /**
     * Sets the date parution document.
     *
     * @param dateParutionDocument
     *            the dateParutionDocument to set
     */
    public void setDateParutionDocument(Date dateParutionDocument) {
        this.persistenceBean.setDateParutionDocument(dateParutionDocument);
    }

    /**
     * Gets the description.
     *
     * @return the description
     */
    public String getDescription() {
        return persistenceBean.getDescription();
    }

    /**
     * Sets the description.
     *
     * @param description
     *            the description to set
     */
    public void setDescription(String description) {
        this.persistenceBean.setDescription(description);
    }

    /**
     * Gets the editeur.
     *
     * @return the editeur
     */
    public String getEditeur() {
        return persistenceBean.getEditeur();
    }

    /**
     * Sets the editeur.
     *
     * @param editeur
     *            the editeur to set
     */
    public void setEditeur(String editeur) {
        this.persistenceBean.setEditeur(editeur);
    }

    /**
     * Gets the prochaine maj periodicite.
     *
     * @return the prochaineMajPeriodicite
     */
    public String getProchaineMajPeriodicite() {
        return persistenceBean.getProchaineMajPeriodicite();
    }

    /**
     * Sets the prochaine maj periodicite.
     *
     * @param prochaineMajPeriodicite
     *            the prochaineMajPeriodicite to set
     */
    public void setProchaineMajPeriodicite(String prochaineMajPeriodicite) {
        this.persistenceBean.setProchaineMajPeriodicite(prochaineMajPeriodicite);
    }

    /**
     * Gets the references isdn.
     *
     * @return the referencesIsdn
     */
    public String getReferencesIsdn() {
        return persistenceBean.getReferencesIsdn();
    }

    /**
     * Sets the references isdn.
     *
     * @param referencesIsdn
     *            the referencesIsdn to set
     */
    public void setReferencesIsdn(String referencesIsdn) {
        this.persistenceBean.setReferencesIsdn(referencesIsdn);
    }

    /**
     * Gets the resume.
     *
     * @return the resume
     */
    public String getResume() {
        return persistenceBean.getResume();
    }

    /**
     * Sets the resume.
     *
     * @param resume
     *            the resume to set
     */
    public void setResume(String resume) {
        this.persistenceBean.setResume(resume);
    }

    /**
     * Gets the source support.
     *
     * @return the sourceSupport
     */
    public String getSourceSupport() {
        return persistenceBean.getSourceSupport();
    }

    /**
     * Sets the source support.
     *
     * @param sourceSupport
     *            the sourceSupport to set
     */
    public void setSourceSupport(String sourceSupport) {
        this.persistenceBean.setSourceSupport(sourceSupport);
    }

    /**
     * Gets the sous titre.
     *
     * @return the sousTitre
     */
    public String getSousTitre() {
        return persistenceBean.getSousTitre();
    }

    /**
     * Sets the sous titre.
     *
     * @param sousTitre
     *            the sousTitre to set
     */
    public void setSousTitre(String sousTitre) {
        this.persistenceBean.setSousTitre(sousTitre);
    }

    /**
     * Gets the thematique.
     *
     * @return the thematique
     */
    public String getThematique() {
        return persistenceBean.getThematique();
    }

    /**
     * Sets the thematique.
     *
     * @param thematique
     *            the thematique to set
     */
    public void setThematique(String thematique) {
        this.persistenceBean.setThematique(thematique);
    }

    /**
     * Gets the theme.
     *
     * @return the theme
     */
    public String getTheme() {
        return persistenceBean.getTheme();
    }

    /**
     * Sets the theme.
     *
     * @param theme
     *            the theme to set
     */
    public void setTheme(String theme) {
        this.persistenceBean.setTheme(theme);
    }

    /**
     * Gets the type document.
     *
     * @return the typeDocument
     */
    public String getTypeDocument() {
        return persistenceBean.getTypeDocument();
    }

    /**
     * Sets the type document.
     *
     * @param typeDocument
     *            the typeDocument to set
     */
    public void setTypeDocument(String typeDocument) {
        this.persistenceBean.setTypeDocument(typeDocument);
    }
}
