/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.processus;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.jsbsoft.jtf.upload.UploadedFile;
import com.kosmos.importexport.ComposantImportExport;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.utils.FileReaderServlet;
import com.univ.utils.URLResolver;
import com.univ.xhtml.pdf.PDFManipulateur;
import com.univ.xml.IXmlCstes;
import com.univ.xml.ImportExportManager;
import com.univ.xml.XMLValidator;
import com.univ.xml.exports.Exporter;
import com.univ.xml.extraction.IExtractor;
import com.univ.xml.imports.IImporter;
import com.univ.xml.util.InfosExport;
import com.univ.xml.util.InfosImport;

/**
 * Processus d'import/export XML.
 */
public class TraitementFichierXML extends ProcessusBean {

    /**
     * Nom de la clé du type d'import dans l'infobean
     */
    public static final String CLE_TYPE_IMPORT = "TYPE_OBJET";

    /**
     * Type d'import pour un import utilisateur
     */
    public static final String TYPE_IMPORT_UTILISATEUR = "UTILISATEUR";

    /**
     * Type d'import pour un import groupe
     */
    public static final String TYPE_IMPORT_GROUPE = "GROUPE";

    protected static final Logger LOG = LoggerFactory.getLogger(TraitementFichierXML.class);

    /**
     * The Constant ECRAN_EXPORT.
     */
    private static final String ECRAN_EXPORT = "SAISIE_EXPORT";

    /**
     * The Constant ECRAN_IMPORT.
     */
    private static final String ECRAN_IMPORT = "SAISIE_IMPORT";

    /** The Constant ECRAN_RESULTAT. */
    private static final String ECRAN_RESULTAT = "RESULTAT";

    /**
     * The Constant ECRAN_TRANSFORMATION.
     */
    private static final String ECRAN_TRANSFORMATION = "SAISIE_TRANSFORMATION";

    private static final String ECRAN_REDIRECTION = "REDIRECTION";

    /**
     * Constructeur.
     *
     * @param ciu com.jsbsoft.jtf.core.InfoBean
     */
    public TraitementFichierXML(final InfoBean ciu) {
        super(ciu);
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     * @throws Exception the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        try {
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            final AutorisationBean autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
            // retour sur login si utilisateur non connecte
            if (autorisations == null) {
                infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
                ecranLogique = "LOGIN";
            } else if (!ComposantImportExport.isAutoriseParActionProcessus(autorisations, action)) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            } else {
                if (ecranLogique == null) {
                    if (action.equals("IMPORTER")) {
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
                        preparerIMPORT();
                    }
                    if (action.equals("EXPORTER")) {
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
                        preparerEXPORT();
                    }
                    if (action.equals("TRANSFORMER")) {
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
                        preparerTRANSFORMATION();
                    }
                    if (action.equals("EXPORTER_FICHIER")) {
                        traiterEXPORT();
                    }
                } else if (ecranLogique.equals(ECRAN_IMPORT)) {
                    traiterIMPORT();
                } else if (ecranLogique.equals(ECRAN_EXPORT)) {
                    traiterEXPORT();
                } else if (ecranLogique.equals(ECRAN_TRANSFORMATION)) {
                    traiterTRANSFORMATION();
                }
            }
            //placer l'état dans le composant d'infoBean
            infoBean.setEcranLogique(ecranLogique);
        } catch (final Exception e) {
            LOG.error(e.getMessage(), e);
            infoBean.addMessageErreur(e.toString());
        }
        return etat == FIN;
    }

    /**
     * Preparer export.
     *
     * @throws Exception the exception
     */
    private void preparerEXPORT() throws Exception {
        infoBean.set("SERIALISATION", "");
        ecranLogique = ECRAN_EXPORT;
    }

    /**
     * Preparer import.
     *
     * @throws Exception the exception
     */
    private void preparerIMPORT() throws Exception {
        infoBean.set("LISTE_SOURCES", TraitementImportUtil.getListeSourcePourInfoBean());
        infoBean.set("LISTE_TRANSFORMATIONS", TraitementImportUtil.getListeTransformationPourInfoBean());
        infoBean.set("LISTE_OBJETS", TraitementImportUtil.getListeObjetPourInfoBean());
        infoBean.set(UploadedFile.KEY_FILE_EXTENSIONS + "_XML_FICHIER", "xml");
        infoBean.set("OBJET_IMPORT", "utilisateur");
        ecranLogique = ECRAN_IMPORT;
        if (TYPE_IMPORT_UTILISATEUR.equals(infoBean.get(CLE_TYPE_IMPORT))) {
            infoBean.set("OBJET_IMPORT", "utilisateur");
        }
        if (TYPE_IMPORT_GROUPE.equals(infoBean.get(CLE_TYPE_IMPORT))) {
            infoBean.set("OBJET_IMPORT", "groupe");
        }
    }

    /**
     * Preparer transformation.
     *
     * @throws Exception the exception
     */
    private void preparerTRANSFORMATION() throws Exception {
        ecranLogique = ECRAN_TRANSFORMATION;
    }

    /**
     * Gestion de l'écran d'export XML.
     *
     * @throws Exception the exception
     */
    private void traiterEXPORT() throws Exception {
        if (action.equals(InfoBean.ACTION_VALIDER) || action.equals("EXPORTER_FICHIER")) {
            final ImportExportManager iem = ImportExportManager.getInstance();
            final InfosExport infosExport = new InfosExport();
            infosExport.setCode(IXmlCstes.CODE_EXPORT_BO);
            infosExport.setLibelle("Export Back Office");
            infosExport.setFormaterEnHtml("1".equals(infoBean.getString("FORMATAGE_EN_HTML")));
            infosExport.setInclusionFichierGW("1".equals(infoBean.getString("INCLUSION_FICHIERGW")));
            infosExport.setInclusionOT("1".equals(infoBean.getString("INCLUSION_OT")));
            if (infoBean.getString("SERIALISATION") != null && infoBean.getString("SERIALISATION").length() > 0) {
                infosExport.setSerialisation(infoBean.getString("SERIALISATION"));
            } else {
                infosExport.setSerialisation(IXmlCstes.CODE_SERIALISATION_XML);
            }
            iem.setInfosExport(infosExport);
            infoBean.set(IXmlCstes.CODE_EXPORT, IXmlCstes.CODE_EXPORT_BO);
            final IExtractor extracteur = iem.getExportManager(IXmlCstes.CODE_EXPORT_BO).getExtracteur();
            final File exportPdfFile = new File(WebAppUtil.getDownloadPrivatePath(), extracteur.getFileName());
            new Exporter(iem).execute(infoBean, this, exportPdfFile);
            // Génération du fichier d'export
            final String url = PropertyHelper.getCoreProperty(FileReaderServlet.URL_PROPERTY) + "?FILE=" + exportPdfFile.getName() + "&TYPE=" + extracteur.getContentType();
            if (action.equals("EXPORTER_FICHIER")) {
                infoBean.setEcranRedirection(URLResolver.getAbsoluteUrl(url, this));
                ecranLogique = ECRAN_REDIRECTION;
            } else {
                infoBean.set("RESULTAT", "Export terminé : <a download=\"export.xml\" href=\"" + url + "\">télécharger le fichier</a>");
                ecranLogique = ECRAN_RESULTAT;
            }
        }
    }

    /**
     * Gestion de l'écran d'import XML.
     *
     * @throws Exception the exception
     */
    private void traiterIMPORT() throws Exception {
        if (action.equals(InfoBean.ACTION_VALIDER)) {
            final UploadedFile file = (UploadedFile) infoBean.get("XML_FICHIER_FILE");
            final InfosImport infosImport = new InfosImport();
            File f = null;
            if ((file != null) && (StringUtils.isNotEmpty(file.getContentFilename()))) {
                infosImport.setNameFichier(file.getContentFilename());
                f = file.getTemporaryFile();
                infoBean.set("XML_FICHIER_FILE", f.getAbsolutePath());
            } else {
                throw new ErreurApplicative("Veuillez saisir le fichier source");
            }
            try {
                XMLValidator.validate(f, null);
            } catch (final Exception e) {
                LOG.error(e.getMessage(), e);
                throw new ErreurApplicative("Le fichier source n'est pas un document XML valide : " + e.getMessage());
            }
            String res = "";
            final ImportExportManager iem = ImportExportManager.getInstance();
            infosImport.setCode(IXmlCstes.CODE_IMPORT_BO);
            infosImport.setLibelle("Import Back Office");
            infosImport.setPathFichier(f.getPath());
            final String infoBeanTransformation = infoBean.getString("TRANSFORMATION");
            if (StringUtils.isNotEmpty(infoBeanTransformation) && !infoBeanTransformation.equals("0000")) {
                infosImport.addTransformation(infoBeanTransformation);
            }
            if (IXmlCstes.TYPE_IMPORT_SYNCHRONISE.equalsIgnoreCase(infoBean.getString("TYPE_IMPORT"))) {
                final String infoBeanSourceImport = infoBean.getString("SOURCE_IMPORT");
                if (StringUtils.isNotEmpty(infoBeanSourceImport) && !infoBeanSourceImport.equals("0000")) {
                    infosImport.setSourceImport(infoBeanSourceImport);
                }
                final String infoBeanObjetImport = infoBean.getString("OBJET_IMPORT");
                if (StringUtils.isNotEmpty(infoBeanObjetImport) && !infoBeanObjetImport.equals("0000")) {
                    infosImport.setTypeImport(infoBeanObjetImport);
                } else {
                    throw new ErreurApplicative("Veuillez spécifier le type d'objet importé");
                }
                if (infoBean.getString("SYNCHRONIZED_IMPORT") != null) {
                    infosImport.setSynchronizedImport(true);
                }
                if (infoBean.getString("ALL_IMPORT") != null) {
                    infosImport.setAllSourceImport(true);
                }
            }
            TraitementImportUtil.processInfosImport(infosImport);
            iem.setInfosImport(infosImport);
            final IImporter importManager = iem.getImportManager(IXmlCstes.CODE_IMPORT_BO);
            res = importManager.execute(this);
            ecranLogique = ECRAN_RESULTAT;
            infoBean.set("RESULTAT", res);
        }
    }

    /**
     * Gestion de l'écran d'export XML.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterTRANSFORMATION() throws Exception {
        if (action.equals(InfoBean.ACTION_VALIDER)) {
            String exportFile = null;
            /* Récupération fichier XML */
            UploadedFile file = (UploadedFile) infoBean.get("XML_FICHIER_FILE");
            File fXML = null;
            if ((file != null) && (file.getContentFilename().length() != 0)) {
                fXML = file.getTemporaryFile();
            } else {
                throw new ErreurApplicative("Fichier XML non renseigné");
            }
            /**************************************************/
            /*   Transformation XSL                           */
            /**************************************************/
            file = (UploadedFile) infoBean.get("XSL_FICHIER_FILE");
            File fXSL = null;
            if ((file != null) && (file.getContentFilename().length() != 0)) {
                fXSL = file.getTemporaryFile();
                exportFile = transformerXML(fXML, fXSL, infoBean.getString("TYPE_TRANSFORMATION"));
            }
            /**************************************************/
            /*    Envoi du fichier XML                        */
            /**************************************************/
            final JSBMailbox mailbox = new JSBMailbox(false);
            mailbox.sendSystemMsgWithAttachedFiles(infoBean.getString("EMAIL"), "fichier XML", "voir la piece attachée ...", new String[] {exportFile});
            ecranLogique = ECRAN_RESULTAT;
            infoBean.set("RESULTAT", "L'export s'est déroulé correctement . ");
        }
    }

    /**
     * Transformation XSL d'un fichier XML.
     *
     * @param _fXML
     *            the _f xml
     * @param _fXSL
     *            the _f xsl
     * @param _typeTransformation
     *            the _type transformation
     *
     * @return the string
     *
     * @throws Exception
     *             the exception
     */
    private String transformerXML(final File _fXML, final File _fXSL, final String _typeTransformation) throws Exception {
        final File fPathExport = new File(WebAppUtil.getDownloadPrivatePath());
        File fExportFile = null;
        FileOutputStream os = null;
        Transformer transformer = null;
        if (_typeTransformation.equals("0001")) {
            final TransformerFactory tFactory = TransformerFactory.newInstance();
            fExportFile = new File(fPathExport, System.currentTimeMillis() + ".export");
            os = new FileOutputStream(fExportFile);
            transformer = tFactory.newTransformer(new StreamSource(_fXSL));
            /*
			The transformation is accomplished by invoking the transform method,
			passing it the the data and the output stream.
			 */
            transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
            transformer.transform(new StreamSource(_fXML), new StreamResult(os));
            os.close();
        }
        if (_typeTransformation.equals("0002")) {
            /* Transformation XML -> FO */
            final File fExportFile2 = new File(fPathExport, System.currentTimeMillis() + ".fo");
            os = new FileOutputStream(fExportFile2);
            final TransformerFactory tFactory = TransformerFactory.newInstance();
            transformer = tFactory.newTransformer(new StreamSource(_fXSL));
            transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
            transformer.transform(new StreamSource(_fXML), new StreamResult(os));
            os.close();
            /* Transformation PDF */
            fExportFile = new File(fPathExport, System.currentTimeMillis() + ".pdf");
            os = new FileOutputStream(fExportFile);
            //PDFManipulateur.createPDF( _fXML, _fXSL, os );
            final InputStream in = new java.io.FileInputStream(fExportFile2);
            PDFManipulateur.createPDF(in, os);
            os.close();
        }
        return fExportFile.getAbsolutePath();
    }
}
