/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.util;

import java.beans.PropertyDescriptor;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;

import org.apache.commons.beanutils.BeanPropertyValueEqualsPredicate;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.upload.UploadedFile;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.Objetpartage;
import com.kportal.extension.IExtension;
import com.univ.batch.imports.ImportXML;
import com.univ.objetspartages.bean.PersistenceBean;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.AbstractOm;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.xml.IXmlCstes;
import com.univ.xml.ImportExportManager;
import com.univ.xml.TraitementImportHelper;
import com.univ.xml.XMLValidator;
import com.univ.xml.imports.EnteteImport.Action;
import com.univ.xml.imports.IImporter;
import com.univ.xml.util.InfosImport;

/**
 * Classe utilitaire pour le traitement d'import.
 */
public class TraitementImportUtil {

    /**
     * Nom de la clé du type d'import dans l'infobean.
     */
    public static final String CLE_TYPE_IMPORT = "TYPE_OBJET";

    /**
     * Type d'import pour un import utilisateur.
     */
    public static final String TYPE_IMPORT_UTILISATEUR = "UTILISATEUR";

    /**
     * Type d'import pour un import groupe.
     */
    public static final String TYPE_IMPORT_GROUPE = "GROUPE";

    /**
     * Type d'import pour un import de fiche.
     */
    public static final String TYPE_IMPORT_FICHE = "FICHE";

    /** The Constant ECRAN_EXPORT. */
    public static final String ECRAN_EXPORT = "SAISIE_EXPORT";

    /** The Constant ECRAN_IMPORT. */
    public static final String ECRAN_IMPORT = "SAISIE_IMPORT";

    /** The Constant ECRAN_RESULTAT. */
    public static final String ECRAN_RESULTAT = "RESULTAT";

    /** The Constant ECRAN_TRANSFORMATION. */
    public static final String ECRAN_TRANSFORMATION = "SAISIE_TRANSFORMATION";

    /** The Constant PATH_EXPORT. */
    public static final String PATH_FOLDER_EXPORT = "export";

    /**
     * Clé de la propriété d'activation du choix d'une transformation XML pour l'import.
     */
    public static final String CLE_ACTIVATION_TRANSFORMATION_IMPORT = "import.transformation";

    /**
     * Code par défaut d'un libellé.
     */
    private static final String CODE_LIBELLE_DEFAUT = "0000";

    private static final Logger LOG = LoggerFactory.getLogger(TraitementImportUtil.class);

    public static final String IMPORT_NOEUDSVIDES = "import.noeudsvides";

    /**
     * Service de gestion des {@link LabelDto}
     */
    private static ServiceLabel serviceLabel;

    /**
     * Constructeur par défaut.
     */
    private TraitementImportUtil() {
    }

    /**
     * Indique si le choix d'une transformation XML est activé.
     * Cette activation est faite via la propriété {@link #CLE_ACTIVATION_TRANSFORMATION_IMPORT}.
     *
     * @return true si le choix est activé, false sinon.
     */
    public static boolean isChoixTransformationXMLActive() {
        return isChoixTransformationXMLActive(ExtensionUtil.getExtension());
    }

    /**
     * Indique si le choix d'une transformation XML est activé.
     * Cette activation est faite via la propriété {@link #CLE_ACTIVATION_TRANSFORMATION_IMPORT}.
     *
     * @param extension
     * 		référence vers l'extension pour permettre une récupération des propriétés pendant les phases d'initialisation / rechargement des extensions.
     * @return true si le choix est activé, false sinon.
     */
    public static boolean isChoixTransformationXMLActive(final IExtension extension) {
        return "1".equals(extension.getProperty(CLE_ACTIVATION_TRANSFORMATION_IMPORT));
    }

    /**
     * Retourne le code du {@link LabelDto} du type indiqué à partir de son libellé.
     * Cette méthode est destinée à être appelée depuis une feuille XSL (nécessite le chargement du contexte Spring).
     *
     * @param typeLibelle
     * 		le type du LabelDto.
     * @param libelle
     * 		l'intitulé du LabelDto.
     * @return le code du LabelDto correspondant, {@link org.apache.commons.lang3.StringUtils}.EMPTY si aucune correspondance n'a été trouvée, {@link #CODE_LIBELLE_DEFAUT} si le code n'a pas pu être récupéréé.
     */
    public static String lireCodeLabelDepuisLibelle(final String typeLibelle, final String libelle) {
        return lireCodeLabelDepuisLibelle(typeLibelle, libelle, null);
    }

    /**
     * Retourne le code du {@link LabelDto} du type indiqué à partir de son libellé.
     * Cette méthode est destinée à être appelée depuis une feuille XSL (nécessite le chargement du contexte Spring).
     *
     * @param typeLibelle
     * 		le type du LabelDto.
     * @param libelle
     * 		l'intitulé du LabelDto.
     * @param langue
     * 		la langue du LabelDto.
     * @return le code du LabelDto correspondant, {@link org.apache.commons.lang3.StringUtils}.EMPTY si aucune correspondance n'a été trouvée, {@link #CODE_LIBELLE_DEFAUT} si le code n'a pas pu être récupéréé.
     */
    public static String lireCodeLabelDepuisLibelle(final String typeLibelle, final String libelle, String langue) {
        String code = StringUtils.EMPTY;
        // Si le service a bien été injecté par Spring
        if (serviceLabel != null) {
            if (StringUtils.isBlank(langue)) {
                langue = LangueUtil.getLangueLocale(LangueUtil.getDefaultLocale());
            }
            final List<LabelDto> listeLabels = serviceLabel.getByTypeMatchingLibelle(typeLibelle, libelle);
            if (CollectionUtils.isNotEmpty(listeLabels)) {
                // Filtrage des Label dont la langue correspond à celle recherchée
                CollectionUtils.filter(listeLabels, new BeanPropertyValueEqualsPredicate("langue", langue));
                // Si des Label correspondent à la langue, on récupère le premier
                if (CollectionUtils.isNotEmpty(listeLabels)) {
                    code = listeLabels.get(0).getCode();
                }
            }
        } else {
            // Si pas de service injecté, on informe l'utilisateur que la méthode nécessite le chargement d'un contexte Spring
            LOG.warn("Un contexte Spring est nécessaire au bon fonctionnement de cette méthode.");
            code = CODE_LIBELLE_DEFAUT;
        }
        return code;
    }

    /**
     * Retourne la liste des transformations XML disponibles pour l'écran d'import manuel.
     *
     * @return
     */
    public static Hashtable<String, String> getListeTransformationPourInfoBean() {
        int i = 1;
        String param;
        final Hashtable<String, String> transformationsPossibles = new Hashtable<String, String>();
        while ((param = ExtensionUtil.getProperty("importsexports.transformationimport" + i + ".source")) != null) {
            final String libelle = ExtensionUtil.getProperty("importsexports.transformationimport" + i + ".libelle");
            transformationsPossibles.put(param, StringUtils.defaultIfEmpty(libelle, "Transformation " + i));
            i++;
        }
        return transformationsPossibles;
    }

    /**
     * Retourne la liste des transformations XML disponibles pour le script d'import automatisé.
     *
     * @param extension
     * 		référence vers l'extension pour permettre une récupération des propriétés pendant les phases d'initialisation / rechargement des extensions.
     * @return une chaîne contenant les noms des fichiers XSL séparés par des ";".
     */
    public static String getListeTransformationPourScriptAutomatise(final IExtension extension) {
        final List<String> listeTransformations = new ArrayList<>();
        // Ajout d'une transformation fictive "Aucune" pour permettre de ne pas en sélectionner (car la sélection est obligatoire)
        listeTransformations.add(ImportXML.AUCUNE_TRANSFORMATION_XML);
        int i = 1;
        String param;
        while ((param = extension.getProperty("importsexports.transformationimport" + i + ".source")) != null) {
            listeTransformations.add(param);
            i++;
        }
        return StringUtils.join(listeTransformations, ";");
    }

    /**
     * Retourne la liste des patterns de fichiers XML disponibles pour le script d'import automatisé.
     *
     * @param extension
     * 		référence vers l'extension pour permettre une récupération des propriétés pendant les phases d'initialisation / rechargement des extensions.
     * @return une chaîne contenant les patterns des fichiers XML séparés par des ";".
     */
    public static String getListePatternsFichiersPourScriptAutomatise(final IExtension extension) {
        final List<String> listePatterns = new ArrayList<>();
        // Ajout d'un pattern fictif "Tous" pour permettre de ne pas en sélectionner (car la sélection est obligatoire)
        listePatterns.add(ImportXML.TOUS_LES_FICHIERS);
        int i = 1;
        String param;
        while ((param = extension.getProperty("import.fichiers.pattern" + i)) != null) {
            listePatterns.add(param);
            i++;
        }
        return StringUtils.join(listePatterns, ";");
    }

    public static Hashtable<String, String> getListeSourcePourInfoBean() {
        int i = 1;
        String param;
        final Hashtable<String, String> sourcesPossibles = new Hashtable<String, String>();
        while ((param = ExtensionUtil.getProperty("importsexports.source" + i + ".code")) != null) {
            final String libelle = ExtensionUtil.getProperty("importsexports.source" + i + ".libelle");
            sourcesPossibles.put(param, StringUtils.defaultIfEmpty(libelle, "Source " + i));
            i++;
        }
        return sourcesPossibles;
    }

    public static Hashtable<String, String> getListeObjetPourInfoBean() throws Exception {
        final Hashtable<String, String> objetsKPortalDispo = new Hashtable<>();
        for (final Objetpartage objet : ReferentielObjets.getObjetsPartagesTries()) {
            if (!objet.isStrictlyCollaboratif()) {
                objetsKPortalDispo.put(objet.getNomObjet(), objet.getLibelleObjet());
            }
        }
        // Ajout des Utilisateurs et des groupes
        objetsKPortalDispo.put("utilisateur", "Utilisateur");
        objetsKPortalDispo.put("groupe", "Groupe");
        return objetsKPortalDispo;
    }

    public static void importerDonnees(final InfoBean infoBean, final OMContext ctx) throws Exception {
        final UploadedFile file = (UploadedFile) infoBean.get("XML_FICHIER_FILE");
        File f = null;
        if ((file != null) && (file.getContentFilename().length() != 0)) {
            f = file.getTemporaryFile();
            infoBean.set("XML_FICHIER_FILE", f.getAbsolutePath());
        } else {
            throw new ErreurApplicative("Veuillez saisir le fichier source");
        }
        try {
            XMLValidator.validate(f, null);
        } catch (final Exception e) {
            throw new ErreurApplicative("Le fichier source n'est pas un document XML valide : " + e.getMessage());
        }
        String res = "";
        final ImportExportManager iem = ImportExportManager.getInstance();
        final InfosImport infosImport = new InfosImport();
        infosImport.setCode(IXmlCstes.CODE_IMPORT_BO);
        infosImport.setLibelle("Import Back Office");
        infosImport.setPathFichier(f.getPath());
        final String infoBeanTransformation = infoBean.getString("TRANSFORMATION");
        if (StringUtils.isNotEmpty(infoBeanTransformation) && !infoBeanTransformation.equals("0000")) {
            infosImport.addTransformation(infoBeanTransformation);
        }
        if (IXmlCstes.TYPE_IMPORT_SYNCHRONISE.equalsIgnoreCase(infoBean.getString("TYPE_IMPORT"))) {
            final String infoBeanSourceImport = infoBean.getString("SOURCE_IMPORT");
            if (StringUtils.isNotEmpty(infoBeanSourceImport) && !infoBeanSourceImport.equals("0000")) {
                infosImport.setSourceImport(infoBeanSourceImport);
            }
            final String infoBeanObjetImport = infoBean.getString("OBJET_IMPORT");
            if (StringUtils.isNotEmpty(infoBeanObjetImport) && !infoBeanObjetImport.equals("0000")) {
                infosImport.setTypeImport(infoBeanObjetImport);
            } else {
                throw new ErreurApplicative("Veuillez spécifier le type d'objet importé");
            }
            if (infoBean.getString("SYNCHRONIZED_IMPORT") != null) {
                infosImport.setSynchronizedImport(true);
            }
            if (infoBean.getString("ALL_IMPORT") != null) {
                infosImport.setAllSourceImport(true);
            }
        }
        processInfosImport(infosImport);
        iem.setInfosImport(infosImport);
        final IImporter importManager = iem.getImportManager(IXmlCstes.CODE_IMPORT_BO);
        res = importManager.execute(ctx);
        infoBean.set("RESULTAT", res);
    }

    /**
     * Gets the full text string.
     *
     * @param ctx
     * 		the ctx
     * @param fiche
     * 		the fiche
     * @return the full text string
     * @throws Exception
     * 		the exception
     */
    public static String getFullTextString(final OMContext ctx, final FicheUniv fiche) throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        String res = serviceStructure.getDisplayableLabel(fiche.getCodeRattachement(), fiche.getLangue());
        if (res.length() > 0) {
            res += " ";
        }
        return res;
    }

    /**
     * Renvoie la valeur d'une propriété d'import en prenant en charge les spécificités liées aux sources d'import.
     * Exemple :
     * Pour la clef import.FORMATION.LIBELLE_COURT, il faut appeler la méthode de cette façon : TraitementImportUtil.getPropertyImport(sourceImport, "FORMATION.LIBELLE_COURT").
     * La méthode se charge de concaténer les composants de la clef de la propriété, puis d'ajouter la source d'import à la clef.
     * Elle récupère d'abord la valeur spécifique à la source d'import si elle existe, sinon elle renvoie la valeur générique.
     * @param sourceImport La source d'import
     * @param clef La clef sans préfixe.
     * @return Valeur de la propriété demandée.
     */
    public static String getPropertyImport(String sourceImport, String clef) {
        if (StringUtils.isNotEmpty(sourceImport)) {
            return StringUtils.defaultIfBlank(ExtensionUtil.getProperty("import." + sourceImport + "." + clef), ExtensionUtil.getProperty("import." + clef));
        }
        return ExtensionUtil.getProperty("import." + clef);
    }

    public static String toSnakeCase(String camelCase) {
        return camelCase.replaceAll("(.)(\\p{Lu})", "$1_$2");
    }

    /**
     * Synchronisation entre la fiche tampon (à l'image du flux XML) et la fiche cible (nouvelle ou existante).
     *
     * @param ficheCible
     * 		La fiche cible
     * @param ficheTampon
     * 		La fiche "buffer", mapping du flux XML avec un objet fiche
     * @param action
     * @param infosImport Informations d'import
     * @return the object
     */
    public static Object synchronizeObject(final Object ficheCible, final Object ficheTampon, final Action action, final InfosImport infosImport) {
        // test si les deux objets sont de la même classe
        if (!ficheCible.getClass().equals(ficheTampon.getClass())) {
            return ficheCible;
        }
        final NullAwareBeanUtils nullAwareBeanUtils = TraitementImportHelper.getNullAwareBeanUtils();
        Long idFiche = (long) 0;
        String etat = StringUtils.EMPTY;
        String langue = StringUtils.EMPTY;
        String nomObjet = StringUtils.EMPTY;
        if (ficheCible instanceof FicheUniv) {
            idFiche = ((FicheUniv) ficheCible).getIdFiche();
            etat = ((FicheUniv) ficheCible).getEtatObjet();
            langue = ((FicheUniv) ficheCible).getLangue();
            nomObjet = ReferentielObjets.getNomObjet((FicheUniv) ficheCible);
        }
        synchronizeDateCreation(ficheCible, ficheTampon, action);
        final Object objectSync = AbstractOm.class.isInstance(ficheCible) ? ((AbstractOm) ficheCible).getPersistenceBean() : ficheCible;
        final Object importedObject = AbstractOm.class.isInstance(ficheTampon) ? ((AbstractOm) ficheTampon).getPersistenceBean() : ficheTampon;
        PropertyDescriptor[] properties = PropertyUtils.getPropertyDescriptors(objectSync.getClass());
        for (PropertyDescriptor currentProperty : properties) {
            final String importDonnee = getPropertyImport(infosImport.getSourceImport(), nomObjet.toUpperCase() + "." + toSnakeCase(currentProperty.getName()).toUpperCase());
            if (currentProperty.getWriteMethod() != null && (action == Action.AJOUTER || action == Action.MODIFIER || (action == Action.IMPORTER && (StringUtils.isEmpty(importDonnee) || !importDonnee.equals("0"))))) {
                try {
                    Object value = PropertyUtils.getProperty(importedObject, currentProperty.getName());
                    // On ne recopie de la fiche tampon vers la fiche à synchroniser que les champs non null (cela permet d'ignorer les champs non transmis).
                    nullAwareBeanUtils.copyProperty(objectSync, currentProperty.getName(), value);
                } catch (IllegalAccessException | InvocationTargetException e) {
                    LOG.error("La propriété " + currentProperty.getName() + " n'a pas pu être synchronisée.", e);
                } catch (NoSuchMethodException e) {
                    if(LOG.isDebugEnabled()) {
                        LOG.debug("La propriété " + currentProperty.getName() + " n'a pas pu être synchronisée : pas de getter associé.", e);
                    } else {
                        LOG.warn("La propriété " + currentProperty.getName() + " n'a pas pu être synchronisée : pas de getter associé.");
                    }
                }
            }
        }
        if (AbstractOm.class.isInstance(ficheCible)) {
            ((AbstractOm) ficheCible).setPersistenceBean((PersistenceBean) objectSync);
        }
        // remise en place des anciennes valeurs
        if (ficheCible instanceof FicheUniv) {
            ((FicheUniv) ficheCible).setIdFiche(idFiche);
            ((FicheUniv) ficheCible).setEtatObjet(etat);
            ((FicheUniv) ficheCible).setLangue(langue);
        }
        return ficheCible;
    }

    /**
     * Permet de positionner la date de création de la fiche éxistante sur la fiche qui sera importée. 
     * Excepté dans le cadre d'une création de fiche ou l'on garde la date positionné par l'import ou l'initialisation de la fiche
     * La Date de création est reprise si les deux objets sont des implémentations de {@link FicheUniv} , et si la date de création est existante sur le fiche référence
     * @param ficheExistante
     * @param ficheAImporter
     * @param action
     */
    public static void synchronizeDateCreation(final Object ficheExistante, final Object ficheAImporter, final Action action) {
        if (Action.AJOUTER != action && ficheExistante instanceof FicheUniv && ficheAImporter instanceof FicheUniv && ((FicheUniv) ficheExistante).getDateCreation() != null) {
            ((FicheUniv) ficheAImporter).setDateCreation(((FicheUniv) ficheExistante).getDateCreation());
        }
    }

    /**
     * Méthode permettant de tester si une classe est de type primitf ou string
     *
     * @param clazz
     * @return
     */
    public static boolean isPrimitiveOrStringType(final Class<?> clazz) {
        if (getPrimitiveTypes().contains(clazz)) {
            return true;
        } else {
            return String.class.equals(clazz);
        }
    }

    private static Set<Class<?>> getPrimitiveTypes() {
        final Set<Class<?>> ret = new HashSet<>();
        ret.add(boolean.class);
        ret.add(char.class);
        ret.add(byte.class);
        ret.add(short.class);
        ret.add(int.class);
        ret.add(long.class);
        ret.add(float.class);
        ret.add(double.class);
        ret.add(void.class);
        return ret;
    }

    /**
     * A partir d'un paramétrage basique d'import, récupération des propriétés associées.
     * <ul>
     *     <li>Import de noeuds vides</li>
     * </ul>
     * @param infosImport Paramétrage d'import
     */
    public static void processInfosImport(final InfosImport infosImport) {
        infosImport.setImportNoeudsVides("1".equals(TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), "noeudsvides")));
    }

    @Required
    public static final void setServiceLabel(final ServiceLabel serviceLabel) {
        TraitementImportUtil.serviceLabel = serviceLabel;
    }
}
