/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws;

import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.batch.LogReportThread;
import com.univ.pws.om.SourceSynchronisation;
import com.univ.pws.om.Synchronisation;
import com.univ.utils.ContexteUtil;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class SynchroXmlRpcClient extends LogReportThread {

    private static final Logger LOG = LoggerFactory.getLogger(SynchroXmlRpcClient.class);

    private static final Object LOCK = new Object();

    private Synchronisation synchronisation = null;

    /**
     * @return Returns the synchronisation.
     */
    public Synchronisation getSynchronisation() {
        return synchronisation;
    }

    /**
     * @param synchronisation The synchronisation to set.
     */
    public void setSynchronisation(final Synchronisation synchronisation) {
        this.synchronisation = synchronisation;
    }

    @Override
    public void perform() {
        // ajout d'un contexte dans le thread local
        ContexteUtil.setContexteSansRequete();
        ContexteUtil.getContexteUniv().setConnectionLongue(true);
        synchronized (LOCK) {
            if (!isHwsRunning()) {
                // on recontrole en base
                synchronisation.setEtat(Synchronisation.ETAT_EN_COURS);
                synchronisation.setCtx(ContexteUtil.getContexteUniv());
                try {
                    synchronisation.update();
                } catch (final Exception e) {
                    LOG.error("Activation de la source impossible " + synchronisation.getLibelle(), e);
                }
            } else {
                LOG.error("Synchronisation d'une source de contenu en cours, impossible d'éxecuter " + synchronisation.getLibelle());
                ContexteUtil.releaseContexteUniv();
                return;
            }
        }
        InputStream xmlInputStream = null;
        try {
            LOG.info("Synchronisation : " + synchronisation.getLibelle());
            LOG.info("Source : " + synchronisation.getSourceSynchronisation().getLibelle());
            LOG.info("Collecte des fiches avant le synchronisation");
            // appel procédure distante
            if (synchronisation.getFichierSynchro() == null) {
                LOG.info("Appel du web service");
                final SourceSynchronisation sourceSynchronisation = synchronisation.getSourceSynchronisation();
                final ExportFicheHandler export = new ExportFicheHandler(sourceSynchronisation.getUrl());
                export.setLogger(LOG);
                final String result = export.execute(sourceSynchronisation.getLogin(), sourceSynchronisation.getPassword(), synchronisation.getRequete());
                String charset = CharEncoding.DEFAULT;
                if (StringUtils.containsIgnoreCase(result, org.apache.commons.codec.CharEncoding.ISO_8859_1)) {
                    charset = org.apache.commons.codec.CharEncoding.ISO_8859_1;
                }
                xmlInputStream = new ByteArrayInputStream(result.getBytes(charset));
            } else {
                LOG.info("Import du fichier");
                xmlInputStream = new FileInputStream(synchronisation.getFichierSynchro());
            }
            boolean transfoXSL = false;
            final String xslPath = ExtensionUtil.getXslPath();
            File xslFile = null;
            if ((synchronisation.getSource() != null) && (xslPath != null)) {
                xslFile = new File(xslPath + File.separator + "PWC_" + synchronisation.getSource() + ".xsl");
                if (xslFile.exists()) {
                    transfoXSL = true;
                }
            }
            Node nodeResult = null;
            if (transfoXSL) {
                LOG.info("Transformation XSL");
                Transformer transformer = null;
                final TransformerFactory tFactory = TransformerFactory.newInstance();
                transformer = tFactory.newTransformer(new StreamSource(xslFile));
                transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
                final DOMResult domResult = new DOMResult();
                final StreamSource source = new StreamSource(xmlInputStream);
                transformer.transform(source, domResult);
                nodeResult = domResult.getNode().getFirstChild();
            } else {
                final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                final DocumentBuilder builder = factory.newDocumentBuilder();
                final Document document = builder.parse(xmlInputStream);
                nodeResult = document.getDocumentElement();
            }
            LOG.info("Traitement de l'import");
            LOG.debug(nodeResult.getNodeName());
            LOG.info(ImportFicheIntrospection.importNode(ContexteUtil.getContexteUniv(), nodeResult, synchronisation));
        } catch (final Exception e) {
            LOG.error("Erreur XML-RPC", e);
        } finally {
            synchronized (LOCK) {
                synchronisation.setEtat(Synchronisation.ETAT_ATTENTE);
                try {
                    synchronisation.update();
                } catch (final Exception e) {
                    LOG.error("Erreur XML-RPC", e);
                }
            }
            // suppression du contexte du thread local
            ContexteUtil.releaseContexteUniv();

            // Fermeture du flux
            if (xmlInputStream != null) {
                try {
                    xmlInputStream.close();
                } catch (final IOException e) {
                    LOG.error("Erreur lors de la fermeture du flux", e);
                }
            }
        }
    }

    public boolean isHwsRunning() {
        try {
            final Synchronisation sync = new Synchronisation();
            sync.init();
            sync.setCtx(ContexteUtil.getContexteUniv());
            sync.select("WHERE ETAT='1'");
            return sync.nextItem();
        } catch (final Exception e) {
            LOG.error("Erreur XML-RPC", e);
        }
        return Boolean.TRUE;
    }
}
