/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.univ.utils.ContexteUniv;
import com.univ.xml.exports.ExportManager;
import com.univ.xml.extraction.ExtracteurFactory;
import com.univ.xml.imports.IImporter;
import com.univ.xml.imports.ImportManager;
import com.univ.xml.util.ImportExportBuilder;
import com.univ.xml.util.InfosExport;
import com.univ.xml.util.InfosImport;
// TODO: Auto-generated Javadoc

/**
 * Gestionnaire du module XML interne au produit (singleton).
 */
public class ImportExportManager implements IImportExportManager, IXmlCstes {

    /** Logger. */
    private static Logger _log = LoggerFactory.getLogger(ImportExportManager.class);

    /** Instance unique de la classe (pattern singleton). */
    private static ImportExportManager _instance = null;

    /**
     * Table contenant le paramétrage des imports<br />
     * clé: <code>java.lang.String</code> représentant le code de l'import valeur: <code>com.kosmos.carif.xml.InfosImport</code> représentant un objet de description de l'import
     */
    private Map<String, InfosImport> tableInfosImport = null;

    /**
     * Table contenant le paramétrage des exports<br />
     * clé: <code>java.lang.String</code> représentant le code de l'export valeur: <code>com.kosmos.carif.xml.InfosExport</code> représentant un objet de description de l'export
     */
    private Map<String, InfosExport> tableInfosExport = null;

    /** Instance unique du gestionnaire d'import. */
    private TransformateurXSL transXSL = null;

    /**
     * constructeur privé.
     */
    private ImportExportManager() {}

    /**
     * Méthode retournant l'instance unique de la classe.
     *
     * @return instance unique de la classe
     */
    public static ImportExportManager getInstance() {
        if (_instance == null) {
            synchronized (ImportExportManager.class) {
                if (_instance == null) {
                    _instance = new ImportExportManager();
                    _instance.initialiser();
                }
            }
        }
        return _instance;
    }

    /**
     * Méthode permettant de retourner l'instance du gestionnaire d'export de libellés.
     *
     * @param codeExport
     *            code de l'export paramétré
     *
     * @return instance du gestionnaire d'export
     *
     * @throws ErreurApplicative
     *             si le code est inconnu, et si le format d'export paramétré n'est pas connu
     */
    public static ExportManager getIExportLibelle(String codeExport) throws ErreurApplicative {
        InfosExport infosExport = getInstance().getInfosExport(codeExport);
        return new ExportManager(infosExport, ExtracteurFactory.newInstance(infosExport.getSerialisation()), getInstance().transXSL);
    }

    /**
     * Gets the infos export.
     *
     * @param codeExport
     *            the code export
     *
     * @return the infos export
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     *
     */
    @Override
    public InfosExport getInfosExport(String codeExport) throws ErreurApplicative {
        if (!tableInfosExport.containsKey(codeExport)) {
            String messErr = "Le code '" + codeExport + "' ne correspond pas à un export paramétré";
            _log.error(messErr);
            throw new ErreurApplicative(messErr);
        }
        return tableInfosExport.get(codeExport);
    }

    /**
     * Méthode permettant d'ajouter les paramètres d'import.
     *
     * @param infosExport
     *            the infos export
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     *
     */
    public void setInfosExport(InfosExport infosExport) throws ErreurApplicative {
        tableInfosExport.put(infosExport.getCode(), infosExport);
    }

    /**
     * Méthode retournant une classe permettant d'effectuer un export.
     *
     * @param codeExport
     *            code de l'export pour lequel on souhaite obtenir un export
     * @param ctx
     *            contexte
     *
     * @return the export manager
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     */
    public ExportManager getExportManager(String codeExport, ContexteUniv ctx) throws ErreurApplicative {
        InfosExport infosExport = getInstance().getInfosExport(codeExport);
        return new ExportManager(ctx, infosExport, ExtracteurFactory.newInstance(infosExport.getSerialisation()), getInstance().transXSL);
    }

    /**
     * Méthode permettant de retourner les paramètres d'import en fonction d'un code d'imports.
     *
     * @param codeImport
     *            the code import
     *
     * @return the infos import
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     *
     */
    @Override
    public InfosImport getInfosImport(String codeImport) throws ErreurApplicative {
        if (!tableInfosImport.containsKey(codeImport)) {
            String messErr = "Le code '" + codeImport + "' ne correspond pas à un import paramétré";
            _log.error(messErr);
            throw new ErreurApplicative(messErr);
        }
        return tableInfosImport.get(codeImport);
    }

    /**
     * Méthode permettant d'ajouter les paramètres d'import.
     *
     * @param infosImport
     *            the infos import
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     *
     */
    public void setInfosImport(InfosImport infosImport) throws ErreurApplicative {
        tableInfosImport.put(infosImport.getCode(), infosImport);
    }

    /**
     * Méthode permettant d'initialiser la classe.
     */
    private void initialiser() {
        try {
            transXSL = new TransformateurXSL();
            ImportExportBuilder importExport = chargerMapping();
            if (importExport != null) {
                tableInfosExport = importExport.getTableInfosExport();
                tableInfosImport = importExport.getTableInfosImport();
            }
        } catch (ErreurApplicative e) {
            // on ne fait rien le mapping est chargé à null;
        } finally {
            if (tableInfosImport == null) {
                tableInfosImport = new HashMap<String, InfosImport>();
            }
            if (tableInfosExport == null) {
                tableInfosExport = new HashMap<String, InfosExport>();
            }
        }
    }

    /**
     * Méthode permettant de charger le mapping.
     *
     * @return ImportExport classe contenant le mapping chargé
     *
     * @throws Exception
     *             * @throws ErreurApplicative the erreur applicative
     * @throws ErreurApplicative
     *             the erreur applicative
     */
    private ImportExportBuilder chargerMapping() throws ErreurApplicative {
        ImportExportBuilder importExport = null;
        return importExport;
    }

    /**
     * Gets the trans xsl.
     *
     * @return the trans xsl
     */
    public TransformateurXSL getTransXSL() {
        return transXSL;
    }

    /**
     * Méthode permettant de retourner l'instance du gestionnaire d'import.
     *
     * @param codeImport
     *            the code import
     *
     * @return instance du gestionnaire d'import
     *
     * @throws ErreurApplicative
     *             si le code est inconnu, et si le format d'export paramétré n'est pas connu
     */
    @Override
    public IImporter getImportManager(String codeImport) throws ErreurApplicative {
        InfosImport infosImport = getInfosImport(codeImport);
        return new ImportManager(infosImport, transXSL);
    }

    /**
     * Methode permettant de retourner un extracteur de données pour le code export.
     *
     * @param codeExport
     *            code export
     *
     * @return Extracteur de données
     *
     * @throws ErreurApplicative
     *             si le code export est inconnu
     */
    @Override
    public ExportManager getExportManager(String codeExport) throws ErreurApplicative {
        InfosExport infosExport = getInfosExport(codeExport);
        return new ExportManager(infosExport, ExtracteurFactory.newInstance(infosExport.getSerialisation()), transXSL);
    }
}
