/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import com.univ.objetspartages.om.FicheUniv;
import java.util.HashMap;
import java.util.Map;

public class ResultatImport {

	/**
	 * Indique si l'import est synchronisé.
	 */
	private final boolean synchronise;

	/**
	 * Source de l'import.
	 */
	private final String sourceImport;

	/**
	 * Map des résultats d'import par objet.
	 */
	private final Map<String, ResultatImportObjet> mapResultatImportParObjet;

	/**
	 * Nombre d'erreurs bloquantes.
	 */
	private int nbErreursBloquantes;

	/**
	 * Constructeur par défaut.
	 *
	 * @param synchronise
	 * 		true si l'import est synchronisé.
	 * @param sourceImport
	 * 		source de l'import.
	 */
	ResultatImport(final boolean synchronise, final String sourceImport) {
		this.synchronise = synchronise;
		this.sourceImport = sourceImport;
		this.mapResultatImportParObjet = new HashMap<>();
	}

	public int getNbErreursBloquantes() {
		return nbErreursBloquantes;
	}

	/**
	 * Incrémente le nombre d'erreurs bloquantes.
	 */
	public void incrementNombreErreursBloquantes() {
		nbErreursBloquantes++;
	}

	/**
	 * Incrémente le nombre d'enregistrements pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet concerné.
	 */
	void incrementeNombreEnregistrements(final String nomObjet) {
		getResultatImportObjet(nomObjet).incrementENombreEnregistrements();
	}

	/**
	 * Incrémente le nombre d'erreurs pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet concerné.
	 */
	void incrementeNombreErreurs(final String nomObjet) {
		getResultatImportObjet(nomObjet).incrementeNombreErreurs();
	}

	/**
	 * Incrémente le nombre d'ajouts pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet concerné.
	 */
	public void incrementeNombreElementsAjoutes(final String nomObjet) {
		getResultatImportObjet(nomObjet).incrementeNombreElementsAjoutes();
	}

	/**
	 * Incrémente le nombre de modifications pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet concerné.
	 */
	public void incrementeNombreElementsModifies(final String nomObjet) {
		getResultatImportObjet(nomObjet).incrementeNombreElementsModifies();
	}

	/**
	 * Incrémente le nombre de suppressions pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet concerné.
	 */
	public void incrementeNombreElementsSupprimes(final String nomObjet) {
		getResultatImportObjet(nomObjet).incrementeNombreElementsSupprimes();
	}

	/**
	 * Enregistre l'ajout de la fiche passée en paramètre pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 * @param fiche
	 * 		la fiche.
	 */
	void enregistrerAjoutFiche(final String nomObjet, final FicheUniv fiche) {
		getResultatImportObjet(nomObjet).enregistrerAjoutFiche(fiche);
	}

	/**
	 * Enregistre la modification de la fiche passée en paramètre pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 * @param fiche
	 * 		la fiche.
	 */
	void enregistrerModificationFiche(final String nomObjet, final FicheUniv fiche) {
		getResultatImportObjet(nomObjet).enregistrerModificationFiche(fiche);
	}

	/**
	 * Enregistre la modification d'une fiche inexistante pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 */
	void enregistrerModificationFicheInexistante(final String nomObjet) {
		getResultatImportObjet(nomObjet).enregistrerModificationFicheInexistante();
	}

	/**
	 * Enregistre la suppression de la fiche passée en paramètre pour l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 * @param fiche
	 * 		la fiche.
	 */
	void enregistrerSuppressionFiche(final String nomObjet, final FicheUniv fiche) {
		getResultatImportObjet(nomObjet).enregistrerSuppressionFiche(fiche);
	}

	public Map<String, ResultatImportObjet> getMapResultatImportParObjet() {
		return mapResultatImportParObjet;
	}

	/**
	 * Retourne le résultat d'import spécifique à l'objet indiqué.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 * @return le résultat d'import associé à cet objet.
	 */
	private ResultatImportObjet getResultatImportObjet(final String nomObjet) {
		ResultatImportObjet resultatImportObjet = mapResultatImportParObjet.get(nomObjet);
		if (resultatImportObjet == null) {
			resultatImportObjet = new ResultatImportObjet(nomObjet, sourceImport, synchronise);
			mapResultatImportParObjet.put(nomObjet, resultatImportObjet);
		}

		return resultatImportObjet;
	}
}