/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import com.univ.objetspartages.om.FicheUniv;
import com.univ.xml.bean.ResultatFiche;
import com.univ.xml.util.ResultatFicheUtil;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.collections.CollectionUtils;

/**
 * Représente le résultat d'un import pour un type d'objet donné.
 *
 * @author josse
 */
public class ResultatImportObjet {

	/**
	 * Le nom de l'objet concerné.
	 */
	private final String nomObjet;

	/**
	 * Indique si l'import est synchronisé.
	 */
	private final boolean synchronise;

	/**
	 * Source de l'import.
	 */
	private final String sourceImport;

	/**
	 * Nombre d'enregistrements (items) traités.
	 */
	private int nbEnregistrements;

	/**
	 * Nombre d'erreur survenues.
	 */
	private int nbErreurs;

	/**
	 * Nombre d'éléments ajoutés.
	 */
	private int nbElementsAjoutes;

	/**
	 * Nombre d'éléments modifiés.
	 */
	private int nbElementsModifies;

	/**
	 * Nombre éléments supprimés.
	 */
	private int nbElementsSupprimes;

	/**
	 * Nombre d'éléments restaurés.
	 */
	private int nbElementsRestaures;

	/**
	 * Nombre d'éléments ne pouvant être modifiés car inexistants.
	 */
	private int nbElementsModifiesInexistants;

	/**
	 * Liste des éléments traités.
	 */
	private final List<ResultatFiche> listeElementsTraites;

	/**
	 * Map des éléments supprimés.
	 */
	private final Map<Long, String> mapElementsSupprimes;

	/**
	 * Constructeur pour un objet.
	 *
	 * @param nomObjet
	 * 		le nom d'objet.
	 * @param sourceImport
	 * 		la source de l'import.
	 * @param synchronise
	 */
	ResultatImportObjet(final String nomObjet, final String sourceImport, final boolean synchronise) {
		this.nomObjet = nomObjet;
		this.sourceImport = sourceImport;
		this.synchronise = synchronise;
		this.listeElementsTraites = new ArrayList<>();
		this.mapElementsSupprimes = new HashMap<>();
	}

	/**
	 * Indique si l'import a conduit au traitement d'au moins un élément.
	 *
	 * @return
	 */
	public boolean possedeElementTraite() {
		return CollectionUtils.isNotEmpty(listeElementsTraites);
	}

	public String getNomObjet() {
		return nomObjet;
	}

	public String getSourceImport() {
		return sourceImport;
	}

	public boolean isSynchronise() {
		return synchronise;
	}

	public int getNbEnregistrements() {
		return nbEnregistrements;
	}

	/**
	 * Incrémente le nombre d'enregistrements.
	 */
	public void incrementENombreEnregistrements() {
		nbEnregistrements++;
	}

	public int getNbErreurs() {
		return nbErreurs;
	}

	/**
	 * Incrémente le nombre d'erreurs.
	 */
	void incrementeNombreErreurs() {
		nbErreurs++;
	}

	public int getNbElementsAjoutes() {
		return nbElementsAjoutes;
	}


	/**
	 * Incrémente le nombre d'éléments ajoutés.
	 */
	public void incrementeNombreElementsAjoutes() {
		nbElementsAjoutes++;
	}

	public int getNbElementsModifies() {
		return nbElementsModifies;
	}

	/**
	 * Incrémente le nombre d'éléments modifiés.
	 */
	public void incrementeNombreElementsModifies() {
		nbElementsModifies++;
	}

	public int getNbElementsSupprimes() {
		return nbElementsSupprimes;
	}

	/**
	 * Incrémente le nombre d'éléments supprimés.
	 */
	public void incrementeNombreElementsSupprimes() {
		nbElementsSupprimes++;
	}

	public int getNbElementsRestaures() {
		return nbElementsRestaures;
	}

	public int getNbElementsModifiesInexistants() {
		return nbElementsModifiesInexistants;
	}

	/**
	 * Enregistre l'ajout de la fiche passée en paramètre.
	 *
	 * @param fiche
	 * 		la fiche.
	 */
	void enregistrerAjoutFiche(final FicheUniv fiche) {
		enregistrerResultat(fiche);
		incrementeNombreElementsAjoutes();
	}

	/**
	 * Enregistre la restauration de la fiche passée en paramètre.
	 *
	 * @param fiche
	 * 		la fiche.
	 */
	public void enregistrerRestaurationFiche(final FicheUniv fiche) {
		enregistrerResultat(fiche);
		nbElementsRestaures++;
	}

	/**
	 * Enregistre la modification de la fiche passée en paramètre.
	 *
	 * @param fiche
	 * 		la fiche.
	 */
	void enregistrerModificationFiche(final FicheUniv fiche) {
		enregistrerResultat(fiche);
		incrementeNombreElementsModifies();
	}

	/**
	 * Enregistre la modification d'une fiche inexistante.
	 */
	void enregistrerModificationFicheInexistante() {
		nbElementsModifiesInexistants++;
	}

	/**
	 * Enregistre la suppression de la fiche passée en paramètre.
	 *
	 * @param fiche
	 * 		la fiche.
	 */
	public void enregistrerSuppressionFiche(final FicheUniv fiche) {
		enregistrerResultat(fiche);
		incrementeNombreElementsSupprimes();
	}

	/**
	 * Enregistre le résultat de la fiche passée en paramètre à la liste des résultats de l'import.
	 *
	 * @param fiche
	 * 		la fiche traitée.
	 */
	private void enregistrerResultat(final FicheUniv fiche) {
		listeElementsTraites.add(ResultatFicheUtil.mapResultatFiche(fiche));
	}

	/**
	 * Consolide le résultat courant avec le résultat passé en paramètre.
	 *
	 * @param resultat
	 * 		le résultat à intégrer au résultat courant.
	 */
	public void consolider(final ResultatImportObjet resultat) {

		nbEnregistrements += resultat.getNbEnregistrements();
		nbErreurs += resultat.getNbErreurs();
		nbElementsAjoutes += resultat.getNbElementsAjoutes();
		nbElementsModifies += resultat.getNbElementsModifies();
		nbElementsSupprimes += resultat.getNbElementsSupprimes();
		nbElementsRestaures += resultat.getNbElementsRestaures();
		nbElementsModifiesInexistants += resultat.getNbElementsModifiesInexistants();
		listeElementsTraites.addAll(resultat.getListeElementsTraites());
		mapElementsSupprimes.putAll(resultat.getMapElementsSupprimes());
	}

	public List<ResultatFiche> getListeElementsTraites() {
		return listeElementsTraites;
	}

	public Map<Long, String> getMapElementsSupprimes() {
		return mapElementsSupprimes;
	}
}