/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.importexport.util.ExtensionUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.util.NullAwareBeanUtils;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.objetspartages.util.XmlDateConverter;
import com.univ.utils.Chaine;

public class TraitementImportHelper {

    // Clé de la propriété recensant les patterns de conversion de dates
    public static final String DATE_PATTERNS_PROPERTIES_KEY = "import.date.patterns";

    // Pattern date de java.util.Date.toString() Exemple Mon Jan 20 00:00:00 CET 2020
    public static final String PATTERN_DATE_MYSQL = "E MMM dd hh:mm:ss zzz yyyy";

    // Préfixe utilisé pour prévaloriser des champs à la création d'une fiche
    private static final String PREFIXE_PREVALORISATION = "/";

    private static final String VALUE_NOT_IMPORT_FIELD = "0";

    private final static String ATTACH_SECTION_CODE_PROPERTY = "migration.fiche-sans-rubrique.code-rubrique";

    private final static String ATTACH_SECTION_NAME_PROPERTY = "migration.fiche-sans-rubrique.libelle-rubrique";

    private final static String ATTACH_SECTION_BREADCRUMB_PROPERTY = "migration.fiche-sans-rubrique.fil-ariane-rubrique";

    /**
     * Initialisation des valeurs par défaut en fonction des paramètres d'import XML.
     *
     * @param fiche
     *            the fiche
     * @param nomDansFichierParametre
     *            the nom dans fichier parametre
     *
     * @param sourceImport
     * @throws Exception
     *             the exception
     */
    public static void initialiserValeursImport(final Object fiche, final String nomDansFichierParametre, final String sourceImport) throws Exception {
        final Method[] methodes = fiche.getClass().getMethods();
        for (final Method accesseur : methodes) {
            if (accesseur.getName().startsWith("set")) {
                /* On détermine le nom dans le fichier properties */
                String nomXML = "";
                final String nomAccesseur = accesseur.getName();
                for (int j = 3; j < nomAccesseur.length(); j++) {
                    if (j > 3 && Character.isUpperCase(nomAccesseur.charAt(j))) {
                        nomXML += "_";
                    }
                    nomXML += nomAccesseur.substring(j, j + 1).toUpperCase();
                }
                /* On regarde dans le jtf si la zone doit être initialisée */
                String importDonnee = TraitementImportUtil.getPropertyImport(sourceImport, nomDansFichierParametre.toUpperCase() + "." + nomXML);
                if (!StringUtils.equals(importDonnee, VALUE_NOT_IMPORT_FIELD)) {
                    if ("CODE_RUBRIQUE".equals(nomXML)) {
                        importDonnee = getDefaultAttachSectionCode(importDonnee);
                    } else if (StringUtils.isNotBlank(importDonnee) && StringUtils.startsWith(importDonnee, PREFIXE_PREVALORISATION)) {
                        importDonnee = StringUtils.substringAfter(importDonnee, PREFIXE_PREVALORISATION);
                    }
                    if (StringUtils.isNotBlank(importDonnee)) {
                        final Class<?>[] parametres = accesseur.getParameterTypes();
                        final Object o[] = new Object[1];
                        o[0] = format(parametres[0], importDonnee);
                        accesseur.invoke(fiche, o);
                    }
                }
            }
        }
    }

    /**
     * Caster.
     *
     * @param classe
     *            the classe
     * @param valeur
     *            the valeur
     *
     * @return the object
     */
    public static Object format(final Class<?> classe, String valeur) {
        Object o = null;
        if (classe == String.class) {
            o = Chaine.encodeUTF8(valeur);
        } else if (classe == Long.TYPE || classe == Long.class) {
            o = new Long(valeur);
        } else if (classe == Integer.TYPE || classe == Integer.class) {
            o = new Integer(valeur);
        } else if (classe == Boolean.TYPE || classe == Boolean.class) {
            o = Boolean.valueOf(valeur);
        } else if (classe.getName().equals("java.util.Date") && valeur.length() > 0) {
            //1970-01-01 01:00:00
            valeur = StringUtils.replacePattern(valeur, "(\\-|:| )", "");
            final SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            if (valeur.length() == 8) {
                valeur = valeur + "000000";
            }
            try {
                o = sdf.parse(valeur);
            } catch (final ParseException e) {
                o = new Date(0);
            }
        }
        return o;
    }

    /**
     * Renvoie le BeanUtils permettant de convertir les dates.
     *
     * @return nullAwareBeanUtils
     */
    public static NullAwareBeanUtils getNullAwareBeanUtils() {
        final NullAwareBeanUtils nullAwareBeanUtils = new NullAwareBeanUtils();
        final String patterns = ExtensionUtil.getProperty(DATE_PATTERNS_PROPERTIES_KEY);
        // Si on a pas réussi à récupérer les patterns, on garde ceux par défaut
        if (StringUtils.isNotEmpty(patterns)) {
            final XmlDateConverter dateConverter = new XmlDateConverter();
            final String[] lstPatterns = patterns.split(";");
            for (final String pattern : lstPatterns) {
                // Chacun des patterns sont ajoutés
                dateConverter.registerPattern(pattern, Locale.FRANCE, false);
            }
            //Ajout du pattern renvoyé par Mysql 
            dateConverter.registerPattern(PATTERN_DATE_MYSQL, Locale.ENGLISH, true);
            nullAwareBeanUtils.getConvertUtils().register(dateConverter, Date.class);
        }
        return nullAwareBeanUtils;
    }

    /**
     * Retourne le code de la rubrique à laquelle la fiche doit etre rattachée
     * @param codeRubrique Code de la rubrique
     * @return codeRubrique
     */
    private static String getDefaultAttachSectionCode(String codeRubrique) {
        // Si un code par défaut est renseigné on le récupère
        if (StringUtils.isNotBlank(codeRubrique) && StringUtils.startsWith(codeRubrique, PREFIXE_PREVALORISATION)) {
            codeRubrique = StringUtils.substringAfter(codeRubrique, PREFIXE_PREVALORISATION);
        }
        ServiceRubrique serviceRubrique = ServiceManager.getService(ServiceRubrique.class);
        RubriqueBean attachSection = serviceRubrique.getRubriqueByCode(codeRubrique);
        // Si la rubrique renseignée par défaut n'existe pas, on attache la fiche à la rubrique de migration par défaut
        if (attachSection == null) {
            attachSection = getDefaultAttachSection(PropertyHelper.getCoreProperty(ATTACH_SECTION_CODE_PROPERTY));
            codeRubrique = attachSection.getCode();
        }
        return codeRubrique;
    }

    /**
     * Retourne la rubrique de rattachement par défaut
     * Si la rubrique n'existe pas, celle-ci est créée
     * @param attachSectionCode Code de la rubrique
     * @return attachSection
     */
    private static RubriqueBean getDefaultAttachSection(String attachSectionCode) {
        ServiceRubrique serviceRubrique = ServiceManager.getService(ServiceRubrique.class);
        RubriqueBean attachSection = serviceRubrique.getRubriqueByCode(attachSectionCode);
        // Si la rubrique n'existe pas, on la créer dans le site principal
        if (attachSection == null) {
            ServiceInfosSite serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
            InfosSite principalSite = serviceInfosSite.getPrincipalSite();
            String principalSiteSectionCode = null;
            if (principalSite != null && StringUtils.isNotEmpty(principalSite.getCodeRubrique())) {
                principalSiteSectionCode = principalSite.getCodeRubrique();
            }
            attachSection = new RubriqueBean();
            attachSection.setCodeRubriqueMere(principalSiteSectionCode);
            attachSection.setCode(attachSectionCode);
            attachSection.setIntitule(PropertyHelper.getCoreProperty(ATTACH_SECTION_NAME_PROPERTY));
            attachSection.setNomOnglet(PropertyHelper.getCoreProperty(ATTACH_SECTION_BREADCRUMB_PROPERTY));
            attachSection.setLangue("0");
            serviceRubrique.save(attachSection);
        }
        return attachSection;
    }
}
