/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.apache.commons.beanutils.BeanUtilsBean;
import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.importexport.dto.ImportExportDto;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.PluginBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.synchro.SynchroGroupedsiStructure;
import com.univ.utils.Chaine;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.IntrospectionWrapper;
import com.univ.utils.json.NamingStrategyHelper;
import com.univ.xml.imports.EnteteImport;
import com.univ.xml.imports.EnteteImport.Action;
import com.univ.xml.imports.ImportPlugin;
import com.univ.xml.util.InfosImport;

/**
 * Classe qui gère l'import d'un ensemble de fiches grâce à la lecture d'un fichier XML cet import passe par les méthodes importNodeFiche de chaque objet pour les parties
 * spécifiques
 * <p>
 * Un bilan par mail est envoyé à la fin du script, pour lister les fiches supprimées
 */
public class TraitementImportNode extends TraitementImport {

    private static final Logger LOG = LoggerFactory.getLogger(TraitementImportNode.class);

    private final ServiceMetatag serviceMetatag;

    /** Doit contenir uniquement les caractères alphanumérique **/
    public static final String REGEX_VALID_CODE = "^\\w*$";

    public static final Pattern PATTERN_VALID_CODE = Pattern.compile(REGEX_VALID_CODE);

    /**
     * The in tree.
     */
    private String inTreeDefaut = "1";

    public TraitementImportNode(final InfosImport infosImport) {
        super(infosImport);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
    }

    /**
     * Cette méthode réalise l'import
     *
     * @param ctx  contexte pour accéder à la base
     * @param xmlr the _xmlr
     * @return String retourne un bilan de l'import
     * @throws Exception the exception
     */
    @Override
    public ResultatImport execute(final OMContext ctx, final XMLEventReader xmlr) throws Exception {
        final String szTree = ExtensionUtil.getProperty("import.structures.intree");
        if ("0".equals(szTree)) {
            inTreeDefaut = "0";
        }
        /* on itère sur tous les élements
         * <ITEM>
         * <ENTETE>
         * 		<ACTION>action</ACTION>
         * 		<OBJET>objet</OBJET>
         * 		<CODE>code</CODE>
         * 		<LANGUE>langue</LANGUE>
         * 		<ETAT>etat</ETAT>
         * 		<SOURCE>source</SOURCE>
         * </ENTETE>
         * <DONNEES>
         * 		...
         * </DONNEES>
         * </ITEM>
         *
         */
        try {
            while (xmlr.hasNext()) {
                // premier tag (un tag = une balise ouvrante/fermante d'un élément)
                XMLEvent xmlEvent = xmlr.nextEvent();
                // on cherche la balise <ITEM>
                if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equals("ITEM")) {
                    String objet = "";
                    EnteteImport entete;
                    // tant que ne change pas d'item </ITEM>
                    while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equals("ITEM"))) {
                        // tag suivant
                        xmlEvent = xmlr.nextEvent();
                        // on cherche la balise <ENTETE>
                        if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equals("ENTETE")) {
                            try {
                                entete = new EnteteImport(xmlr);
                                if (StringUtils.isEmpty(entete.getObjet())) {
                                    throw new ErreurApplicative(ExtensionUtil.getMessage("TRAITEMENT_FICHIER_XML.ERREUR.OBJET.VIDE"));
                                }
                                if (StringUtils.isEmpty(entete.getCode())) {
                                    throw new ErreurApplicative(ExtensionUtil.getMessage("TRAITEMENT_FICHIER_XML.ERREUR.CODE.VIDE"));
                                }
                                if (!isValidCode(entete.getCode())) {
                                    throw new ErreurApplicative(String.format(ExtensionUtil.getMessage("TRAITEMENT_FICHIER_XML.ERREUR.CODE.INVALIDE"), entete.getCode()));
                                }
                                objet = entete.getObjet().toLowerCase();
                                logger.debug("Element " + objet + " trouvé");
                                getResultatImport().incrementeNombreEnregistrements(objet);
                            } catch (final Exception e) {
                                getResultatImport().incrementeNombreErreurs(objet);
                                logger.error("Erreur dans l'entête de l'item", e);
                                // si pas d'entete calculée on passe à l'item suivant
                                break;
                            }
                            // pour chaque item on va traiter les données
                            try {
                                final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(objet);
                                if (ficheUniv != null) {
                                    ficheUniv.init();
                                    importItemFiche(ctx, entete, ficheUniv, xmlr);
                                } else {
                                    getResultatImport().incrementeNombreErreurs(objet);
                                    logger.error("Objet " + objet + " non instanciable");
                                }
                                if (ficheUniv != null && ficheUniv instanceof StructureModele) {
                                    // Mise à jour des groupes sur la structure
                                    SynchroGroupedsiStructure.synchroniserGroupeDsi((StructureModele) ficheUniv);
                                }
                            } catch (final Exception e) {
                                logger.error(e.getMessage(), e);
                                getResultatImport().incrementeNombreErreurs(objet);
                            }
                        }
                    }
                }
            }
        } catch (final Exception e) {
            logger.info("Erreur bloquante dans le parsing des données xml", e);
            getResultatImport().incrementNombreErreursBloquantes();
        }
        return getResultatImport();
    }

    /**
     * Méthode qui vérfie que le code importé est conforme.
     * @param code le code à tester
     * @return vrai si le code est valide, faux dans le cas échéant
     */
    protected static boolean isValidCode(final String code) {
        try {
            Chaine.controlerCodeMetier(code);
            return true;
        } catch (ErreurApplicative erreurApplicative) {
            LOG.debug("Code '{}' non valide", code, erreurApplicative);
        }
        return false;
    }

    /**
     * Import item fiche set donnees.
     *
     * @param ctx      the ctx
     * @param fiche    the fiche
     * @param nomObjet the nom objet
     * @param xmlr     the xmlr
     * @param action   the action
     * @throws Exception the exception
     */
    private ImportExportDto importItemFicheSetDonnees(final OMContext ctx, final FicheUniv fiche, final List<PluginBean> lstObjets, final String nomObjet, final XMLEventReader xmlr, final
    Action action) throws Exception {
        
        final ImportExportDto importExportDto = new ImportExportDto();
        /*
         * valeurs par défaut pour les champs non saisis (qui ne sont pas passés
         * dans la boucle) qui ont une valeur par défaut dans le properties
         */
        final IntrospectionWrapper wrapper = new IntrospectionWrapper(fiche.getClass());
        if (action == EnteteImport.Action.AJOUTER) {
            TraitementImportHelper.initialiserValeursImport(fiche, nomObjet, infosImport.getSourceImport());
        }
        // on est dans la balise <DONNEES>, on passe au tag suivant
        XMLEvent xmlEvent = xmlr.nextEvent();
        // tant que l'on reste dans l'entete </DONNEES>
        Set<String> attributeName = getAttributeNameOfImportDto(importExportDto);
        while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equals("DONNEES"))) {
            if (xmlEvent.isStartElement()) {
                final String nodeName = xmlEvent.asStartElement().getName().getLocalPart();
                logger.debug("NodeName : " + nodeName);
                String nomAccesseur = "";
                // nom accesseur : ID_FORMATION devent idFormation
                final String tokens[] = nodeName.split("_", -2);
                for (final String token : tokens) {
                    nomAccesseur += token.substring(0, 1).toUpperCase() + token.substring(1).toLowerCase();
                }
                logger.debug("nomAccesseur : " + nomAccesseur);
                // on regarde la valeur du noeud suivant
                String nomSousObjet = "";
                xmlEvent = xmlr.peek();
                while (xmlEvent != null && xmlEvent.isCharacters() && xmlEvent.asCharacters().getData().trim().equals("")) {
                    xmlEvent = xmlr.nextEvent();
                    xmlEvent = xmlr.peek();
                }
                /* Si au moins un des noeuds fils n'est pas de type TEXT, alors
                 * il s'agit d'une sous-arborescence et donc d'un sous-objet
                 */
                if (xmlEvent != null && xmlEvent.isStartElement()) {
                    nomSousObjet = xmlEvent.asStartElement().getName().getLocalPart();
                }
                /*Sous-objets techniques
                 * <DONNEES>
                 *	<PARCOURSS>
                 * 		<PARCOURS>
                 * 			...
                 * 		</PARCOURS>
                 * 		<PARCOURS>
                 * 			...
                 * 		</PARCOURS>
                 * 	</PARCOURSS>
                 * </DONNEES>
                 */

                // Gestion comportement Legacy (ne supprime pas le comportement précédent sur les noeuds imbriqués)
                if (nomSousObjet.length() > 0) {
                    importItemFichePlugins(ctx, fiche, lstObjets, xmlr, nomSousObjet);

                } else {
                    // attributs simples
                    importItemFicheAttribut(fiche, nomObjet, action, xmlEvent, nodeName, importExportDto,attributeName);
                }// fin type attribut simple
            }
            xmlEvent = xmlr.nextEvent();
        }// fin </DONNEES>
        importExportDto.setFicheUniv(fiche);
        return importExportDto;
    }

    private Set<String> getAttributeNameOfImportDto(final ImportExportDto importExportDto) {
        Class<?> classz = importExportDto.getClass();
        return Arrays.stream(classz.getDeclaredFields()).map(field -> field.getName()).collect(Collectors.toSet());
    }

    /**
     * Import des attributs simples de la fiche (qui n'ont pas de sous noeuds)
     * @param fiche
     * @param nomObjet
     * @param action
     * @param xmlEvent
     * @param nodeName
     * @param importExportDto
     * @param attributeName
     */
    private void importItemFicheAttribut(final FicheUniv fiche, final String nomObjet, final Action action, final XMLEvent xmlEvent, final String nodeName, final ImportExportDto importExportDto, final Set<String> attributeName) {
        final BeanUtilsBean beanUtilsBean = TraitementImportHelper.getNullAwareBeanUtils();
        String valeur = null;
        if (xmlEvent != null && xmlEvent.isCharacters()) {
            valeur = xmlEvent.asCharacters().getData();
        }
        try {
            String currentPropertyName = NamingStrategyHelper.translateUnderscoreToUppercase(nodeName.toLowerCase());
            final String isImportDonnee = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), nomObjet.toUpperCase() + "." + TraitementImportUtil.toSnakeCase(currentPropertyName).toUpperCase());
            if(action == Action.AJOUTER || (StringUtils.isBlank(isImportDonnee) || !"0".equals(isImportDonnee))) {
                if (infosImport.isImportNoeudsVides() && StringUtils.isBlank(valeur)) {
                    beanUtilsBean.setProperty(fiche, NamingStrategyHelper.translateUnderscoreToUppercase(nodeName.toLowerCase()), StringUtils.EMPTY);
                } else {
                    if (valeur != null) {
                        final String nameProperty = NamingStrategyHelper.translateUnderscoreToUppercase(nodeName.toLowerCase());
                        beanUtilsBean.setProperty(fiche, nameProperty, valeur);
                        if(attributeName.contains(nameProperty)){
                            beanUtilsBean.setProperty(importExportDto, nameProperty, valeur);
                        }
                    }
                }
            }
        } catch (ConversionException | IllegalAccessException | InvocationTargetException e) {
            logger.warn("La propriété " + nodeName + " n'a pas pu être extraite.", e);
        }
        if (nodeName.equals("VISIBLE")) {
            inTree = valeur;
        } else {
            inTree = inTreeDefaut;
        }
    }

    protected void importItemFichePlugins(final OMContext ctx, final FicheUniv fiche, final List<PluginBean> lstObjets, final XMLEventReader xmlr, final String nomSousObjet)
        throws Exception {
        final List<String> classNames = new ArrayList<>();
        for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
            if (plugin.isActive(fiche.getClass().getName())) {
                if (plugin.getObjets() != null) {
                    classNames.addAll(plugin.getObjets());
                }
            }
        }
        Class<?> classeDeclarationObjet = null;
        // On boucle sur tous les sous-objets de la fiche courante
        for (final String name : classNames) {
            final Class<?> classeTemp = Class.forName(name);
            // Le nom de la classe doit correspondre au nom dans le fichier XML
            final String tokensSousObjet[] = nomSousObjet.split("_", -2);
            String classeXMLSousObjet = "";
            for (final String element : tokensSousObjet) {
                classeXMLSousObjet += element.substring(0, 1).toUpperCase() + element.substring(1).toLowerCase();
            }
            String nomDeclarationObjet = classeTemp.getName();
            if (nomDeclarationObjet.lastIndexOf('.') != -1) {
                nomDeclarationObjet = nomDeclarationObjet.substring(nomDeclarationObjet.lastIndexOf('.') + 1);
            }
            // si correspondance on sort
            if (nomDeclarationObjet.equals(classeXMLSousObjet)) {
                classeDeclarationObjet = classeTemp;
                break;
            }
        }
        if (classeDeclarationObjet != null) {
            // Appel de l'import du sous-objet
            lstObjets.addAll(importSousObjet(ctx, getInfosImport().getTypeImport(), nomSousObjet, classeDeclarationObjet, xmlr));
        }
    }

    /**
     * Cette méthode réalise l'import, elle est appelée par com.univ.objetspartages.processus.TraitementFichierXML.importNodeFiche()
     *
     * @param _ctx    contexte pour accéder à la base
     * @param _entete the _entete
     * @param _fiche  the _fiche
     * @param _xmlr   the _xmlr
     * @throws Exception lors des requêtes SQL et des
     */
    private void importItemFiche(final OMContext _ctx, final EnteteImport _entete, FicheUniv _fiche, final XMLEventReader _xmlr) throws Exception {
        int countRs = 0;
        final String authorHistoryImport = this.getNameAuthorImport();
        final String nomObjet = ReferentielObjets.getNomObjet(_fiche);
        _fiche.setCtx(_ctx);
        final ArrayList<PluginBean> lstObjets = new ArrayList<>();
        final List<ImportPlugin> importPluginsRaw = new ArrayList<>();
        MetatagBean meta;

        //L'objet ficheTampon est l'objet qui sera chargé à partir des données XML.
        FicheUniv ficheTampon = ReferentielObjets.instancierFiche(ReferentielObjets.getNomObjet(ReferentielObjets.getCodeObjet(_fiche)));
        nullifyEmptyString(ficheTampon);
        // traitement de l'entete
        //L'objet _fiche est l'objet qui sera sérialisé en BDD.
        if (_entete.getCode().length() > 0 && _entete.getLangue().length() > 0) {
            // On cherche en priorité la fiche en ligne. Si il en existe une, on l'initialise
            // avec nextItem avant de récuperer l'ensemble des versions
            countRs = _fiche.selectCodeLangueEtat(_entete.getCode(), _entete.getLangue(), "0003");
            if (countRs != 0) {
                _fiche.nextItem();
                countRs = _fiche.selectCodeLangueEtat(_entete.getCode(), _entete.getLangue(), "");
            } else {
                countRs = _fiche.selectCodeLangueEtat(_entete.getCode(), _entete.getLangue(), "");
                _fiche.nextItem();
            }
        }
        // on cherche la balise <DONNEES> pour remplir la fiche
        XMLEvent xmlEvent = _xmlr.nextEvent();
        while (!(xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equals("DONNEES"))) {
            xmlEvent = _xmlr.nextEvent();
        }
        // traitement des données
        /*  si on est en import et en ajout de fiche,
         *  on importe tous les champs (donc on ne passe pas le param "importer" mais "ajouter".
         */
        //Alimentation de l'objet ficheReference à partir des donnees du fichier XML
        ImportExportDto importExportDto = null;
        if (countRs == 0 && _entete.getAction() == EnteteImport.Action.IMPORTER) {
            importExportDto = importItemFicheSetDonnees(_ctx, ficheTampon, lstObjets, _entete.getObjet(), _xmlr, EnteteImport.Action.AJOUTER);
        } else {
            importExportDto = importItemFicheSetDonnees(_ctx, ficheTampon, lstObjets, _entete.getObjet(), _xmlr, _entete.getAction());
        }
        ficheTampon = importExportDto.getFicheUniv();
        
        // </DONNEES>

        importePluginsRootNode(_xmlr, importPluginsRaw, ficheTampon);

        // récupération du code et de la langue dans l'entête
        ficheTampon.setCode(_entete.getCode());
        ficheTampon.setLangue(_entete.getLangue());
        // ACTION=IMPORTER
        if (_entete.getAction() == EnteteImport.Action.IMPORTER) {
            // NEW
            if (countRs == 0) {
                //La fiche n'existe pas. On la créé à partir des données importées
                checkCode(ficheTampon);
                _fiche.setCode(ficheTampon.getCode());
                _fiche.setLangue(ficheTampon.getLangue());
                // Dans le cas d'une nouvelle fiche, on est sur une action d'Ajout.
                _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, Action.AJOUTER, infosImport);

                // Il est possible de créer une fiche en état brouillon ou en ligne. Par défaut en ligne
                if (_entete.getEtat() != null && _entete.getEtat().equals("0001")) {
                    _fiche.setEtatObjet("0001");
                } else {
                    _fiche.setEtatObjet("0003");
                }

                // ajout de la fiche
                _fiche.add();
                // gestion de l'état visible en front ou non
                meta = MetatagUtils.lireMeta(_fiche);
                if (StringUtils.isNotEmpty(_entete.getSource())) {
                    meta.setMetaSourceImport(_entete.getSource());
                }
                meta.setMetaInTree(getInTree());
                synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto, MetatagUtils.HISTORIQUE_CREATION);
                // ajout des objets techniques
                PluginFicheHelper.ajouterPlugins(_fiche, meta, lstObjets, null);
                this.synchronizePluginXml(_fiche, meta, importPluginsRaw);
                getResultatImport().enregistrerAjoutFiche(nomObjet, _fiche);
            }
            // UPDATE
            else {
                //La fiche existe, on met à jour les champs à partir des données du fichier XML
                meta = MetatagUtils.lireMeta(_fiche);
                if (meta.getMetaSourceImport().equalsIgnoreCase(_entete.getSource())) {
                    // mise a jour du premier état de la fiche
                    _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, _entete.getAction(), infosImport);
                    // sauvegarde des données de la fiche
                    _fiche.update();
                    // gestion de l'état visible en front ou non
                    meta.setMetaInTree(getInTree());
                    synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto,MetatagUtils.HISTORIQUE_MODIFICATION);
                    // synchronise la liste des sous objets
                    PluginFicheHelper.synchroniserListePlugins(_fiche, meta, lstObjets);
                    this.synchronizePluginXml(_fiche, meta, importPluginsRaw);

                    getResultatImport().enregistrerModificationFiche(nomObjet, _fiche);
                    // on boucle sur la fiche pour supprimer tous les autres etats
                    while (_fiche.nextItem()) {
                        if (!"0003".equals(_fiche.getEtatObjet())) {
                            meta = MetatagUtils.lireMeta(_fiche);
                            if (meta.getMetaSourceImport().equalsIgnoreCase(_entete.getSource())) {
                                // on synchronise les autres fiches
                                _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, _entete.getAction(), infosImport);
                                _fiche.update();
                                // gestion de l'état visible en front ou non
                                meta.setMetaInTree(getInTree());
                                synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto, MetatagUtils.HISTORIQUE_MODIFICATION);
                                // synchronise la liste des sous objets
                                PluginFicheHelper.synchroniserListePlugins(_fiche, meta, lstObjets);
                                this.synchronizePluginXml(_fiche, meta, importPluginsRaw);
                            }
                        }
                    }
                } else {
                    LOG.info("Le metatag ID:{} n'est pas traité car la source d'import dans le fichier XML {} est différente de la source de la fiche {} ", meta.getIdMetatag(), _entete.getSource(), meta.getMetaSourceImport());
                }
            }
        }
        // ACTION=AJOUTER
        else if (_entete.getAction() == EnteteImport.Action.AJOUTER && countRs == 0) {
            checkCode(ficheTampon);
            _fiche.setCode(ficheTampon.getCode());
            _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, _entete.getAction(), infosImport);
            // Il est possible de créer une fiche en état brouillon ou en ligne. Par défaut en ligne
            if (_entete.getEtat() != null && _entete.getEtat().equals("0001")) {
                _fiche.setEtatObjet("0001");
            } else {
                _fiche.setEtatObjet("0003");
            }
            _fiche.add();
            // gestion de l'état visible en front ou non
            meta = MetatagUtils.lireMeta(_fiche);
            meta.setMetaInTree(getInTree());
            synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto, MetatagUtils.HISTORIQUE_CREATION);
            PluginFicheHelper.ajouterPlugins(_fiche, meta, lstObjets, null);
            this.synchronizePluginXml(_fiche, meta, importPluginsRaw);
            getResultatImport().enregistrerAjoutFiche(nomObjet, _fiche);
        }
        // ACTION=MODIFIER
        else if (_entete.getAction() == EnteteImport.Action.MODIFIER) {
            if (countRs > 0) {
                _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, _entete.getAction(), infosImport);
                // mise a jour du premier état de la fiche
                _fiche.update();
                // gestion de l'état visible en front ou non
                meta = MetatagUtils.lireMeta(_fiche);
                meta.setMetaInTree(getInTree());
                synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto, MetatagUtils.HISTORIQUE_MODIFICATION);
                // synchronise la liste des sous objets
                PluginFicheHelper.synchroniserListePlugins(_fiche, meta, lstObjets);
                this.synchronizePluginXml(_fiche, meta, importPluginsRaw);
                // sauvegarde des données de la fiche
                getResultatImport().enregistrerModificationFiche(nomObjet, _fiche);
                // on boucle sur la fiche pour supprimer tous les autres etats
                while (_fiche.nextItem()) {
                    if (!"0003".equals(_fiche.getEtatObjet())) {
                        // on synchronise les autres fiches
                        _fiche = (FicheUniv) TraitementImportUtil.synchronizeObject(_fiche, ficheTampon, _entete.getAction(), infosImport);
                        _fiche.update();
                        // gestion de l'état visible en front ou non
                        meta = MetatagUtils.lireMeta(_fiche);
                        meta.setMetaInTree(getInTree());
                        synchronizeAndSaveMeta(_fiche, authorHistoryImport, meta, importExportDto, MetatagUtils.HISTORIQUE_MODIFICATION);
                        // synchronise la liste des sous objets
                        PluginFicheHelper.synchroniserListePlugins(_fiche, meta, lstObjets);
                        this.synchronizePluginXml(_fiche, meta, importPluginsRaw);
                    }
                }
            } else {
                getResultatImport().enregistrerModificationFicheInexistante(nomObjet);
            }
        }
        // ACTION=SUPPRIMER
        else if (_entete.getAction() == EnteteImport.Action.SUPPRIMER) {
            if (countRs > 0) {
                FicheUnivMgr.supprimerFiche(_fiche, true);
                getResultatImport().enregistrerSuppressionFiche(nomObjet, _fiche);
                // on boucle sur la fiche pour supprimer tous les états
                while (_fiche.nextItem()) {
                    FicheUnivMgr.supprimerFiche(_fiche, true);
                }
            } else {
                getResultatImport().enregistrerModificationFicheInexistante(nomObjet);
            }
        }
    }

    /**
     * Synchronisation du métatag avec les données de l'export.
     * @param _fiche {@link FicheUniv}
     * @param authorHistoryImport Le nom de l'auteur dans l'historique de la fiche.
     * @param meta {@link MetatagBean}
     * @param importExportDto {@link ImportExportDto}
     * @param historique Le type d'historique 
     */
    private void synchronizeAndSaveMeta(final FicheUniv _fiche, final String authorHistoryImport, final MetatagBean meta, final ImportExportDto importExportDto, final String historique) {
        serviceMetatag.synchroniser(meta, _fiche.getPersistanceBean(), false);
        synchronizeImportDtoMeta(meta, importExportDto);
        serviceMetatag.save(meta);
        serviceMetatag.addHistory(meta, historique, authorHistoryImport, meta.getMetaEtatObjet());
    }

    /**
     * Synchronisation du DTO et du métatag.
     * @param meta {@link MetatagBean}
     * @param importExportDto {@link ImportExportDto}
     */
    private void synchronizeImportDtoMeta(final MetatagBean meta, final ImportExportDto importExportDto) {
        meta.setMetaDateArchivage(importExportDto.getDateArchivage());
        meta.setMetaDateSuppression(importExportDto.getDateSuppression());
    }

    /**
     * Passage à null des propriétés initialisées à StringUtils.EMPTY.
     * @param ficheTampon : Fiche tampon passée en référence
     */
    private void nullifyEmptyString(final FicheUniv ficheTampon) {
        PropertyDescriptor[] properties = PropertyUtils.getPropertyDescriptors(ficheTampon.getClass());
        for (PropertyDescriptor currentProperty : properties) {
            if (currentProperty.getPropertyType().equals(String.class) && PropertyUtils.isWriteable(ficheTampon, currentProperty.getName())) {
                try {
                    String initValue = (String) PropertyUtils.getProperty(ficheTampon, currentProperty.getName());
                    if (StringUtils.EMPTY.equals(initValue)) {
                        PropertyUtils.setProperty(ficheTampon, currentProperty.getName(), null);
                    }
                } catch (IllegalAccessException | InvocationTargetException e) {
                    LOG.error("La propriété " + currentProperty.getName() + " n'a pas pu être récupérée pour initialisation.", e);
                } catch (NoSuchMethodException e) {
                    if(LOG.isDebugEnabled()) {
                        LOG.debug("La propriété " + currentProperty.getName() + " n'a pas pu être récupérée pour initialisation.", e);
                    } else {
                        LOG.warn("La propriété " + currentProperty.getName() + " n'a pas pu être récupérée pour initialisation.");
                    }
                }
            }
        }
    }

    /**
     * Gère l'import de la balise PLUGINS (facultatif).
     * @param xmlr
     * @param importPluginsRaw Liste des données de plugins (brutes). Extrait du flux XML de plugin
     * @param ficheReference Fiche, permet de vérifier que le(s) plugin(s) est actif sur cette fiche
     * @throws Exception
     */
    private void importePluginsRootNode(final XMLEventReader xmlr, final List<ImportPlugin> importPluginsRaw, final FicheUniv ficheReference) throws XMLStreamException {
        // Plugins
        XMLEvent pluginEvent = xmlr.peek();
        // Les caractères saut de ligne / tab doivent être ignorés
        while (!pluginEvent.isStartElement() && !pluginEvent.isEndElement()) {
            xmlr.nextEvent();
            pluginEvent = xmlr.peek();
        }
        if (pluginEvent.isStartElement() && "PLUGINS".equals(pluginEvent.asStartElement().getName().getLocalPart())) {
            importePlugins(xmlr, importPluginsRaw, ficheReference);
        }
    }

    /**
     * Importe les plugins depuis le flux XML.
     *
     * @param xmlEventReader
     * @param importPluginsRaw
     * @param ficheReference
     * @throws Exception
     */
    private void importePlugins(final XMLEventReader xmlEventReader, final List<ImportPlugin> importPluginsRaw, final FicheUniv ficheReference) throws XMLStreamException {
        // On rentre dans le noeud PLUGINS
        XMLEvent xmlEvent = xmlEventReader.nextEvent();

        if (!xmlEvent.isEndElement()) {
            // On prend le prochain noeud (qu'on ne consomme pas)
            XMLEvent nextEvent = xmlEventReader.peek();
            // tant que l'on reste dans l'entete </PLUGINS>
            while (!(nextEvent.isEndElement() && "PLUGINS".equals(nextEvent.asEndElement().getName().getLocalPart()))) {
                if (nextEvent.isStartElement()) {
                    final String nodeName = nextEvent.asStartElement().getName().getLocalPart();
                    logger.debug("NodeName Plugins : " + nodeName);
                    // On reparcourt chaque plugin pour avoir les informations utiles

                    boolean nodePluginTraite = false;
                    for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
                        if (plugin.isActive(ficheReference.getClass().getName()) && nodeName.equals(plugin.getNodeName())) {
                            final String xmlFragment = readElementBody(xmlEventReader);
                            importPluginsRaw.add(new ImportPlugin(plugin, xmlFragment));
                            nodePluginTraite = true;
                            break;
                        }
                    }
                    if (!nodePluginTraite) {
                        // Permet de lire le flux et d'avancer dans le XML
                        final String xmlFragment = readElementBody(xmlEventReader);
                        logger.info("Aucun import du noeud '{}', pas de plugin actif correspondant", nodeName);
                        logger.debug("Flux XML ignoré lors de l'import : {}", xmlFragment);
                    }
                } else {
                    xmlEventReader.nextEvent();
                }
                nextEvent = xmlEventReader.peek();
            }
        }
        // on sort du <PLUGINS>
        xmlEventReader.nextEvent();
    }

    /**
     * Calcul depuis l'infobean le nom de l'auteur qui sera affiché dans le suivi de la fiche.
     *
     * @return Le nom du fichier importé
     */
    private String getNameAuthorImport() {
        StringBuilder str = new StringBuilder(MetatagUtils.KEY_USER_IMPORT_XML);
        if (getInfosImport() != null && StringUtils.isNotEmpty(getInfosImport().getNameFichier())) {
            str.append(" ");
            str.append("(");
            str.append(getInfosImport().getNameFichier());
            str.append(")");
        }
        return str.toString();
    }

    private void checkCode(final FicheUniv fiche) throws Exception {
        if (fiche instanceof StructureModele) {
            final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
            serviceStructure.checkCode(fiche.getCode(), fiche.getLangue());
        }
        if (fiche instanceof FicheAnnuaire) {
            FicheAnnuaire.checkCode(fiche.getCode(), fiche.getLangue());
        }
    }

    @Override
    public void setLogger(final Logger logger) {
        this.logger = logger;
    }

    /**
     * Synchronise les plugins avec meta + fiche.
     * @param fiche Fiche Univ
     * @param metatagBean Metatag
     * @param importPlugins Plugins à importer, avec le contenu XML brut.
     */
    protected void synchronizePluginXml(final FicheUniv fiche, final MetatagBean metatagBean, final Collection<ImportPlugin> importPlugins) {
        importPlugins.stream().forEach(ip -> {
            logger.debug("Désérialisation du contenu de plugin {}", ip.getPluginFiche().getClass());
            // Gestion de désérialisation complète sur les contenus de
            ip.getPluginFiche().deserializeAndSave(fiche, metatagBean, ip.getPluginContent());
        });
    }
}
