/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.exception.ErreurTechnique;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.xml.util.InfosExportImport;
import com.univ.xml.util.ListeItem;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
// TODO: Auto-generated Javadoc

/**
 * Classe utilisÃ© pour effectuer les transformations XSL.
 */
public class TransformateurXSL implements IXmlCstes {

    /** Logger. */
    private static final Logger _log = LoggerFactory.getLogger(TransformateurXSL.class);

    /** Fabrique de transformer. Cette fabrique doit Ãªtre utilisÃ©e dans un environnement sÃ©curisÃ© pour Ãªtre sÃ»r qu'elle ne distribue pas une nouvelle instance */
    private TransformerFactory tFactory = null;

    /**
     * Constructeur Ã visibilitÃ© package pour assurer l'unicitÃ©.
     */
    public TransformateurXSL() {
        tFactory = TransformerFactory.newInstance();
    }

    /**
     * Methode permettant d'obtenir une instance de transformer Ã partir de la fabrique. Cette mÃ©thode est <code>thread-safe</code>, elle peut donc Ãªtre appelÃ©e simultanÃ©ment
     * par deux threads concurrents
     *
     * @param source
     *            <code>javax.xml.transform.Source</code> source Ã partir de laquelle on souhaite obtenir un transformer
     *
     * @return instance du transformer
     *
     * @throws ErreurTechnique
     *             erreur liÃ©e Ã l'instanciation du transformer
     * @throws ErreurApplicative
     *             the erreur applicative
     */
    private synchronized Transformer getTransformer(final Source source) throws ErreurApplicative {
        Transformer transformer = null;
        try {
            transformer = tFactory.newTransformer(source);
        } catch (final TransformerConfigurationException e) {
            final String messErr = "Erreur lors de l'obtention d'un nouveau transformer";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
        return transformer;
    }

    /**
     * Gets the path xsl.
     *
     * @param file
     *            the file
     *
     * @return the path xsl
     */
    private String getPathXsl(final String file) {
        String nomXsl = file;
        if (!file.endsWith(".xsl")) {
            nomXsl += ".xsl";
        }
        return ExtensionUtil.getXslPath() + nomXsl;
    }

    /**
     * MÃ©thode vÃ©rifiant que le flux reÃ§u est valide (non nul).
     *
     * @param input
     *            flux Ã valider
     *
     * @throws ErreurApplicative
     *             erreur applicative si le flux est null
     */
    private void controlerValiditeInput(final InputStream input) throws ErreurApplicative {
        if (input == null) {
            final String messErr = "Tentative de transformation sur un flux XML nul";
            _log.error(messErr);
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Permet de lancer les transformations xsl dÃ©finies dans le paramÃ©trage.
     *
     * @param input
     *            flux xml en entrÃ©e
     * @param importExport
     *            objet contenant les transformations Ã exÃ©cuter.
     *
     * @return rÃ©sultat de la transformation
     *
     * @throws ErreurTechnique
     *             erreur technique
     * @throws ErreurApplicative
     *             erreur applicative
     */
    public InputStream lancerTransformations(final InputStream input, final InfosExportImport importExport) throws ErreurApplicative {
        final List<String> lstTransformation = importExport.getTransformations();
        InputStream resInputStream = input;
        controlerValiditeInput(input);
        for (final String string : lstTransformation) {
            // test le fichier
            final String pathFichierXsl = string;
            File xsl = new File(pathFichierXsl);
            final String pathXsl = getPathXsl(pathFichierXsl);
            if (xsl == null || !xsl.exists()) {
                xsl = new File(pathXsl);
            }
            if (xsl != null && xsl.exists()) {
                resInputStream = transformer(input, new StreamSource(xsl));
            } else {
                final String messErr = "Erreur de transformation: le chemin '" + pathXsl + "' est inexistant.";
                _log.error(messErr);
                throw new ErreurApplicative(messErr);
            }
        }
        return resInputStream;
    }

    /**
     * MÃ©thode permettant d'effectuer une transformation xsl.
     *
     * @param input
     *            flux entrÃ©e
     * @param source
     *            xsl
     *
     * @return flux sortie
     *
     * @throws ErreurTechnique
     *             erreur technique
     * @throws ErreurApplicative
     *             erreur applicative
     */
    public InputStream transformer(final InputStream input, final Source source) throws ErreurApplicative {
        InputStream resInputStream = null;
        controlerValiditeInput(input);
        try {
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            final Transformer transformer = getTransformer(source);
            transformer.setOutputProperty(OutputKeys.ENCODING, CharEncoding.DEFAULT.toLowerCase());
            transformer.transform(new StreamSource(input), new StreamResult(baos));
            resInputStream = new ByteArrayInputStream(baos.toByteArray());
        } catch (final IllegalArgumentException e) {
            final String messErr = "Erreur pendant la transformation";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } catch (final TransformerException e) {
            final String messErr = "Erreur pendant la transformation";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
        return resInputStream;
    }

    /**
     * MÃ©thode permettant de convertir un flux d'import au format CSV vers un format XML.
     *
     * @param input
     *            flux Ã convertir
     *
     * @return flux XML
     *
     * @throws ErreurApplicative
     *             Erreur lors de la transformation du flux CSV
     */
    public InputStream convertirCsvXml(final InputStream input) throws ErreurApplicative {
        InputStream resInput = null;
        controlerValiditeInput(input);
        final InputStreamReader isr = new InputStreamReader(input);
        final BufferedReader bfr = new BufferedReader(isr);
        final ListeItem listeItem = new ListeItem();
        try {
            if (bfr.ready()) {
                listeItem.setColNames(bfr.readLine().split(";"));
                int c = 0;
                boolean finLigne = false;
                boolean debutGuill = false;
                String res = "";
                final List<String> listeCols = new ArrayList<String>();
                String car = "";
                while ((c = bfr.read()) != -1 && bfr.ready()) {
                    car = String.valueOf((char) c);
                    if ("\"".equals(car)) {
                        debutGuill = !debutGuill;
                        res += car;
                    } else if (!debutGuill && (";".equals(car) || "\n".equals(car))) {
                        listeCols.add(res);
                        res = "";
                        finLigne = "\n".equals(car);
                    } else {
                        res += car;
                    }
                    if (finLigne) {
                        listeItem.traiterLigne(listeCols.toArray(new String[] {}));
                        listeCols.clear();
                        finLigne = false;
                    }
                }
                //on traite la derniÃ¨re ligne
                if (listeCols.size() > 0) {
                    listeItem.traiterLigne(listeCols.toArray(new String[] {}));
                }
                //on transforme le rÃ©sultat en InputStream pour le retourner
                final StringReader sr = new StringReader(listeItem.toString());
                final ByteArrayOutputStream baos = new ByteArrayOutputStream();
                int b = 0;
                while ((b = sr.read()) != -1) {
                    baos.write(b);
                }
                resInput = new ByteArrayInputStream(baos.toByteArray());
            }
        } catch (final IOException e) {
            final String messErr = "Erreur pendant la lecture du flux CSV";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
        return resInput;
    }
}
