/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.exports;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.HTMLUtil;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.importexport.dto.ImportExportDto;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.PluginBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUniv;
import com.univ.utils.IntrospectionWrapper;
import com.univ.utils.UnivWebFmt;
import com.univ.xml.IXmlCstes;
import com.univ.xml.TransformateurXSL;
import com.univ.xml.extraction.IExtractor;
import com.univ.xml.util.InfosExport;

/**
 * Gestionnaire d'export.
 *
 * @author EBO
 */
public class ExportManager implements IXmlCstes {

    /**
     * Logger.
     */
    private static final Logger LOG = LoggerFactory.getLogger(ExportManager.class);

    /**
     * Contexte applicatif.
     */
    protected ContexteUniv ctx = null;

    /**
     * Sax writer pour la génération flux de données.
     */
    protected XMLStreamWriter xmlWriter = null;

    /**
     * The output.
     */
    protected OutputStream output = null;

    /**
     * The traitement item.
     */
    protected boolean traitementItem = false;

    /**
     * Nombre de noeuds métier ajoutés. Cette variable est utilisée pour savoir s'il faut ajouter ou non le noeud METIER (qui ne s'affiche pas dans le flux XML si celui ci est
     * vide)
     */
    protected int nbNoeudsMetier = 0;

    /**
     * Objet contenant les informations sur l'export en cours.
     */
    private InfosExport infosExport = null;

    /**
     * Extracteur associé au gestionnaire d'export.
     */
    private IExtractor extracteur = null;

    /**
     * Transformateur xsl.
     */
    private TransformateurXSL transXSL = null;

    private final ServiceRessource serviceRessource;

    private final ServiceMetatag serviceMetatag;

    /**
     * Map des champs formattés
     */
    private Map<String, String> formatedFields = new HashMap<>();

    /**
     * Liste des champs exclus de l'import 
     */
    private static List<String> FIELDS_EXCLUDE_EXPORT = Arrays.asList("persistenceBean","langue","ficheUniv","etatObjet");

    /**
     * Constructeur de la classe.
     *
     * @param ctx         contexte applicatif
     * @param infosExport conteneur de paramètres de l'export
     * @param extracteur  extracteur associé à l'export
     * @param transXSL    transformateur XSL
     */
    public ExportManager(final ContexteUniv ctx, final InfosExport infosExport, final IExtractor extracteur, final TransformateurXSL transXSL) {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        this.infosExport = infosExport;
        this.ctx = ctx;
        this.extracteur = extracteur;
        this.transXSL = transXSL;
    }

    /**
     * Constructeur avec seulement un paramètre.
     *
     * @param infosExport the infos export
     * @param extracteur  the extracteur
     * @param transXSL    the trans xsl
     */
    public ExportManager(final InfosExport infosExport, final IExtractor extracteur, final TransformateurXSL transXSL) {
        this(null, infosExport, extracteur, transXSL);
    }

    /**
     * Initialisation de l'export.
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void demarrerExport() throws ErreurApplicative {
        final XMLOutputFactory xmlof = XMLOutputFactory.newInstance();
        try {
            output = new ByteArrayOutputStream();
            xmlWriter = xmlof.createXMLStreamWriter(output, CharEncoding.DEFAULT);
            xmlWriter.writeStartDocument(CharEncoding.DEFAULT.toLowerCase(), "1.0");
            xmlWriter.writeCharacters("\n");
            createElement("LISTE_ITEMS");
            xmlWriter.writeCharacters("\n");
        } catch (final XMLStreamException e) {
            final String messErr = "Erreur lors de la création du parser xml";
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Méthode permettant d'exporter en xml un objet.
     *
     * @param objetAConvertir fiche que l'on souhaite convertir
     * @param nomElement      nom du noeud Dom dans lequel on souhaite ajouter les informations
     * @throws XMLStreamException
     */
    private void writeFicheInXml(final Object objetAConvertir, final String nomElement) throws XMLStreamException {
        createElement(nomElement);
        try {
            writeObjectXml(objetAConvertir);
        } catch (final Exception e) {
            LOG.error("L'objet '" + objetAConvertir.getClass().getName() + "' n'a pas été pris en compte dans l'export", e);
        } finally {
            closeElement();
        }
    }

    /**
     * Permet de convertir plusieurs objets dans une même balise
     * @param objetsAConvertir
     * @param nomElement
     * @throws XMLStreamException
     */
    private void writeImportExportDtoXml(final ImportExportDto importExportDto, final String nomElement) throws XMLStreamException {
        createElement(nomElement);
        try {
            LOG.debug("Ecriture de la ficheUniv");
            writeObjectXml(importExportDto.getFicheUniv());
            LOG.debug("Ecriture des information complémentaires non présents dans la fiche");
            writeObjectXml(importExportDto);
        } catch (final XMLStreamException e) {
            LOG.error("Erreur lors de l'écriture du fichier xml", e);
        } finally {
            closeElement();
        }
    }

    private void writeObjectXml(final Object objetAConvertir) throws XMLStreamException {
        final IntrospectionWrapper reflect = IntrospectionWrapper.getIntrospectionWrapper(objetAConvertir.getClass());
        // on obtient la liste des getters
        final Map<String, Method> tableGetters = reflect.getLstGetter();
        // on récupère la liste des méthodes et noms de champs
        final Iterator<String> itKeySet = tableGetters.keySet().iterator();
        String nomChamp = null;
        Method method = null;
        final Object[] args = new Object[] {};
        Object res = null;
        String nomChampFormate = null;
        final SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        while (itKeySet.hasNext()) {
            nomChamp = itKeySet.next();
            nomChampFormate = formaterChamp(nomChamp);
            if (FIELDS_EXCLUDE_EXPORT.stream().noneMatch(nomChamp::equalsIgnoreCase)) {
                method = tableGetters.get(nomChamp);
                try {
                    res = executerMethode(method, args, objetAConvertir);
                    if (method.getReturnType().isAssignableFrom(Date.class) && res != null) {
                        res = sdf.format(res);
                    }
                } catch (final ErreurApplicative e) {
                    LOG.error("Le contenu de la méthode '" + method.getName() + "' n'est pas pris en compte dans l'export", e);
                    res = "";
                }
                if (res != null) {
                    writeElement(nomChampFormate, res.toString());
                }
            }
        }
    }

    /**
     * Méthode permettant de convertir le nom d'un champ objet vers un tag xml.
     *
     * @param champ champ à convertir
     * @return champ formatté
     */
    private String formaterChamp(final String champ) {
        String res = StringUtils.defaultString(formatedFields.get(champ));
        if (StringUtils.isEmpty(res)) {
            for (int i = 0; i < champ.length(); i++) {
                if (i == 0 || Character.isLowerCase(champ.charAt(i))) {
                    res += champ.charAt(i);
                } else {
                    res += "_" + champ.charAt(i);
                }
            }
            res = res.toUpperCase();
            formatedFields.put(champ, res);
        }
        return res;
    }

    /**
     * Méthode permettant d'écrire la balise ouvrante d'un noeud "<element>".
     *
     * @param element le nom du noeud
     * @throws XMLStreamException the XML stream exception
     */
    public void createElement(final String element) throws XMLStreamException {
        xmlWriter.writeStartElement(element);
        xmlWriter.writeCharacters("\n");
    }

    /**
     * Méthode permettant d'écrire un élément XML (UTF-8) directement dans le flux de sortie.
     * <p>
     * Attention, il s'agit d'une méthode d'accès bas niveau, permettant d'écrire du contenu XML sans
     * tenir compte de l'API {@link XMLStreamWriter}.
     * </p>
     *
     * @param rawElement Chaîne XML des données.
     * @throws IOException        Erreur d'I/O sur l'écriture des données
     * @throws XMLStreamException erreur lors du flush
     */
    public void writeRawElement(final String rawElement) throws IOException, XMLStreamException {
        xmlWriter.flush();
        output.write(rawElement.getBytes(CharEncoding.DEFAULT));
        xmlWriter.flush();
    }

    /**
     * Méthode permettant d'écrire la balise fermante du noeud courant ouvert "</element>".
     *
     * @throws XMLStreamException the XML stream exception
     */
    public void closeElement() throws XMLStreamException {
        xmlWriter.writeEndElement();
        xmlWriter.writeCharacters("\n");
    }

    /**
     * Méthode permettant d'écrire un noeud complet "<element>value</element>".
     *
     * @param element the element
     * @param value   the value
     * @throws XMLStreamException the XML stream exception
     */
    public void writeElement(final String element, String value) throws XMLStreamException {
        xmlWriter.writeCharacters("	");
        xmlWriter.writeStartElement(element);
        // si l'option formaterEnHtml est à true alors
        if (value != null && infosExport.isFormaterEnHtml()) {
            if (value.contains("[")) {
                try {
                    value = UnivWebFmt.formaterEnHTML(ctx, value);
                } catch (final Exception e) {
                    LOG.error("Erreur de formatage pour le contenu du champ " + element, e);
                    value = "";
                }
            }
        }
        // suppression des balises html et autres en cas de besoin (ex edition pdf)
        if (infosExport.isFormaterEnTexte()) {
            value = HTMLUtil.stripHtml(value);
        }
        if (value != null && value.contains("<")) {
            xmlWriter.writeCData(Chaine.encode(value, CharEncoding.DEFAULT));
        } else {
            xmlWriter.writeCharacters(Chaine.encode(value, CharEncoding.DEFAULT));
        }
        xmlWriter.writeEndElement();
        xmlWriter.writeCharacters("\n");
    }

    /**
     * Méthode permettant d'éxécuter une méthode de l'objet que l'on souhaite exporter.
     *
     * @param method      méthode à exécuter
     * @param params      paramètres à passer à la méthode
     * @param objetParent objet sur lequel on souhaite exécuter la méthode
     * @return résultat de la méthode
     * @throws ErreurApplicative exception survenant sur la méthode
     */
    private Object executerMethode(final Method method, final Object[] params, final Object objetParent) throws ErreurApplicative {
        Object resMethode = null;
        try {
            resMethode = method.invoke(objetParent);
        } catch (final IllegalArgumentException e) {
            final String messErr = "Nombre de paramètres incorrect dans l'appel de la méthode '" + method.getName() + "' sur la classe '" + objetParent.getClass().getName() + "'.";
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } catch (final IllegalAccessException e) {
            final String messErr = "La méthode '" + method.getName() + "' n'est pas accessible sur la classe '" + objetParent.getClass().getName() + "'.";
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } catch (final InvocationTargetException e) {
            final String messErr = "Exception à l'appel de la méthode '" + method.getName() + "' sur la classe '" + objetParent.getClass().getName() + "'.";
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
        return resMethode;
    }

    /**
     * Write entete fiche.
     *
     * @param ficheUniv the fiche univ
     * @throws ErreurApplicative the erreur applicative
     */
    private void writeEnteteFiche(final FicheUniv ficheUniv) throws ErreurApplicative {
        String objet = ficheUniv.getClass().getName();
        objet = objet.substring(objet.lastIndexOf(".") + 1);
        try {
            createElement("ENTETE");
            writeElement("OBJET", objet);
            writeElement("CODE", ficheUniv.getCode());
            writeElement("LANGUE", ficheUniv.getLangue());
            writeElement("SOURCE", infosExport.getCode());
            closeElement();
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Exporter donnees fiche.
     *
     * @param ficheUniv the fiche univ
     * @throws ErreurApplicative the erreur applicative
     */
    private void exporterDonneesFiche(final FicheUniv ficheUniv) throws ErreurApplicative {
        if (ficheUniv != null) {
            final MetatagBean metatagBean = serviceMetatag.getByCodeAndIdFiche(ReferentielObjets.getCodeObjetParClasse(ficheUniv.getClass().getName()), ficheUniv.getIdFiche());
            final ImportExportDto importExportDto = new ImportExportDto();
            importExportDto.setFicheUniv(ficheUniv);
            importExportDto.setDateSuppression(metatagBean.getMetaDateSuppression());
            importExportDto.setDateArchivage(metatagBean.getMetaDateArchivage());
            try {
                writeImportExportDtoXml(importExportDto, "DONNEES");
            } catch (final XMLStreamException e) {
                LOG.error("impossible d'écrire le fichier XML", e);
                throw new ErreurApplicative(e.getMessage());
            }
        } else {
            final String messErr = "pas de résultat exportable";
            LOG.debug(messErr);
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Exporter fichiers gw.
     *
     * @param ficheUniv the fiche univ
     * @throws ErreurApplicative the erreur applicative
     */
    private void exporterFichiersGw(final FicheUniv ficheUniv) throws ErreurApplicative {
        if (ficheUniv != null) {
            try {
                String nomTag = RessourceBean.class.getName();
                nomTag = nomTag.substring(nomTag.lastIndexOf(".") + 1);
                nomTag = nomTag.toUpperCase();
                if (infosExport.isInclusionFichierGW()) {
                    final List<RessourceBean> listeFichiers = serviceRessource.getFiles(ficheUniv);
                    // ouverture du noeud global pour les fichiers joints
                    createElement(nomTag + "S");
                    for (final Object object : listeFichiers) {
                        // ecriture de l'item fichier
                        writeFicheInXml(object, nomTag);
                    }
                    // fermeture du noeud
                    closeElement();
                }
            } catch (final Exception e) {
                LOG.error("L'objet '" + ficheUniv.getClass().getName() + "' n'a pas été pris en compte dans l'export", e);
            }
        } else {
            final String messErr = "pas de résultat exportable";
            LOG.debug(messErr);
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Exporter objets techniques.
     *
     * @param ficheUniv the fiche univ
     */
    private void exporterObjetsTechniques(final FicheUniv ficheUniv) {
        // Le noeud plugins est toujours présent.
        try {
            // ouverture du noeud global pour tous les objets techniques
            createElement("PLUGINS");
            if (infosExport.isInclusionOT()) {
                final MetatagBean meta = MetatagUtils.lireMeta(ficheUniv);
                addXmlPlugins(ficheUniv, meta);
            }
            // fermeture du noeud
            closeElement();
        } catch (final Exception e) {
            LOG.error("L'objet {} n'a pas été pris en compte dans l'export", ficheUniv.getClass().getName(), e);
        }
    }

    /**
     * Ajout de l'XML de plugins de fiche.
     * @param ficheUniv
     * @param meta
     * @throws Exception
     */
    private void addXmlPlugins(final FicheUniv ficheUniv, final MetatagBean meta) throws Exception {
        // Fonctionnement standard des beans de contenu.
        for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
            if (plugin.isActive(ficheUniv.getClass().getName())) {
                // Appel de la méthode de sérialisation si elle est implémentée sur les projets. Par défaut elle renvoie null.
                String serialise = plugin.serialize(ficheUniv, meta);
                // Indicateur sur un traitement générique de fiche.
                boolean rawElementWrite = false;
                if (serialise != null) {
                    writeRawElement(serialise);
                    rawElementWrite = true;
                }
                if (!rawElementWrite && plugin.getObjets() != null) {
                    for (final String nomClasse : plugin.getObjets()) {
                        addPluginObjet(ficheUniv, meta, nomClasse);
                    }
                }
            }
        }
    }

    private void addPluginObjet(final FicheUniv ficheUniv, final MetatagBean meta, final String nomClasse) throws Exception {
        final String classeOT;
        final int indexDebutNomClasse = nomClasse.lastIndexOf(".");
        classeOT = nomClasse.substring(indexDebutNomClasse + 1);
        // ouverture du noeud global pour un type d'objet technique
        createElement(classeOT.toUpperCase() + "S");
        // on récupère une liste d'objet que l'on va convertir en XML
        final List<PluginBean> objets = PluginFicheHelper.lirePlugins(ficheUniv, meta, nomClasse);
        for (final PluginBean objetPluginContenu : objets) {
            // ecriture de l'item objet technique
            writeFicheInXml(objetPluginContenu, classeOT.toUpperCase());
        }
        // fermeture du noeud
        closeElement();
    }

    /**
     * Gets the requete.
     *
     * @return the requete
     */
    public String getRequete() {
        return infosExport.getRequete();
    }

    /**
     * Permet de démarrer l'ajout d'un nouvel ITEM.
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void demarrerAjouterItem() throws ErreurApplicative {
        if (xmlWriter == null) {
            demarrerExport();
        }
        try {
            traitementItem = true;
            xmlWriter.writeStartElement("ITEM");
            xmlWriter.writeCharacters("\n");
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Termine l'ajout de l'ITEM.
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void terminerAjoutItem() throws ErreurApplicative {
        try {
            xmlWriter.writeEndElement();
            xmlWriter.writeCharacters("\n");
            traitementItem = false;
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Permet d'ajouter l'entête de la fiche dans le noeud XML.
     *
     * @param fiche the fiche
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterEnteteFiche(final FicheUniv fiche) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        writeEnteteFiche(fiche);
    }

    /**
     * Permet d'ajouter les données de la fiche.
     *
     * @param fiche the fiche
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterDonneesFiche(final FicheUniv fiche) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        exporterDonneesFiche(fiche);
    }

    /**
     * Permet d'ajouter les noeud correspondant aux objets techniques.
     *
     * @param fiche the fiche
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterOTS(final FicheUniv fiche) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        exporterObjetsTechniques(fiche);
    }

    /**
     * Permet d'ajouter les noeuds correspondant aux fichiers GW.
     *
     * @param fiche the fiche
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterFWS(final FicheUniv fiche) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        exporterFichiersGw(fiche);
    }

    /**
     * Permet de démarrer l'ajout de données métier.
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void demarrerAjoutDonneesMetier() throws ErreurApplicative {
        try {
            createElement("METIER");
            traitementItem = true;
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Permet d'ajouter les données métier.
     *
     * @param nomNoeud the nom noeud
     * @param donnee   the donnee
     * @throws Exception the exception
     */
    public void ajouterDonneesMetier(final String nomNoeud, final String donnee) throws Exception {
        if (donnee != null && !"".equals(donnee)) {
            nbNoeudsMetier++;
            writeElement(nomNoeud, donnee);
        }
    }

    /**
     * Permet de terminer l'ajout de données métier (si le nombre de noeuds à ajouter est > 0 alors on ajoute le noeud METIER au noeud ITEM).
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void terminerAjoutDonneesMetier() throws ErreurApplicative {
        try {
            closeElement();
            traitementItem = false;
            nbNoeudsMetier = 0;
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Méthode permettant de terminer l'export.
     *
     * @throws ErreurApplicative the erreur applicative
     */
    public void terminerExport() throws ErreurApplicative {
        try {
            closeElement();
            xmlWriter.writeEndDocument();
            xmlWriter.flush();
            xmlWriter.close();
            output.flush();
            output.close();
        } catch (final Exception e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Extrait les données de l'export.
     *
     * @param out the out
     * @throws ErreurApplicative the erreur applicative
     */
    public void extraireDonneesExport(final Writer out) throws ErreurApplicative {
        // sortie du résultat
        try (final InputStream input = new ByteArrayInputStream(((ByteArrayOutputStream) output).toByteArray());
             final InputStream is = transXSL.lancerTransformations(input, infosExport);
             InputStreamReader isReader = new InputStreamReader(is, CharEncoding.DEFAULT_CHARSET);
             BufferedReader reader = new BufferedReader(isReader)) {
            String line = reader.readLine();
            while (line != null) {
                out.write(line);
                line = reader.readLine();
            }
        } catch (final IOException e) {
            LOG.error("Erreur lors de l'extraction des donnees de l'export", e);
        }
    }

    /**
     * Ajouter donnees libelle.
     *
     * @param valeurs the valeurs
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterDonneesLibelle(final Map<String, String> valeurs) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        writeLibellesInXml(valeurs, "DONNEES");
    }

    /**
     * Ajouter entete libelle.
     *
     * @param code   the code
     * @param langue the langue
     * @throws ErreurApplicative the erreur applicative
     */
    public void ajouterEnteteLibelle(final String code, final String langue) throws ErreurApplicative {
        if (!traitementItem) {
            demarrerAjouterItem();
        }
        writeEnteteLibelle(code, langue);
    }

    /**
     * Write entete libelle.
     *
     * @param code   the code
     * @param langue the langue
     * @throws ErreurApplicative the erreur applicative
     */
    public void writeEnteteLibelle(final String code, final String langue) throws ErreurApplicative {
        String objet = LabelDto.class.getName();
        objet = objet.substring(objet.lastIndexOf(".") + 1);
        try {
            createElement("ENTETE");
            writeElement("OBJET", objet);
            writeElement("CODE", code);
            writeElement("LANGUE", langue);
            closeElement();
        } catch (final XMLStreamException e) {
            throw new ErreurApplicative(e.getMessage(), e);
        }
    }

    /**
     * Méthode permettant d'exporter en xml un objet.
     *
     * @param nomElement nom du noeud Dom dans lequel on souhaite ajouter les informations
     * @param valeurs    the valeurs
     * @throws ErreurApplicative the erreur applicative
     */
    private void writeLibellesInXml(final Map<String, String> valeurs, final String nomElement) throws ErreurApplicative {
        try {
            createElement(nomElement);
            String val = "";
            for (final String cle : valeurs.keySet()) {
                try {
                    createElement("DONNEE");
                    val = valeurs.get(cle);
                    writeElement("CLE", cle);
                    writeElement("VALEUR", val);
                    closeElement();
                } catch (final Exception e) {
                    LOG.error("La donnée '" + val + "' n'a pas été pris en compte dans l'export", e);
                }
            }
            closeElement();
        } catch (final Exception e) {
            LOG.error("L'objet '" + nomElement + "' n'a pas été pris en compte dans l'export", e);
        }
    }

    public IExtractor getExtracteur() {
        return extracteur;
    }
}
