/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.exports;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpCookie;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.jsbsoft.jtf.webutils.ContextePage;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.extension.ExtensionHelper;
import com.univ.utils.SessionUtil;
import com.univ.utils.URLResolver;
import com.univ.xml.IImportExportManager;
import com.univ.xml.IXmlCstes;
import com.univ.xml.util.InfosExport;

/**
 * Prend en charge l'exportation.
 *
 * @author jbiard
 */
public class Exporter implements IXmlCstes {

    /** The _log. */
    private static Logger LOG = LoggerFactory.getLogger(Exporter.class);

    /** The export manager. */
    private final IImportExportManager exportManager;

    /** The session. */
    private HttpSession session;

    /**
     * Utilise pour le XML-RPC pour passer la session rattachee a l'utilisateur appelant.
     *
     * @param session
     *            the session
     * @param exportManager
     *            the export manager
     */
    public Exporter(final IImportExportManager exportManager, final HttpSession session) {
        this.session = session;
        this.exportManager = exportManager;
    }

    /**
     * Instantiates a new exporter.
     *
     * @param exportManager
     *            the export manager
     */
    public Exporter(final IImportExportManager exportManager) {
        this(exportManager, null);
    }

    /**
     * Lance une exportation a partir des parametres de la requete HTTP.
     *
     * @param request
     *            requete HTTP
     * @param response
     *            the response
     *
     * @throws Exception
     *             the exception
     */
    public void execute(final HttpServletRequest request, final HttpServletResponse response) throws Exception {
        session = request.getSession();
        // passage des parametres de la requete dans une Map
        // request.getParameterMap( ) renvoie une map cle, tableau de chaines =>
        // pasage en cle, chaine
        final Map<String, String> mapParams = new HashMap<>(request.getParameterMap().size());
        final Enumeration<String> eParams = request.getParameterNames();
        String nomParam;
        while (eParams.hasMoreElements()) {
            nomParam = eParams.nextElement();
            mapParams.put(nomParam, request.getParameter(nomParam));
        }
        IOUtils.copyLarge(getFluxResultantJsp(null, construireRequete(null, null, null, false, mapParams)), response.getOutputStream());
    }

    /**
     * Methode appelee par XML-RPC.
     *
     * @param mapParams
     *            parametres de la methode XML-RPC
     *
     * @return XML resultant
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     */
    public String execute(final Hashtable<String, String> mapParams) throws ErreurApplicative {
        try {
            return IOUtils.toString(getFluxResultantJsp(null, construireRequete(null, null, null, false, mapParams)), CharEncoding.DEFAULT);
        } catch (final IOException e) {
            throw new ErreurApplicative(e.getMessage());
        }
    }

    /**
     * Méthode appelée par un processus K-Portal (TraitementFichierXML ou TraitementPDF).
     *
     * @param infoBean infobean contenant le parametrage de l'exportation
     * @param ctx      contexte
     * @throws ErreurApplicative the erreur applicative
     */
    public void execute(final InfoBean infoBean, final OMContext ctx, final File file) throws ErreurApplicative {
        execute(infoBean, ctx, null, file);
    }

    /**
     * Execute.
     *
     * @param ctx the ctx
     * @param mapParams the map params
     * @throws ErreurApplicative the erreur applicative
     */
    public void execute(final OMContext ctx, final Map<String, String> mapParams, final File file) throws ErreurApplicative {
        execute(null, ctx, mapParams, file);
    }

    /**
     * Exporte suivant le paramétrage.
     *
     * @param infoBean  s'il s'agit d'un processus, sinon vaut null
     * @param ctx       contexte, peut valoir null
     * @param mapParams parametres de l'export, non lus si un infobean est passe, peut valoir null
     * @throws ErreurApplicative the erreur applicative
     */
    private void execute(final InfoBean infoBean, final OMContext ctx, final Map<String, String> mapParams, final File file) throws ErreurApplicative {
        final boolean infoBeanOk = infoBean != null;
        String codeExport;
        if (infoBeanOk) {
            session = infoBean.getSessionHttp();
            codeExport = infoBean.getString(CODE_EXPORT);
        } else {
            codeExport = mapParams.get(CODE_EXPORT);
        }
        if (codeExport == null || "".equals(codeExport)) {
            throw new ErreurApplicative("Code de l'export " + CODE_EXPORT + " non transmis dans les parametres.");
        }
        final InfosExport infosExport = exportManager.getInfosExport(codeExport);
        final Map<String, String> params = infosExport.getParametres();
        // construit la requete passee en front
        final String requete = construireRequete(codeExport, params, infoBean, infoBeanOk, mapParams);
        exportManager.getExportManager(codeExport).getExtracteur().writeToFile(getFluxResultantJsp(ctx, requete), file);
    }

    /**
     * Retourne le flux généré par la JSP d'export.
     *
     * @param ctx the contexte
     * @param requete URL parametree pour l'export
     * @return flux
     * @throws ErreurApplicative the erreur applicative
     */
    public InputStream getFluxResultantJsp(OMContext ctx, final String requete) throws ErreurApplicative {
        //TODO : param ctx à supprimer (on prend le site courant)
        boolean contexteCree = false;
        // creation contexte pour recuperation de l'URL
        if (ctx == null) {
            contexteCree = true;
            ctx = new ContextePage("");
        }
        final String sUrl = URLResolver.getAbsoluteUrl(requete, ctx);
        if (contexteCree) {
            ((ContextePage) ctx).release();
        }
        InputStream is;
        try {
            final URL url = new URL(sUrl);
            final URLConnection urlConnection = url.openConnection();
            final HttpCookie cookie = SessionUtil.copyTrackingCookie(session);
            // on passe l'identifiant de session dans le cas d'appel interserveur
            if (cookie != null) {
                final String cookieSession = cookie.getName() + "=" + cookie.getValue();
                urlConnection.setRequestProperty(HttpHeaders.COOKIE, cookieSession);
            }
            is = urlConnection.getInputStream();
        } catch (final MalformedURLException e) {
            final String messErr = "L'url d'accès aux exports paramétrés n'est pas correcte: " + requete;
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } catch (final IOException e) {
            final String messErr = "Erreur à la lecture de la jsp spécifique de l'export paramétré";
            LOG.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
        return is;
    }

    /**
     * Construit la requête d'export.
     * @param ctx le contexte d'execution
     * @param codeExport le codeExport
     * @param params les parametres de l'export
     * @param infoBean l'infoBean
     * @param infoBeanOk un boolean indiquant si l'infobean est OK (?)
     * @param mapParams une map de paramètres
     * @return une chaine
     * @deprecated Contexte plus utilisé + cette méthode n'est pas utilisée en dehors de la classe (=> privée)
     */
    @Deprecated
    public String construireRequete(final OMContext ctx, final String codeExport, final Map<String, String> params, final InfoBean infoBean, final boolean infoBeanOk, final Map<String, String> mapParams) {
        return construireRequete(codeExport, params, infoBean, infoBeanOk, mapParams);
    }

    /**
     * Construit la requête d'export.
     *
     * @param codeExport the code export
     * @param params     the params
     * @param infoBean   the info bean
     * @param infoBeanOk the info bean ok
     * @param mapParams  the map params
     * @return the string
     */
    private String construireRequete(final String codeExport, final Map<String, String> params, final InfoBean infoBean, final boolean infoBeanOk, final Map<String, String> mapParams) {
        String jspExport = JSP_EXPORT_FICHE;
        if (CODE_EXPORT_LIBELLE.equals(codeExport)) {
            jspExport = JSP_EXPORT_LIBELLE;
        } else if (CODE_EXPORT_BO.equals(codeExport)) {
            jspExport = JSP_EXPORT_BO;
        } else if (CODE_EXPORT_FO.equals(codeExport)) {
            jspExport = JSP_EXPORT_FO;
        }
        String requete = ExtensionHelper.getTemplateExtension(ExtensionUtil.getExtension().getId(), ExtensionUtil.getProperty(jspExport), true) + "?";
        requete = insererDansRequete(requete, CODE_EXPORT, codeExport.toUpperCase());
        if (infoBeanOk && infoBean.getString("REQUETE") != null) {
            requete += "&" + infoBean.getString("REQUETE");
        }
        for (final Object element : params.keySet()) {
            String cle = (String) element;
            if (cle != null) {
                String val;
                if (infoBeanOk) {
                    val = infoBean.getString(cle);
                } else {
                    val = mapParams.get(cle);
                }
                if ((val != null) && !"".equals(val)) {
                    requete = insererDansRequete(requete, cle, val);
                } else if ((val == null) && !"".equals(params.get(cle))) {
                    requete = insererDansRequete(requete, cle, params.get(cle));
                }
            }
        }
        return requete;
    }

    /**
     * Inserer dans requete.
     *
     * @param requete
     *            the requete
     * @param cle
     *            the cle
     * @param val
     *            the val
     *
     * @return the string
     */
    protected String insererDansRequete(String requete, final String cle, final String val) {
        if (requete.charAt(requete.length() - 1) != '?') {
            requete += "&";
        }
        requete += cle + "=" + val;
        return requete;
    }
}
