/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.imports;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.comparator.LastModifiedFileComparator;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.io.PatternFilenameFilter;
import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.CodeLibelle;
import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.webutils.ContextePage;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.logging.LogHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.util.compress.TarGz;
import com.univ.batch.imports.XMLFilter;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.cache.ICacheFicheManager;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.URLResolver;
import com.univ.xml.ITraitementImport;
import com.univ.xml.IXmlCstes;
import com.univ.xml.ResultatImport;
import com.univ.xml.ResultatImportObjet;
import com.univ.xml.TraitementImportEvent;
import com.univ.xml.TraitementImportGroupe;
import com.univ.xml.TraitementImportNode;
import com.univ.xml.TraitementImportUtilisateur;
import com.univ.xml.TransformateurXSL;
import com.univ.xml.XMLValidator;
import com.univ.xml.bean.ResultatFiche;
import com.univ.xml.imports.EnteteImport.Action;
import com.univ.xml.util.InfosImport;
import com.univ.xml.util.ResultatFicheUtil;

/**
 * Classe réalisant les imports.
 *
 * @author EBO
 */
public class ImportManager implements IImporter, IXmlCstes {

	private final ServiceUser serviceUser;

	private final ServiceMetatag serviceMetatag;

	private UrlProvider urlProvider;

    /** The logger */
    private Logger logger = LoggerFactory.getLogger(ImportManager.class);

	/** Objet permettant les transformations sur l'import en cours. */
	private TransformateurXSL transXSL = null;

	/** Objet contenant les informations sur l'import en cours. */
	private InfosImport infosImport = null;

	/**
	 * Constructeur de la classe.
	 *
	 * @param infosImport
	 * 		the infos import
	 * @param transXSL
	 * 		the trans xsl
	 */
	public ImportManager(final InfosImport infosImport, final TransformateurXSL transXSL) {
		this.infosImport = infosImport;
		this.transXSL = transXSL;
		urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
		serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
		serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
	}

	private static boolean isFicheASynchroniserAvecUtilisateur(final boolean bSynchro, final String codeObjet) {
		return bSynchro && ("0006".equals(codeObjet) || "0007".equals(codeObjet));
	}

	private static boolean existInImport(final FicheUniv ficheUniv, final Collection<ResultatFiche> listeFiches) {
		final ResultatFiche ficheCourante = ResultatFicheUtil.mapResultatFiche(ficheUniv);
		return listeFiches.stream()
			.anyMatch(resultat -> ficheCourante.getCodeFiche().equals(resultat.getCodeFiche()) && ficheCourante.getCodeLangue().equals(resultat.getCodeLangue()));
	}

    public void setLogger(final Logger logger) {
        this.logger = logger;
	}

	/**
	 * Execute.
	 *
	 * @param ctx
	 * 		the ctx
	 * @return the string result
	 * @throws Exception
	 * 		the exception
	 */
	@Override
	public String execute(final OMContext ctx) throws Exception {
		int erreurBloquantes = 0;

		// Stockage du type d'import initialement demandé pour permettre la gestion de plusieurs fichiers
		final String typeImport = infosImport.getTypeImport();

		final String path = infosImport.getPathFichier();
		final File folderorfile = new File(path);
		final boolean isFolder = folderorfile.isDirectory();
		if (!folderorfile.exists()) {
			throw new Exception((isFolder ? "Dossier " : "Fichier ") + path + " inexistant");
		}
		if (!folderorfile.canRead()) {
			throw new Exception((isFolder ? "Dossier " : "Fichier ") + path + " non accessible en lecture");
		}
		logger.info("Lecture du " + (isFolder ? "dossier" : "fichier") + " source : " + folderorfile.getAbsolutePath());
		// Définition du filtre sur les fichiers
		final FilenameFilter filtre;
		final String patternFichiers = infosImport.getPatternFichiers();
		if (StringUtils.isNotBlank(patternFichiers)) {
			// Si un pattern a été sélectionné, on filtre sur le pattern
			filtre = new PatternFilenameFilter(patternFichiers);
			logger.info("Traitement des fichiers correspondant au pattern '{}'", patternFichiers);
		} else {
			// Par défaut, on filtre sur les fichiers XML
			filtre = new XMLFilter();
			logger.info("Traitement de tous les fichiers *.xml");
		}
		final File[] listeFiles;
		if (isFolder) {
			listeFiles = folderorfile.listFiles(filtre);
		} else {
			listeFiles = new File[]{folderorfile};
		}
		// Map des résultats d'import par objet traité
		final Map<String, ResultatImportObjet> resultatsConsolidesParObjet = new HashMap<>();
		Arrays.sort(listeFiles, LastModifiedFileComparator.LASTMODIFIED_COMPARATOR);
		// boucle sur tous les fichiers xml trouvés
		for (final File xmlFile : listeFiles) {
			logger.info("Traitement du fichier : " + xmlFile.getName());
			FileInputStream fis = null;
			try {
				if (xmlFile.exists()) {
					if (!"CSV".equalsIgnoreCase(infosImport.getFormat())) {
						XMLValidator.validate(xmlFile, null);
					}
					// On ajoute le nom du fichier uniquement si un autre process ne l'a pas déjà ajouté
					if(StringUtils.isNotEmpty(xmlFile.getName()) && StringUtils.isEmpty(infosImport.getNameFichier())) {
						infosImport.setNameFichier(xmlFile.getName());
					}
					// Initialisaton des paramètres de l'import
					infosImport.setPathFichier(xmlFile.getAbsolutePath());
					infosImport.setTypeImport(typeImport);

					fis = new FileInputStream(xmlFile);
					final ResultatImport resultat = execute(ctx, fis);
					erreurBloquantes += resultat.getNbErreursBloquantes();

					// Consolidation des résultats par objet
					final Map<String, ResultatImportObjet> mapResultatsImportObjet = resultat.getMapResultatImportParObjet();
					for (final String nomObjet : mapResultatsImportObjet.keySet()) {

						final ResultatImportObjet resultatImportObjet = mapResultatsImportObjet.get(nomObjet);

						final ResultatImportObjet resultatConsolide = resultatsConsolidesParObjet.get(nomObjet);
						if (resultatConsolide != null) {
							resultatConsolide.consolider(resultatImportObjet);
						} else {
							resultatsConsolidesParObjet.put(nomObjet, resultatImportObjet);
						}
					}

				}
			} catch (final Exception e) {
				logger.error("Erreur bloquante sur le fichier " + xmlFile.getName(), e);
				erreurBloquantes++;
			} finally {
				if (fis != null) {
					fis.close();
				}
			}
			// Fin du traitement du fichier, on rénitialise le nom 
			infosImport.setNameFichier(StringUtils.EMPTY);
		}
		// compression et sauvegarde du fichier xml source
		try {
			File folderToSave = new File(WebAppUtil.getSauvegardePath() + SAVE_PATH);
			FileUtils.forceMkdir( folderToSave );
			final String archivePrefix = folderorfile.getName();
			final File gzFile = new File(folderToSave, archivePrefix + "_" + System.currentTimeMillis() + ".tar.gz");
			TarGz.compressFiles(Arrays.asList(listeFiles), gzFile, Boolean.TRUE);
		} catch (final IOException e) {
			logger.error("Impossible de sauvegarder le fichier xml d'import " + folderorfile, e);
		}

		final Collection<ResultatImportObjet> listeResultatsConsolides = resultatsConsolidesParObjet.values();
		for (final ResultatImportObjet resultatImportObjet : listeResultatsConsolides) {
			if (resultatImportObjet.isSynchronise() && resultatImportObjet.possedeElementTraite()) {
				// boucle sur toutes les fiches une fois tous les imports terminés
				try {
					synchroniseFichesASupprimer(ctx, resultatImportObjet);
					logger.info("Synchronisation des fiches '{}'", resultatImportObjet.getNomObjet());
				} catch (final Exception e) {
					logger.error("Erreur lors de la synchronisation", e);
					erreurBloquantes++;
				}
			}
		}
		// Rechargement mémoire des fiches
		for (final ICacheFicheManager bean : ApplicationContextManager.getAllBeansOfType(ICacheFicheManager.class).values()) {
			bean.flush();
		}
		logger.info("Bilan de l'import :");
		String message = "<ul>";
		if (CollectionUtils.isNotEmpty(listeResultatsConsolides)) {
			for (final ResultatImportObjet resultatImportObjet : listeResultatsConsolides) {
				final String nomObjet = resultatImportObjet.getNomObjet();
				final String codeObjet = ReferentielObjets.getCodeObjet(nomObjet);
				String libelleElement = StringUtils.EMPTY;
				if (StringUtils.isNotBlank(codeObjet)) {
					libelleElement = ReferentielObjets.getLibelleObjet(codeObjet);
				}
				if (StringUtils.isEmpty(libelleElement)) {
					libelleElement = nomObjet;
				}
				message += "<li>Traitement des éléments <strong>" + libelleElement + "</strong> :" + "<ul style=\"list-style:disc inside none;margin-bottom:1em\">";
				logger.info("Traitement des éléments " + libelleElement + " :");
				message += "<li>Enregistrement(s) traité(s)          : " + resultatImportObjet.getNbEnregistrements() + "</li>";
				logger.info("	Enregistrement(s) traité(s)         : " + resultatImportObjet.getNbEnregistrements());
				message += "<li>Élément(s) ajouté(s)                 : " + resultatImportObjet.getNbElementsAjoutes() + "</li>";
				logger.info("	Élément(s) ajouté(s)                : " + resultatImportObjet.getNbElementsAjoutes());
				message += "<li>Élément(s) mis à jour                : " + resultatImportObjet.getNbElementsModifies() + "</li>";
				logger.info("	Élément(s) mis à jour               : " + resultatImportObjet.getNbElementsModifies());
				message += "<li>Élément(s) restauré(s)               : " + resultatImportObjet.getNbElementsRestaures() + "</li>";
				logger.info("	Élément(s) restauré(s)              : " + resultatImportObjet.getNbElementsRestaures());
				message += "<li>Élément(s) supprimé(s)               : " + resultatImportObjet.getNbElementsSupprimes() + "</li>";
				logger.info("	Élément(s) supprimé(s)              : " + resultatImportObjet.getNbElementsSupprimes());
				message += "<li>Erreur(s)                            : " + resultatImportObjet.getNbErreurs() + "</li>";
				logger.info("	Erreur(s)                           : " + resultatImportObjet.getNbErreurs());
				message += "<li>Élément(s) à modifier non trouvé(s)  : " + resultatImportObjet.getNbElementsModifiesInexistants() + "</li>";
				logger.info("	Élément(s) à modifier non trouvé(s) : " + resultatImportObjet.getNbElementsModifiesInexistants());
				message += "</ul></li>";
			}
		} else {
			message += "<li>Aucun élément traité</li>";
			logger.info("Aucun élément traité");
		}
		if (erreurBloquantes > 0) {
			message += "<li>Erreur(s) bloquantes(s)        : " + erreurBloquantes + "</li>";
			logger.info("Erreur(s) bloquante(s)         : " + erreurBloquantes);
		}
		message += "</ul>";
		return message;
	}

	/**
	 * Execute.
	 *
	 * @param ctx
	 * 		the _ctx
	 * @param input
	 * 		the _input
	 * @return the string
	 * @throws Exception
	 * 		the exception
	 */
	public ResultatImport execute(final OMContext ctx, InputStream input) throws Exception {
		ResultatImport resultat;
		if (input == null) {
			throw new ErreurApplicative("Import impossible, pas de données à importer.");
		}
		if (transXSL != null) {
			// tranformations liées au paramétrage
			if ("CSV".equalsIgnoreCase(infosImport.getFormat())) {
				//Transformation du format csv en XML
				input = transXSL.convertirCsvXml(input);
			}
			input = transXSL.lancerTransformations(input, infosImport);
		}
		final XMLInputFactory xmlif = XMLInputFactory.newInstance();
		xmlif.setProperty(XMLInputFactory.IS_COALESCING, true);
		XMLEventReader xmlr = xmlif.createXMLEventReader(input);
		// on se positionne juste avant le premier élément
		while (!xmlr.peek().isStartElement()) {
			xmlr.nextEvent();
		}
		logger.debug("Element " + xmlr.peek().asStartElement().getName().getLocalPart() + " trouvé");
		final ITraitementImport traitementImport;

		try (FileInputStream fis = new FileInputStream(new File(infosImport.getPathFichier()))) {
			// on regarde la première balise
			if ("groupe".equalsIgnoreCase(infosImport.getTypeImport()) || xmlr.peek().isStartElement() && xmlr.peek().asStartElement().getName().getLocalPart().equalsIgnoreCase("GROUPES")) {
				infosImport.setTypeImport("groupe");
				traitementImport = new TraitementImportGroupe(infosImport);
			} else if ("utilisateur".equalsIgnoreCase(infosImport.getTypeImport()) || xmlr.peek().isStartElement() && xmlr.peek().asStartElement().getName().getLocalPart().equalsIgnoreCase("UTILISATEURS")) {
				infosImport.setTypeImport("utilisateur");
				traitementImport = new TraitementImportUtilisateur(infosImport);
			} else if (StringUtils.isEmpty(infosImport.getTypeImport())) {
				EnteteImport entete;
				String objet = "";
				Action action;
				// on itère sur tous les élements
				while (xmlr.hasNext()) {
					// on cherche la balise <ENTETE>
					if (xmlr.peek().isStartElement() && xmlr.peek().asStartElement().getName().getLocalPart().equalsIgnoreCase("ENTETE")) {
						try {
							entete = new EnteteImport(xmlr);
							objet = entete.getObjet();
							action = entete.getAction();
							if (action == Action.IMPORTER) {
								infosImport.setSynchronizedImport(true);
								infosImport.setSourceImport(entete.getSource());
							}
						} catch (final Exception e) {
							// si pas d'entete calculée on passe à l'item suivant
							objet = "";
						}
					}
					if (objet.length() > 0) {
						infosImport.setTypeImport(objet.toLowerCase());
						logger.info("Element de type " + objet.toLowerCase() + " trouvé");
						break;
					}
					xmlr.nextEvent();
				}
				TraitementImportUtil.processInfosImport(infosImport);
				// on reprend le fichier au début
				if (CollectionUtils.isNotEmpty(infosImport.getTransformations())) {
					// Si une transformation XSL a été faite, l'InputStream supporte le reset
					input.reset();
					xmlr = xmlif.createXMLEventReader(input);
				} else {
					xmlr = xmlif.createXMLEventReader(fis);
				}
				// import historique où les actions sont pilotées par le xml
				traitementImport = new TraitementImportNode(infosImport);
			} else {
				traitementImport = new TraitementImportEvent(infosImport);
			}
			// execution de l'import
			traitementImport.setLogger(logger);
			resultat = traitementImport.execute(ctx, xmlr);
		} finally {
			xmlr.close();
		}
		return resultat;
	}

	/**
	 * lors de l'import synchronisé de fiches, on regarde si des fiches présentes dans la base ont été supprimées depuis (comparaison avec import) si c'est le cas, pour chaque
	 * fiche> envoi d'un mail pour prévenir que la fiche est périmée.
	 *
	 * @param _ctx
	 * 		the _ctx
	 * @param resultat
	 * 		le bean resultat
	 * @throws Exception
	 * 		the exception
	 */
	private void synchroniseFichesASupprimer(final OMContext _ctx, final ResultatImportObjet resultat) throws Exception {
		// les suppressions pour l'annuaire et l'annuaire étudiants peuvent être basées sur la table utilisateur
		// dans ce cas, la table utilisateur sert de référence pour déterminer la liste des personnes valides
		final boolean bSynchro = "1".equals(TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), "suppression.synchroutilisateurs"));
		// date du jour pour retrouver les nouvelles fiches
		final Date today = new Date(System.currentTimeMillis());
		final DateFormat dF = DateFormat.getDateInstance(DateFormat.LONG, LangueUtil.getDefaultLocale());
		final String szToday = dF.format(today);
		// les fiches qui ont été archivées, et qui sont réimportées doivent-elles être remises en ligne ?
		String etat_objet_archives = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), "etat.fiches_restaurees");
		if (etat_objet_archives == null || (!"0001".equals(etat_objet_archives) && !"0003".equals(etat_objet_archives))) {
			etat_objet_archives = "";
		}
		Date dateArchivage = new Date(0);

		// récupération du type de fiche
		final String nomObjet = resultat.getNomObjet();
		final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(nomObjet);
		final String codeObjet = ReferentielObjets.getCodeObjet(ficheUniv);
		final String libelleObjet = ReferentielObjets.getLibelleObjet(codeObjet);
		final Collection<ResultatFiche> listeFiches = resultat.getListeElementsTraites();
		// pas de synchro avec la table utilisateur
		Collection<String> listeCodes = new ArrayList<>();
		// les fiches annuaire et annuaire étudiant sont les seules concernées pour la synchro
		if (isFicheASynchroniserAvecUtilisateur(bSynchro, codeObjet)) {
			logger.debug("Traitement des utilisateurs");
			final List<UtilisateurBean> users = serviceUser.getAllUsers();
			listeCodes = new ArrayList<>();
			for (final UtilisateurBean currentUser : users) {
				listeCodes.add(currentUser.getCode());
			}
		} else {
			logger.debug("Traitement des fiches " + libelleObjet);
		}
		logger.debug("Nombre de code de fiche " + libelleObjet + " à traiter : " + listeCodes.size());
		// Si ce paramètre vaut 0, la suppression est désactivée, mais le webmaster est informé par mail de la liste des fiches obsolètes.
		// S'il vaut 1, la suppression est effective, et le webmaster reçoit également le mail.
		// S'il vaut 2, la suppression est effective, et le compte-rendu est enregistré dans le fichier de log
		final String typeSuppression = StringUtils.defaultIfBlank(TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), nomObjet.toUpperCase() + ".suppressionauto"), "0");
		// possibilité d'archiver les fiches au lieu de les supprimer, choisir entre la suppression (0004) et l'archivage (0007) d'une fiche obsolète.
		final String etatSuppression = StringUtils.defaultIfBlank(TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), nomObjet.toUpperCase() + ".etatsuppression"), "0004");
		// archivage en positionnant la date du lendemain
		// si vide, archivage immédiat
		// valeurs possibles : +n : permet d'archiver n jours après l'import (permet de gérer l'archivage en fonction
		// de l'heure à laquelle a été lancée le script (avant ou après minuit) )
		// doit être inférieur à 10 (on teste sur charAt)
		final String postArchivage = StringUtils.defaultString(TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), nomObjet.toUpperCase() + ".postarchivage"));
		// selection de toutes les fiches dans la base
		final StringBuilder texteAjout = new StringBuilder("");
		final StringBuilder texteSuppression = new StringBuilder("");
		ficheUniv.setCtx(_ctx);
		ficheUniv.select("");
		while (ficheUniv.nextItem()) {
			// fiche ajoutée ou à restaurer
			if (existInImport(ficheUniv, listeFiches) || listeCodes.contains(ficheUniv.getCode()) && isFicheASynchroniserAvecUtilisateur(bSynchro, codeObjet)) {
				// fiche ajoutée
				if (ficheUniv.getDateCreation() != null && dF.format(ficheUniv.getDateCreation()).equals(szToday)) {
					// récupération du libelle
					texteAjout.append(ficheUniv.getLibelleAffichable());
					final String etatObjet = ficheUniv.getEtatObjet();
					// création de l'url avec les code et la langue
					texteAjout.append(" (code : ");
					texteAjout.append(ficheUniv.getCode());
					texteAjout.append(", langue : ");
					texteAjout.append(ficheUniv.getLangue());
					texteAjout.append(", état de l'objet : ");
					texteAjout.append((CodeLibelle.lireLibelle(null, "etat_objet", etatObjet)).trim());
					texteAjout.append(")\nCette fiche est accessible ici : ");
					// url de la fiche
					final String urlUniv = urlProvider.provideAbsoluteUrl(ficheUniv.getPersistanceBean());
					texteAjout.append(URLResolver.getAbsoluteUrl(urlUniv, _ctx));
					texteAjout.append("\n\n");
				} else if (etat_objet_archives.length() > 0 && "0007".equals(ficheUniv.getEtatObjet())) {
					// restauration de la fiche
					ficheUniv.setEtatObjet(etat_objet_archives);
					final MetatagBean meta = MetatagUtils.lireMeta(ficheUniv);
					if (Formateur.estSaisie(meta.getMetaDateArchivage())) {
						meta.setMetaDateArchivage(new Date(0));
						meta.setMetaDateOperation(ficheUniv.getDateModification());
					}
					meta.setMetaEtatObjet(etat_objet_archives);
					serviceMetatag.save(meta);
					ficheUniv.update();
					resultat.enregistrerRestaurationFiche(ficheUniv);
				}
			} else {
				// fiche à supprimer
				final String etatObjet = ficheUniv.getEtatObjet();
				final ContextePage ctx = new ContextePage();
				try {
					final MetatagBean meta = MetatagUtils.lireMeta(ficheUniv);
					if (meta.getMetaSourceImport().equals(resultat.getSourceImport())) {
						//si la fiche est déjà supprimée, déjà archivée, ou programmée à l'archivage, on ne la met pas dans le mail
						if (("0".equals(typeSuppression) || "1".equals(typeSuppression)) && (!etatSuppression.equals(etatObjet) && ("".equals(postArchivage) || !Formateur.estSaisie(meta.getMetaDateArchivage())))) {
							//récupération du libelle
							texteSuppression.append(ficheUniv.getLibelleAffichable());
							//création de l'url avec les code et la langue
							texteSuppression.append(" (code : ").append(ficheUniv.getCode()).append(", langue : ").append(ficheUniv.getLangue()).append(", état de l'objet : ").append((ReferentielObjets.getLibelleEtatObjet(etatObjet)).trim()).append(")\nCette fiche est accessible ici : ");
							// url de la fiche
							final String urlUniv = urlProvider.provideAbsoluteUrl(ficheUniv.getPersistanceBean());
							texteSuppression.append(URLResolver.getAbsoluteUrl(urlUniv, ctx));
							texteSuppression.append("\n\n");
						}
						// suppression auto.
						if (!"0".equals(typeSuppression)) {
							//état à supprimer : archivage via la date
							if ("0007".equals(etatSuppression) && StringUtils.isNotBlank(postArchivage)) {
								String valuePostArchivage = StringUtils.substring(postArchivage, 1);
								if ('+' != (postArchivage.charAt(0))){
									throw new ErreurApplicative("La valeur de post-archivage est incorrecte.");
								}
								if (!StringUtils.isNumeric(valuePostArchivage)){
									throw new ErreurApplicative("La valeur saisie en post-archivage n'est pas un entier.");
								}
								if (!Formateur.estSaisie(meta.getMetaDateArchivage())) {
									//on ne met une date d'archivage que si elle n'a pas déjà été saisie
									final int iNbJours = Integer.parseInt(String.valueOf(valuePostArchivage));
									LocalDateTime localDateTime = LocalDateTime.now().plusDays(iNbJours);
									Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
									dateArchivage = date;
									meta.setMetaDateArchivage(date);
								}
							} else {
								ficheUniv.setEtatObjet(etatSuppression);
							}
							ficheUniv.setDateProposition(new Date(System.currentTimeMillis()));
							ficheUniv.update();
							meta.setMetaEtatObjet(ficheUniv.getEtatObjet());
							serviceMetatag.synchroniser(meta, ficheUniv.getPersistanceBean(), true);
							serviceMetatag.save(meta);
							resultat.enregistrerSuppressionFiche(ficheUniv);
						}
					}
				} catch (final Exception e) {
					logger.error("Erreur mise à jour du metatag de la fiche " + ficheUniv.getLibelleAffichable(), e);
				} finally {
					ctx.release();
				}
			}
		}
		// envoi du mail pour les fiches ajoutées
		if (texteAjout.length() > 0) {
			final StringBuilder texte = new StringBuilder("Voici les fiches " + libelleObjet + " qui ont été ajoutées le " + szToday + " : \n\n");
			texte.append(texteAjout);
			LogHelper.logInfoFromLine(logger, texte.toString());
			texte.append("\nCordialement.");
			try {
				//Initialisation du mailbox
				final JSBMailbox mailbox = new JSBMailbox(false);
				final String sujet = "[IMPORTS XML] Fiche(s) " + libelleObjet + " ajoutée(s) ou modifiée(s)";
				final String to = PropertyHelper.getCoreProperty("mail.webmaster");
				//envoi du message
				mailbox.sendSystemMsg(to, sujet, texte.toString());
			} catch (final Exception e) {
				logger.error("Le mail de rapport des imports n'a pas pu être envoyé.", e);
			}
		}
		// envoi mail de suppression
		if (texteSuppression.length() > 0) {
			final StringBuilder texte = new StringBuilder();
			texte.append("Voici la liste des fiches ").append(libelleObjet).append(" existantes et non importées : \n\n");
			texte.append(texteSuppression);
			//Suppression auto.
			final String url_site = URLResolver.getAbsoluteBoUrl("/adminsite/", _ctx.getInfosSite());
			if (!"0".equals(typeSuppression) && "0004".equals(etatSuppression)) {
				// état à supprimer
				texte.append("Ces fiches sont maintenant en état 'A Supprimer' et seront définitivement supprimées lors de l'exécution du batch de maintenace.\n");
				texte.append("Si vous souhaitez conserver ces fiches, connectez-vous sur le module d'administration du site web(").append(url_site).append(") pour annuler la suppression.");
			} else if (!"0".equals(typeSuppression) && "0007".equals(etatSuppression)) {
				// etat archivé
				if ("".equals(postArchivage)) {
					texte.append("Ces fiches sont maintenant en état 'Archive'. Vous pouvez les restaurer à tout moment en vous connectant sur le module d'administration du site web ( ").append(url_site).append(" ).");
				} else {
					texte.append("Ces fiches seront archivées le ").append(dateArchivage).append(", lors de l'exécution du batch de maintenance.\n").append("Si vous souhaitez conserver ces fiches 'En ligne', connectez-vous sur le module d'administration du site web (").append(url_site).append(") pour modifier ou supprimer la date d'archivage.");
				}
			} else {
				texte.append("Si vous souhaitez supprimer ces fiches, connectez-vous sur le module d'administration du site web(").append(url_site).append(") pour les supprimer manuellement.");
			}
			if (!"2".equals(typeSuppression)) {
				texte.append("\nCordialement.");
				// envoi du mail
				try {
					// Initialisation du mailbox
					final JSBMailbox mailbox = new JSBMailbox(false);
					final String sujet = "[IMPORTS XML] Synchronisation des fiches " + libelleObjet;
					final String to = PropertyHelper.getCoreProperty("mail.webmaster");
					//envoi du message
					mailbox.sendSystemMsg(to, sujet, texte.toString());
				} catch (final Exception e) {
					logger.warn("Le mail de rapport des imports n'a pas pu être envoyé.");
				}
			} else {
				LogHelper.logInfoFromLine(logger, texte.toString());
			}
		}
	}
}
