/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package fr.kosmos.ofin.cdmfr.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MultiValuedMap;
import org.apache.commons.collections4.multimap.ArrayListValuedHashMap;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.layout.card.bean.CardBean;
import com.kosmos.layout.card.service.ServiceCard;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kportal.core.config.Langue;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ResponsableBean;
import com.ksup.objetspartages.bean.ResponseRechercheParcoursBean;
import com.ksup.objetspartages.om.Cours;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.Parcours;
import com.ksup.objetspartages.om.Ueup;
import com.ksup.objetspartages.service.ServiceFormation;
import com.ksup.objetspartages.service.ServiceRechercheParcours;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceStructure;

import fr.kosmos.cdmfr.batch.ConfigurationExportCDMfr;
import fr.kosmos.cdmfr.bean.DonneesFormationExportCDMfr;
import fr.kosmos.cdmfr.service.ConstantesExportCDMfr;
import fr.kosmos.cdmfr.service.ServiceExportCDMfr;

/**
 * Services liés à l'export CDM-fr.
 * Cette classe est destinée à être surchargée par les projets pour permettre de personnaliser les traitements.
 *
 * @author josse
 */
public class ServiceExportCDMfrOFIN extends ServiceExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> {

    protected ServiceFormation serviceFormation;

    protected ServiceStructure serviceStructure;

    protected ServiceLabel serviceLabel;

    protected ServiceRechercheParcours serviceRechercheParcours;

    protected ServiceCard serviceCard;

    /**
     * Constructeur par défaut.
     */
    public ServiceExportCDMfrOFIN() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Set<Formation> lireListeFormationsAExporter() {
        Set<Formation> listeFormations = new HashSet<>();
        // Lecture des formations pour chaque langue déclarée
        for (Langue langue : LangueUtil.getLangues().values()) {
            final List<FormationBean> listeFormationsLangue = serviceFormation.getAll(langue, EtatFiche.EN_LIGNE);
            listeFormationsLangue.stream().map(bean -> new Formation(bean)).collect(Collectors.toCollection(() -> listeFormations));
        }
        LOGGER.info("{} formation(s) chargée(s) pour l'export CDM-fr.", CollectionUtils.size(listeFormations));
        return listeFormations;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void chargerParcours(final DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr) {
        final Formation formation = donneesFormationExportCDMfr.getFormation();
        try {
            // Chargement des parcours liés à la formation
            List<Parcours> parcoursDeFormation = formation.getFichesParcoursFiche();
            // Chargement des parcours liés aux parcours
            final List<Parcours> listeParcoursLies = new ArrayList<>();
            for (Parcours parcours : parcoursDeFormation) {
                listeParcoursLies.addAll(lireParcoursLies(parcours));
            }
            for (Parcours parcoursLie : listeParcoursLies) {
                donneesFormationExportCDMfr.ajouterParcours(parcoursLie);
            }
        } catch (Exception e) {
            LOGGER.error("Erreur lors de la récupération des parcours de la formation '{}", formation.getIdFormation(), e);
        }
    }

    /**
     * Lit la liste des parcours liés à un parcours.
     * @param parcours le parcours pour lequel on souhaite récupérer les parcours liés.
     * @return la liste des {@link ParcoursBean} liés
     */
    protected List<Parcours> lireParcoursLies(final Parcours parcours) {
        final List<Parcours> listeParcours = new ArrayList<>();
        try {
            final ResponseRechercheParcoursBean resultatRechercheParcours = serviceRechercheParcours.findTousParcoursForProgramme(parcours.getPersistenceBean());
            if (resultatRechercheParcours != null) {
                final Set<ParcoursBean> listeParcoursDuParcours = resultatRechercheParcours.getParcours();
                if (CollectionUtils.isNotEmpty(listeParcoursDuParcours)) {
                    for (ParcoursBean parcoursBean : listeParcoursDuParcours) {
                        listeParcours.add(new Parcours(parcoursBean));
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("Erreur lors de la récupération des parcours liés au parcours '{}", parcours.getIdParcours(), e);
        }
        return listeParcours;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void chargerElementsPedagogiques(final DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr) {
        // Chargement des éléments pédagogiques liés aux parcours
        final List<Ueup> listeUeupDeParcours = new ArrayList<>();
        final List<Parcours> listeParcours = donneesFormationExportCDMfr.getListeParcours();
        if (CollectionUtils.isNotEmpty(listeParcours)) {
            for (Parcours parcours : listeParcours) {
                try {
                    listeUeupDeParcours.addAll(parcours.getFicheUeups());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors de la récupération des éléments pédagogiques du parcours '{}", parcours.getIdParcours(), e);
                }
            }
        }
        // Chargement des éléments pédagogiques enfants d'éléments pédagogiques
        if (CollectionUtils.isNotEmpty(listeUeupDeParcours)) {
            for (Ueup ueup : listeUeupDeParcours) {
                donneesFormationExportCDMfr.ajouterElementPedagogique(ueup);
                try {
                    donneesFormationExportCDMfr.ajouterElementsPedagogiques(ueup.getFichesCodeUeupfils());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors de la récupération des éléments pédagogiques fils de l'élément pédagogique '{}", ueup.getIdUeup(), e);
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void chargerCours(final DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr) {
        // Chargement des cours rattachés aux éléments pédagogiques
        final List<Ueup> listeElementsPedagogiques = donneesFormationExportCDMfr.getListeElementsPedagogiques();
        if (CollectionUtils.isNotEmpty(listeElementsPedagogiques)) {
            for (Ueup ueup : listeElementsPedagogiques) {
                try {
                    final List<Cours> listeCoursUeup = ueup.getFichesCodeCours();
                    if (CollectionUtils.isNotEmpty(listeCoursUeup)) {
                        donneesFormationExportCDMfr.ajouterCours(listeCoursUeup);
                    }
                } catch (Exception e) {
                    LOGGER.error("Erreur lors de la récupération des cours de l'élément pédagogique '{}", ueup.getIdUeup(), e);
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void chargerStructures(DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr) {
        // Extraction des structures rattachées à la formation
        Formation formation = donneesFormationExportCDMfr.getFormation();
        try {
            donneesFormationExportCDMfr.ajouterStructure(formation.getInfosStructure());
            donneesFormationExportCDMfr.ajouterStructures(formation.getFicheCodeRattachementAutres());
        } catch (Exception e) {
            LOGGER.error("Erreur lors du chargement des structures de la formation id='{}'", formation.getIdFormation());
        }
        // Extraction des structures rattachées aux parcours
        final List<Parcours> listeParcours = donneesFormationExportCDMfr.getListeParcours();
        if (CollectionUtils.isNotEmpty(listeParcours)) {
            for (Parcours parcours : listeParcours) {
                try {
                    donneesFormationExportCDMfr.ajouterStructure(parcours.getInfosStructure());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des structures du parcours id='{}'", parcours.getIdParcours());
                }
            }
        }
        // Extraction des structures rattachées aux éléments pédagogiques
        final List<Ueup> listeElementsPedagogiques = donneesFormationExportCDMfr.getListeElementsPedagogiques();
        if (CollectionUtils.isNotEmpty(listeElementsPedagogiques)) {
            for (Ueup ueup : listeElementsPedagogiques) {
                try {
                    donneesFormationExportCDMfr.ajouterStructure(serviceStructure.getByCodeLanguage(ueup.getCodeRattachement(), ueup.getLangue()));
                    donneesFormationExportCDMfr.ajouterStructures(ueup.getFichesCodeRattachementsAutres());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des structures du l'élément pédagogiques id='{}'", ueup.getIdUeup());
                }
            }
        }
        // Extraction des structures rattachées aux cours
        final List<Cours> listeCours = donneesFormationExportCDMfr.getListeCours();
        if (CollectionUtils.isNotEmpty(listeCours)) {
            for (Cours cours : listeCours) {
                try {
                    donneesFormationExportCDMfr.ajouterStructure(serviceStructure.getByCodeLanguage(cours.getCodeRattachement(), cours.getLangue()));
                    donneesFormationExportCDMfr.ajouterStructures(cours.getFichesCodeRattachementsAutres());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des structures du cours id='{}'", cours.getIdCours());
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void chargerAnnuaires(DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr) {
        // Extraction des responsables de la formation
        Formation formation = donneesFormationExportCDMfr.getFormation();
        try {
            donneesFormationExportCDMfr.ajouterAnnuaires(formation.getFicheResponsablesFiche());
        } catch (Exception e) {
            LOGGER.error("Erreur lors du chargement des responsables de la formation id='{}'", formation.getIdFormation());
        }
        // Extraction des responsables des éléments pédagogiques
        final List<Ueup> listeElementsPedagogiques = donneesFormationExportCDMfr.getListeElementsPedagogiques();
        if (CollectionUtils.isNotEmpty(listeElementsPedagogiques)) {
            for (Ueup ueup : listeElementsPedagogiques) {
                try {
                    donneesFormationExportCDMfr.ajouterAnnuaires(ueup.getFichesCodeResponsable());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des responsables de l'élément pédagogique id='{}'", ueup.getIdUeup());
                }
            }
        }
        // Extraction des responsables des cours
        final List<Cours> listeCours = donneesFormationExportCDMfr.getListeCours();
        if (CollectionUtils.isNotEmpty(listeCours)) {
            for (Cours cours : listeCours) {
                try {
                    donneesFormationExportCDMfr.ajouterAnnuaires(cours.getFichesCodeResponsable());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des responsables du cours id='{}'", cours.getIdCours());
                }
            }
        }
        // Extraction des responsables des structures
        final List<StructureModele> listeStructures = donneesFormationExportCDMfr.getListeStructures();
        if (CollectionUtils.isNotEmpty(listeStructures)) {
            for (StructureModele structure : listeStructures) {
                try {
                    donneesFormationExportCDMfr.ajouterAnnuaires(structure.getFichesCodeResponsable());
                } catch (Exception e) {
                    LOGGER.error("Erreur lors du chargement des responsables de la strcuture id='{}'", structure.getIdFiche());
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String genererSectionsPrograms(DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr, ConfigurationExportCDMfr configurationExportCDMfr) {
        StringBuilder programs = new StringBuilder(super.genererSectionsPrograms(donneesFormationExportCDMfr, configurationExportCDMfr));
        // Génération des parcours non diplômants
        final List<Parcours> listeParcours = donneesFormationExportCDMfr.getListeParcours();
        if (CollectionUtils.isNotEmpty(listeParcours)) {
            for (Parcours parcours : listeParcours) {
                final Map<String, Object> modeleParcours = peuplerModeleSectionProgram(parcours, donneesFormationExportCDMfr, configurationExportCDMfr);
                programs.append(genererDepuisTemplate(ConstantesExportCDMfr.Templates.PROGRAM_PARCOURS, modeleParcours));
            }
        }
        return programs.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Map<String, Object> peuplerModeleSectionProgram(final DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr, final ConfigurationExportCDMfr configurationExportCDMfr) {
        Map<String, Object> modele = super.peuplerModeleSectionProgram(donneesFormationExportCDMfr, configurationExportCDMfr);
        Formation formation = donneesFormationExportCDMfr.getFormation();
        List<LabelDto> metierLabeList = serviceLabel.getFromCodes(formation.getMetiers(), formation.getLangue(), ";", MetierSerializer.TYPE_LIBELLE_METIER);
        modele.put(ConstantesExportCDMfr.ModeleTemplateProgramFormation.LISTE_METIERS, metierLabeList);
        modele.put(ConstantesExportCDMfr.ModeleTemplateProgramFormation.LISTE_RESPONSABLES, chargerListeResponsables(donneesFormationExportCDMfr.getFormation()));
        modele.put(ConstantesExportCDMfr.ModeleTemplateProgramFormation.MAP_CARTES, chargerMapCartes(donneesFormationExportCDMfr.getFormation()));
        return modele;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String genererSectionsSubProgram(DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr, ConfigurationExportCDMfr configurationExportCDMfr) {
        StringBuilder subPrograms = new StringBuilder();
        final List<Parcours> listeParcours = donneesFormationExportCDMfr.getListeParcours();
        if (CollectionUtils.isNotEmpty(listeParcours)) {
            for (Parcours parcours : listeParcours) {
                if (parcours.isDiplomantEtape()) {
                    final Map<String, Object> modele = peuplerModeleSectionSubProgram(parcours, donneesFormationExportCDMfr, configurationExportCDMfr);
                    subPrograms.append(genererDepuisTemplate(ConstantesExportCDMfr.Templates.SUB_PROGRAM, modele));
                }
            }
        }
        return subPrograms.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Map<String, Object> peuplerModeleSectionSubProgram(final Parcours parcoursDiplomant, final DonneesFormationExportCDMfr<Formation, Parcours, Ueup, Cours, StructureModele, AnnuaireModele> donneesFormationExportCDMfr, final ConfigurationExportCDMfr configurationExportCDMfr) {
        Map<String, Object> modele = super.peuplerModeleSectionSubProgram(parcoursDiplomant, donneesFormationExportCDMfr, configurationExportCDMfr);
        // Récupération de la liste des parcours liés au parcours diplômant
        final List<Parcours> listeParcours = lireParcoursLies(parcoursDiplomant);
        // Classement des parcours par numéro du premier semestre
        Map<String, TreeSet<Parcours>> mapParcoursParNumeroSemestre = new HashMap<>();
        Comparator<Parcours> parcoursComparator = (Parcours parcours1, Parcours parcours2) -> new CompareToBuilder().append(parcours1.getIntituleLong(), parcours2.getIntituleLong()).toComparison();
        for (Parcours parcours : listeParcours) {
            final String numeroSemestre = String.valueOf(parcours.getNumeroPremierSemestre());
            TreeSet<Parcours> listeParcoursDuSemestre = mapParcoursParNumeroSemestre.get(numeroSemestre);
            if (listeParcoursDuSemestre == null) {
                listeParcoursDuSemestre = new TreeSet<>(parcoursComparator);
                mapParcoursParNumeroSemestre.put(numeroSemestre, listeParcoursDuSemestre);
            }
            listeParcoursDuSemestre.add(parcours);
        }
        modele.put(ConstantesExportCDMfr.ModeleTemplateSubProgram.MAP_PARCOURS, mapParcoursParNumeroSemestre);
        return modele;
    }

    /**
     * Charge la liste des responsables de la formation.
     * @param formation la formation
     * @return la liste des {@link ResponsableBean} associés à la formation
     */
    protected Collection<ResponsableBean> chargerListeResponsables(final Formation formation) {
        Collection<ResponsableBean> listeResponsables = new ArrayList<>();
        try {
            final Map<String, List<AnnuaireModele>> fichesAnnuaireParTypeResponsable = formation.getFichesAnnuaireDepuisReferenceParTypeResponsable();
            for (String codeTitreResponsable : fichesAnnuaireParTypeResponsable.keySet()) {
                // Chargement du libellé correspondant au titre
                LabelDto labelTitreRepsonsable = serviceLabel.getByTypeCodeLanguage("64", codeTitreResponsable, formation.getLangue());
                String titreResponsable = StringUtils.EMPTY;
                if (labelTitreRepsonsable != null) {
                    titreResponsable = labelTitreRepsonsable.getLibelle();
                }
                final List<AnnuaireModele> listeResponsable = fichesAnnuaireParTypeResponsable.get(codeTitreResponsable);
                if (CollectionUtils.isNotEmpty(listeResponsable)) {
                    for (AnnuaireModele annuaireModele : listeResponsable) {
                        listeResponsables.add(new ResponsableBean(annuaireModele, titreResponsable));
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("Erreur lors de la récupération des responsables de la formation '{}'", formation.getIdFormation(), e);
        }
        return listeResponsables;
    }

    /**
     * Charge la map des cartes de la formation classées par nom.
     * @param formation la formation
     * @return la map des {@link CardBean} associées à la formation
     */
    protected MultiValuedMap<String, CardBean> chargerMapCartes(final Formation formation) {
        MultiValuedMap<String, CardBean> mapCartes = new ArrayListValuedHashMap<>();
        // Chargement des cartes
        final Collection<CardBean> listeCartes = serviceCard.getCards(formation).values();
        if (CollectionUtils.isNotEmpty(listeCartes)) {
            // Constitution d'une map avec le nom de la carte comme clé
            for (CardBean carte : listeCartes) {
                mapCartes.put(StringUtils.replace(carte.getClass().getSimpleName(), "CardBean", StringUtils.EMPTY), carte);
            }
        }
        return mapCartes;
    }

    public void setServiceFormation(final ServiceFormation serviceFormation) {
        this.serviceFormation = serviceFormation;
    }

    public void setServiceStructure(final ServiceStructure serviceStructure) {
        this.serviceStructure = serviceStructure;
    }

    public void setServiceLabel(final ServiceLabel serviceLabel) {
        this.serviceLabel = serviceLabel;
    }

    public void setServiceRechercheParcours(final ServiceRechercheParcours serviceRechercheParcours) {
        this.serviceRechercheParcours = serviceRechercheParcours;
    }

    public void setServiceCard(final ServiceCard serviceCard) {
        this.serviceCard = serviceCard;
    }
}